=== modified file '.bzrignore'
--- .bzrignore	2005-07-11 05:46:55 +0000
+++ .bzrignore	2005-08-26 02:31:37 +0000
@@ -1,5 +1,6 @@
 ## *.diff
 ./doc/*.html
+./tutorial.html
 *.py[oc]
 *~
 .arch-ids
@@ -14,3 +15,6 @@
 api
 test.log
 ./tmp
+build
+./ID
+./tags

=== added file 'HACKING'
--- HACKING	1970-01-01 00:00:00 +0000
+++ HACKING	2005-08-25 12:46:42 +0000
@@ -0,0 +1,99 @@
+============================
+guidelines for modifying bzr
+============================
+
+* New functionality should have test cases.  Preferably write the
+  test before writing the code.
+
+  In general, you can test at either the command-line level or the
+  internal API level.  Choose whichever is appropriate: if adding a
+  new command, or a new command option, then call through run_bzr().
+  It is not necessary to do both.
+
+* Before fixing a bug, write a test case so that it does not regress.
+
+* Exceptions should be defined inside bzrlib.errors, so that we can
+  see the whole tree at a glance.
+
+* Imports should be done at the top-level of the file, unless there is
+  a strong reason to have them lazily loaded when a particular
+  function runs.  Import statements have a cost, so try to make sure
+  they don't run inside hot functions.
+
+* Please write PEP-8__ compliant code.  
+
+  One often-missed requirement is that the first line of docstrings
+  should be a self-contained one-sentence summary.
+
+__ http://www.python.org/peps/pep-0008.html
+
+* Module names should always be given fully-qualified,
+  i.e. ``bzrlib.hashcache`` not just ``hashcache``.
+
+
+
+Documentation
+=============
+
+If you change the behaviour of a command, please update its docstring
+in bzrlib/commands.py.  This is displayed by the 'bzr help' command.
+
+If you make a user-visible change, please add a note to the NEWS file.
+The description should be written to make sense to someone who's just
+a user of bzr, not a developer: new functions or classes shouldn't be
+mentioned, but new commands, changes in behaviour or fixed nontrivial
+bugs should be listed.  See the existing entries for an idea of what
+should be done.
+
+
+
+Writing output
+==============
+
+(The strategy described here is what we want to get to, but it's not
+consistently followed in the code at the moment.)
+
+bzrlib is intended to be a generically reusable library.  It shouldn't
+write messages to stdout or stderr, because some programs that use it
+might want to display that information through a GUI or some other
+mechanism.
+
+We can distinguish two types of output from the library:
+
+ 1. Structured data representing the progress or result of an
+    operation.  For example, for a commit command this will be a list
+    of the modified files and the finally committed revision number
+    and id.
+
+    These should be exposed either through the return code or by calls
+    to a callback parameter.
+
+    A special case of this is progress indicators for long-lived
+    operations, where the caller should pass a ProgressBar object.
+
+ 2. Unstructured log/debug messages, mostly for the benefit of the
+    developers or users trying to debug problems.  This should always
+    be sent through ``bzrlib.trace`` and Python ``logging``, so that
+    it can be redirected by the client.
+
+The distinction between the two is a bit subjective, but in general if
+there is any chance that a library would want to see something as
+structured data, we should make it so.
+
+The policy about how output is presented in the text-mode client
+should be only in the command-line tool.
+
+Writing tests
+=============
+In general tests should be placed in a file named test_FOO.py where 
+FOO is the logical thing under test. That file should be placed in the
+tests subdirectory under the package being tested.
+
+For example, tests for merge3 in bzrlib belong in bzrlib/tests/test_merge3.py.
+
+Running tests
+=============
+Currently, bzr selftest is used to invoke tests.
+You can provide a pattern argument to run a subset. For example, 
+to run just the whitebox tests, run bzr selftest --pattern .*whitebox.*
+

=== added file 'Makefile'
--- Makefile	1970-01-01 00:00:00 +0000
+++ Makefile	2005-09-01 06:32:52 +0000
@@ -0,0 +1,13 @@
+all:
+	chmod u+x ./bzr
+
+check:
+	./bzr selftest
+
+.PHONY: all
+
+
+# build emacs cross-reference
+tag_files=./bzr ./bzrlib/*py ./bzrlib/selftest/*.py
+TAGS: $(tag_files)
+	ctags-exuberant -e $(tag_files)

=== modified file 'NEWS'
--- NEWS	2005-07-23 13:53:01 +0000
+++ NEWS	2005-09-12 01:29:49 +0000
@@ -1,4 +1,82 @@
-DEVELOPMENT HEAD
+development head
+
+  IMPROVEMENTS:
+
+    * Adding a file whose parent directory is not versioned will
+      implicitly add the parent, and so on up to the root. This means
+      you should never need to explictly add a directory, they'll just
+      get added when you add a file in the directory.  Contributed by
+      Michael Ellerman.
+
+
+  INTERNALS:
+
+    * Refactor xml packing/unpacking.
+
+
+  BUG FIXES: 
+
+    * Fixed 'bzr mv' by Ollie Rutherfurd.
+
+    * Fixed strange error when trying to access a nonexistent http
+      branch.
+
+
+  PORTABILITY:
+
+    * Various Windows fixes from Ollie Rutherfurd.
+
+
+bzr-0.0.7 2005-09-02
+
+  NEW FEATURES:
+
+    * ``bzr shell-complete`` command contributed by Clint Adams to
+      help with intelligent shell completion.
+
+    * New expert command ``bzr find-merge-base`` for debugging merges.
+
+
+  ENHANCEMENTS:
+
+    * Much better merge support.
+
+    * merge3 conflicts are now reported with markers like '<<<<<<<'
+      (seven characters) which is the same as CVS and pleases things
+      like emacs smerge.
+
+
+  BUG FIXES:
+
+    * ``bzr upgrade`` no longer fails when trying to fix trees that
+      mention revisions that are not present.
+
+    * Fixed bugs in listing plugins from ``bzr plugins``.
+
+    * Fix case of $EDITOR containing options for the editor.
+
+    * Fix log -r refusing to show the last revision.
+      (Patch from Goffredo Baroncelli.)
+
+
+  CHANGES:
+
+    * ``bzr log --show-ids`` shows the revision ids of all parents.
+
+    * Externally provided commands on your $BZRPATH no longer need
+      to recognize --bzr-usage to work properly, and can just handle
+      --help themselves.
+
+
+  LIBRARY:
+
+    * Changed trace messages to go through the standard logging
+      framework, so that they can more easily be redirected by
+      libraries.
+
+
+
+bzr-0.0.6 2005-08-18
 
   NEW FEATURES:
 
@@ -14,13 +92,36 @@
       FOO.
       
     * ``bzr add`` with no arguments adds everything under the current directory.
-    
+
+    * ``bzr mv`` does move or rename depending on its arguments, like
+      the Unix command.
+
+    * ``bzr missing`` command shows a summary of the differences
+      between two trees.  (Merged from John Arbash-Meinel.)
+
+    * An email address for commits to a particular tree can be
+      specified by putting it into .bzr/email within a branch.  (Based
+      on a patch from Heikki Paajanen.)
+
+
   ENHANCEMENTS:
 
     * Faster working tree operations.
 
+
   CHANGES:
 
+    * 3rd-party modules shipped with bzr are copied within the bzrlib
+      python package, so that they can be installed by the setup
+      script without clashing with anything already existing on the
+      system.  (Contributed by Gustavo Niemeyer.)
+
+    * Moved plugins directory to bzrlib/, so that there's a standard
+      plugin directory which is not only installed with bzr itself but
+      is also available when using bzr from the development tree.
+      BZR_PLUGIN_PATH and DEFAULT_PLUGIN_PATH are then added to the
+      standard plugins directory.
+
     * When exporting to a tarball with ``bzr export --format tgz``, put 
       everything under a top directory rather than dumping it into the
       current directory.   This can be overridden with the ``--root`` 
@@ -39,6 +140,22 @@
     * By default the commit command refuses to record a revision with
       no changes unless the ``--unchanged`` option is given.
 
+    * The ``--no-plugins``, ``--profile`` and ``--builtin`` command
+      line options must come before the command name because they 
+      affect what commands are available; all other options must come 
+      after the command name because their interpretation depends on
+      it.
+
+    * ``branch`` and ``clone`` added as aliases for ``branch``.
+
+    * Default log format is back to the long format; the compact one
+      is available with ``--short``.
+      
+      
+  BUG FIXES:
+  
+    * Fix bugs in committing only selected files or within a subdirectory.
+
 
 bzr-0.0.5  2005-06-15
   

=== modified file 'TODO'
--- TODO	2005-07-22 23:43:26 +0000
+++ TODO	2005-09-01 12:34:34 +0000
@@ -17,7 +17,18 @@
 Small things
 ------------
 
-* is_within_any is very very dodgy
+* ``bzr status`` should show some description of patches that are merged but 
+  not yet committed.  Preferably in a very compact format so that they
+  just fit on a single line; this could also be used for say ``log
+  --tiny``::
+
+  2005-11-05      mbp  Flowers for stephane
+
+* print a message at the end of running the tests telling them that the
+  test log and output exists but can be removed
+
+* tests for running the commit editor, and fix problem of not passing in 
+  multiple arguments
 
 * Merging add of a new file clashing with an existing file doesn't
   work; add gets an error that it's already versioned and the merge
@@ -68,7 +79,9 @@
 * ``bzr status DIR`` should give status on all files under that
   directory.
 
-* ``bzr log DIR`` should give changes to any files within DIR.
+* ``bzr log DIR`` should give changes to any files within DIR; at the
+  moment it only lists things which modify the specific named file
+  (and not its contents)
 
 * ``bzr inventory -r REV`` and perhaps unify this with ``bzr ls``,
   giving options to display ids, types, etc.
@@ -156,13 +169,52 @@
 * Reproducible performance benchmark to measure whether performance is 
   getting better or worse.
 
+* ``bzr log -m foo`` should perhaps error if nothing matches?
+
+* ``bzr diff -r 30 -r 40 foo.c`` or ``bzr diff -r30..40 foo.c``
+
+  If diffing between two branches then we probably want two -r
+  options, since the revisions don't form a range that can be
+  evaluated on either one.
+
+* bzr diff shouldn't diff binary files
+
+* setup.py install when run from a bzr tree should freeze the tree
+  revision-id into the installed bzr.
+
+* bzr script should trap ImportError and perhaps give a better error
+  message?
+
+* revert after a merge should possibly remove all the BASE/THIS/OTHER
+  files to get you back to where you were.
+
+* files that are added and then deleted are still reported as added
+
+* stores should raise KeyError, not IndexError
+
+* merging from a remote branch seems to sometimes raise errors not
+  present locally
+
+* should be possible to give a related branch when pulling from a
+  remote branch to make things faster
+
+* sometimes gives "conflicting add" even when the contents are in fact
+  the same???
+
+* BZRDIR should be in branch.py not __init__.py.
 
 Medium things
 -------------
 
-* Merge revert patch.
-
-* ``bzr mv`` that does either rename or move as in Unix.
+* ~/.bzr.log is not written anymore for some reason.
+
+* merge should add all revision and inventory XML to the local store.
+
+* check should give a warning for revisions that are named in the
+  chain but not actually present in the store.
+
+* remove anything outside of the branch implementation that directly
+  accesses the stores.
 
 * More efficient diff of only selected files.  We should be able to
   just get the id for the selected files, look up their location and
@@ -182,6 +234,8 @@
 
 * More test cases.
 
+  - ``missing`` command
+
   - Selected-file commit
 
   - Impossible selected-file commit: adding things in non-versioned
@@ -201,13 +255,9 @@
 
   Given this we might be able to import patches at 1/second or better.
 
-* Get branch over http.
-
-* Pull pure updates over http.
-
 * revfile compression.
 
-* Split inventory into per-directory files.
+* Split inventory into per-directory files?
 
 * Fix ignore file parsing:
 
@@ -328,9 +378,9 @@
 * Function to list a directory, saying in which revision each file was
   last modified.  Useful for web and gui interfaces, and slow to
   compute one file at a time.
-
-* unittest is standard, but the results are kind of ugly; would be
-  nice to make it cleaner.
+  
+  This will be done when we track file texts by referring to the
+  version that created them. 
 
 * Check locking is correct during merge-related operations.
 
@@ -345,6 +395,33 @@
 
 * Track all merged-in revisions in a versioned add-only metafile.
 
+* ``pull --clobber`` should discard any local changes not present
+  remotely.  Not generally what you want, but possibly useful when
+  you're just mirroring another branch and want to keep tracking it
+  even when they e.g. uncommit or make similar non-forward movements.
+  Also for push I suppose.  Clobber may not be the best name, maybe
+  ``--destroy``?
+
+* ``uncommit`` command that removes a revision from the end of the
+  revision-history; just doing this is enough to remove the commit,
+  and a new commit will automatically be made against the
+  predecessor.  This can be repeated.
+
+  It only makes sense to delete from the tail of history, not from the
+  end.
+
+  The revision, its inventory and texts remain floating in the store.
+  We should perhaps add the revision to a list of removed-commits, so
+  that it can be restored or at least accounted for when checking
+  consistency.  This file would not be versioned, and probably should
+  not propagate when branched.
+
+  If we track merged revisions then we need to update this list too.
+  If the list is stored in a weave it's easy (implicit): the version
+  of the list can remain but it won't be referenced anymore.  It's
+  probably best to just store this list in a weave in the first place
+  and be done.
+
 
 Large things
 ------------

=== modified file 'bzr'
--- bzr	2005-07-26 14:06:11 +0000
+++ bzr	2005-08-25 07:46:11 +0000
@@ -42,7 +42,10 @@
     sys.exit(1)
 os.unsetenv(REINVOKE)
 
-import bzrlib, bzrlib.commands
+import bzrlib
+import bzrlib.commands
+import bzrlib.trace
 
 if __name__ == '__main__':
+    bzrlib.trace.enable_default_logging()
     sys.exit(bzrlib.commands.main(sys.argv))

=== modified file 'bzrlib/__init__.py'
--- bzrlib/__init__.py	2005-07-23 13:52:38 +0000
+++ bzrlib/__init__.py	2005-09-05 04:07:23 +0000
@@ -16,10 +16,6 @@
 
 """bzr library"""
 
-# TODO: Do less imports here
-from branch import Branch, ScratchBranch, find_branch
-from errors import BzrError
-
 BZRDIR = ".bzr"
 
 DEFAULT_IGNORE = ['.bzr.log',
@@ -42,11 +38,14 @@
 
 __copyright__ = "Copyright 2005 Canonical Development Ltd."
 __author__ = "Martin Pool <mbp@canonical.com>"
-__version__ = '0.0.5'
+__version__ = '0.1pre'
 
 
 def get_bzr_revision():
     """If bzr is run from a branch, return (revno,revid) or None"""
+    import bzrlib.errors
+    from bzrlib.branch import Branch
+    
     try:
         branch = Branch(__path__[0])
         rh = branch.revision_history()
@@ -54,6 +53,9 @@
             return len(rh), rh[-1]
         else:
             return None
-    except BzrError:
+    except bzrlib.errors.BzrError:
         return None
     
+def test_suite():
+    import selftest
+    return selftest.test_suite()

=== modified file 'bzrlib/add.py'
--- bzrlib/add.py	2005-07-23 14:06:37 +0000
+++ bzrlib/add.py	2005-09-12 01:29:49 +0000
@@ -14,7 +14,11 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-from trace import mutter, note
+from bzrlib.trace import mutter, note, warning
+from bzrlib.errors import NotBranchError
+from bzrlib.branch import Branch
+from bzrlib.osutils import quotefn
+from os.path import dirname
 
 def glob_expand_for_win32(file_list):
     import glob
@@ -31,35 +35,64 @@
             expanded_file_list += glob_files
     return expanded_file_list
 
-def smart_add(file_list, verbose=True, recurse=True):
-    """Add files to version, optionally recursing into directories.
-
-    This is designed more towards DWIM for humans than API simplicity.
-    For the specific behaviour see the help for cmd_add().
-    """
-    import os
+
+def add_reporter_null(path, kind, entry):
+    """Absorb add reports and do nothing."""
+    pass
+
+def add_reporter_print(path, kind, entry):
+    """Print a line to stdout for each file that's added."""
+    print "added", quotefn(path)
+    
+def _prepare_file_list(file_list):
+    """Prepare a file list for use by smart_add_*."""
     import sys
-    from bzrlib.osutils import quotefn, kind_marker
-    from bzrlib.errors import BadFileKindError, ForbiddenFileError
-    import bzrlib.branch
-    import bzrlib.osutils
-
     if sys.platform == 'win32':
         file_list = glob_expand_for_win32(file_list)
-        
     if not file_list:
         file_list = ['.']
-    
+    file_list = list(file_list)
+    return file_list
+
+
+def smart_add(file_list, recurse=True, reporter=add_reporter_null):
+    """Add files to version, optionally recursing into directories.
+
+    This is designed more towards DWIM for humans than API simplicity.
+    For the specific behaviour see the help for cmd_add().
+
+    Returns the number of files added.
+    """
+    file_list = _prepare_file_list(file_list)
+    b = Branch(file_list[0], find_root=True)
+    return smart_add_branch(b, file_list, recurse, reporter)
+
+        
+def smart_add_branch(branch, file_list, recurse=True, reporter=add_reporter_null):
+    """Add files to version, optionally recursing into directories.
+
+    This is designed more towards DWIM for humans than API simplicity.
+    For the specific behaviour see the help for cmd_add().
+
+    This yields a sequence of (path, kind, file_id) for added files.
+
+    Returns the number of files added.
+    """
+    import os
+    from bzrlib.errors import BadFileKindError, ForbiddenFileError
+    import bzrlib.branch
+
+    assert isinstance(recurse, bool)
+
+    file_list = _prepare_file_list(file_list)
     user_list = file_list[:]
-    assert not isinstance(file_list, basestring)
-    b = bzrlib.branch.Branch(file_list[0], find_root=True)
-    inv = b.read_working_inventory()
-    tree = b.working_tree()
+    inv = branch.read_working_inventory()
+    tree = branch.working_tree()
     count = 0
 
     for f in file_list:
-        rf = b.relpath(f)
-        af = b.abspath(rf)
+        rf = branch.relpath(f)
+        af = branch.abspath(rf)
 
         kind = bzrlib.osutils.file_kind(af)
 
@@ -67,7 +100,7 @@
             if f in user_list:
                 raise BadFileKindError("cannot add %s of type %s" % (f, kind))
             else:
-                print "skipping %s (can't add file of kind '%s')" % (f, kind)
+                warning("skipping %s (can't add file of kind '%s')", f, kind)
                 continue
 
         mutter("smart add of %r, abs=%r" % (f, af))
@@ -77,19 +110,26 @@
             
         versioned = (inv.path2id(rf) != None)
 
+        if kind == 'directory':
+            try:
+                sub_branch = Branch(af, find_root=False)
+                sub_tree = True
+            except NotBranchError:
+                sub_tree = False
+        else:
+            sub_tree = False
+
         if rf == '':
             mutter("branch root doesn't need to be added")
+            sub_tree = False
         elif versioned:
             mutter("%r is already versioned" % f)
+        elif sub_tree:
+            mutter("%r is a bzr tree" %f)
         else:
-            file_id = bzrlib.branch.gen_file_id(rf)
-            inv.add_path(rf, kind=kind, file_id=file_id)
-            mutter("added %r kind %r file_id={%s}" % (rf, kind, file_id))
-            count += 1 
-
-            print 'added', quotefn(f)
-
-        if kind == 'directory' and recurse:
+            count += __add_one(branch, inv, rf, kind, reporter)
+
+        if kind == 'directory' and recurse and not sub_tree:
             for subf in os.listdir(af):
                 subp = os.path.join(rf, subf)
                 if subf == bzrlib.BZRDIR:
@@ -98,12 +138,30 @@
                     mutter("skip ignored sub-file %r" % subp)
                 else:
                     mutter("queue to add sub-file %r" % subp)
-                    file_list.append(b.abspath(subp))
-
+                    file_list.append(branch.abspath(subp))
+
+
+    mutter('added %d entries', count)
+    
     if count > 0:
-        if verbose:
-            note('added %d' % count)
-        b._write_inventory(inv)
-    else:
-        print "nothing new to add"
-        # should this return 1 to the shell?
+        branch._write_inventory(inv)
+
+    return count
+
+def __add_one(branch, inv, path, kind, reporter):
+    """Add a file or directory, automatically add unversioned parents."""
+
+    # Nothing to do if path is already versioned.
+    # This is safe from infinite recursion because the branch root is
+    # always versioned.
+    if inv.path2id(path) != None:
+        return 0
+
+    # add parent
+    count = __add_one(branch, inv, dirname(path), 'directory', reporter)
+
+    entry = inv.add_path(path, kind=kind)
+    mutter("added %r kind %r file_id={%s}" % (path, kind, entry.file_id))
+    reporter(path, kind, entry)
+
+    return count + 1

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2005-07-22 22:31:40 +0000
+++ bzrlib/branch.py	2005-09-12 08:21:24 +0000
@@ -15,18 +15,37 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-import sys, os
+import sys
+import os
 
 import bzrlib
 from bzrlib.trace import mutter, note
-from bzrlib.osutils import isdir, quotefn, compact_date, rand_bytes, splitpath, \
+from bzrlib.osutils import isdir, quotefn, compact_date, rand_bytes, \
+     splitpath, \
      sha_file, appendpath, file_kind
-from bzrlib.errors import BzrError
+
+from bzrlib.errors import BzrError, InvalidRevisionNumber, InvalidRevisionId
+import bzrlib.errors
+from bzrlib.textui import show_status
+from bzrlib.revision import Revision
+from bzrlib.delta import compare_trees
+from bzrlib.tree import EmptyTree, RevisionTree
+import bzrlib.xml
+import bzrlib.ui
+
+
 
 BZR_BRANCH_FORMAT = "Bazaar-NG branch, format 0.0.4\n"
 ## TODO: Maybe include checks for common corruption of newlines, etc?
 
 
+# TODO: Some operations like log might retrieve the same revisions
+# repeatedly to calculate deltas.  We could perhaps have a weakref
+# cache in memory to make this faster.
+
+# TODO: please move the revision-string syntax stuff out of the branch
+# object; it's clutter
+
 
 def find_branch(f, **args):
     if f and (f.startswith('http://') or f.startswith('https://')):
@@ -89,7 +108,8 @@
     It is not necessary that f exists.
 
     Basically we keep looking up until we find the control directory or
-    run into the root."""
+    run into the root.  If there isn't one, raises NotBranchError.
+    """
     if f == None:
         f = os.getcwd()
     elif hasattr(os.path, 'realpath'):
@@ -108,9 +128,12 @@
         head, tail = os.path.split(f)
         if head == f:
             # reached the root, whatever that may be
-            raise BzrError('%r is not in a branch' % orig_f)
+            raise bzrlib.errors.NotBranchError('%s is not in a branch' % orig_f)
         f = head
-    
+
+
+
+# XXX: move into bzrlib.errors; subclass BzrError    
 class DivergedBranches(Exception):
     def __init__(self, branch1, branch2):
         self.branch1 = branch1
@@ -118,14 +141,6 @@
         Exception.__init__(self, "These branches have diverged.")
 
 
-class NoSuchRevision(BzrError):
-    def __init__(self, branch, revision):
-        self.branch = branch
-        self.revision = revision
-        msg = "Branch %s has no revision %d" % (branch, revision)
-        BzrError.__init__(self, msg)
-
-
 ######################################################################
 # branch objects
 
@@ -204,7 +219,6 @@
             self._lock.unlock()
 
 
-
     def lock_write(self):
         if self._lock_mode:
             if self._lock_mode != 'w':
@@ -220,7 +234,6 @@
             self._lock_count = 1
 
 
-
     def lock_read(self):
         if self._lock_mode:
             assert self._lock_mode in ('r', 'w'), \
@@ -233,8 +246,6 @@
             self._lock_mode = 'r'
             self._lock_count = 1
                         
-
-            
     def unlock(self):
         if not self._lock_mode:
             from errors import LockError
@@ -247,19 +258,16 @@
             self._lock = None
             self._lock_mode = self._lock_count = None
 
-
     def abspath(self, name):
         """Return absolute filename for something in the branch"""
         return os.path.join(self.base, name)
 
-
     def relpath(self, path):
         """Return path relative to this branch of something inside it.
 
         Raises an error if path is not in this branch."""
         return _relpath(self.base, path)
 
-
     def controlfilename(self, file_or_path):
         """Return location relative to branch."""
         if isinstance(file_or_path, basestring):
@@ -292,11 +300,8 @@
         else:
             raise BzrError("invalid controlfile mode %r" % mode)
 
-
-
     def _make_control(self):
         from bzrlib.inventory import Inventory
-        from bzrlib.xml import pack_xml
         
         os.mkdir(self.controlfilename([]))
         self.controlfile('README', 'w').write(
@@ -312,7 +317,11 @@
             self.controlfile(f, 'w').write('')
         mutter('created control directory in ' + self.base)
 
-        pack_xml(Inventory(gen_root_id()), self.controlfile('inventory','w'))
+        # if we want per-tree root ids then this is the place to set
+        # them; they're not needed for now and so ommitted for
+        # simplicity.
+        f = self.controlfile('inventory','w')
+        bzrlib.xml.serializer_v4.write_inventory(Inventory(), f)
 
 
     def _check_format(self):
@@ -327,7 +336,7 @@
         # on Windows from Linux and so on.  I think it might be better
         # to always make all internal files in unix format.
         fmt = self.controlfile('branch-format', 'r').read()
-        fmt.replace('\r\n', '')
+        fmt = fmt.replace('\r\n', '\n')
         if fmt != BZR_BRANCH_FORMAT:
             raise BzrError('sorry, branch format %r not supported' % fmt,
                            ['use a different bzr version',
@@ -353,18 +362,12 @@
     def read_working_inventory(self):
         """Read the working inventory."""
         from bzrlib.inventory import Inventory
-        from bzrlib.xml import unpack_xml
-        from time import time
-        before = time()
         self.lock_read()
         try:
             # ElementTree does its own conversion from UTF-8, so open in
             # binary.
-            inv = unpack_xml(Inventory,
-                             self.controlfile('inventory', 'rb'))
-            mutter("loaded inventory of %d items in %f"
-                   % (len(inv), time() - before))
-            return inv
+            f = self.controlfile('inventory', 'rb')
+            return bzrlib.xml.serializer_v4.read_inventory(f)
         finally:
             self.unlock()
             
@@ -376,13 +379,12 @@
         will be committed to the next revision.
         """
         from bzrlib.atomicfile import AtomicFile
-        from bzrlib.xml import pack_xml
         
         self.lock_write()
         try:
             f = AtomicFile(self.controlfilename('inventory'), 'wb')
             try:
-                pack_xml(inv, f)
+                bzrlib.xml.serializer_v4.write_inventory(inv, f)
                 f.commit()
             finally:
                 f.close()
@@ -396,10 +398,11 @@
                          """Inventory for the working copy.""")
 
 
-    def add(self, files, verbose=False, ids=None):
+    def add(self, files, ids=None):
         """Make files versioned.
 
-        Note that the command line normally calls smart_add instead.
+        Note that the command line normally calls smart_add instead,
+        which can automatically recurse.
 
         This puts the files in the Added state, so that they will be
         recorded by the next commit.
@@ -415,14 +418,8 @@
         TODO: Perhaps have an option to add the ids even if the files do
               not (yet) exist.
 
-        TODO: Perhaps return the ids of the files?  But then again it
-              is easy to retrieve them if they're needed.
-
-        TODO: Adding a directory should optionally recurse down and
-              add all non-ignored children.  Perhaps do that in a
-              higher-level method.
+        TODO: Perhaps yield the ids and paths as they're added.
         """
-        from bzrlib.textui import show_status
         # TODO: Re-adding a file that is removed in the working copy
         # should probably put it back with the previous ID.
         if isinstance(files, basestring):
@@ -463,9 +460,6 @@
                     file_id = gen_file_id(f)
                 inv.add_path(f, kind=kind, file_id=file_id)
 
-                if verbose:
-                    print 'added', quotefn(f)
-
                 mutter("add file %s file_id:{%s} kind=%r" % (f, file_id, kind))
 
             self._write_inventory(inv)
@@ -501,7 +495,6 @@
         is the opposite of add.  Removing it is consistent with most
         other tools.  Maybe an option.
         """
-        from bzrlib.textui import show_status
         ## TODO: Normalize names
         ## TODO: Remove nested loops; better scalability
         if isinstance(files, basestring):
@@ -582,21 +575,61 @@
             f.close()
 
 
+    def get_revision_xml_file(self, revision_id):
+        """Return XML file object for revision object."""
+        if not revision_id or not isinstance(revision_id, basestring):
+            raise InvalidRevisionId(revision_id)
+
+        self.lock_read()
+        try:
+            try:
+                return self.revision_store[revision_id]
+            except IndexError:
+                raise bzrlib.errors.NoSuchRevision(self, revision_id)
+        finally:
+            self.unlock()
+
+
+    #deprecated
+    get_revision_xml = get_revision_xml_file
+
+
     def get_revision(self, revision_id):
         """Return the Revision object for a named revision"""
-        from bzrlib.revision import Revision
-        from bzrlib.xml import unpack_xml
+        xml_file = self.get_revision_xml_file(revision_id)
 
-        self.lock_read()
         try:
-            if not revision_id or not isinstance(revision_id, basestring):
-                raise ValueError('invalid revision-id: %r' % revision_id)
-            r = unpack_xml(Revision, self.revision_store[revision_id])
-        finally:
-            self.unlock()
+            r = bzrlib.xml.serializer_v4.read_revision(xml_file)
+        except SyntaxError, e:
+            raise bzrlib.errors.BzrError('failed to unpack revision_xml',
+                                         [revision_id,
+                                          str(e)])
             
         assert r.revision_id == revision_id
         return r
+
+
+    def get_revision_delta(self, revno):
+        """Return the delta for one revision.
+
+        The delta is relative to its mainline predecessor, or the
+        empty tree for revision 1.
+        """
+        assert isinstance(revno, int)
+        rh = self.revision_history()
+        if not (1 <= revno <= len(rh)):
+            raise InvalidRevisionNumber(revno)
+
+        # revno is 1-based; list is 0-based
+
+        new_tree = self.revision_tree(rh[revno-1])
+        if revno == 1:
+            old_tree = EmptyTree()
+        else:
+            old_tree = self.revision_tree(rh[revno-2])
+
+        return compare_trees(old_tree, new_tree)
+
         
 
     def get_revision_sha1(self, revision_id):
@@ -607,7 +640,7 @@
         # the revision, (add signatures/remove signatures) and still
         # have all hash pointers stay consistent.
         # But for now, just hash the contents.
-        return sha_file(self.revision_store[revision_id])
+        return bzrlib.osutils.sha_file(self.get_revision_xml(revision_id))
 
 
     def get_inventory(self, inventory_id):
@@ -617,15 +650,22 @@
                parameter which can be either an integer revno or a
                string hash."""
         from bzrlib.inventory import Inventory
-        from bzrlib.xml import unpack_xml
-
-        return unpack_xml(Inventory, self.inventory_store[inventory_id])
+
+        f = self.get_inventory_xml_file(inventory_id)
+        return bzrlib.xml.serializer_v4.read_inventory(f)
+
+
+    def get_inventory_xml(self, inventory_id):
+        """Get inventory XML as a file object."""
+        return self.inventory_store[inventory_id]
+
+    get_inventory_xml_file = get_inventory_xml
             
 
     def get_inventory_sha1(self, inventory_id):
         """Return the sha1 hash of the inventory entry
         """
-        return sha_file(self.inventory_store[inventory_id])
+        return sha_file(self.get_inventory_xml(inventory_id))
 
 
     def get_revision_inventory(self, revision_id):
@@ -697,27 +737,6 @@
                 return r+1, my_history[r]
         return None, None
 
-    def enum_history(self, direction):
-        """Return (revno, revision_id) for history of branch.
-
-        direction
-            'forward' is from earliest to latest
-            'reverse' is from latest to earliest
-        """
-        rh = self.revision_history()
-        if direction == 'forward':
-            i = 1
-            for rid in rh:
-                yield i, rid
-                i += 1
-        elif direction == 'reverse':
-            i = len(rh)
-            while i > 0:
-                yield i, rh[i-1]
-                i -= 1
-        else:
-            raise ValueError('invalid history direction', direction)
-
 
     def revno(self):
         """Return current revision number for this branch.
@@ -738,7 +757,7 @@
             return None
 
 
-    def missing_revisions(self, other, stop_revision=None):
+    def missing_revisions(self, other, stop_revision=None, diverged_ok=False):
         """
         If self and other have not diverged, return a list of the revisions
         present in other, but missing from self.
@@ -777,55 +796,64 @@
         if stop_revision is None:
             stop_revision = other_len
         elif stop_revision > other_len:
-            raise NoSuchRevision(self, stop_revision)
+            raise bzrlib.errors.NoSuchRevision(self, stop_revision)
         
         return other_history[self_len:stop_revision]
 
 
     def update_revisions(self, other, stop_revision=None):
         """Pull in all new revisions from other branch.
-        
-        >>> from bzrlib.commit import commit
-        >>> bzrlib.trace.silent = True
-        >>> br1 = ScratchBranch(files=['foo', 'bar'])
-        >>> br1.add('foo')
-        >>> br1.add('bar')
-        >>> commit(br1, "lala!", rev_id="REVISION-ID-1", verbose=False)
-        >>> br2 = ScratchBranch()
-        >>> br2.update_revisions(br1)
-        Added 2 texts.
-        Added 1 inventories.
-        Added 1 revisions.
-        >>> br2.revision_history()
-        [u'REVISION-ID-1']
-        >>> br2.update_revisions(br1)
-        Added 0 texts.
-        Added 0 inventories.
-        Added 0 revisions.
-        >>> br1.text_store.total_size() == br2.text_store.total_size()
-        True
         """
-        from bzrlib.progress import ProgressBar
-
-        pb = ProgressBar()
-
+        from bzrlib.fetch import greedy_fetch
+        from bzrlib.revision import get_intervening_revisions
+
+        pb = bzrlib.ui.ui_factory.progress_bar()
         pb.update('comparing histories')
-        revision_ids = self.missing_revisions(other, stop_revision)
-
+
+        try:
+            revision_ids = self.missing_revisions(other, stop_revision)
+        except DivergedBranches, e:
+            try:
+                if stop_revision is None:
+                    end_revision = other.last_patch()
+                revision_ids = get_intervening_revisions(self.last_patch(), 
+                                                         end_revision, other)
+                assert self.last_patch() not in revision_ids
+            except bzrlib.errors.NotAncestor:
+                raise e
+
+        if len(revision_ids) > 0:
+            count = greedy_fetch(self, other, revision_ids[-1], pb)[0]
+        else:
+            count = 0
+        self.append_revision(*revision_ids)
+        ## note("Added %d revisions." % count)
+        pb.clear()
+
+    def install_revisions(self, other, revision_ids, pb):
         if hasattr(other.revision_store, "prefetch"):
             other.revision_store.prefetch(revision_ids)
         if hasattr(other.inventory_store, "prefetch"):
             inventory_ids = [other.get_revision(r).inventory_id
                              for r in revision_ids]
             other.inventory_store.prefetch(inventory_ids)
+
+        if pb is None:
+            pb = bzrlib.ui.ui_factory.progress_bar()
                 
         revisions = []
         needed_texts = set()
         i = 0
-        for rev_id in revision_ids:
-            i += 1
-            pb.update('fetching revision', i, len(revision_ids))
-            rev = other.get_revision(rev_id)
+
+        failures = set()
+        for i, rev_id in enumerate(revision_ids):
+            pb.update('fetching revision', i+1, len(revision_ids))
+            try:
+                rev = other.get_revision(rev_id)
+            except bzrlib.errors.NoSuchRevision:
+                failures.add(rev_id)
+                continue
+
             revisions.append(rev)
             inv = other.get_inventory(str(rev.inventory_id))
             for key, entry in inv.iter_entries():
@@ -836,20 +864,22 @@
 
         pb.clear()
                     
-        count = self.text_store.copy_multi(other.text_store, needed_texts)
-        print "Added %d texts." % count 
+        count, cp_fail = self.text_store.copy_multi(other.text_store, 
+                                                    needed_texts)
+        #print "Added %d texts." % count 
         inventory_ids = [ f.inventory_id for f in revisions ]
-        count = self.inventory_store.copy_multi(other.inventory_store, 
-                                                inventory_ids)
-        print "Added %d inventories." % count 
+        count, cp_fail = self.inventory_store.copy_multi(other.inventory_store, 
+                                                         inventory_ids)
+        #print "Added %d inventories." % count 
         revision_ids = [ f.revision_id for f in revisions]
-        count = self.revision_store.copy_multi(other.revision_store, 
-                                               revision_ids)
-        for revision_id in revision_ids:
-            self.append_revision(revision_id)
-        print "Added %d revisions." % count
-                    
-        
+
+        count, cp_fail = self.revision_store.copy_multi(other.revision_store, 
+                                                          revision_ids,
+                                                          permit_failure=True)
+        assert len(cp_fail) == 0 
+        return count, failures
+       
+
     def commit(self, *args, **kw):
         from bzrlib.commit import commit
         commit(self, *args, **kw)
@@ -857,9 +887,19 @@
 
     def lookup_revision(self, revision):
         """Return the revision identifier for a given revision information."""
-        revno, info = self.get_revision_info(revision)
+        revno, info = self._get_revision_info(revision)
         return info
 
+
+    def revision_id_to_revno(self, revision_id):
+        """Given a revision id, return its revno"""
+        history = self.revision_history()
+        try:
+            return history.index(revision_id) + 1
+        except ValueError:
+            raise bzrlib.errors.NoSuchRevision(self, revision_id)
+
+
     def get_revision_info(self, revision):
         """Return (revno, revision id) for revision identifier.
 
@@ -868,6 +908,34 @@
         revision can also be a string, in which case it is parsed for something like
             'date:' or 'revid:' etc.
         """
+        revno, rev_id = self._get_revision_info(revision)
+        if revno is None:
+            raise bzrlib.errors.NoSuchRevision(self, revision)
+        return revno, rev_id
+
+    def get_rev_id(self, revno, history=None):
+        """Find the revision id of the specified revno."""
+        if revno == 0:
+            return None
+        if history is None:
+            history = self.revision_history()
+        elif revno <= 0 or revno > len(history):
+            raise bzrlib.errors.NoSuchRevision(self, revno)
+        return history[revno - 1]
+
+    def _get_revision_info(self, revision):
+        """Return (revno, revision id) for revision specifier.
+
+        revision can be an integer, in which case it is assumed to be revno
+        (though this will translate negative values into positive ones)
+        revision can also be a string, in which case it is parsed for something
+        like 'date:' or 'revid:' etc.
+
+        A revid is always returned.  If it is None, the specifier referred to
+        the null revision.  If the revid does not occur in the revision
+        history, revno will be None.
+        """
+        
         if revision is None:
             return 0, None
         revno = None
@@ -877,40 +945,48 @@
             pass
         revs = self.revision_history()
         if isinstance(revision, int):
-            if revision == 0:
-                return 0, None
-            # Mabye we should do this first, but we don't need it if revision == 0
             if revision < 0:
                 revno = len(revs) + revision + 1
             else:
                 revno = revision
+            rev_id = self.get_rev_id(revno, revs)
         elif isinstance(revision, basestring):
             for prefix, func in Branch.REVISION_NAMESPACES.iteritems():
                 if revision.startswith(prefix):
-                    revno = func(self, revs, revision)
+                    result = func(self, revs, revision)
+                    if len(result) > 1:
+                        revno, rev_id = result
+                    else:
+                        revno = result[0]
+                        rev_id = self.get_rev_id(revno, revs)
                     break
             else:
-                raise BzrError('No namespace registered for string: %r' % revision)
+                raise BzrError('No namespace registered for string: %r' %
+                               revision)
+        else:
+            raise TypeError('Unhandled revision type %s' % revision)
 
-        if revno is None or revno <= 0 or revno > len(revs):
-            raise BzrError("no such revision %s" % revision)
-        return revno, revs[revno-1]
+        if revno is None:
+            if rev_id is None:
+                raise bzrlib.errors.NoSuchRevision(self, revision)
+        return revno, rev_id
 
     def _namespace_revno(self, revs, revision):
         """Lookup a revision by revision number"""
         assert revision.startswith('revno:')
         try:
-            return int(revision[6:])
+            return (int(revision[6:]),)
         except ValueError:
             return None
     REVISION_NAMESPACES['revno:'] = _namespace_revno
 
     def _namespace_revid(self, revs, revision):
         assert revision.startswith('revid:')
+        rev_id = revision[len('revid:'):]
         try:
-            return revs.index(revision[6:]) + 1
+            return revs.index(rev_id) + 1, rev_id
         except ValueError:
-            return None
+            return None, rev_id
     REVISION_NAMESPACES['revid:'] = _namespace_revid
 
     def _namespace_last(self, revs, revision):
@@ -918,11 +994,11 @@
         try:
             offset = int(revision[5:])
         except ValueError:
-            return None
+            return (None,)
         else:
             if offset <= 0:
                 raise BzrError('You must supply a positive value for --revision last:XXX')
-            return len(revs) - offset + 1
+            return (len(revs) - offset + 1,)
     REVISION_NAMESPACES['last:'] = _namespace_last
 
     def _namespace_tag(self, revs, revision):
@@ -1003,14 +1079,14 @@
                 # TODO: Handle timezone.
                 dt = datetime.datetime.fromtimestamp(r.timestamp)
                 if first >= dt and (last is None or dt >= last):
-                    return i+1
+                    return (i+1,)
         else:
             for i in range(len(revs)):
                 r = self.get_revision(revs[i])
                 # TODO: Handle timezone.
                 dt = datetime.datetime.fromtimestamp(r.timestamp)
                 if first <= dt and (last is None or dt <= last):
-                    return i+1
+                    return (i+1,)
     REVISION_NAMESPACES['date:'] = _namespace_date
 
     def revision_tree(self, revision_id):
@@ -1018,11 +1094,10 @@
 
         `revision_id` may be None for the null revision, in which case
         an `EmptyTree` is returned."""
-        from bzrlib.tree import EmptyTree, RevisionTree
         # TODO: refactor this to use an existing revision object
         # so we don't need to read it in twice.
         if revision_id == None:
-            return EmptyTree(self.get_root_id())
+            return EmptyTree()
         else:
             inv = self.get_revision_inventory(revision_id)
             return RevisionTree(self.text_store, inv)
@@ -1039,10 +1114,9 @@
 
         If there are no revisions yet, return an `EmptyTree`.
         """
-        from bzrlib.tree import EmptyTree, RevisionTree
         r = self.last_patch()
         if r == None:
-            return EmptyTree(self.get_root_id())
+            return EmptyTree()
         else:
             return RevisionTree(self.text_store, self.get_revision_inventory(r))
 
@@ -1083,8 +1157,6 @@
 
             inv.rename(file_id, to_dir_id, to_tail)
 
-            print "%s => %s" % (from_rel, to_rel)
-
             from_abs = self.abspath(from_rel)
             to_abs = self.abspath(to_rel)
             try:
@@ -1109,7 +1181,11 @@
 
         Note that to_name is only the last component of the new name;
         this doesn't change the directory.
+
+        This returns a list of (from_path, to_path) pairs for each
+        entry that is moved.
         """
+        result = []
         self.lock_write()
         try:
             ## TODO: Option to move IDs only
@@ -1150,7 +1226,7 @@
             for f in from_paths:
                 name_tail = splitpath(f)[-1]
                 dest_path = appendpath(to_name, name_tail)
-                print "%s => %s" % (f, dest_path)
+                result.append((f, dest_path))
                 inv.rename(inv.path2id(f), to_dir_id, name_tail)
                 try:
                     os.rename(self.abspath(f), self.abspath(dest_path))
@@ -1162,6 +1238,8 @@
         finally:
             self.unlock()
 
+        return result
+
 
     def revert(self, filenames, old_tree=None, backups=True):
         """Restore selected files to the versions from a previous tree.
@@ -1249,6 +1327,56 @@
             self.unlock()
 
 
+    def get_parent(self):
+        """Return the parent location of the branch.
+
+        This is the default location for push/pull/missing.  The usual
+        pattern is that the user can override it by specifying a
+        location.
+        """
+        import errno
+        _locs = ['parent', 'pull', 'x-pull']
+        for l in _locs:
+            try:
+                return self.controlfile(l, 'r').read().strip('\n')
+            except IOError, e:
+                if e.errno != errno.ENOENT:
+                    raise
+        return None
+
+
+    def set_parent(self, url):
+        # TODO: Maybe delete old location files?
+        from bzrlib.atomicfile import AtomicFile
+        self.lock_write()
+        try:
+            f = AtomicFile(self.controlfilename('parent'))
+            try:
+                f.write(url + '\n')
+                f.commit()
+            finally:
+                f.close()
+        finally:
+            self.unlock()
+
+    def check_revno(self, revno):
+        """\
+        Check whether a revno corresponds to any revision.
+        Zero (the NULL revision) is considered valid.
+        """
+        if revno != 0:
+            self.check_real_revno(revno)
+            
+    def check_real_revno(self, revno):
+        """\
+        Check whether a revno corresponds to a real revision.
+        Zero (the NULL revision) is considered invalid
+        """
+        if revno < 1 or revno > self.revno():
+            raise InvalidRevisionNumber(revno)
+        
+        
+
 
 class ScratchBranch(Branch):
     """Special test class: a branch that cleans up after itself.
@@ -1296,6 +1424,8 @@
         os.rmdir(base)
         copytree(self.base, base, symlinks=True)
         return ScratchBranch(base=base)
+
+
         
     def __del__(self):
         self.destroy()
@@ -1371,3 +1501,42 @@
     """Return a new tree-root file id."""
     return gen_file_id('TREE_ROOT')
 
+
+def pull_loc(branch):
+    # TODO: Should perhaps just make attribute be 'base' in
+    # RemoteBranch and Branch?
+    if hasattr(branch, "baseurl"):
+        return branch.baseurl
+    else:
+        return branch.base
+
+
+def copy_branch(branch_from, to_location, revision=None):
+    """Copy branch_from into the existing directory to_location.
+
+    revision
+        If not None, only revisions up to this point will be copied.
+        The head of the new branch will be that revision.
+
+    to_location
+        The name of a local directory that exists but is empty.
+    """
+    from bzrlib.merge import merge
+    from bzrlib.branch import Branch
+
+    assert isinstance(branch_from, Branch)
+    assert isinstance(to_location, basestring)
+    
+    br_to = Branch(to_location, init=True)
+    br_to.set_root_id(branch_from.get_root_id())
+    if revision is None:
+        revno = branch_from.revno()
+    else:
+        revno, rev_id = branch_from.get_revision_info(revision)
+    br_to.update_revisions(branch_from, stop_revision=revno)
+    merge((to_location, -1), (to_location, 0), this_dir=to_location,
+          check_clean=False, ignore_zero=True)
+    
+    from_location = pull_loc(branch_from)
+    br_to.set_parent(pull_loc(branch_from))
+

=== added file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	1970-01-01 00:00:00 +0000
+++ bzrlib/builtins.py	2005-09-12 01:29:49 +0000
@@ -0,0 +1,1329 @@
+# Copyright (C) 2004, 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+import sys
+import os
+
+import bzrlib
+import bzrlib.trace
+from bzrlib.trace import mutter, note, log_error, warning
+from bzrlib.errors import BzrError, BzrCheckError, BzrCommandError
+from bzrlib.branch import find_branch
+from bzrlib import BZRDIR
+from bzrlib.commands import Command
+
+
+class cmd_status(Command):
+    """Display status summary.
+
+    This reports on versioned and unknown files, reporting them
+    grouped by state.  Possible states are:
+
+    added
+        Versioned in the working copy but not in the previous revision.
+
+    removed
+        Versioned in the previous revision but removed or deleted
+        in the working copy.
+
+    renamed
+        Path of this file changed from the previous revision;
+        the text may also have changed.  This includes files whose
+        parent directory was renamed.
+
+    modified
+        Text has changed since the previous revision.
+
+    unchanged
+        Nothing about this file has changed since the previous revision.
+        Only shown with --all.
+
+    unknown
+        Not versioned and not matching an ignore pattern.
+
+    To see ignored files use 'bzr ignored'.  For details in the
+    changes to file texts, use 'bzr diff'.
+
+    If no arguments are specified, the status of the entire working
+    directory is shown.  Otherwise, only the status of the specified
+    files or directories is reported.  If a directory is given, status
+    is reported for everything inside that directory.
+    """
+
+    takes_args = ['file*']
+    takes_options = ['all', 'show-ids']
+    aliases = ['st', 'stat']
+    
+    def run(self, all=False, show_ids=False, file_list=None):
+        if file_list:
+            b = find_branch(file_list[0])
+            file_list = [b.relpath(x) for x in file_list]
+            # special case: only one path was given and it's the root
+            # of the branch
+            if file_list == ['']:
+                file_list = None
+        else:
+            b = find_branch('.')
+            
+        from bzrlib.status import show_status
+        show_status(b, show_unchanged=all, show_ids=show_ids,
+                    specific_files=file_list)
+
+
+class cmd_cat_revision(Command):
+    """Write out metadata for a revision."""
+
+    hidden = True
+    takes_args = ['revision_id']
+    
+    def run(self, revision_id):
+        b = find_branch('.')
+        sys.stdout.write(b.get_revision_xml_file(revision_id).read())
+
+
+class cmd_revno(Command):
+    """Show current revision number.
+
+    This is equal to the number of revisions on this branch."""
+    def run(self):
+        print find_branch('.').revno()
+
+
+class cmd_revision_info(Command):
+    """Show revision number and revision id for a given revision identifier.
+    """
+    hidden = True
+    takes_args = ['revision_info*']
+    takes_options = ['revision']
+    def run(self, revision=None, revision_info_list=None):
+        from bzrlib.branch import find_branch
+
+        revs = []
+        if revision is not None:
+            revs.extend(revision)
+        if revision_info_list is not None:
+            revs.extend(revision_info_list)
+        if len(revs) == 0:
+            raise BzrCommandError('You must supply a revision identifier')
+
+        b = find_branch('.')
+
+        for rev in revs:
+            print '%4d %s' % b.get_revision_info(rev)
+
+    
+class cmd_add(Command):
+    """Add specified files or directories.
+
+    In non-recursive mode, all the named items are added, regardless
+    of whether they were previously ignored.  A warning is given if
+    any of the named files are already versioned.
+
+    In recursive mode (the default), files are treated the same way
+    but the behaviour for directories is different.  Directories that
+    are already versioned do not give a warning.  All directories,
+    whether already versioned or not, are searched for files or
+    subdirectories that are neither versioned or ignored, and these
+    are added.  This search proceeds recursively into versioned
+    directories.  If no names are given '.' is assumed.
+
+    Therefore simply saying 'bzr add' will version all files that
+    are currently unknown.
+
+    Adding a file whose parent directory is not versioned will
+    implicitly add the parent, and so on up to the root. This means
+    you should never need to explictly add a directory, they'll just
+    get added when you add a file in the directory.
+    """
+    takes_args = ['file*']
+    takes_options = ['verbose', 'no-recurse']
+    
+    def run(self, file_list, verbose=False, no_recurse=False):
+        # verbose currently has no effect
+        from bzrlib.add import smart_add, add_reporter_print
+        smart_add(file_list, not no_recurse, add_reporter_print)
+
+
+
+class cmd_mkdir(Command):
+    """Create a new versioned directory.
+
+    This is equivalent to creating the directory and then adding it.
+    """
+    takes_args = ['dir+']
+
+    def run(self, dir_list):
+        b = None
+        
+        for d in dir_list:
+            os.mkdir(d)
+            if not b:
+                b = find_branch(d)
+            b.add([d])
+            print 'added', d
+
+
+class cmd_relpath(Command):
+    """Show path of a file relative to root"""
+    takes_args = ['filename']
+    hidden = True
+    
+    def run(self, filename):
+        print find_branch(filename).relpath(filename)
+
+
+
+class cmd_inventory(Command):
+    """Show inventory of the current working copy or a revision."""
+    takes_options = ['revision', 'show-ids']
+    
+    def run(self, revision=None, show_ids=False):
+        b = find_branch('.')
+        if revision == None:
+            inv = b.read_working_inventory()
+        else:
+            if len(revision) > 1:
+                raise BzrCommandError('bzr inventory --revision takes'
+                    ' exactly one revision identifier')
+            inv = b.get_revision_inventory(b.lookup_revision(revision[0]))
+
+        for path, entry in inv.entries():
+            if show_ids:
+                print '%-50s %s' % (path, entry.file_id)
+            else:
+                print path
+
+
+class cmd_move(Command):
+    """Move files to a different directory.
+
+    examples:
+        bzr move *.txt doc
+
+    The destination must be a versioned directory in the same branch.
+    """
+    takes_args = ['source$', 'dest']
+    def run(self, source_list, dest):
+        b = find_branch('.')
+
+        # TODO: glob expansion on windows?
+        b.move([b.relpath(s) for s in source_list], b.relpath(dest))
+
+
+class cmd_rename(Command):
+    """Change the name of an entry.
+
+    examples:
+      bzr rename frob.c frobber.c
+      bzr rename src/frob.c lib/frob.c
+
+    It is an error if the destination name exists.
+
+    See also the 'move' command, which moves files into a different
+    directory without changing their name.
+
+    TODO: Some way to rename multiple files without invoking bzr for each
+    one?"""
+    takes_args = ['from_name', 'to_name']
+    
+    def run(self, from_name, to_name):
+        b = find_branch('.')
+        b.rename_one(b.relpath(from_name), b.relpath(to_name))
+
+
+
+class cmd_mv(Command):
+    """Move or rename a file.
+
+    usage:
+        bzr mv OLDNAME NEWNAME
+        bzr mv SOURCE... DESTINATION
+
+    If the last argument is a versioned directory, all the other names
+    are moved into it.  Otherwise, there must be exactly two arguments
+    and the file is changed to a new name, which must not already exist.
+
+    Files cannot be moved between branches.
+    """
+    takes_args = ['names*']
+    def run(self, names_list):
+        if len(names_list) < 2:
+            raise BzrCommandError("missing file argument")
+        b = find_branch(names_list[0])
+
+        rel_names = [b.relpath(x) for x in names_list]
+        
+        if os.path.isdir(names_list[-1]):
+            # move into existing directory
+            for pair in b.move(rel_names[:-1], rel_names[-1]):
+                print "%s => %s" % pair
+        else:
+            if len(names_list) != 2:
+                raise BzrCommandError('to mv multiple files the destination '
+                                      'must be a versioned directory')
+            for pair in b.move(rel_names[0], rel_names[1]):
+                print "%s => %s" % pair
+            
+    
+
+
+class cmd_pull(Command):
+    """Pull any changes from another branch into the current one.
+
+    If the location is omitted, the last-used location will be used.
+    Both the revision history and the working directory will be
+    updated.
+
+    This command only works on branches that have not diverged.  Branches are
+    considered diverged if both branches have had commits without first
+    pulling from the other.
+
+    If branches have diverged, you can use 'bzr merge' to pull the text changes
+    from one into the other.
+    """
+    takes_args = ['location?']
+
+    def run(self, location=None):
+        from bzrlib.merge import merge
+        import tempfile
+        from shutil import rmtree
+        import errno
+        from bzrlib.branch import pull_loc
+        
+        br_to = find_branch('.')
+        stored_loc = None
+        try:
+            stored_loc = br_to.controlfile("x-pull", "rb").read().rstrip('\n')
+        except IOError, e:
+            if e.errno != errno.ENOENT:
+                raise
+        if location is None:
+            if stored_loc is None:
+                raise BzrCommandError("No pull location known or specified.")
+            else:
+                print "Using last location: %s" % stored_loc
+                location = stored_loc
+        cache_root = tempfile.mkdtemp()
+        from bzrlib.branch import DivergedBranches
+        br_from = find_branch(location)
+        location = pull_loc(br_from)
+        old_revno = br_to.revno()
+        try:
+            from branch import find_cached_branch, DivergedBranches
+            br_from = find_cached_branch(location, cache_root)
+            location = pull_loc(br_from)
+            old_revno = br_to.revno()
+            try:
+                br_to.update_revisions(br_from)
+            except DivergedBranches:
+                raise BzrCommandError("These branches have diverged."
+                    "  Try merge.")
+                
+            merge(('.', -1), ('.', old_revno), check_clean=False)
+            if location != stored_loc:
+                br_to.controlfile("x-pull", "wb").write(location + "\n")
+        finally:
+            rmtree(cache_root)
+
+
+
+class cmd_branch(Command):
+    """Create a new copy of a branch.
+
+    If the TO_LOCATION is omitted, the last component of the FROM_LOCATION will
+    be used.  In other words, "branch ../foo/bar" will attempt to create ./bar.
+
+    To retrieve the branch as of a particular revision, supply the --revision
+    parameter, as in "branch foo/bar -r 5".
+    """
+    takes_args = ['from_location', 'to_location?']
+    takes_options = ['revision']
+    aliases = ['get', 'clone']
+
+    def run(self, from_location, to_location=None, revision=None):
+        from bzrlib.branch import copy_branch, find_cached_branch
+        import tempfile
+        import errno
+        from shutil import rmtree
+        cache_root = tempfile.mkdtemp()
+        try:
+            if revision is None:
+                revision = [None]
+            elif len(revision) > 1:
+                raise BzrCommandError(
+                    'bzr branch --revision takes exactly 1 revision value')
+            try:
+                br_from = find_cached_branch(from_location, cache_root)
+            except OSError, e:
+                if e.errno == errno.ENOENT:
+                    raise BzrCommandError('Source location "%s" does not'
+                                          ' exist.' % to_location)
+                else:
+                    raise
+            if to_location is None:
+                to_location = os.path.basename(from_location.rstrip("/\\"))
+            try:
+                os.mkdir(to_location)
+            except OSError, e:
+                if e.errno == errno.EEXIST:
+                    raise BzrCommandError('Target directory "%s" already'
+                                          ' exists.' % to_location)
+                if e.errno == errno.ENOENT:
+                    raise BzrCommandError('Parent of "%s" does not exist.' %
+                                          to_location)
+                else:
+                    raise
+            try:
+                copy_branch(br_from, to_location, revision[0])
+            except bzrlib.errors.NoSuchRevision:
+                rmtree(to_location)
+                msg = "The branch %s has no revision %d." % (from_location, revision[0])
+                raise BzrCommandError(msg)
+        finally:
+            rmtree(cache_root)
+
+
+class cmd_renames(Command):
+    """Show list of renamed files.
+
+    TODO: Option to show renames between two historical versions.
+
+    TODO: Only show renames under dir, rather than in the whole branch.
+    """
+    takes_args = ['dir?']
+
+    def run(self, dir='.'):
+        b = find_branch(dir)
+        old_inv = b.basis_tree().inventory
+        new_inv = b.read_working_inventory()
+
+        renames = list(bzrlib.tree.find_renames(old_inv, new_inv))
+        renames.sort()
+        for old_name, new_name in renames:
+            print "%s => %s" % (old_name, new_name)        
+
+
+class cmd_info(Command):
+    """Show statistical information about a branch."""
+    takes_args = ['branch?']
+    
+    def run(self, branch=None):
+        import info
+
+        b = find_branch(branch)
+        info.show_info(b)
+
+
+class cmd_remove(Command):
+    """Make a file unversioned.
+
+    This makes bzr stop tracking changes to a versioned file.  It does
+    not delete the working copy.
+    """
+    takes_args = ['file+']
+    takes_options = ['verbose']
+    
+    def run(self, file_list, verbose=False):
+        b = find_branch(file_list[0])
+        b.remove([b.relpath(f) for f in file_list], verbose=verbose)
+
+
+class cmd_file_id(Command):
+    """Print file_id of a particular file or directory.
+
+    The file_id is assigned when the file is first added and remains the
+    same through all revisions where the file exists, even when it is
+    moved or renamed.
+    """
+    hidden = True
+    takes_args = ['filename']
+    def run(self, filename):
+        b = find_branch(filename)
+        i = b.inventory.path2id(b.relpath(filename))
+        if i == None:
+            raise BzrError("%r is not a versioned file" % filename)
+        else:
+            print i
+
+
+class cmd_file_path(Command):
+    """Print path of file_ids to a file or directory.
+
+    This prints one line for each directory down to the target,
+    starting at the branch root."""
+    hidden = True
+    takes_args = ['filename']
+    def run(self, filename):
+        b = find_branch(filename)
+        inv = b.inventory
+        fid = inv.path2id(b.relpath(filename))
+        if fid == None:
+            raise BzrError("%r is not a versioned file" % filename)
+        for fip in inv.get_idpath(fid):
+            print fip
+
+
+class cmd_revision_history(Command):
+    """Display list of revision ids on this branch."""
+    hidden = True
+    def run(self):
+        for patchid in find_branch('.').revision_history():
+            print patchid
+
+
+class cmd_directories(Command):
+    """Display list of versioned directories in this branch."""
+    def run(self):
+        for name, ie in find_branch('.').read_working_inventory().directories():
+            if name == '':
+                print '.'
+            else:
+                print name
+
+
+class cmd_init(Command):
+    """Make a directory into a versioned branch.
+
+    Use this to create an empty branch, or before importing an
+    existing project.
+
+    Recipe for importing a tree of files:
+        cd ~/project
+        bzr init
+        bzr add -v .
+        bzr status
+        bzr commit -m 'imported project'
+    """
+    def run(self):
+        from bzrlib.branch import Branch
+        Branch('.', init=True)
+
+
+class cmd_diff(Command):
+    """Show differences in working tree.
+    
+    If files are listed, only the changes in those files are listed.
+    Otherwise, all changes for the tree are listed.
+
+    TODO: Allow diff across branches.
+
+    TODO: Option to use external diff command; could be GNU diff, wdiff,
+          or a graphical diff.
+
+    TODO: Python difflib is not exactly the same as unidiff; should
+          either fix it up or prefer to use an external diff.
+
+    TODO: If a directory is given, diff everything under that.
+
+    TODO: Selected-file diff is inefficient and doesn't show you
+          deleted files.
+
+    TODO: This probably handles non-Unix newlines poorly.
+
+    examples:
+        bzr diff
+        bzr diff -r1
+        bzr diff -r1..2
+    """
+    
+    takes_args = ['file*']
+    takes_options = ['revision', 'diff-options']
+    aliases = ['di', 'dif']
+
+    def run(self, revision=None, file_list=None, diff_options=None):
+        from bzrlib.diff import show_diff
+
+        if file_list:
+            b = find_branch(file_list[0])
+            file_list = [b.relpath(f) for f in file_list]
+            if file_list == ['']:
+                # just pointing to top-of-tree
+                file_list = None
+        else:
+            b = find_branch('.')
+
+        if revision is not None:
+            if len(revision) == 1:
+                show_diff(b, revision[0], specific_files=file_list,
+                          external_diff_options=diff_options)
+            elif len(revision) == 2:
+                show_diff(b, revision[0], specific_files=file_list,
+                          external_diff_options=diff_options,
+                          revision2=revision[1])
+            else:
+                raise BzrCommandError('bzr diff --revision takes exactly one or two revision identifiers')
+        else:
+            show_diff(b, None, specific_files=file_list,
+                      external_diff_options=diff_options)
+
+        
+
+
+class cmd_deleted(Command):
+    """List files deleted in the working tree.
+
+    TODO: Show files deleted since a previous revision, or between two revisions.
+    """
+    def run(self, show_ids=False):
+        b = find_branch('.')
+        old = b.basis_tree()
+        new = b.working_tree()
+
+        ## TODO: Much more efficient way to do this: read in new
+        ## directories with readdir, rather than stating each one.  Same
+        ## level of effort but possibly much less IO.  (Or possibly not,
+        ## if the directories are very large...)
+
+        for path, ie in old.inventory.iter_entries():
+            if not new.has_id(ie.file_id):
+                if show_ids:
+                    print '%-50s %s' % (path, ie.file_id)
+                else:
+                    print path
+
+
+class cmd_modified(Command):
+    """List files modified in working tree."""
+    hidden = True
+    def run(self):
+        from bzrlib.delta import compare_trees
+
+        b = find_branch('.')
+        td = compare_trees(b.basis_tree(), b.working_tree())
+
+        for path, id, kind in td.modified:
+            print path
+
+
+
+class cmd_added(Command):
+    """List files added in working tree."""
+    hidden = True
+    def run(self):
+        b = find_branch('.')
+        wt = b.working_tree()
+        basis_inv = b.basis_tree().inventory
+        inv = wt.inventory
+        for file_id in inv:
+            if file_id in basis_inv:
+                continue
+            path = inv.id2path(file_id)
+            if not os.access(b.abspath(path), os.F_OK):
+                continue
+            print path
+                
+        
+
+class cmd_root(Command):
+    """Show the tree root directory.
+
+    The root is the nearest enclosing directory with a .bzr control
+    directory."""
+    takes_args = ['filename?']
+    def run(self, filename=None):
+        """Print the branch root."""
+        b = find_branch(filename)
+        print getattr(b, 'base', None) or getattr(b, 'baseurl')
+
+
+class cmd_log(Command):
+    """Show log of this branch.
+
+    To request a range of logs, you can use the command -r begin:end
+    -r revision requests a specific revision, -r :end or -r begin: are
+    also valid.
+
+    --message allows you to give a regular expression, which will be evaluated
+    so that only matching entries will be displayed.
+
+    TODO: Make --revision support uuid: and hash: [future tag:] notation.
+  
+    """
+
+    takes_args = ['filename?']
+    takes_options = ['forward', 'timezone', 'verbose', 'show-ids', 'revision',
+                     'long', 'message', 'short',]
+    
+    def run(self, filename=None, timezone='original',
+            verbose=False,
+            show_ids=False,
+            forward=False,
+            revision=None,
+            message=None,
+            long=False,
+            short=False):
+        from bzrlib.log import log_formatter, show_log
+        import codecs
+
+        direction = (forward and 'forward') or 'reverse'
+        
+        if filename:
+            b = find_branch(filename)
+            fp = b.relpath(filename)
+            if fp:
+                file_id = b.read_working_inventory().path2id(fp)
+            else:
+                file_id = None  # points to branch root
+        else:
+            b = find_branch('.')
+            file_id = None
+
+        if revision is None:
+            rev1 = None
+            rev2 = None
+        elif len(revision) == 1:
+            rev1 = rev2 = b.get_revision_info(revision[0])[0]
+        elif len(revision) == 2:
+            rev1 = b.get_revision_info(revision[0])[0]
+            rev2 = b.get_revision_info(revision[1])[0]
+        else:
+            raise BzrCommandError('bzr log --revision takes one or two values.')
+
+        if rev1 == 0:
+            rev1 = None
+        if rev2 == 0:
+            rev2 = None
+
+        mutter('encoding log as %r' % bzrlib.user_encoding)
+
+        # use 'replace' so that we don't abort if trying to write out
+        # in e.g. the default C locale.
+        outf = codecs.getwriter(bzrlib.user_encoding)(sys.stdout, errors='replace')
+
+        if not short:
+            log_format = 'long'
+        else:
+            log_format = 'short'
+        lf = log_formatter(log_format,
+                           show_ids=show_ids,
+                           to_file=outf,
+                           show_timezone=timezone)
+
+        show_log(b,
+                 lf,
+                 file_id,
+                 verbose=verbose,
+                 direction=direction,
+                 start_revision=rev1,
+                 end_revision=rev2,
+                 search=message)
+
+
+
+class cmd_touching_revisions(Command):
+    """Return revision-ids which affected a particular file.
+
+    A more user-friendly interface is "bzr log FILE"."""
+    hidden = True
+    takes_args = ["filename"]
+    def run(self, filename):
+        b = find_branch(filename)
+        inv = b.read_working_inventory()
+        file_id = inv.path2id(b.relpath(filename))
+        for revno, revision_id, what in bzrlib.log.find_touching_revisions(b, file_id):
+            print "%6d %s" % (revno, what)
+
+
+class cmd_ls(Command):
+    """List files in a tree.
+
+    TODO: Take a revision or remote path and list that tree instead.
+    """
+    hidden = True
+    def run(self, revision=None, verbose=False):
+        b = find_branch('.')
+        if revision == None:
+            tree = b.working_tree()
+        else:
+            tree = b.revision_tree(b.lookup_revision(revision))
+
+        for fp, fc, kind, fid in tree.list_files():
+            if verbose:
+                if kind == 'directory':
+                    kindch = '/'
+                elif kind == 'file':
+                    kindch = ''
+                else:
+                    kindch = '???'
+
+                print '%-8s %s%s' % (fc, fp, kindch)
+            else:
+                print fp
+
+
+
+class cmd_unknowns(Command):
+    """List unknown files."""
+    def run(self):
+        from bzrlib.osutils import quotefn
+        for f in find_branch('.').unknowns():
+            print quotefn(f)
+
+
+
+class cmd_ignore(Command):
+    """Ignore a command or pattern.
+
+    To remove patterns from the ignore list, edit the .bzrignore file.
+
+    If the pattern contains a slash, it is compared to the whole path
+    from the branch root.  Otherwise, it is comapred to only the last
+    component of the path.
+
+    Ignore patterns are case-insensitive on case-insensitive systems.
+
+    Note: wildcards must be quoted from the shell on Unix.
+
+    examples:
+        bzr ignore ./Makefile
+        bzr ignore '*.class'
+    """
+    takes_args = ['name_pattern']
+    
+    def run(self, name_pattern):
+        from bzrlib.atomicfile import AtomicFile
+        import os.path
+
+        b = find_branch('.')
+        ifn = b.abspath('.bzrignore')
+
+        if os.path.exists(ifn):
+            f = open(ifn, 'rt')
+            try:
+                igns = f.read().decode('utf-8')
+            finally:
+                f.close()
+        else:
+            igns = ''
+
+        # TODO: If the file already uses crlf-style termination, maybe
+        # we should use that for the newly added lines?
+
+        if igns and igns[-1] != '\n':
+            igns += '\n'
+        igns += name_pattern + '\n'
+
+        try:
+            f = AtomicFile(ifn, 'wt')
+            f.write(igns.encode('utf-8'))
+            f.commit()
+        finally:
+            f.close()
+
+        inv = b.working_tree().inventory
+        if inv.path2id('.bzrignore'):
+            mutter('.bzrignore is already versioned')
+        else:
+            mutter('need to make new .bzrignore file versioned')
+            b.add(['.bzrignore'])
+
+
+
+class cmd_ignored(Command):
+    """List ignored files and the patterns that matched them.
+
+    See also: bzr ignore"""
+    def run(self):
+        tree = find_branch('.').working_tree()
+        for path, file_class, kind, file_id in tree.list_files():
+            if file_class != 'I':
+                continue
+            ## XXX: Slightly inefficient since this was already calculated
+            pat = tree.is_ignored(path)
+            print '%-50s %s' % (path, pat)
+
+
+class cmd_lookup_revision(Command):
+    """Lookup the revision-id from a revision-number
+
+    example:
+        bzr lookup-revision 33
+    """
+    hidden = True
+    takes_args = ['revno']
+    
+    def run(self, revno):
+        try:
+            revno = int(revno)
+        except ValueError:
+            raise BzrCommandError("not a valid revision-number: %r" % revno)
+
+        print find_branch('.').lookup_revision(revno)
+
+
+class cmd_export(Command):
+    """Export past revision to destination directory.
+
+    If no revision is specified this exports the last committed revision.
+
+    Format may be an "exporter" name, such as tar, tgz, tbz2.  If none is
+    given, try to find the format with the extension. If no extension
+    is found exports to a directory (equivalent to --format=dir).
+
+    Root may be the top directory for tar, tgz and tbz2 formats. If none
+    is given, the top directory will be the root name of the file."""
+    # TODO: list known exporters
+    takes_args = ['dest']
+    takes_options = ['revision', 'format', 'root']
+    def run(self, dest, revision=None, format=None, root=None):
+        import os.path
+        b = find_branch('.')
+        if revision is None:
+            rev_id = b.last_patch()
+        else:
+            if len(revision) != 1:
+                raise BzrError('bzr export --revision takes exactly 1 argument')
+            revno, rev_id = b.get_revision_info(revision[0])
+        t = b.revision_tree(rev_id)
+        root, ext = os.path.splitext(dest)
+        if not format:
+            if ext in (".tar",):
+                format = "tar"
+            elif ext in (".gz", ".tgz"):
+                format = "tgz"
+            elif ext in (".bz2", ".tbz2"):
+                format = "tbz2"
+            else:
+                format = "dir"
+        t.export(dest, format, root)
+
+
+class cmd_cat(Command):
+    """Write a file's text from a previous revision."""
+
+    takes_options = ['revision']
+    takes_args = ['filename']
+
+    def run(self, filename, revision=None):
+        if revision == None:
+            raise BzrCommandError("bzr cat requires a revision number")
+        elif len(revision) != 1:
+            raise BzrCommandError("bzr cat --revision takes exactly one number")
+        b = find_branch('.')
+        b.print_file(b.relpath(filename), revision[0])
+
+
+class cmd_local_time_offset(Command):
+    """Show the offset in seconds from GMT to local time."""
+    hidden = True    
+    def run(self):
+        print bzrlib.osutils.local_time_offset()
+
+
+
+class cmd_commit(Command):
+    """Commit changes into a new revision.
+    
+    If no arguments are given, the entire tree is committed.
+
+    If selected files are specified, only changes to those files are
+    committed.  If a directory is specified then the directory and everything 
+    within it is committed.
+
+    A selected-file commit may fail in some cases where the committed
+    tree would be invalid, such as trying to commit a file in a
+    newly-added directory that is not itself committed.
+
+    TODO: Run hooks on tree to-be-committed, and after commit.
+
+    TODO: Strict commit that fails if there are unknown or deleted files.
+    """
+    takes_args = ['selected*']
+    takes_options = ['message', 'file', 'verbose', 'unchanged']
+    aliases = ['ci', 'checkin']
+
+    # TODO: Give better message for -s, --summary, used by tla people
+    
+    def run(self, message=None, file=None, verbose=True, selected_list=None,
+            unchanged=False):
+        from bzrlib.errors import PointlessCommit
+        from bzrlib.msgeditor import edit_commit_message
+        from bzrlib.status import show_status
+        from cStringIO import StringIO
+
+        b = find_branch('.')
+        if selected_list:
+            selected_list = [b.relpath(s) for s in selected_list]
+            
+        if not message and not file:
+            catcher = StringIO()
+            show_status(b, specific_files=selected_list,
+                        to_file=catcher)
+            message = edit_commit_message(catcher.getvalue())
+            
+            if message is None:
+                raise BzrCommandError("please specify a commit message"
+                                      " with either --message or --file")
+        elif message and file:
+            raise BzrCommandError("please specify either --message or --file")
+        
+        if file:
+            import codecs
+            message = codecs.open(file, 'rt', bzrlib.user_encoding).read()
+
+        try:
+            b.commit(message, verbose=verbose,
+                     specific_files=selected_list,
+                     allow_pointless=unchanged)
+        except PointlessCommit:
+            # FIXME: This should really happen before the file is read in;
+            # perhaps prepare the commit; get the message; then actually commit
+            raise BzrCommandError("no changes to commit",
+                                  ["use --unchanged to commit anyhow"])
+
+
+class cmd_check(Command):
+    """Validate consistency of branch history.
+
+    This command checks various invariants about the branch storage to
+    detect data corruption or bzr bugs.
+
+    If given the --update flag, it will update some optional fields
+    to help ensure data consistency.
+    """
+    takes_args = ['dir?']
+
+    def run(self, dir='.'):
+        from bzrlib.check import check
+
+        check(find_branch(dir))
+
+
+class cmd_scan_cache(Command):
+    hidden = True
+    def run(self):
+        from bzrlib.hashcache import HashCache
+
+        c = HashCache('.')
+        c.read()
+        c.scan()
+            
+        print '%6d stats' % c.stat_count
+        print '%6d in hashcache' % len(c._cache)
+        print '%6d files removed from cache' % c.removed_count
+        print '%6d hashes updated' % c.update_count
+        print '%6d files changed too recently to cache' % c.danger_count
+
+        if c.needs_write:
+            c.write()
+            
+
+
+class cmd_upgrade(Command):
+    """Upgrade branch storage to current format.
+
+    The check command or bzr developers may sometimes advise you to run
+    this command.
+    """
+    takes_args = ['dir?']
+
+    def run(self, dir='.'):
+        from bzrlib.upgrade import upgrade
+        upgrade(find_branch(dir))
+
+
+
+class cmd_whoami(Command):
+    """Show bzr user id."""
+    takes_options = ['email']
+    
+    def run(self, email=False):
+        try:
+            b = bzrlib.branch.find_branch('.')
+        except:
+            b = None
+        
+        if email:
+            print bzrlib.osutils.user_email(b)
+        else:
+            print bzrlib.osutils.username(b)
+
+
+class cmd_selftest(Command):
+    """Run internal test suite"""
+    hidden = True
+    takes_options = ['verbose', 'pattern']
+    def run(self, verbose=False, pattern=".*"):
+        import bzrlib.ui
+        from bzrlib.selftest import selftest
+        # we don't want progress meters from the tests to go to the
+        # real output; and we don't want log messages cluttering up
+        # the real logs.
+        save_ui = bzrlib.ui.ui_factory
+        bzrlib.trace.info('running tests...')
+        try:
+            bzrlib.ui.ui_factory = bzrlib.ui.SilentUIFactory()
+            result = selftest(verbose=verbose, pattern=pattern)
+            if result:
+                bzrlib.trace.info('tests passed')
+            else:
+                bzrlib.trace.info('tests failed')
+            return int(not result)
+        finally:
+            bzrlib.ui.ui_factory = save_ui
+
+
+def show_version():
+    print "bzr (bazaar-ng) %s" % bzrlib.__version__
+    # is bzrlib itself in a branch?
+    bzrrev = bzrlib.get_bzr_revision()
+    if bzrrev:
+        print "  (bzr checkout, revision %d {%s})" % bzrrev
+    print bzrlib.__copyright__
+    print "http://bazaar-ng.org/"
+    print
+    print "bzr comes with ABSOLUTELY NO WARRANTY.  bzr is free software, and"
+    print "you may use, modify and redistribute it under the terms of the GNU"
+    print "General Public License version 2 or later."
+
+
+class cmd_version(Command):
+    """Show version of bzr."""
+    def run(self):
+        show_version()
+
+class cmd_rocks(Command):
+    """Statement of optimism."""
+    hidden = True
+    def run(self):
+        print "it sure does!"
+
+
+class cmd_find_merge_base(Command):
+    """Find and print a base revision for merging two branches.
+
+    TODO: Options to specify revisions on either side, as if
+          merging only part of the history.
+    """
+    takes_args = ['branch', 'other']
+    hidden = True
+    
+    def run(self, branch, other):
+        from bzrlib.revision import common_ancestor, MultipleRevisionSources
+        
+        branch1 = find_branch(branch)
+        branch2 = find_branch(other)
+
+        history_1 = branch1.revision_history()
+        history_2 = branch2.revision_history()
+
+        last1 = branch1.last_patch()
+        last2 = branch2.last_patch()
+
+        source = MultipleRevisionSources(branch1, branch2)
+        
+        base_rev_id = common_ancestor(last1, last2, source)
+
+        print 'merge base is revision %s' % base_rev_id
+        
+        return
+
+        if base_revno is None:
+            raise bzrlib.errors.UnrelatedBranches()
+
+        print ' r%-6d in %s' % (base_revno, branch)
+
+        other_revno = branch2.revision_id_to_revno(base_revid)
+        
+        print ' r%-6d in %s' % (other_revno, other)
+
+
+
+class cmd_merge(Command):
+    """Perform a three-way merge.
+    
+    The branch is the branch you will merge from.  By default, it will
+    merge the latest revision.  If you specify a revision, that
+    revision will be merged.  If you specify two revisions, the first
+    will be used as a BASE, and the second one as OTHER.  Revision
+    numbers are always relative to the specified branch.
+
+    By default bzr will try to merge in all new work from the other
+    branch, automatically determining an appropriate base.  If this
+    fails, you may need to give an explicit base.
+    
+    Examples:
+
+    To merge the latest revision from bzr.dev
+    bzr merge ../bzr.dev
+
+    To merge changes up to and including revision 82 from bzr.dev
+    bzr merge -r 82 ../bzr.dev
+
+    To merge the changes introduced by 82, without previous changes:
+    bzr merge -r 81..82 ../bzr.dev
+    
+    merge refuses to run if there are any uncommitted changes, unless
+    --force is given.
+    """
+    takes_args = ['branch?']
+    takes_options = ['revision', 'force', 'merge-type']
+
+    def run(self, branch='.', revision=None, force=False, 
+            merge_type=None):
+        from bzrlib.merge import merge
+        from bzrlib.merge_core import ApplyMerge3
+        if merge_type is None:
+            merge_type = ApplyMerge3
+
+        if revision is None or len(revision) < 1:
+            base = [None, None]
+            other = [branch, -1]
+        else:
+            if len(revision) == 1:
+                other = [branch, revision[0]]
+                base = [None, None]
+            else:
+                assert len(revision) == 2
+                if None in revision:
+                    raise BzrCommandError(
+                        "Merge doesn't permit that revision specifier.")
+                base = [branch, revision[0]]
+                other = [branch, revision[1]]
+
+        try:
+            merge(other, base, check_clean=(not force), merge_type=merge_type)
+        except bzrlib.errors.AmbiguousBase, e:
+            m = ("sorry, bzr can't determine the right merge base yet\n"
+                 "candidates are:\n  "
+                 + "\n  ".join(e.bases)
+                 + "\n"
+                 "please specify an explicit base with -r,\n"
+                 "and (if you want) report this to the bzr developers\n")
+            log_error(m)
+
+
+class cmd_revert(Command):
+    """Reverse all changes since the last commit.
+
+    Only versioned files are affected.  Specify filenames to revert only 
+    those files.  By default, any files that are changed will be backed up
+    first.  Backup files have a '~' appended to their name.
+    """
+    takes_options = ['revision', 'no-backup']
+    takes_args = ['file*']
+    aliases = ['merge-revert']
+
+    def run(self, revision=None, no_backup=False, file_list=None):
+        from bzrlib.merge import merge
+        from bzrlib.branch import Branch
+        from bzrlib.commands import parse_spec
+
+        if file_list is not None:
+            if len(file_list) == 0:
+                raise BzrCommandError("No files specified")
+        if revision is None:
+            revision = [-1]
+        elif len(revision) != 1:
+            raise BzrCommandError('bzr revert --revision takes exactly 1 argument')
+        merge(('.', revision[0]), parse_spec('.'),
+              check_clean=False,
+              ignore_zero=True,
+              backup_files=not no_backup,
+              file_list=file_list)
+        if not file_list:
+            Branch('.').set_pending_merges([])
+
+
+class cmd_assert_fail(Command):
+    """Test reporting of assertion failures"""
+    hidden = True
+    def run(self):
+        assert False, "always fails"
+
+
+class cmd_help(Command):
+    """Show help on a command or other topic.
+
+    For a list of all available commands, say 'bzr help commands'."""
+    takes_options = ['long']
+    takes_args = ['topic?']
+    aliases = ['?']
+    
+    def run(self, topic=None, long=False):
+        import help
+        if topic is None and long:
+            topic = "commands"
+        help.help(topic)
+
+
+class cmd_shell_complete(Command):
+    """Show appropriate completions for context.
+
+    For a list of all available commands, say 'bzr shell-complete'."""
+    takes_args = ['context?']
+    aliases = ['s-c']
+    hidden = True
+    
+    def run(self, context=None):
+        import shellcomplete
+        shellcomplete.shellcomplete(context)
+
+
+class cmd_missing(Command):
+    """What is missing in this branch relative to other branch.
+    """
+    takes_args = ['remote?']
+    aliases = ['mis', 'miss']
+    # We don't have to add quiet to the list, because 
+    # unknown options are parsed as booleans
+    takes_options = ['verbose', 'quiet']
+
+    def run(self, remote=None, verbose=False, quiet=False):
+        from bzrlib.errors import BzrCommandError
+        from bzrlib.missing import show_missing
+
+        if verbose and quiet:
+            raise BzrCommandError('Cannot pass both quiet and verbose')
+
+        b = find_branch('.')
+        parent = b.get_parent()
+        if remote is None:
+            if parent is None:
+                raise BzrCommandError("No missing location known or specified.")
+            else:
+                if not quiet:
+                    print "Using last location: %s" % parent
+                remote = parent
+        elif parent is None:
+            # We only update x-pull if it did not exist, missing should not change the parent
+            b.controlfile('x-pull', 'wb').write(remote + '\n')
+        br_remote = find_branch(remote)
+
+        return show_missing(b, br_remote, verbose=verbose, quiet=quiet)
+
+
+
+class cmd_plugins(Command):
+    """List plugins"""
+    hidden = True
+    def run(self):
+        import bzrlib.plugin
+        from inspect import getdoc
+        for plugin in bzrlib.plugin.all_plugins:
+            if hasattr(plugin, '__path__'):
+                print plugin.__path__[0]
+            elif hasattr(plugin, '__file__'):
+                print plugin.__file__
+            else:
+                print `plugin`
+                
+            d = getdoc(plugin)
+            if d:
+                print '\t', d.split('\n')[0]
+
+

=== modified file 'bzrlib/changeset.py'
--- bzrlib/changeset.py	2005-07-22 19:54:37 +0000
+++ bzrlib/changeset.py	2005-08-27 04:42:41 +0000
@@ -18,9 +18,16 @@
 import patch
 import stat
 from bzrlib.trace import mutter
-"""
-Represent and apply a changeset
-"""
+
+# XXX: mbp: I'm not totally convinced that we should handle conflicts
+# as part of changeset application, rather than only in the merge
+# operation.
+
+"""Represent and apply a changeset
+
+Conflicts in applying a changeset are represented as exceptions.
+"""
+
 __docformat__ = "restructuredtext"
 
 NULL_ID = "!NULL"
@@ -36,55 +43,7 @@
     return newdict
 
 
-class PatchApply(object):
-    """Patch application as a kind of content change"""
-    def __init__(self, contents):
-        """Constructor.
-
-        :param contents: The text of the patch to apply
-        :type contents: str"""
-        self.contents = contents
-
-    def __eq__(self, other):
-        if not isinstance(other, PatchApply):
-            return False
-        elif self.contents != other.contents:
-            return False
-        else:
-            return True
-
-    def __ne__(self, other):
-        return not (self == other)
-
-    def apply(self, filename, conflict_handler, reverse=False):
-        """Applies the patch to the specified file.
-
-        :param filename: the file to apply the patch to
-        :type filename: str
-        :param reverse: If true, apply the patch in reverse
-        :type reverse: bool
-        """
-        input_name = filename+".orig"
-        try:
-            os.rename(filename, input_name)
-        except OSError, e:
-            if e.errno != errno.ENOENT:
-                raise
-            if conflict_handler.patch_target_missing(filename, self.contents)\
-                == "skip":
-                return
-            os.rename(filename, input_name)
-            
-
-        status = patch.patch(self.contents, input_name, filename, 
-                                    reverse)
-        os.chmod(filename, os.stat(input_name).st_mode)
-        if status == 0:
-            os.unlink(input_name)
-        elif status == 1:
-            conflict_handler.failed_hunks(filename)
-
-        
+       
 class ChangeUnixPermissions(object):
     """This is two-way change, suitable for file modification, creation,
     deletion"""
@@ -367,27 +326,30 @@
 
 
 class Diff3Merge(object):
-    def __init__(self, base_file, other_file):
-        self.base_file = base_file
-        self.other_file = other_file
+    def __init__(self, file_id, base, other):
+        self.file_id = file_id
+        self.base = base
+        self.other = other
 
     def __eq__(self, other):
         if not isinstance(other, Diff3Merge):
             return False
-        return (self.base_file == other.base_file and 
-                self.other_file == other.other_file)
+        return (self.base == other.base and 
+                self.other == other.other and self.file_id == other.file_id)
 
     def __ne__(self, other):
         return not (self == other)
 
     def apply(self, filename, conflict_handler, reverse=False):
-        new_file = filename+".new" 
+        new_file = filename+".new"
+        base_file = self.base.readonly_path(self.file_id)
+        other_file = self.other.readonly_path(self.file_id)
         if not reverse:
-            base = self.base_file
-            other = self.other_file
+            base = base_file
+            other = other_file
         else:
-            base = self.other_file
-            other = self.base_file
+            base = other_file
+            other = base_file
         status = patch.diff3(new_file, filename, base, other)
         if status == 0:
             os.chmod(new_file, os.stat(filename).st_mode)
@@ -395,7 +357,14 @@
             return
         else:
             assert(status == 1)
-            conflict_handler.merge_conflict(new_file, filename, base, other)
+            def get_lines(filename):
+                my_file = file(base, "rb")
+                lines = my_file.readlines()
+                my_file.close()
+            base_lines = get_lines(base)
+            other_lines = get_lines(other)
+            conflict_handler.merge_conflict(new_file, filename, base_lines, 
+                                            other_lines)
 
 
 def CreateDir():
@@ -657,7 +626,7 @@
                 return None
             return self.path
 
-    def summarize_name(self, changeset, reverse=False):
+    def summarize_name(self, reverse=False):
         """Produce a one-line summary of the filename.  Indicates renames as
         old => new, indicates creation as None => new, indicates deletion as
         old => None.
@@ -694,7 +663,7 @@
         :type reverse: bool
         :rtype: str
         """
-        mutter("Finding new path for %s" % self.summarize_name(changeset))
+        mutter("Finding new path for %s" % self.summarize_name())
         if reverse:
             parent = self.parent
             to_dir = self.dir
@@ -1028,6 +997,12 @@
 
 
 class ExceptionConflictHandler(object):
+    """Default handler for merge exceptions.
+
+    This throws an error on any kind of conflict.  Conflict handlers can
+    descend from this class if they have a better way to handle some or
+    all types of conflict.
+    """
     def __init__(self, dir):
         self.dir = dir
     
@@ -1047,13 +1022,10 @@
     def rename_conflict(self, id, this_name, base_name, other_name):
         raise RenameConflict(id, this_name, base_name, other_name)
 
-    def move_conflict(self, id, inventory):
-        this_dir = inventory.this.get_dir(id)
-        base_dir = inventory.base.get_dir(id)
-        other_dir = inventory.other.get_dir(id)
+    def move_conflict(self, id, this_dir, base_dir, other_dir):
         raise MoveConflict(id, this_dir, base_dir, other_dir)
 
-    def merge_conflict(self, new_file, this_path, base_path, other_path):
+    def merge_conflict(self, new_file, this_path, base_lines, other_lines):
         os.unlink(new_file)
         raise MergeConflict(this_path)
 
@@ -1087,13 +1059,13 @@
     def missing_for_rename(self, filename):
         raise MissingForRename(filename)
 
-    def missing_for_merge(self, file_id, inventory):
-        raise MissingForMerge(inventory.other.get_path(file_id))
+    def missing_for_merge(self, file_id, other_path):
+        raise MissingForMerge(other_path)
 
     def new_contents_conflict(self, filename, other_contents):
         raise NewContentsConflict(filename)
 
-    def finalize():
+    def finalize(self):
         pass
 
 def apply_changeset(changeset, inventory, dir, conflict_handler=None, 
@@ -1315,49 +1287,30 @@
         self.full_path = full_path
         self.stat_result = stat_result
 
-def generate_changeset(tree_a, tree_b, inventory_a=None, inventory_b=None):
-    return ChangesetGenerator(tree_a, tree_b, inventory_a, inventory_b)()
+def generate_changeset(tree_a, tree_b, interesting_ids=None):
+    return ChangesetGenerator(tree_a, tree_b, interesting_ids)()
 
 class ChangesetGenerator(object):
-    def __init__(self, tree_a, tree_b, inventory_a=None, inventory_b=None):
+    def __init__(self, tree_a, tree_b, interesting_ids=None):
         object.__init__(self)
         self.tree_a = tree_a
         self.tree_b = tree_b
-        if inventory_a is not None:
-            self.inventory_a = inventory_a
-        else:
-            self.inventory_a = tree_a.inventory()
-        if inventory_b is not None:
-            self.inventory_b = inventory_b
-        else:
-            self.inventory_b = tree_b.inventory()
-        self.r_inventory_a = self.reverse_inventory(self.inventory_a)
-        self.r_inventory_b = self.reverse_inventory(self.inventory_b)
+        self._interesting_ids = interesting_ids
 
-    def reverse_inventory(self, inventory):
-        r_inventory = {}
-        for entry in inventory.itervalues():
-            if entry.id is None:
-                continue
-            r_inventory[entry.id] = entry
-        return r_inventory
+    def iter_both_tree_ids(self):
+        for file_id in self.tree_a:
+            yield file_id
+        for file_id in self.tree_b:
+            if file_id not in self.tree_a:
+                yield file_id
 
     def __call__(self):
         cset = Changeset()
-        for entry in self.inventory_a.itervalues():
-            if entry.id is None:
-                continue
-            cs_entry = self.make_entry(entry.id)
+        for file_id in self.iter_both_tree_ids():
+            cs_entry = self.make_entry(file_id)
             if cs_entry is not None and not cs_entry.is_boring():
                 cset.add_entry(cs_entry)
 
-        for entry in self.inventory_b.itervalues():
-            if entry.id is None:
-                continue
-            if not self.r_inventory_a.has_key(entry.id):
-                cs_entry = self.make_entry(entry.id)
-                if cs_entry is not None and not cs_entry.is_boring():
-                    cset.add_entry(cs_entry)
         for entry in list(cset.entries.itervalues()):
             if entry.parent != entry.new_parent:
                 if not cset.entries.has_key(entry.parent) and\
@@ -1371,54 +1324,58 @@
                     cset.add_entry(parent_entry)
         return cset
 
-    def get_entry_parent(self, entry, inventory):
-        if entry is None:
-            return None
-        if entry.path == "./.":
-            return NULL_ID
-        dirname = os.path.dirname(entry.path)
-        if dirname == ".":
-            dirname = "./."
-        parent = inventory[dirname]
-        return parent.id
-
-    def get_paths(self, entry, tree):
-        if entry is None:
-            return (None, None)
-        full_path = tree.readonly_path(entry.id)
-        if entry.path == ".":
-            return ("", full_path)
-        return (entry.path, full_path)
-
-    def make_basic_entry(self, id, only_interesting):
-        entry_a = self.r_inventory_a.get(id)
-        entry_b = self.r_inventory_b.get(id)
+    def iter_inventory(self, tree):
+        for file_id in tree:
+            yield self.get_entry(file_id, tree)
+
+    def get_entry(self, file_id, tree):
+        if not tree.has_or_had_id(file_id):
+            return None
+        return tree.tree.inventory[file_id]
+
+    def get_entry_parent(self, entry):
+        if entry is None:
+            return None
+        return entry.parent_id
+
+    def get_path(self, file_id, tree):
+        if not tree.has_or_had_id(file_id):
+            return None
+        path = tree.id2path(file_id)
+        if path == '':
+            return './.'
+        else:
+            return path
+
+    def make_basic_entry(self, file_id, only_interesting):
+        entry_a = self.get_entry(file_id, self.tree_a)
+        entry_b = self.get_entry(file_id, self.tree_b)
         if only_interesting and not self.is_interesting(entry_a, entry_b):
-            return (None, None, None)
-        parent = self.get_entry_parent(entry_a, self.inventory_a)
-        (path, full_path_a) = self.get_paths(entry_a, self.tree_a)
-        cs_entry = ChangesetEntry(id, parent, path)
-        new_parent = self.get_entry_parent(entry_b, self.inventory_b)
-
-
-        (new_path, full_path_b) = self.get_paths(entry_b, self.tree_b)
+            return None
+        parent = self.get_entry_parent(entry_a)
+        path = self.get_path(file_id, self.tree_a)
+        cs_entry = ChangesetEntry(file_id, parent, path)
+        new_parent = self.get_entry_parent(entry_b)
+
+        new_path = self.get_path(file_id, self.tree_b)
 
         cs_entry.new_path = new_path
         cs_entry.new_parent = new_parent
-        return (cs_entry, full_path_a, full_path_b)
+        return cs_entry
 
     def is_interesting(self, entry_a, entry_b):
+        if self._interesting_ids is None:
+            return True
         if entry_a is not None:
-            if entry_a.interesting:
-                return True
-        if entry_b is not None:
-            if entry_b.interesting:
-                return True
-        return False
+            file_id = entry_a.file_id
+        elif entry_b is not None:
+            file_id = entry_b.file_id
+        else:
+            return False
+        return file_id in self._interesting_ids
 
     def make_boring_entry(self, id):
-        (cs_entry, full_path_a, full_path_b) = \
-            self.make_basic_entry(id, only_interesting=False)
+        cs_entry = self.make_basic_entry(id, only_interesting=False)
         if cs_entry.is_creation_or_deletion():
             return self.make_entry(id, only_interesting=False)
         else:
@@ -1426,17 +1383,20 @@
         
 
     def make_entry(self, id, only_interesting=True):
-        (cs_entry, full_path_a, full_path_b) = \
-            self.make_basic_entry(id, only_interesting)
+        cs_entry = self.make_basic_entry(id, only_interesting)
 
         if cs_entry is None:
             return None
-       
+        if id in self.tree_a and id in self.tree_b:
+            a_sha1 = self.tree_a.get_file_sha1(id)
+            b_sha1 = self.tree_b.get_file_sha1(id)
+            if None not in (a_sha1, b_sha1) and a_sha1 == b_sha1:
+                return cs_entry
+
+        full_path_a = self.tree_a.readonly_path(id)
+        full_path_b = self.tree_b.readonly_path(id)
         stat_a = self.lstat(full_path_a)
         stat_b = self.lstat(full_path_b)
-        if stat_b is None:
-            cs_entry.new_parent = None
-            cs_entry.new_path = None
         
         cs_entry.metadata_change = self.make_mode_change(stat_a, stat_b)
         cs_entry.contents_change = self.make_contents_change(full_path_a,
@@ -1467,15 +1427,6 @@
             if stat_a.st_ino == stat_b.st_ino and \
                 stat_a.st_dev == stat_b.st_dev:
                 return None
-            if file(full_path_a, "rb").read() == \
-                file(full_path_b, "rb").read():
-                return None
-
-            patch_contents = patch.diff(full_path_a, 
-                                        file(full_path_b, "rb").read())
-            if patch_contents is None:
-                return None
-            return PatchApply(patch_contents)
 
         a_contents = self.get_contents(stat_a, full_path_a)
         b_contents = self.get_contents(stat_b, full_path_b)

=== modified file 'bzrlib/check.py'
--- bzrlib/check.py	2005-07-11 07:33:45 +0000
+++ bzrlib/check.py	2005-09-04 02:59:56 +0000
@@ -15,6 +15,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+import bzrlib.ui
+from bzrlib.trace import note, warning
 
 def _update_store_entry(obj, obj_id, branch, store_name, store):
     """This is just a meta-function, which handles both revision entries
@@ -73,6 +75,7 @@
     _update_store_entry(inv, inv_id, branch,
             'inventory-store', branch.inventory_store)
 
+
 def check(branch):
     """Run consistency checks on a branch.
 
@@ -81,20 +84,19 @@
     TODO: Check for extra files in the control directory.
     """
     from bzrlib.trace import mutter
-    from bzrlib.errors import BzrCheckError
+    from bzrlib.errors import BzrCheckError, NoSuchRevision
     from bzrlib.osutils import fingerprint_file
-    from bzrlib.progress import ProgressBar
     from bzrlib.inventory import ROOT_ID
     from bzrlib.branch import gen_root_id
 
     branch.lock_read()
 
     try:
-        pb = ProgressBar(show_spinner=True)
         last_rev_id = None
 
         missing_inventory_sha_cnt = 0
         missing_revision_sha_cnt = 0
+        missing_revision_cnt = 0
 
         history = branch.revision_history()
         revno = 0
@@ -104,10 +106,12 @@
         # for all texts checked, text_id -> sha1
         checked_texts = {}
 
+        progress = bzrlib.ui.ui_factory.progress_bar()
+
         for rev_id in history:
             revno += 1
-            pb.update('checking revision', revno, revcount)
-            mutter('    revision {%s}' % rev_id)
+            progress.update('checking revision', revno, revcount)
+            # mutter('    revision {%s}' % rev_id)
             rev = branch.get_revision(rev_id)
             if rev.revision_id != rev_id:
                 raise BzrCheckError('wrong internal revision id in revision {%s}'
@@ -132,7 +136,15 @@
                         missing_revision_sha_cnt += 1
                         continue
                     prid = prr.revision_id
-                    actual_sha = branch.get_revision_sha1(prid)
+                    
+                    try:
+                        actual_sha = branch.get_revision_sha1(prid)
+                    except NoSuchRevision:
+                        missing_revision_cnt += 1
+                        mutter("parent {%s} of {%s} not present in store",
+                               prid, rev_id)
+                        continue
+                        
                     if prr.revision_sha1 != actual_sha:
                         raise BzrCheckError("mismatched revision sha1 for "
                                             "parent {%s} of {%s}: %s vs %s"
@@ -173,7 +185,7 @@
             for file_id in inv:
                 i += 1
                 if i & 31 == 0:
-                    pb.tick()
+                    progress.tick()
 
                 ie = inv[file_id]
 
@@ -202,7 +214,7 @@
                         raise BzrCheckError('directory {%s} has text in revision {%s}'
                                 % (file_id, rev_id))
 
-            pb.tick()
+            progress.tick()
             for path, ie in inv.iter_entries():
                 if path in seen_names:
                     raise BzrCheckError('duplicated path %s '
@@ -214,21 +226,29 @@
     finally:
         branch.unlock()
 
-    pb.clear()
+    progress.clear()
 
-    print 'checked %d revisions, %d file texts' % (revcount, len(checked_texts))
+    note('checked %d revisions, %d file texts' % (revcount, len(checked_texts)))
     
     if missing_inventory_sha_cnt:
-        print '%d revisions are missing inventory_sha1' % missing_inventory_sha_cnt
+        note('%d revisions are missing inventory_sha1' % missing_inventory_sha_cnt)
 
     if missing_revision_sha_cnt:
-        print '%d parent links are missing revision_sha1' % missing_revision_sha_cnt
-
-    if (missing_inventory_sha_cnt
-        or missing_revision_sha_cnt):
-        print '  (use "bzr upgrade" to fix them)'
+        note('%d parent links are missing revision_sha1' % missing_revision_sha_cnt)
+
+    if missing_revision_cnt:
+        note('%d revisions are mentioned but not present' % missing_revision_cnt)
+
+    if missing_revision_cnt:
+        print '%d revisions are mentioned but not present' % missing_revision_cnt
+
+    # stub this out for now because the main bzr branch has references
+    # to revisions that aren't present in the store -- mbp 20050804
+#    if (missing_inventory_sha_cnt
+#        or missing_revision_sha_cnt):
+#        print '  (use "bzr upgrade" to fix them)'
 
     if mismatch_inv_id:
-        print '%d revisions have mismatched inventory ids:' % len(mismatch_inv_id)
+        warning('%d revisions have mismatched inventory ids:' % len(mismatch_inv_id))
         for rev_id in mismatch_inv_id:
-            print '  ', rev_id
+            warning('  %s', rev_id)

=== modified file 'bzrlib/commands.py'
--- bzrlib/commands.py	2005-07-29 17:19:16 +0000
+++ bzrlib/commands.py	2005-09-04 02:59:56 +0000
@@ -15,16 +15,29 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-
-import sys, os
+# TODO: probably should say which arguments are candidates for glob
+# expansion on windows and do that at the command level.
+
+# TODO: Help messages for options.
+
+# TODO: Define arguments by objects, rather than just using names.
+# Those objects can specify the expected type of the argument, which
+# would help with validation and shell completion.
+
+
+
+import sys
+import os
+from warnings import warn
+from inspect import getdoc
 
 import bzrlib
+import bzrlib.trace
 from bzrlib.trace import mutter, note, log_error, warning
 from bzrlib.errors import BzrError, BzrCheckError, BzrCommandError
 from bzrlib.branch import find_branch
 from bzrlib import BZRDIR
 
-
 plugin_cmds = {}
 
 
@@ -38,6 +51,7 @@
         k_unsquished = k
     if not plugin_cmds.has_key(k_unsquished):
         plugin_cmds[k_unsquished] = cmd
+        mutter('registered plugin command %s', k_unsquished)      
     else:
         log_error('Two plugins defined the same command: %r' % k)
         log_error('Not loading the one in %r' % sys.modules[cmd.__module__])
@@ -136,19 +150,46 @@
         raise BzrCommandError(msg)
     
 
+def get_merge_type(typestring):
+    """Attempt to find the merge class/factory associated with a string."""
+    from merge import merge_types
+    try:
+        return merge_types[typestring][0]
+    except KeyError:
+        templ = '%s%%7s: %%s' % (' '*12)
+        lines = [templ % (f[0], f[1][1]) for f in merge_types.iteritems()]
+        type_list = '\n'.join(lines)
+        msg = "No known merge type %s. Supported types are:\n%s" %\
+            (typestring, type_list)
+        raise BzrCommandError(msg)
+
+
+def _builtin_commands():
+    import bzrlib.builtins
+    r = {}
+    builtins = bzrlib.builtins.__dict__
+    for name in builtins:
+        if name.startswith("cmd_"):
+            real_name = _unsquish_command_name(name)        
+            r[real_name] = builtins[name]
+    return r
+
+            
+
+def builtin_command_names():
+    """Return list of builtin command names."""
+    return _builtin_commands().keys()
+    
+
+def plugin_command_names():
+    return plugin_cmds.keys()
+
 
 def _get_cmd_dict(plugins_override=True):
-    d = {}
-    for k, v in globals().iteritems():
-        if k.startswith("cmd_"):
-            d[_unsquish_command_name(k)] = v
-    # If we didn't load plugins, the plugin_cmds dict will be empty
+    """Return name->class mapping for all commands."""
+    d = _builtin_commands()
     if plugins_override:
         d.update(plugin_cmds)
-    else:
-        d2 = plugin_cmds.copy()
-        d2.update(d)
-        d = d2
     return d
 
     
@@ -158,37 +199,56 @@
         yield k,v
 
 
-def get_cmd_class(cmd, plugins_override=True):
+def get_cmd_object(cmd_name, plugins_override=True):
     """Return the canonical name and command class for a command.
+
+    plugins_override
+        If true, plugin commands can override builtins.
     """
-    cmd = str(cmd)                      # not unicode
+    from bzrlib.externalcommand import ExternalCommand
+
+    cmd_name = str(cmd_name)            # not unicode
 
     # first look up this command under the specified name
     cmds = _get_cmd_dict(plugins_override=plugins_override)
     try:
-        return cmd, cmds[cmd]
+        return cmds[cmd_name]()
     except KeyError:
         pass
 
     # look for any command which claims this as an alias
-    for cmdname, cmdclass in cmds.iteritems():
-        if cmd in cmdclass.aliases:
-            return cmdname, cmdclass
-
-    cmdclass = ExternalCommand.find_command(cmd)
-    if cmdclass:
-        return cmd, cmdclass
-
-    raise BzrCommandError("unknown command %r" % cmd)
+    for real_cmd_name, cmd_class in cmds.iteritems():
+        if cmd_name in cmd_class.aliases:
+            return cmd_class()
+
+    cmd_obj = ExternalCommand.find_command(cmd_name)
+    if cmd_obj:
+        return cmd_obj
+
+    raise BzrCommandError("unknown command %r" % cmd_name)
 
 
 class Command(object):
     """Base class for commands.
 
+    Commands are the heart of the command-line bzr interface.
+
+    The command object mostly handles the mapping of command-line
+    parameters into one or more bzrlib operations, and of the results
+    into textual output.
+
+    Commands normally don't have any state.  All their arguments are
+    passed in to the run method.  (Subclasses may take a different
+    policy if the behaviour of the instance needs to depend on e.g. a
+    shell plugin and not just its Python class.)
+
     The docstring for an actual command should give a single-line
     summary, then a complete description of the command.  A grammar
     description will be inserted.
 
+    aliases
+        Other accepted names for this command.
+
     takes_args
         List of argument forms, marked with whether they are optional,
         repeated, etc.
@@ -197,7 +257,8 @@
         List of options that may be given for this command.
 
     hidden
-        If true, this command isn't advertised.
+        If true, this command isn't advertised.  This is typically
+        for commands intended for expert users.
     """
     aliases = []
     
@@ -206,1150 +267,62 @@
 
     hidden = False
     
-    def __init__(self, options, arguments):
-        """Construct and run the command.
-
-        Sets self.status to the return value of run()."""
-        assert isinstance(options, dict)
-        assert isinstance(arguments, dict)
-        cmdargs = options.copy()
-        cmdargs.update(arguments)
+    def __init__(self):
+        """Construct an instance of this command."""
         if self.__doc__ == Command.__doc__:
-            from warnings import warn
             warn("No help message set for %r" % self)
-        self.status = self.run(**cmdargs)
-        if self.status is None:
-            self.status = 0
+
+
+    def run_argv(self, argv):
+        """Parse command line and run."""
+        args, opts = parse_args(argv)
+
+        if 'help' in opts:  # e.g. bzr add --help
+            from bzrlib.help import help_on_command
+            help_on_command(self.name())
+            return 0
+
+        # check options are reasonable
+        allowed = self.takes_options
+        for oname in opts:
+            if oname not in allowed:
+                raise BzrCommandError("option '--%s' is not allowed for command %r"
+                                      % (oname, self.name()))
+
+        # mix arguments and options into one dictionary
+        cmdargs = _match_argform(self.name(), self.takes_args, args)
+        cmdopts = {}
+        for k, v in opts.items():
+            cmdopts[k.replace('-', '_')] = v
+
+        all_cmd_args = cmdargs.copy()
+        all_cmd_args.update(cmdopts)
+
+        return self.run(**all_cmd_args)
 
     
     def run(self):
-        """Override this in sub-classes.
+        """Actually run the command.
 
         This is invoked with the options and arguments bound to
         keyword parameters.
 
-        Return 0 or None if the command was successful, or a shell
-        error code if not.
+        Return 0 or None if the command was successful, or a non-zero
+        shell error code if not.  It's OK for this method to allow
+        an exception to raise up.
         """
-        return 0
-
-
-class ExternalCommand(Command):
-    """Class to wrap external commands.
-
-    We cheat a little here, when get_cmd_class() calls us we actually give it back
-    an object we construct that has the appropriate path, help, options etc for the
-    specified command.
-
-    When run_bzr() tries to instantiate that 'class' it gets caught by the __call__
-    method, which we override to call the Command.__init__ method. That then calls
-    our run method which is pretty straight forward.
-
-    The only wrinkle is that we have to map bzr's dictionary of options and arguments
-    back into command line options and arguments for the script.
-    """
-
-    def find_command(cls, cmd):
-        import os.path
-        bzrpath = os.environ.get('BZRPATH', '')
-
-        for dir in bzrpath.split(os.pathsep):
-            path = os.path.join(dir, cmd)
-            if os.path.isfile(path):
-                return ExternalCommand(path)
-
-        return None
-
-    find_command = classmethod(find_command)
-
-    def __init__(self, path):
-        self.path = path
-
-        pipe = os.popen('%s --bzr-usage' % path, 'r')
-        self.takes_options = pipe.readline().split()
-
-        for opt in self.takes_options:
-            if not opt in OPTIONS:
-                raise BzrError("Unknown option '%s' returned by external command %s"
-                               % (opt, path))
-
-        # TODO: Is there any way to check takes_args is valid here?
-        self.takes_args = pipe.readline().split()
-
-        if pipe.close() is not None:
-            raise BzrError("Failed funning '%s --bzr-usage'" % path)
-
-        pipe = os.popen('%s --bzr-help' % path, 'r')
-        self.__doc__ = pipe.read()
-        if pipe.close() is not None:
-            raise BzrError("Failed funning '%s --bzr-help'" % path)
-
-    def __call__(self, options, arguments):
-        Command.__init__(self, options, arguments)
-        return self
-
-    def run(self, **kargs):
-        opts = []
-        args = []
-
-        keys = kargs.keys()
-        keys.sort()
-        for name in keys:
-            optname = name.replace('_','-')
-            value = kargs[name]
-            if OPTIONS.has_key(optname):
-                # it's an option
-                opts.append('--%s' % optname)
-                if value is not None and value is not True:
-                    opts.append(str(value))
-            else:
-                # it's an arg, or arg list
-                if type(value) is not list:
-                    value = [value]
-                for v in value:
-                    if v is not None:
-                        args.append(str(v))
-
-        self.status = os.spawnv(os.P_WAIT, self.path, [self.path] + opts + args)
-        return self.status
-
-
-class cmd_status(Command):
-    """Display status summary.
-
-    This reports on versioned and unknown files, reporting them
-    grouped by state.  Possible states are:
-
-    added
-        Versioned in the working copy but not in the previous revision.
-
-    removed
-        Versioned in the previous revision but removed or deleted
-        in the working copy.
-
-    renamed
-        Path of this file changed from the previous revision;
-        the text may also have changed.  This includes files whose
-        parent directory was renamed.
-
-    modified
-        Text has changed since the previous revision.
-
-    unchanged
-        Nothing about this file has changed since the previous revision.
-        Only shown with --all.
-
-    unknown
-        Not versioned and not matching an ignore pattern.
-
-    To see ignored files use 'bzr ignored'.  For details in the
-    changes to file texts, use 'bzr diff'.
-
-    If no arguments are specified, the status of the entire working
-    directory is shown.  Otherwise, only the status of the specified
-    files or directories is reported.  If a directory is given, status
-    is reported for everything inside that directory.
-
-    If a revision is specified, the changes since that revision are shown.
-    """
-    takes_args = ['file*']
-    takes_options = ['all', 'show-ids', 'revision']
-    aliases = ['st', 'stat']
-    
-    def run(self, all=False, show_ids=False, file_list=None):
-        if file_list:
-            b = find_branch(file_list[0])
-            file_list = [b.relpath(x) for x in file_list]
-            # special case: only one path was given and it's the root
-            # of the branch
-            if file_list == ['']:
-                file_list = None
-        else:
-            b = find_branch('.')
-            
-        from bzrlib.status import show_status
-        show_status(b, show_unchanged=all, show_ids=show_ids,
-                    specific_files=file_list)
-
-
-class cmd_cat_revision(Command):
-    """Write out metadata for a revision."""
-
-    hidden = True
-    takes_args = ['revision_id']
-    
-    def run(self, revision_id):
-        from bzrlib.xml import pack_xml
-        pack_xml(find_branch('.').get_revision(revision_id), sys.stdout)
-
-
-class cmd_revno(Command):
-    """Show current revision number.
-
-    This is equal to the number of revisions on this branch."""
-    def run(self):
-        print find_branch('.').revno()
-
-class cmd_revision_info(Command):
-    """Show revision number and revision id for a given revision identifier.
-    """
-    hidden = True
-    takes_args = ['revision_info*']
-    takes_options = ['revision']
-    def run(self, revision=None, revision_info_list=None):
-        from bzrlib.branch import find_branch
-
-        revs = []
-        if revision is not None:
-            revs.extend(revision)
-        if revision_info_list is not None:
-            revs.extend(revision_info_list)
-        if len(revs) == 0:
-            raise BzrCommandError('You must supply a revision identifier')
-
-        b = find_branch('.')
-
-        for rev in revs:
-            print '%4d %s' % b.get_revision_info(rev)
-
-    
-class cmd_add(Command):
-    """Add specified files or directories.
-
-    In non-recursive mode, all the named items are added, regardless
-    of whether they were previously ignored.  A warning is given if
-    any of the named files are already versioned.
-
-    In recursive mode (the default), files are treated the same way
-    but the behaviour for directories is different.  Directories that
-    are already versioned do not give a warning.  All directories,
-    whether already versioned or not, are searched for files or
-    subdirectories that are neither versioned or ignored, and these
-    are added.  This search proceeds recursively into versioned
-    directories.  If no names are given '.' is assumed.
-
-    Therefore simply saying 'bzr add' will version all files that
-    are currently unknown.
-
-    TODO: Perhaps adding a file whose directly is not versioned should
-    recursively add that parent, rather than giving an error?
-    """
-    takes_args = ['file*']
-    takes_options = ['verbose', 'no-recurse']
-    
-    def run(self, file_list, verbose=False, no_recurse=False):
-        from bzrlib.add import smart_add
-        smart_add(file_list, verbose, not no_recurse)
-
-
-
-class cmd_mkdir(Command):
-    """Create a new versioned directory.
-
-    This is equivalent to creating the directory and then adding it.
-    """
-    takes_args = ['dir+']
-
-    def run(self, dir_list):
-        b = None
-        
-        for d in dir_list:
-            os.mkdir(d)
-            if not b:
-                b = find_branch(d)
-            b.add([d], verbose=True)
-
-
-class cmd_relpath(Command):
-    """Show path of a file relative to root"""
-    takes_args = ['filename']
-    hidden = True
-    
-    def run(self, filename):
-        print find_branch(filename).relpath(filename)
-
-
-
-class cmd_inventory(Command):
-    """Show inventory of the current working copy or a revision."""
-    takes_options = ['revision', 'show-ids']
-    
-    def run(self, revision=None, show_ids=False):
-        b = find_branch('.')
-        if revision == None:
-            inv = b.read_working_inventory()
-        else:
-            if len(revision) > 1:
-                raise BzrCommandError('bzr inventory --revision takes'
-                    ' exactly one revision identifier')
-            inv = b.get_revision_inventory(b.lookup_revision(revision[0]))
-
-        for path, entry in inv.entries():
-            if show_ids:
-                print '%-50s %s' % (path, entry.file_id)
-            else:
-                print path
-
-
-class cmd_move(Command):
-    """Move files to a different directory.
-
-    examples:
-        bzr move *.txt doc
-
-    The destination must be a versioned directory in the same branch.
-    """
-    takes_args = ['source$', 'dest']
-    def run(self, source_list, dest):
-        b = find_branch('.')
-
-        b.move([b.relpath(s) for s in source_list], b.relpath(dest))
-
-
-class cmd_rename(Command):
-    """Change the name of an entry.
-
-    examples:
-      bzr rename frob.c frobber.c
-      bzr rename src/frob.c lib/frob.c
-
-    It is an error if the destination name exists.
-
-    See also the 'move' command, which moves files into a different
-    directory without changing their name.
-
-    TODO: Some way to rename multiple files without invoking bzr for each
-    one?"""
-    takes_args = ['from_name', 'to_name']
-    
-    def run(self, from_name, to_name):
-        b = find_branch('.')
-        b.rename_one(b.relpath(from_name), b.relpath(to_name))
-
-
-
-
-
-class cmd_pull(Command):
-    """Pull any changes from another branch into the current one.
-
-    If the location is omitted, the last-used location will be used.
-    Both the revision history and the working directory will be
-    updated.
-
-    This command only works on branches that have not diverged.  Branches are
-    considered diverged if both branches have had commits without first
-    pulling from the other.
-
-    If branches have diverged, you can use 'bzr merge' to pull the text changes
-    from one into the other.
-    """
-    takes_args = ['location?']
-
-    def run(self, location=None):
-        from bzrlib.merge import merge
-        import tempfile
-        from shutil import rmtree
-        import errno
-        
-        br_to = find_branch('.')
-        stored_loc = None
-        try:
-            stored_loc = br_to.controlfile("x-pull", "rb").read().rstrip('\n')
-        except IOError, e:
-            if e.errno != errno.ENOENT:
-                raise
-        if location is None:
-            if stored_loc is None:
-                raise BzrCommandError("No pull location known or specified.")
-            else:
-                print "Using last location: %s" % stored_loc
-                location = stored_loc
-        cache_root = tempfile.mkdtemp()
-        from bzrlib.branch import DivergedBranches
-        br_from = find_branch(location)
-        location = pull_loc(br_from)
-        old_revno = br_to.revno()
-        try:
-            from branch import find_cached_branch, DivergedBranches
-            br_from = find_cached_branch(location, cache_root)
-            location = pull_loc(br_from)
-            old_revno = br_to.revno()
-            try:
-                br_to.update_revisions(br_from)
-            except DivergedBranches:
-                raise BzrCommandError("These branches have diverged."
-                    "  Try merge.")
-                
-            merge(('.', -1), ('.', old_revno), check_clean=False)
-            if location != stored_loc:
-                br_to.controlfile("x-pull", "wb").write(location + "\n")
-        finally:
-            rmtree(cache_root)
-
-
-
-class cmd_branch(Command):
-    """Create a new copy of a branch.
-
-    If the TO_LOCATION is omitted, the last component of the FROM_LOCATION will
-    be used.  In other words, "branch ../foo/bar" will attempt to create ./bar.
-
-    To retrieve the branch as of a particular revision, supply the --revision
-    parameter, as in "branch foo/bar -r 5".
-    """
-    takes_args = ['from_location', 'to_location?']
-    takes_options = ['revision']
-
-    def run(self, from_location, to_location=None, revision=None):
-        import errno
-        from bzrlib.merge import merge
-        from bzrlib.branch import DivergedBranches, NoSuchRevision, \
-             find_cached_branch, Branch
-        from shutil import rmtree
-        from meta_store import CachedStore
-        import tempfile
-        cache_root = tempfile.mkdtemp()
-
-        if revision is None:
-            revision = [None]
-        elif len(revision) > 1:
-            raise BzrCommandError('bzr branch --revision takes exactly 1 revision value')
-
-        try:
-            try:
-                br_from = find_cached_branch(from_location, cache_root)
-            except OSError, e:
-                if e.errno == errno.ENOENT:
-                    raise BzrCommandError('Source location "%s" does not'
-                                          ' exist.' % to_location)
-                else:
-                    raise
-
-            if to_location is None:
-                to_location = os.path.basename(from_location.rstrip("/\\"))
-
-            try:
-                os.mkdir(to_location)
-            except OSError, e:
-                if e.errno == errno.EEXIST:
-                    raise BzrCommandError('Target directory "%s" already'
-                                          ' exists.' % to_location)
-                if e.errno == errno.ENOENT:
-                    raise BzrCommandError('Parent of "%s" does not exist.' %
-                                          to_location)
-                else:
-                    raise
-            br_to = Branch(to_location, init=True)
-
-            br_to.set_root_id(br_from.get_root_id())
-
-            if revision:
-                if revision[0] is None:
-                    revno = br_from.revno()
-                else:
-                    revno, rev_id = br_from.get_revision_info(revision[0])
-                try:
-                    br_to.update_revisions(br_from, stop_revision=revno)
-                except NoSuchRevision:
-                    rmtree(to_location)
-                    msg = "The branch %s has no revision %d." % (from_location,
-                                                                 revno)
-                    raise BzrCommandError(msg)
-            
-            merge((to_location, -1), (to_location, 0), this_dir=to_location,
-                  check_clean=False, ignore_zero=True)
-            from_location = pull_loc(br_from)
-            br_to.controlfile("x-pull", "wb").write(from_location + "\n")
-        finally:
-            rmtree(cache_root)
-
-
-def pull_loc(branch):
-    # TODO: Should perhaps just make attribute be 'base' in
-    # RemoteBranch and Branch?
-    if hasattr(branch, "baseurl"):
-        return branch.baseurl
-    else:
-        return branch.base
-
-
-
-class cmd_renames(Command):
-    """Show list of renamed files.
-
-    TODO: Option to show renames between two historical versions.
-
-    TODO: Only show renames under dir, rather than in the whole branch.
-    """
-    takes_args = ['dir?']
-
-    def run(self, dir='.'):
-        b = find_branch(dir)
-        old_inv = b.basis_tree().inventory
-        new_inv = b.read_working_inventory()
-
-        renames = list(bzrlib.tree.find_renames(old_inv, new_inv))
-        renames.sort()
-        for old_name, new_name in renames:
-            print "%s => %s" % (old_name, new_name)        
-
-
-class cmd_info(Command):
-    """Show statistical information about a branch."""
-    takes_args = ['branch?']
-    
-    def run(self, branch=None):
-        import info
-
-        b = find_branch(branch)
-        info.show_info(b)
-
-
-class cmd_remove(Command):
-    """Make a file unversioned.
-
-    This makes bzr stop tracking changes to a versioned file.  It does
-    not delete the working copy.
-    """
-    takes_args = ['file+']
-    takes_options = ['verbose']
-    
-    def run(self, file_list, verbose=False):
-        b = find_branch(file_list[0])
-        b.remove([b.relpath(f) for f in file_list], verbose=verbose)
-
-
-class cmd_file_id(Command):
-    """Print file_id of a particular file or directory.
-
-    The file_id is assigned when the file is first added and remains the
-    same through all revisions where the file exists, even when it is
-    moved or renamed.
-    """
-    hidden = True
-    takes_args = ['filename']
-    def run(self, filename):
-        b = find_branch(filename)
-        i = b.inventory.path2id(b.relpath(filename))
-        if i == None:
-            raise BzrError("%r is not a versioned file" % filename)
-        else:
-            print i
-
-
-class cmd_file_path(Command):
-    """Print path of file_ids to a file or directory.
-
-    This prints one line for each directory down to the target,
-    starting at the branch root."""
-    hidden = True
-    takes_args = ['filename']
-    def run(self, filename):
-        b = find_branch(filename)
-        inv = b.inventory
-        fid = inv.path2id(b.relpath(filename))
-        if fid == None:
-            raise BzrError("%r is not a versioned file" % filename)
-        for fip in inv.get_idpath(fid):
-            print fip
-
-
-class cmd_revision_history(Command):
-    """Display list of revision ids on this branch."""
-    hidden = True
-    def run(self):
-        for patchid in find_branch('.').revision_history():
-            print patchid
-
-
-class cmd_directories(Command):
-    """Display list of versioned directories in this branch."""
-    def run(self):
-        for name, ie in find_branch('.').read_working_inventory().directories():
-            if name == '':
-                print '.'
-            else:
-                print name
-
-
-class cmd_init(Command):
-    """Make a directory into a versioned branch.
-
-    Use this to create an empty branch, or before importing an
-    existing project.
-
-    Recipe for importing a tree of files:
-        cd ~/project
-        bzr init
-        bzr add -v .
-        bzr status
-        bzr commit -m 'imported project'
-    """
-    def run(self):
-        from bzrlib.branch import Branch
-        Branch('.', init=True)
-
-
-class cmd_diff(Command):
-    """Show differences in working tree.
-    
-    If files are listed, only the changes in those files are listed.
-    Otherwise, all changes for the tree are listed.
-
-    TODO: Given two revision arguments, show the difference between them.
-
-    TODO: Allow diff across branches.
-
-    TODO: Option to use external diff command; could be GNU diff, wdiff,
-          or a graphical diff.
-
-    TODO: Python difflib is not exactly the same as unidiff; should
-          either fix it up or prefer to use an external diff.
-
-    TODO: If a directory is given, diff everything under that.
-
-    TODO: Selected-file diff is inefficient and doesn't show you
-          deleted files.
-
-    TODO: This probably handles non-Unix newlines poorly.
-    """
-    
-    takes_args = ['file*']
-    takes_options = ['revision', 'diff-options']
-    aliases = ['di', 'dif']
-
-    def run(self, revision=None, file_list=None, diff_options=None):
-        from bzrlib.diff import show_diff
-
-        if file_list:
-            b = find_branch(file_list[0])
-            file_list = [b.relpath(f) for f in file_list]
-            if file_list == ['']:
-                # just pointing to top-of-tree
-                file_list = None
-        else:
-            b = find_branch('.')
-
-        # TODO: Make show_diff support taking 2 arguments
-        base_rev = None
-        if revision is not None:
-            if len(revision) != 1:
-                raise BzrCommandError('bzr diff --revision takes exactly one revision identifier')
-            base_rev = revision[0]
-    
-        show_diff(b, base_rev, specific_files=file_list,
-                  external_diff_options=diff_options)
-
-
-        
-
-
-class cmd_deleted(Command):
-    """List files deleted in the working tree.
-
-    TODO: Show files deleted since a previous revision, or between two revisions.
-    """
-    def run(self, show_ids=False):
-        b = find_branch('.')
-        old = b.basis_tree()
-        new = b.working_tree()
-
-        ## TODO: Much more efficient way to do this: read in new
-        ## directories with readdir, rather than stating each one.  Same
-        ## level of effort but possibly much less IO.  (Or possibly not,
-        ## if the directories are very large...)
-
-        for path, ie in old.inventory.iter_entries():
-            if not new.has_id(ie.file_id):
-                if show_ids:
-                    print '%-50s %s' % (path, ie.file_id)
-                else:
-                    print path
-
-
-class cmd_modified(Command):
-    """List files modified in working tree."""
-    hidden = True
-    def run(self):
-        from bzrlib.diff import compare_trees
-
-        b = find_branch('.')
-        td = compare_trees(b.basis_tree(), b.working_tree())
-
-        for path, id, kind in td.modified:
-            print path
-
-
-
-class cmd_added(Command):
-    """List files added in working tree."""
-    hidden = True
-    def run(self):
-        b = find_branch('.')
-        wt = b.working_tree()
-        basis_inv = b.basis_tree().inventory
-        inv = wt.inventory
-        for file_id in inv:
-            if file_id in basis_inv:
-                continue
-            path = inv.id2path(file_id)
-            if not os.access(b.abspath(path), os.F_OK):
-                continue
-            print path
-                
-        
-
-class cmd_root(Command):
-    """Show the tree root directory.
-
-    The root is the nearest enclosing directory with a .bzr control
-    directory."""
-    takes_args = ['filename?']
-    def run(self, filename=None):
-        """Print the branch root."""
-        b = find_branch(filename)
-        print getattr(b, 'base', None) or getattr(b, 'baseurl')
-
-
-class cmd_log(Command):
-    """Show log of this branch.
-
-    To request a range of logs, you can use the command -r begin:end
-    -r revision requests a specific revision, -r :end or -r begin: are
-    also valid.
-
-    --message allows you to give a regular expression, which will be evaluated
-    so that only matching entries will be displayed.
-
-    TODO: Make --revision support uuid: and hash: [future tag:] notation.
-  
-    """
-
-    takes_args = ['filename?']
-    takes_options = ['forward', 'timezone', 'verbose', 'show-ids', 'revision','long', 'message']
-    
-    def run(self, filename=None, timezone='original',
-            verbose=False,
-            show_ids=False,
-            forward=False,
-            revision=None,
-            message=None,
-            long=False):
-        from bzrlib.branch import find_branch
-        from bzrlib.log import log_formatter, show_log
-        import codecs
-
-        direction = (forward and 'forward') or 'reverse'
-        
-        if filename:
-            b = find_branch(filename)
-            fp = b.relpath(filename)
-            if fp:
-                file_id = b.read_working_inventory().path2id(fp)
-            else:
-                file_id = None  # points to branch root
-        else:
-            b = find_branch('.')
-            file_id = None
-
-        if revision is None:
-            rev1 = None
-            rev2 = None
-        elif len(revision) == 1:
-            rev1 = rev2 = b.get_revision_info(revision[0])[0]
-        elif len(revision) == 2:
-            rev1 = b.get_revision_info(revision[0])[0]
-            rev2 = b.get_revision_info(revision[1])[0]
-        else:
-            raise BzrCommandError('bzr log --revision takes one or two values.')
-
-        if rev1 == 0:
-            rev1 = None
-        if rev2 == 0:
-            rev2 = None
-
-        mutter('encoding log as %r' % bzrlib.user_encoding)
-
-        # use 'replace' so that we don't abort if trying to write out
-        # in e.g. the default C locale.
-        outf = codecs.getwriter(bzrlib.user_encoding)(sys.stdout, errors='replace')
-
-        if long:
-            log_format = 'long'
-        else:
-            log_format = 'short'
-        lf = log_formatter(log_format,
-                           show_ids=show_ids,
-                           to_file=outf,
-                           show_timezone=timezone)
-
-        show_log(b,
-                 lf,
-                 file_id,
-                 verbose=verbose,
-                 direction=direction,
-                 start_revision=rev1,
-                 end_revision=rev2,
-                 search=message)
-
-
-
-class cmd_touching_revisions(Command):
-    """Return revision-ids which affected a particular file.
-
-    A more user-friendly interface is "bzr log FILE"."""
-    hidden = True
-    takes_args = ["filename"]
-    def run(self, filename):
-        b = find_branch(filename)
-        inv = b.read_working_inventory()
-        file_id = inv.path2id(b.relpath(filename))
-        for revno, revision_id, what in bzrlib.log.find_touching_revisions(b, file_id):
-            print "%6d %s" % (revno, what)
-
-
-class cmd_ls(Command):
-    """List files in a tree.
-
-    TODO: Take a revision or remote path and list that tree instead.
-    """
-    hidden = True
-    def run(self, revision=None, verbose=False):
-        b = find_branch('.')
-        if revision == None:
-            tree = b.working_tree()
-        else:
-            tree = b.revision_tree(b.lookup_revision(revision))
-
-        for fp, fc, kind, fid in tree.list_files():
-            if verbose:
-                if kind == 'directory':
-                    kindch = '/'
-                elif kind == 'file':
-                    kindch = ''
-                else:
-                    kindch = '???'
-
-                print '%-8s %s%s' % (fc, fp, kindch)
-            else:
-                print fp
-
-
-
-class cmd_unknowns(Command):
-    """List unknown files."""
-    def run(self):
-        from bzrlib.osutils import quotefn
-        for f in find_branch('.').unknowns():
-            print quotefn(f)
-
-
-
-class cmd_ignore(Command):
-    """Ignore a command or pattern.
-
-    To remove patterns from the ignore list, edit the .bzrignore file.
-
-    If the pattern contains a slash, it is compared to the whole path
-    from the branch root.  Otherwise, it is comapred to only the last
-    component of the path.
-
-    Ignore patterns are case-insensitive on case-insensitive systems.
-
-    Note: wildcards must be quoted from the shell on Unix.
-
-    examples:
-        bzr ignore ./Makefile
-        bzr ignore '*.class'
-    """
-    takes_args = ['name_pattern']
-    
-    def run(self, name_pattern):
-        from bzrlib.atomicfile import AtomicFile
-        import os.path
-
-        b = find_branch('.')
-        ifn = b.abspath('.bzrignore')
-
-        if os.path.exists(ifn):
-            f = open(ifn, 'rt')
-            try:
-                igns = f.read().decode('utf-8')
-            finally:
-                f.close()
-        else:
-            igns = ''
-
-        # TODO: If the file already uses crlf-style termination, maybe
-        # we should use that for the newly added lines?
-
-        if igns and igns[-1] != '\n':
-            igns += '\n'
-        igns += name_pattern + '\n'
-
-        try:
-            f = AtomicFile(ifn, 'wt')
-            f.write(igns.encode('utf-8'))
-            f.commit()
-        finally:
-            f.close()
-
-        inv = b.working_tree().inventory
-        if inv.path2id('.bzrignore'):
-            mutter('.bzrignore is already versioned')
-        else:
-            mutter('need to make new .bzrignore file versioned')
-            b.add(['.bzrignore'])
-
-
-
-class cmd_ignored(Command):
-    """List ignored files and the patterns that matched them.
-
-    See also: bzr ignore"""
-    def run(self):
-        tree = find_branch('.').working_tree()
-        for path, file_class, kind, file_id in tree.list_files():
-            if file_class != 'I':
-                continue
-            ## XXX: Slightly inefficient since this was already calculated
-            pat = tree.is_ignored(path)
-            print '%-50s %s' % (path, pat)
-
-
-class cmd_lookup_revision(Command):
-    """Lookup the revision-id from a revision-number
-
-    example:
-        bzr lookup-revision 33
-    """
-    hidden = True
-    takes_args = ['revno']
-    
-    def run(self, revno):
-        try:
-            revno = int(revno)
-        except ValueError:
-            raise BzrCommandError("not a valid revision-number: %r" % revno)
-
-        print find_branch('.').lookup_revision(revno)
-
-
-class cmd_export(Command):
-    """Export past revision to destination directory.
-
-    If no revision is specified this exports the last committed revision.
-
-    Format may be an "exporter" name, such as tar, tgz, tbz2.  If none is
-    given, try to find the format with the extension. If no extension
-    is found exports to a directory (equivalent to --format=dir).
-
-    Root may be the top directory for tar, tgz and tbz2 formats. If none
-    is given, the top directory will be the root name of the file."""
-    # TODO: list known exporters
-    takes_args = ['dest']
-    takes_options = ['revision', 'format', 'root']
-    def run(self, dest, revision=None, format=None, root=None):
-        import os.path
-        b = find_branch('.')
-        if revision is None:
-            rev_id = b.last_patch()
-        else:
-            if len(revision) != 1:
-                raise BzrError('bzr export --revision takes exactly 1 argument')
-            revno, rev_id = b.get_revision_info(revision[0])
-        t = b.revision_tree(rev_id)
-        root, ext = os.path.splitext(dest)
-        if not format:
-            if ext in (".tar",):
-                format = "tar"
-            elif ext in (".gz", ".tgz"):
-                format = "tgz"
-            elif ext in (".bz2", ".tbz2"):
-                format = "tbz2"
-            else:
-                format = "dir"
-        t.export(dest, format, root)
-
-
-class cmd_cat(Command):
-    """Write a file's text from a previous revision."""
-
-    takes_options = ['revision']
-    takes_args = ['filename']
-
-    def run(self, filename, revision=None):
-        if revision == None:
-            raise BzrCommandError("bzr cat requires a revision number")
-        elif len(revision) != 1:
-            raise BzrCommandError("bzr cat --revision takes exactly one number")
-        b = find_branch('.')
-        b.print_file(b.relpath(filename), revision[0])
-
-
-class cmd_local_time_offset(Command):
-    """Show the offset in seconds from GMT to local time."""
-    hidden = True    
-    def run(self):
-        print bzrlib.osutils.local_time_offset()
-
-
-
-class cmd_commit(Command):
-    """Commit changes into a new revision.
-
-    If selected files are specified, only changes to those files are
-    committed.  If a directory is specified then its contents are also
-    committed.
-
-    A selected-file commit may fail in some cases where the committed
-    tree would be invalid, such as trying to commit a file in a
-    newly-added directory that is not itself committed.
-
-    TODO: Run hooks on tree to-be-committed, and after commit.
-
-    TODO: Strict commit that fails if there are unknown or deleted files.
-    """
-    takes_args = ['selected*']
-    takes_options = ['message', 'file', 'verbose', 'unchanged']
-    aliases = ['ci', 'checkin']
-
-    def run(self, message=None, file=None, verbose=True, selected_list=None,
-            unchanged=False):
-        from bzrlib.errors import PointlessCommit
-        from bzrlib.osutils import get_text_message
-
-        ## Warning: shadows builtin file()
-        if not message and not file:
-            import cStringIO
-            stdout = sys.stdout
-            catcher = cStringIO.StringIO()
-            sys.stdout = catcher
-            cmd_status({"file_list":selected_list}, {})
-            info = catcher.getvalue()
-            sys.stdout = stdout
-            message = get_text_message(info)
-            
-            if message is None:
-                raise BzrCommandError("please specify a commit message",
-                                      ["use either --message or --file"])
-        elif message and file:
-            raise BzrCommandError("please specify either --message or --file")
-        
-        if file:
-            import codecs
-            message = codecs.open(file, 'rt', bzrlib.user_encoding).read()
-
-        b = find_branch('.')
-
-        try:
-            b.commit(message, verbose=verbose,
-                     specific_files=selected_list,
-                     allow_pointless=unchanged)
-        except PointlessCommit:
-            # FIXME: This should really happen before the file is read in;
-            # perhaps prepare the commit; get the message; then actually commit
-            raise BzrCommandError("no changes to commit",
-                                  ["use --unchanged to commit anyhow"])
-
-
-class cmd_check(Command):
-    """Validate consistency of branch history.
-
-    This command checks various invariants about the branch storage to
-    detect data corruption or bzr bugs.
-
-    If given the --update flag, it will update some optional fields
-    to help ensure data consistency.
-    """
-    takes_args = ['dir?']
-
-    def run(self, dir='.'):
-        from bzrlib.check import check
-        check(find_branch(dir))
-
-
-
-class cmd_scan_cache(Command):
-    hidden = True
-    def run(self):
-        from bzrlib.hashcache import HashCache
-        import os
-
-        c = HashCache('.')
-        c.read()
-        c.scan()
-            
-        print '%6d stats' % c.stat_count
-        print '%6d in hashcache' % len(c._cache)
-        print '%6d files removed from cache' % c.removed_count
-        print '%6d hashes updated' % c.update_count
-        print '%6d files changed too recently to cache' % c.danger_count
-
-        if c.needs_write:
-            c.write()
-            
-
-
-class cmd_upgrade(Command):
-    """Upgrade branch storage to current format.
-
-    This should normally be used only after the check command tells
-    you to run it.
-    """
-    takes_args = ['dir?']
-
-    def run(self, dir='.'):
-        from bzrlib.upgrade import upgrade
-        upgrade(find_branch(dir))
-
-
-
-class cmd_whoami(Command):
-    """Show bzr user id."""
-    takes_options = ['email']
-    
-    def run(self, email=False):
-        if email:
-            print bzrlib.osutils.user_email()
-        else:
-            print bzrlib.osutils.username()
-
-
-class cmd_selftest(Command):
-    """Run internal test suite"""
-    hidden = True
-    takes_options = ['verbose']
-    def run(self, verbose=False):
-        from bzrlib.selftest import selftest
-        return int(not selftest(verbose=verbose))
-
-
-class cmd_version(Command):
-    """Show version of bzr."""
-    def run(self):
-        show_version()
-
-def show_version():
-    print "bzr (bazaar-ng) %s" % bzrlib.__version__
-    # is bzrlib itself in a branch?
-    bzrrev = bzrlib.get_bzr_revision()
-    if bzrrev:
-        print "  (bzr checkout, revision %d {%s})" % bzrrev
-    print bzrlib.__copyright__
-    print "http://bazaar-ng.org/"
-    print
-    print "bzr comes with ABSOLUTELY NO WARRANTY.  bzr is free software, and"
-    print "you may use, modify and redistribute it under the terms of the GNU"
-    print "General Public License version 2 or later."
-
-
-class cmd_rocks(Command):
-    """Statement of optimism."""
-    hidden = True
-    def run(self):
-        print "it sure does!"
+        raise NotImplementedError()
+
+
+    def help(self):
+        """Return help message for this class."""
+        if self.__doc__ is Command.__doc__:
+            return None
+        return getdoc(self)
+
+    def name(self):
+        return _unsquish_command_name(self.__class__.__name__)
+
 
 def parse_spec(spec):
     """
@@ -1384,113 +357,6 @@
 
 
 
-class cmd_merge(Command):
-    """Perform a three-way merge of trees.
-    
-    The SPEC parameters are working tree or revision specifiers.  Working trees
-    are specified using standard paths or urls.  No component of a directory
-    path may begin with '@'.
-    
-    Working tree examples: '.', '..', 'foo@', but NOT 'foo/@bar'
-
-    Revisions are specified using a dirname/@revno pair, where dirname is the
-    branch directory and revno is the revision within that branch.  If no revno
-    is specified, the latest revision is used.
-
-    Revision examples: './@127', 'foo/@', '../@1'
-
-    The OTHER_SPEC parameter is required.  If the BASE_SPEC parameter is
-    not supplied, the common ancestor of OTHER_SPEC the current branch is used
-    as the BASE.
-
-    merge refuses to run if there are any uncommitted changes, unless
-    --force is given.
-    """
-    takes_args = ['other_spec', 'base_spec?']
-    takes_options = ['force', 'merge-type']
-
-    def run(self, other_spec, base_spec=None, force=False, merge_type=None):
-        from bzrlib.merge import merge
-        from bzrlib.merge_core import ApplyMerge3
-        if merge_type is None:
-            merge_type = ApplyMerge3
-        merge(parse_spec(other_spec), parse_spec(base_spec),
-              check_clean=(not force), merge_type=merge_type)
-
-
-
-class cmd_revert(Command):
-    """Restore selected files from a previous revision.
-    """
-    takes_args = ['file+']
-    def run(self, file_list):
-        from bzrlib.branch import find_branch
-        
-        if not file_list:
-            file_list = ['.']
-            
-        b = find_branch(file_list[0])
-
-        b.revert([b.relpath(f) for f in file_list])
-
-
-class cmd_merge_revert(Command):
-    """Reverse all changes since the last commit.
-
-    Only versioned files are affected.  By default, any files that are changed
-    will be backed up first.  Backup files have a '~' appended to their name.
-    """
-    takes_options = ['revision', 'no-backup']
-
-    def run(self, revision=None, no_backup=False):
-        from bzrlib.merge import merge
-        if revision is None:
-            revision = [-1]
-        elif len(revision) != 1:
-            raise BzrCommandError('bzr merge-revert --revision takes exactly 1 argument')
-        merge(('.', revision[0]), parse_spec('.'),
-              check_clean=False,
-              ignore_zero=True,
-              backup_files=not no_backup)
-
-
-class cmd_assert_fail(Command):
-    """Test reporting of assertion failures"""
-    hidden = True
-    def run(self):
-        assert False, "always fails"
-
-
-class cmd_help(Command):
-    """Show help on a command or other topic.
-
-    For a list of all available commands, say 'bzr help commands'."""
-    takes_args = ['topic?']
-    aliases = ['?']
-    
-    def run(self, topic=None):
-        import help
-        help.help(topic)
-
-
-
-
-class cmd_plugins(Command):
-    """List plugins"""
-    hidden = True
-    def run(self):
-        import bzrlib.plugin
-        from inspect import getdoc
-        from pprint import pprint
-        for plugin in bzrlib.plugin.all_plugins:
-            print plugin.__path__[0]
-            d = getdoc(plugin)
-            if d:
-                print '\t', d.split('\n')[0]
-
-        #pprint(bzrlib.plugin.all_plugins)
-
-
 
 # list of all available options; the rhs can be either None for an
 # option that takes no argument, or a constructor function that checks
@@ -1507,6 +373,7 @@
     'no-recurse':             None,
     'profile':                None,
     'revision':               _parse_revision_str,
+    'short':                  None,
     'show-ids':               None,
     'timezone':               str,
     'verbose':                None,
@@ -1518,6 +385,7 @@
     'root':                   str,
     'no-backup':              None,
     'merge-type':             get_merge_type,
+    'pattern':                str,
     }
 
 SHORT_OPTIONS = {
@@ -1540,6 +408,8 @@
 
     >>> parse_args('--help'.split())
     ([], {'help': True})
+    >>> parse_args('help -- --invalidcmd'.split())
+    (['help', '--invalidcmd'], {})
     >>> parse_args('--version'.split())
     ([], {'version': True})
     >>> parse_args('status --all'.split())
@@ -1558,15 +428,18 @@
     args = []
     opts = {}
 
-    # TODO: Maybe handle '--' to end options?
-
+    argsover = False
     while argv:
         a = argv.pop(0)
-        if a[0] == '-':
+        if not argsover and a[0] == '-':
             # option names must not be unicode
             a = str(a)
             optarg = None
             if a[1] == '-':
+                if a == '--':
+                    # We've received a standalone -- No more flags
+                    argsover = True
+                    continue
                 mutter("  got option %r" % a)
                 if '=' in a:
                     optname, optarg = a[2:].split('=', 1)
@@ -1673,72 +546,30 @@
     return argdict
 
 
-def _parse_master_args(argv):
-    """Parse the arguments that always go with the original command.
-    These are things like bzr --no-plugins, etc.
-
-    There are now 2 types of option flags. Ones that come *before* the command,
-    and ones that come *after* the command.
-    Ones coming *before* the command are applied against all possible commands.
-    And are generally applied before plugins are loaded.
-
-    The current list are:
-        --builtin   Allow plugins to load, but don't let them override builtin commands,
-                    they will still be allowed if they do not override a builtin.
-        --no-plugins    Don't load any plugins. This lets you get back to official source
-                        behavior.
-        --profile   Enable the hotspot profile before running the command.
-                    For backwards compatibility, this is also a non-master option.
-        --version   Spit out the version of bzr that is running and exit.
-                    This is also a non-master option.
-        --help      Run help and exit, also a non-master option (I think that should stay, though)
-
-    >>> argv, opts = _parse_master_args(['--test'])
-    Traceback (most recent call last):
-    ...
-    BzrCommandError: Invalid master option: 'test'
-    >>> argv, opts = _parse_master_args(['--version', 'command'])
-    >>> print argv
-    ['command']
-    >>> print opts['version']
-    True
-    >>> argv, opts = _parse_master_args(['--profile', 'command', '--more-options'])
-    >>> print argv
-    ['command', '--more-options']
-    >>> print opts['profile']
-    True
-    >>> argv, opts = _parse_master_args(['--no-plugins', 'command'])
-    >>> print argv
-    ['command']
-    >>> print opts['no-plugins']
-    True
-    >>> print opts['profile']
-    False
-    >>> argv, opts = _parse_master_args(['command', '--profile'])
-    >>> print argv
-    ['command', '--profile']
-    >>> print opts['profile']
-    False
-    """
-    master_opts = {'builtin':False,
-        'no-plugins':False,
-        'version':False,
-        'profile':False,
-        'help':False
-    }
-
-    for arg in argv[:]:
-        if arg[:2] != '--': # at the first non-option, we return the rest
-            break
-        arg = arg[2:] # Remove '--'
-        if arg not in master_opts:
-            # We could say that this is not an error, that we should
-            # just let it be handled by the main section instead
-            raise BzrCommandError('Invalid master option: %r' % arg)
-        argv.pop(0) # We are consuming this entry
-        master_opts[arg] = True
-    return argv, master_opts
-
+
+def apply_profiled(the_callable, *args, **kwargs):
+    import hotshot
+    import tempfile
+    pffileno, pfname = tempfile.mkstemp()
+    try:
+        prof = hotshot.Profile(pfname)
+        try:
+            ret = prof.runcall(the_callable, *args, **kwargs) or 0
+        finally:
+            prof.close()
+
+        import hotshot.stats
+        stats = hotshot.stats.load(pfname)
+        #stats.strip_dirs()
+        stats.sort_stats('time')
+        ## XXX: Might like to write to stderr or the trace file instead but
+        ## print_stats seems hardcoded to stdout
+        stats.print_stats(20)
+
+        return ret
+    finally:
+        os.close(pffileno)
+        os.remove(pfname)
 
 
 def run_bzr(argv):
@@ -1748,146 +579,105 @@
     logging and error handling.  
     
     argv
-       The command-line arguments, without the program name.
+       The command-line arguments, without the program name from argv[0]
     
     Returns a command status or raises an exception.
+
+    Special master options: these must come before the command because
+    they control how the command is interpreted.
+
+    --no-plugins
+        Do not load plugin modules at all
+
+    --builtin
+        Only use builtin commands.  (Plugins are still allowed to change
+        other behaviour.)
+
+    --profile
+        Run under the Python profiler.
     """
+    
     argv = [a.decode(bzrlib.user_encoding) for a in argv]
 
-    # some options like --builtin and --no-plugins have special effects
-    argv, master_opts = _parse_master_args(argv)
-    if not master_opts['no-plugins']:
+    opt_profile = opt_no_plugins = opt_builtin = False
+
+    # --no-plugins is handled specially at a very early stage. We need
+    # to load plugins before doing other command parsing so that they
+    # can override commands, but this needs to happen first.
+
+    for a in argv:
+        if a == '--profile':
+            opt_profile = True
+        elif a == '--no-plugins':
+            opt_no_plugins = True
+        elif a == '--builtin':
+            opt_builtin = True
+        else:
+            break
+        argv.remove(a)
+
+    if (not argv) or (argv[0] == '--help'):
+        from bzrlib.help import help
+        if len(argv) > 1:
+            help(argv[1])
+        else:
+            help()
+        return 0
+
+    if argv[0] == '--version':
+        from bzrlib.builtins import show_version
+        show_version()
+        return 0
+        
+    if not opt_no_plugins:
         from bzrlib.plugin import load_plugins
         load_plugins()
 
-    args, opts = parse_args(argv)
-
-    if master_opts.get('help') or 'help' in opts:
-        from bzrlib.help import help
-        if argv:
-            help(argv[0])
-        else:
-            help()
-        return 0            
-        
-    if 'version' in opts:
-        show_version()
-        return 0
-    
-    if args and args[0] == 'builtin':
-        include_plugins=False
-        args = args[1:]
-    
-    try:
-        cmd = str(args.pop(0))
-    except IndexError:
-        print >>sys.stderr, "please try 'bzr help' for help"
-        return 1
-
-    plugins_override = not (master_opts['builtin'])
-    canonical_cmd, cmd_class = get_cmd_class(cmd, plugins_override=plugins_override)
-
-    profile = master_opts['profile']
-    # For backwards compatibility, I would rather stick with --profile being a
-    # master/global option
-    if 'profile' in opts:
-        profile = True
-        del opts['profile']
-
-    # check options are reasonable
-    allowed = cmd_class.takes_options
-    for oname in opts:
-        if oname not in allowed:
-            raise BzrCommandError("option '--%s' is not allowed for command %r"
-                                  % (oname, cmd))
-
-    # mix arguments and options into one dictionary
-    cmdargs = _match_argform(cmd, cmd_class.takes_args, args)
-    cmdopts = {}
-    for k, v in opts.items():
-        cmdopts[k.replace('-', '_')] = v
-
-    if profile:
-        import hotshot, tempfile
-        pffileno, pfname = tempfile.mkstemp()
-        try:
-            prof = hotshot.Profile(pfname)
-            ret = prof.runcall(cmd_class, cmdopts, cmdargs) or 0
-            prof.close()
-
-            import hotshot.stats
-            stats = hotshot.stats.load(pfname)
-            #stats.strip_dirs()
-            stats.sort_stats('time')
-            ## XXX: Might like to write to stderr or the trace file instead but
-            ## print_stats seems hardcoded to stdout
-            stats.print_stats(20)
-            
-            return ret.status
-
-        finally:
-            os.close(pffileno)
-            os.remove(pfname)
+    cmd = str(argv.pop(0))
+
+    cmd_obj = get_cmd_object(cmd, plugins_override=not opt_builtin)
+
+    if opt_profile:
+        ret = apply_profiled(cmd_obj.run_argv, argv)
     else:
-        return cmd_class(cmdopts, cmdargs).status 
-
-
-def _report_exception(summary, quiet=False):
-    import traceback
-    log_error('bzr: ' + summary)
-    bzrlib.trace.log_exception()
-
-    if not quiet:
-        tb = sys.exc_info()[2]
-        exinfo = traceback.extract_tb(tb)
-        if exinfo:
-            sys.stderr.write('  at %s:%d in %s()\n' % exinfo[-1][:3])
-        sys.stderr.write('  see ~/.bzr.log for debug information\n')
-
+        ret = cmd_obj.run_argv(argv)
+    return ret or 0
 
 
 def main(argv):
-    
-    bzrlib.trace.open_tracefile(argv)
+    import bzrlib.ui
+    bzrlib.trace.log_startup(argv)
+    bzrlib.ui.ui_factory = bzrlib.ui.TextUIFactory()
 
     try:
         try:
-            try:
-                return run_bzr(argv[1:])
-            finally:
-                # do this here inside the exception wrappers to catch EPIPE
-                sys.stdout.flush()
-        except BzrError, e:
-            quiet = isinstance(e, (BzrCommandError))
-            _report_exception('error: ' + e.args[0], quiet=quiet)
-            if len(e.args) > 1:
-                for h in e.args[1]:
-                    # some explanation or hints
-                    log_error('  ' + h)
-            return 1
-        except AssertionError, e:
-            msg = 'assertion failed'
-            if str(e):
-                msg += ': ' + str(e)
-            _report_exception(msg)
-            return 2
-        except KeyboardInterrupt, e:
-            _report_exception('interrupted', quiet=True)
-            return 2
-        except Exception, e:
-            import errno
-            quiet = False
-            if (isinstance(e, IOError) 
-                and hasattr(e, 'errno')
-                and e.errno == errno.EPIPE):
-                quiet = True
-                msg = 'broken pipe'
-            else:
-                msg = str(e).rstrip('\n')
-            _report_exception(msg, quiet)
-            return 2
-    finally:
-        bzrlib.trace.close_trace()
+            return run_bzr(argv[1:])
+        finally:
+            # do this here inside the exception wrappers to catch EPIPE
+            sys.stdout.flush()
+    except BzrCommandError, e:
+        # command line syntax error, etc
+        log_error(str(e))
+        return 1
+    except BzrError, e:
+        bzrlib.trace.log_exception()
+        return 1
+    except AssertionError, e:
+        bzrlib.trace.log_exception('assertion failed: ' + str(e))
+        return 3
+    except KeyboardInterrupt, e:
+        bzrlib.trace.note('interrupted')
+        return 2
+    except Exception, e:
+        import errno
+        if (isinstance(e, IOError) 
+            and hasattr(e, 'errno')
+            and e.errno == errno.EPIPE):
+            bzrlib.trace.note('broken pipe')
+            return 2
+        else:
+            bzrlib.trace.log_exception()
+            return 2
 
 
 if __name__ == '__main__':

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2005-07-22 19:44:09 +0000
+++ bzrlib/commit.py	2005-09-05 09:27:11 +0000
@@ -16,8 +16,6 @@
 
 
 
-# FIXME: "bzr commit doc/format" commits doc/format.txt!
-
 def commit(branch, message,
            timestamp=None,
            timezone=None,
@@ -66,7 +64,7 @@
     from bzrlib.errors import BzrError, PointlessCommit
     from bzrlib.revision import Revision, RevisionReference
     from bzrlib.trace import mutter, note
-    from bzrlib.xml import pack_xml
+    from bzrlib.xml import serializer_v4
 
     branch.lock_write()
 
@@ -85,7 +83,10 @@
         basis_inv = basis.inventory
 
         if verbose:
-            note('looking for changes...')
+            # note('looking for changes...')
+            # print 'looking for changes...'
+            # disabled; should be done at a higher level
+            pass
 
         pending_merges = branch.pending_merges()
 
@@ -115,13 +116,13 @@
             if work_inv.has_id(file_id):
                 del work_inv[file_id]
 
-
         if rev_id is None:
-            rev_id = _gen_revision_id(time.time())
+            rev_id = _gen_revision_id(branch, time.time())
         inv_id = rev_id
 
         inv_tmp = tempfile.TemporaryFile()
-        pack_xml(new_inv, inv_tmp)
+        
+        serializer_v4.write_inventory(new_inv, inv_tmp)
         inv_tmp.seek(0)
         branch.inventory_store.add(inv_tmp, inv_id)
         mutter('new inventory_id is {%s}' % inv_id)
@@ -137,7 +138,7 @@
             timestamp = time.time()
 
         if committer == None:
-            committer = username()
+            committer = username(branch)
 
         if timezone == None:
             timezone = local_time_offset()
@@ -160,7 +161,7 @@
             rev.parents.append(RevisionReference(merge_rev))            
 
         rev_tmp = tempfile.TemporaryFile()
-        pack_xml(rev, rev_tmp)
+        serializer_v4.write_revision(rev, rev_tmp)
         rev_tmp.seek(0)
         branch.revision_store.add(rev_tmp, rev_id)
         mutter("new revision_id is {%s}" % rev_id)
@@ -180,18 +181,21 @@
         branch.set_pending_merges([])
 
         if verbose:
-            note("commited r%d" % branch.revno())
+            # disabled; should go through logging
+            # note("commited r%d" % branch.revno())
+            # print ("commited r%d" % branch.revno())
+            pass
     finally:
         branch.unlock()
 
 
 
-def _gen_revision_id(when):
+def _gen_revision_id(branch, when):
     """Return new revision-id."""
     from binascii import hexlify
-    from osutils import rand_bytes, compact_date, user_email
+    from bzrlib.osutils import rand_bytes, compact_date, user_email
 
-    s = '%s-%s-' % (user_email(), compact_date(when))
+    s = '%s-%s-' % (user_email(branch), compact_date(when))
     s += hexlify(rand_bytes(8))
     return s
 

=== added file 'bzrlib/delta.py'
--- bzrlib/delta.py	1970-01-01 00:00:00 +0000
+++ bzrlib/delta.py	2005-08-25 05:58:05 +0000
@@ -0,0 +1,227 @@
+#! /usr/bin/env python
+# -*- coding: UTF-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+from bzrlib.trace import mutter
+
+class TreeDelta(object):
+    """Describes changes from one tree to another.
+
+    Contains four lists:
+
+    added
+        (path, id, kind)
+    removed
+        (path, id, kind)
+    renamed
+        (oldpath, newpath, id, kind, text_modified)
+    modified
+        (path, id, kind)
+    unchanged
+        (path, id, kind)
+
+    Each id is listed only once.
+
+    Files that are both modified and renamed are listed only in
+    renamed, with the text_modified flag true.
+
+    Files are only considered renamed if their name has changed or
+    their parent directory has changed.  Renaming a directory
+    does not count as renaming all its contents.
+
+    The lists are normally sorted when the delta is created.
+    """
+    def __init__(self):
+        self.added = []
+        self.removed = []
+        self.renamed = []
+        self.modified = []
+        self.unchanged = []
+
+    def __eq__(self, other):
+        if not isinstance(other, TreeDelta):
+            return False
+        return self.added == other.added \
+               and self.removed == other.removed \
+               and self.renamed == other.renamed \
+               and self.modified == other.modified \
+               and self.unchanged == other.unchanged
+
+    def __ne__(self, other):
+        return not (self == other)
+
+    def __repr__(self):
+        return "TreeDelta(added=%r, removed=%r, renamed=%r, modified=%r," \
+            " unchanged=%r)" % (self.added, self.removed, self.renamed,
+            self.modified, self.unchanged)
+
+    def has_changed(self):
+        changes = len(self.added) + len(self.removed) + len(self.renamed)
+        changes += len(self.modified) 
+        return (changes != 0)
+
+    def touches_file_id(self, file_id):
+        """Return True if file_id is modified by this delta."""
+        for l in self.added, self.removed, self.modified:
+            for v in l:
+                if v[1] == file_id:
+                    return True
+        for v in self.renamed:
+            if v[2] == file_id:
+                return True
+        return False
+            
+
+    def show(self, to_file, show_ids=False, show_unchanged=False):
+        def show_list(files):
+            for path, fid, kind in files:
+                if kind == 'directory':
+                    path += '/'
+                elif kind == 'symlink':
+                    path += '@'
+                    
+                if show_ids:
+                    print >>to_file, '  %-30s %s' % (path, fid)
+                else:
+                    print >>to_file, ' ', path
+            
+        if self.removed:
+            print >>to_file, 'removed:'
+            show_list(self.removed)
+                
+        if self.added:
+            print >>to_file, 'added:'
+            show_list(self.added)
+
+        if self.renamed:
+            print >>to_file, 'renamed:'
+            for oldpath, newpath, fid, kind, text_modified in self.renamed:
+                if show_ids:
+                    print >>to_file, '  %s => %s %s' % (oldpath, newpath, fid)
+                else:
+                    print >>to_file, '  %s => %s' % (oldpath, newpath)
+                    
+        if self.modified:
+            print >>to_file, 'modified:'
+            show_list(self.modified)
+            
+        if show_unchanged and self.unchanged:
+            print >>to_file, 'unchanged:'
+            show_list(self.unchanged)
+
+
+
+def compare_trees(old_tree, new_tree, want_unchanged=False, specific_files=None):
+    """Describe changes from one tree to another.
+
+    Returns a TreeDelta with details of added, modified, renamed, and
+    deleted entries.
+
+    The root entry is specifically exempt.
+
+    This only considers versioned files.
+
+    want_unchanged
+        If true, also list files unchanged from one version to
+        the next.
+
+    specific_files
+        If true, only check for changes to specified names or
+        files within them.
+    """
+
+    from osutils import is_inside_any
+    
+    old_inv = old_tree.inventory
+    new_inv = new_tree.inventory
+    delta = TreeDelta()
+    mutter('start compare_trees')
+
+    # TODO: match for specific files can be rather smarter by finding
+    # the IDs of those files up front and then considering only that.
+
+    for file_id in old_tree:
+        if file_id in new_tree:
+            old_ie = old_inv[file_id]
+            new_ie = new_inv[file_id]
+
+            kind = old_ie.kind
+            assert kind == new_ie.kind
+            
+            assert kind in ('file', 'directory', 'symlink', 'root_directory'), \
+                   'invalid file kind %r' % kind
+
+            if kind == 'root_directory':
+                continue
+            
+            if specific_files:
+                if (not is_inside_any(specific_files, old_inv.id2path(file_id)) 
+                    and not is_inside_any(specific_files, new_inv.id2path(file_id))):
+                    continue
+
+            if kind == 'file':
+                old_sha1 = old_tree.get_file_sha1(file_id)
+                new_sha1 = new_tree.get_file_sha1(file_id)
+                text_modified = (old_sha1 != new_sha1)
+            else:
+                ## mutter("no text to check for %r %r" % (file_id, kind))
+                text_modified = False
+
+            # TODO: Can possibly avoid calculating path strings if the
+            # two files are unchanged and their names and parents are
+            # the same and the parents are unchanged all the way up.
+            # May not be worthwhile.
+            
+            if (old_ie.name != new_ie.name
+                or old_ie.parent_id != new_ie.parent_id):
+                delta.renamed.append((old_inv.id2path(file_id),
+                                      new_inv.id2path(file_id),
+                                      file_id, kind,
+                                      text_modified))
+            elif text_modified:
+                delta.modified.append((new_inv.id2path(file_id), file_id, kind))
+            elif want_unchanged:
+                delta.unchanged.append((new_inv.id2path(file_id), file_id, kind))
+        else:
+            kind = old_inv.get_file_kind(file_id)
+            if kind == 'root_directory':
+                continue
+            old_path = old_inv.id2path(file_id)
+            if specific_files:
+                if not is_inside_any(specific_files, old_path):
+                    continue
+            delta.removed.append((old_path, file_id, kind))
+
+    mutter('start looking for new files')
+    for file_id in new_inv:
+        if file_id in old_inv:
+            continue
+        kind = new_inv.get_file_kind(file_id)
+        if kind == 'root_directory':
+            continue
+        new_path = new_inv.id2path(file_id)
+        if specific_files:
+            if not is_inside_any(specific_files, new_path):
+                continue
+        delta.added.append((new_path, file_id, kind))
+            
+    delta.removed.sort()
+    delta.added.sort()
+    delta.renamed.sort()
+    delta.modified.sort()
+    delta.unchanged.sort()
+
+    return delta

=== modified file 'bzrlib/diff.py'
--- bzrlib/diff.py	2005-07-27 16:42:41 +0000
+++ bzrlib/diff.py	2005-08-25 05:58:05 +0000
@@ -15,9 +15,9 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-from trace import mutter
-from errors import BzrError
-
+from bzrlib.trace import mutter
+from bzrlib.errors import BzrError
+from bzrlib.delta import compare_trees
 
 # TODO: Rather than building a changeset object, we should probably
 # invoke callbacks on an object.  That object can either accumulate a
@@ -143,7 +143,8 @@
     
 
 
-def show_diff(b, revision, specific_files, external_diff_options=None):
+def show_diff(b, revision, specific_files, external_diff_options=None,
+              revision2=None):
     """Shortcut for showing the diff to the working tree.
 
     b
@@ -161,8 +162,11 @@
         old_tree = b.basis_tree()
     else:
         old_tree = b.revision_tree(b.lookup_revision(revision))
-        
-    new_tree = b.working_tree()
+
+    if revision2 == None:
+        new_tree = b.working_tree()
+    else:
+        new_tree = b.revision_tree(b.lookup_revision(revision2))
 
     show_diff_trees(old_tree, new_tree, sys.stdout, specific_files,
                     external_diff_options)
@@ -242,207 +246,5 @@
 
 
 
-class TreeDelta(object):
-    """Describes changes from one tree to another.
-
-    Contains four lists:
-
-    added
-        (path, id, kind)
-    removed
-        (path, id, kind)
-    renamed
-        (oldpath, newpath, id, kind, text_modified)
-    modified
-        (path, id, kind)
-    unchanged
-        (path, id, kind)
-
-    Each id is listed only once.
-
-    Files that are both modified and renamed are listed only in
-    renamed, with the text_modified flag true.
-
-    Files are only considered renamed if their name has changed or
-    their parent directory has changed.  Renaming a directory
-    does not count as renaming all its contents.
-
-    The lists are normally sorted when the delta is created.
-    """
-    def __init__(self):
-        self.added = []
-        self.removed = []
-        self.renamed = []
-        self.modified = []
-        self.unchanged = []
-
-    def __eq__(self, other):
-        if not isinstance(other, TreeDelta):
-            return False
-        return self.added == other.added \
-               and self.removed == other.removed \
-               and self.renamed == other.renamed \
-               and self.modified == other.modified \
-               and self.unchanged == other.unchanged
-
-    def __ne__(self, other):
-        return not (self == other)
-
-    def __repr__(self):
-        return "TreeDelta(added=%r, removed=%r, renamed=%r, modified=%r," \
-            " unchanged=%r)" % (self.added, self.removed, self.renamed,
-            self.modified, self.unchanged)
-
-    def has_changed(self):
-        changes = len(self.added) + len(self.removed) + len(self.renamed)
-        changes += len(self.modified) 
-        return (changes != 0)
-
-    def touches_file_id(self, file_id):
-        """Return True if file_id is modified by this delta."""
-        for l in self.added, self.removed, self.modified:
-            for v in l:
-                if v[1] == file_id:
-                    return True
-        for v in self.renamed:
-            if v[2] == file_id:
-                return True
-        return False
-            
-
-    def show(self, to_file, show_ids=False, show_unchanged=False):
-        def show_list(files):
-            for path, fid, kind in files:
-                if kind == 'directory':
-                    path += '/'
-                elif kind == 'symlink':
-                    path += '@'
-                    
-                if show_ids:
-                    print >>to_file, '  %-30s %s' % (path, fid)
-                else:
-                    print >>to_file, ' ', path
-            
-        if self.removed:
-            print >>to_file, 'removed:'
-            show_list(self.removed)
-                
-        if self.added:
-            print >>to_file, 'added:'
-            show_list(self.added)
-
-        if self.renamed:
-            print >>to_file, 'renamed:'
-            for oldpath, newpath, fid, kind, text_modified in self.renamed:
-                if show_ids:
-                    print >>to_file, '  %s => %s %s' % (oldpath, newpath, fid)
-                else:
-                    print >>to_file, '  %s => %s' % (oldpath, newpath)
-                    
-        if self.modified:
-            print >>to_file, 'modified:'
-            show_list(self.modified)
-            
-        if show_unchanged and self.unchanged:
-            print >>to_file, 'unchanged:'
-            show_list(self.unchanged)
-
-
-
-def compare_trees(old_tree, new_tree, want_unchanged=False, specific_files=None):
-    """Describe changes from one tree to another.
-
-    Returns a TreeDelta with details of added, modified, renamed, and
-    deleted entries.
-
-    The root entry is specifically exempt.
-
-    This only considers versioned files.
-
-    want_unchanged
-        If true, also list files unchanged from one version to
-        the next.
-
-    specific_files
-        If true, only check for changes to specified names or
-        files within them.
-    """
-
-    from osutils import is_inside_any
-    
-    old_inv = old_tree.inventory
-    new_inv = new_tree.inventory
-    delta = TreeDelta()
-    mutter('start compare_trees')
-
-    # TODO: match for specific files can be rather smarter by finding
-    # the IDs of those files up front and then considering only that.
-
-    for file_id in old_tree:
-        if file_id in new_tree:
-            old_ie = old_inv[file_id]
-            new_ie = new_inv[file_id]
-
-            kind = old_ie.kind
-            assert kind == new_ie.kind
-            
-            assert kind in ('file', 'directory', 'symlink', 'root_directory'), \
-                   'invalid file kind %r' % kind
-
-            if kind == 'root_directory':
-                continue
-            
-            if specific_files:
-                if (not is_inside_any(specific_files, old_inv.id2path(file_id)) 
-                    and not is_inside_any(specific_files, new_inv.id2path(file_id))):
-                    continue
-
-            if kind == 'file':
-                old_sha1 = old_tree.get_file_sha1(file_id)
-                new_sha1 = new_tree.get_file_sha1(file_id)
-                text_modified = (old_sha1 != new_sha1)
-            else:
-                ## mutter("no text to check for %r %r" % (file_id, kind))
-                text_modified = False
-
-            # TODO: Can possibly avoid calculating path strings if the
-            # two files are unchanged and their names and parents are
-            # the same and the parents are unchanged all the way up.
-            # May not be worthwhile.
-            
-            if (old_ie.name != new_ie.name
-                or old_ie.parent_id != new_ie.parent_id):
-                delta.renamed.append((old_inv.id2path(file_id),
-                                      new_inv.id2path(file_id),
-                                      file_id, kind,
-                                      text_modified))
-            elif text_modified:
-                delta.modified.append((new_inv.id2path(file_id), file_id, kind))
-            elif want_unchanged:
-                delta.unchanged.append((new_inv.id2path(file_id), file_id, kind))
-        else:
-            kind = old_inv.get_file_kind(file_id)
-            old_path = old_inv.id2path(file_id)
-            if specific_files:
-                if not is_inside_any(specific_files, old_path):
-                    continue
-            delta.removed.append((old_path, file_id, kind))
-
-    mutter('start looking for new files')
-    for file_id in new_inv:
-        if file_id in old_inv:
-            continue
-        new_path = new_inv.id2path(file_id)
-        if specific_files:
-            if not is_inside_any(specific_files, new_path):
-                continue
-        kind = new_inv.get_file_kind(file_id)
-        delta.added.append((new_path, file_id, kind))
-            
-    delta.removed.sort()
-    delta.added.sort()
-    delta.renamed.sort()
-    delta.modified.sort()
-    delta.unchanged.sort()
-
-    return delta
+
+

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2005-07-11 03:40:02 +0000
+++ bzrlib/errors.py	2005-09-04 02:59:56 +0000
@@ -29,6 +29,18 @@
     pass
 
 
+class InvalidRevisionNumber(BzrError):
+    def __init__(self, revno):
+        self.args = [revno]
+        
+    def __str__(self):
+        return 'invalid revision number: %r' % self.args[0]
+
+
+class InvalidRevisionId(BzrError):
+    pass
+
+
 class BzrCommandError(BzrError):
     # Error from malformed user command
     pass
@@ -70,3 +82,43 @@
 
 class PointlessCommit(Exception):
     """Commit failed because nothing was changed."""
+
+
+class NoSuchRevision(BzrError):
+    def __init__(self, branch, revision):
+        self.branch = branch
+        self.revision = revision
+        msg = "Branch %s has no revision %s" % (branch, revision)
+        BzrError.__init__(self, msg)
+
+
+class UnrelatedBranches(BzrCommandError):
+    def __init__(self):
+        msg = "Branches have no common ancestor, and no base revision"\
+            " specified."
+        BzrCommandError.__init__(self, msg)
+
+
+class NotAncestor(BzrError):
+    def __init__(self, rev_id, not_ancestor_id):
+        self.rev_id = rev_id
+        self.not_ancestor_id = not_ancestor_id
+        msg = "Revision %s is not an ancestor of %s" % (not_ancestor_id, 
+                                                        rev_id)
+        BzrError.__init__(self, msg)
+
+
+class InstallFailed(BzrError):
+    def __init__(self, revisions):
+        self.revisions = revisions
+        msg = "Could not install revisions:\n%s" % " ,".join(revisions)
+        BzrError.__init__(self, msg)
+
+
+class AmbiguousBase(BzrError):
+    def __init__(self, bases):
+        msg = "The correct base is unclear, becase %s are all equally close" %\
+            ", ".join(bases)
+        BzrError.__init__(self, msg)
+        self.bases = bases
+

=== added file 'bzrlib/externalcommand.py'
--- bzrlib/externalcommand.py	1970-01-01 00:00:00 +0000
+++ bzrlib/externalcommand.py	2005-09-01 11:53:02 +0000
@@ -0,0 +1,62 @@
+# Copyright (C) 2004, 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+# TODO: Perhaps rather than mapping options and arguments back and
+# forth, we should just pass in the whole argv, and allow
+# ExternalCommands to handle it differently to internal commands?
+
+
+import os
+import sys
+from bzrlib.commands import Command
+
+
+class ExternalCommand(Command):
+    """Class to wrap external commands."""
+
+    @classmethod
+    def find_command(cls, cmd):
+        import os.path
+        bzrpath = os.environ.get('BZRPATH', '')
+
+        for dir in bzrpath.split(os.pathsep):
+            path = os.path.join(dir, cmd)
+            if os.path.isfile(path):
+                return ExternalCommand(path)
+
+        return None
+
+
+    def __init__(self, path):
+        self.path = path
+
+ 
+    def name(self):
+        return self.path.split(os.sep)[-1]
+
+
+    def run(self, *args, **kwargs):
+        raise NotImplementedError('should not be called on %r' % self)
+
+
+    def run_argv(self, argv):
+        return os.spawnv(os.P_WAIT, self.path, [self.path] + argv)
+
+
+    def help(self):
+        m = 'external command from %s\n\n' % self.path
+        pipe = os.popen('%s --help' % self.path)
+        return m + pipe.read()

=== added file 'bzrlib/fetch.py'
--- bzrlib/fetch.py	1970-01-01 00:00:00 +0000
+++ bzrlib/fetch.py	2005-09-04 02:59:56 +0000
@@ -0,0 +1,82 @@
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+import bzrlib.errors
+from bzrlib.trace import mutter, note
+from bzrlib.branch import Branch
+from bzrlib.progress import ProgressBar
+import sys
+import os
+
+def has_revision(branch, revision_id):
+    try:
+        branch.get_revision_xml(revision_id)
+        return True
+    except bzrlib.errors.NoSuchRevision:
+        return False
+
+
+def greedy_fetch(to_branch, from_branch, revision=None, pb=None):
+    """Copy a revision and all available ancestors from one branch to another
+    If no revision is specified, uses the last revision in the source branch's
+    revision history.
+    """
+    from_history = from_branch.revision_history()
+    required_revisions = set(from_history)
+    all_failed = set()
+    if revision is not None:
+        required_revisions.add(revision)
+        try:
+            rev_index = from_history.index(revision)
+        except ValueError:
+            rev_index = None
+        if rev_index is not None:
+            from_history = from_history[:rev_index + 1]
+        else:
+            from_history = [revision]
+    to_history = to_branch.revision_history()
+    missing = []
+    for rev_id in from_history:
+        if not has_revision(to_branch, rev_id):
+            missing.append(rev_id)
+    
+    count = 0
+    while len(missing) > 0:
+        installed, failed = to_branch.install_revisions(from_branch, 
+                                                        revision_ids=missing,
+                                                        pb=pb)
+        count += installed
+        required_failed = failed.intersection(required_revisions)
+        if len(required_failed) > 0:
+            raise bzrlib.errors.InstallFailed(required_failed)
+        for rev_id in failed:
+            note("Failed to install %s" % rev_id)
+        all_failed.update(failed)
+        new_missing = set() 
+        for rev_id in missing:
+            try:
+                revision = from_branch.get_revision(rev_id)
+            except bzrlib.errors.NoSuchRevision:
+                if revision in from_history:
+                    raise
+                else:
+                    continue
+            for parent in [p.revision_id for p in revision.parents]:
+                if not has_revision(to_branch, parent):
+                    new_missing.add(parent)
+        missing = new_missing
+    return count, all_failed
+
+

=== added file 'bzrlib/graph.py'
--- bzrlib/graph.py	1970-01-01 00:00:00 +0000
+++ bzrlib/graph.py	2005-09-12 02:53:07 +0000
@@ -0,0 +1,93 @@
+# (C) 2005 Canonical
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+def max_distance(node, ancestors, distances, root_descendants):
+    """Calculate the max distance to an ancestor.  
+    Return None if not all possible ancestors have known distances"""
+    best = None
+    if node in distances:
+        best = distances[node]
+    for ancestor in ancestors[node]:
+        # skip ancestors we will never traverse:
+        if root_descendants is not None and ancestor not in root_descendants:
+            continue
+        # An ancestor which is not listed in ancestors will never be in
+        # distances, so we pretend it never existed.
+        if ancestor not in ancestors:
+            continue
+        if ancestor not in distances:
+            return None
+        if best is None or distances[ancestor] > best:
+            best = distances[ancestor] + 1
+    return best
+
+    
+def node_distances(graph, ancestors, start, root_descendants=None):
+    """Produce a list of nodes, sorted by distance from a start node.
+    This is an algorithm devised by Aaron Bentley, because applying Dijkstra
+    backwards seemed too complicated.
+
+    For each node, we walk its descendants.  If all the descendant's ancestors
+    have a max-distance-to-start, (excluding ones that can never reach start),
+    we calculate their max-distance-to-start, and schedule their descendants.
+
+    So when a node's last parent acquires a distance, it will acquire a
+    distance on the next iteration.
+
+    Once we know the max distances for all nodes, we can return a list sorted
+    by distance, farthest first.
+    """
+    distances = {start: 0}
+    lines = set([start])
+    while len(lines) > 0:
+        new_lines = set()
+        for line in lines:
+            line_descendants = graph[line]
+            assert line not in line_descendants, "%s refers to itself" % line
+            for descendant in line_descendants:
+                distance = max_distance(descendant, ancestors, distances,
+                                        root_descendants)
+                if distance is None:
+                    continue
+                distances[descendant] = distance
+                new_lines.add(descendant)
+        lines = new_lines
+    return distances
+
+def farthest_nodes(graph, ancestors, start):
+    def by_distance(n):
+        return distances[n],n
+
+    distances = node_distances(graph, ancestors, start)
+    node_list = distances.keys()
+    node_list.sort(key=by_distance, reverse=True)
+    return node_list
+
+def all_descendants(descendants, start):
+    result = set()
+    lines = set([start])
+    while len(lines) > 0:
+        new_lines = set()
+        for line in lines:
+            if line not in descendants:
+                continue
+            for descendant in descendants[line]:
+                if descendant in result:
+                    continue
+                result.add(descendant)
+                new_lines.add(descendant)
+        lines = new_lines
+    return result

=== modified file 'bzrlib/help.py'
--- bzrlib/help.py	2005-06-11 01:55:29 +0000
+++ bzrlib/help.py	2005-09-05 04:08:03 +0000
@@ -14,45 +14,40 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+# TODO: Some way to get a list of external commands (defined by shell
+# scripts) so that they can be included in the help listing as well.
+# It should be enough to just list the plugin directory and look for
+# executable files with reasonable names.
+
+# TODO: `help commands --all` should show hidden commands
+
 global_help = \
 """Bazaar-NG -- a free distributed version-control tool
 http://bazaar-ng.org/
 
-**WARNING: THIS IS AN UNSTABLE DEVELOPMENT VERSION**
-
-* Metadata format is not stable yet -- you may need to
-  discard history in the future.
-
-* Many commands unimplemented or partially implemented.
-
-* Space-inefficient storage.
-
-* No merge operators yet.
-
-
-To make a branch, use 'bzr init' in an existing directory, then 'bzr
-add' to make files versioned.  'bzr add .' will recursively add all
-non-ignored files.
-
-'bzr status' describes files that are unknown, ignored, or modified.
-'bzr diff' shows the text changes to the tree or named files.
-'bzr commit -m <MESSAGE>' commits all changes in that branch.
-
-'bzr move' and 'bzr rename' allow you to rename files or directories.
-'bzr remove' makes a file unversioned but keeps the working copy;
-to delete that too simply delete the file.
-
-'bzr log' shows a history of changes, and
-'bzr info' gives summary statistical information.
-'bzr check' validates all files are stored safely.
-
-Files can be ignored by giving a path or a glob in .bzrignore at the
-top of the tree.  Use 'bzr ignored' to see what files are ignored and
-why, and 'bzr unknowns' to see files that are neither versioned or
-ignored.
-
-For more help on any command, type 'bzr help COMMAND', or 'bzr help
-commands' for a list.
+WARNING: This is an unstable development version.
+         Please keep backups.
+
+Basic commands:
+
+  bzr init      makes this branch versioned
+  bzr branch    make a copy of another branch
+
+  bzr add       make files or directories versioned
+  bzr ignore    ignore a file or pattern
+  bzr mv        move or rename a versioned file
+
+  bzr status    summarize changes in working copy
+  bzr diff      show detailed diffs
+
+  bzr merge     pull in changes from another branch
+  bzr commit    
+
+  bzr log       show history of changes
+  bzr check     validate storage
+
+Use e.g. 'bzr help init' for more details, or
+'bzr help commands' for all commands.
 """
 
 
@@ -70,13 +65,13 @@
         help_on_command(topic, outfile = outfile)
 
 
-def command_usage(cmdname, cmdclass):
+def command_usage(cmd_object):
     """Return single-line grammar for command.
 
     Only describes arguments, not options.
     """
-    s = cmdname + ' '
-    for aname in cmdclass.takes_args:
+    s = 'bzr ' + cmd_object.name() + ' '
+    for aname in cmd_object.takes_args:
         aname = aname.upper()
         if aname[-1] in ['$', '+']:
             aname = aname[:-1] + '...'
@@ -92,34 +87,37 @@
     return s
 
 
-def help_on_command(cmdname, outfile = None):
+def help_on_command(cmdname, outfile=None):
+    from bzrlib.commands import get_cmd_object
+
     cmdname = str(cmdname)
 
     if outfile == None:
         outfile = sys.stdout
 
-    from inspect import getdoc
-    import commands
-    topic, cmdclass = commands.get_cmd_class(cmdname)
+    cmd_object = get_cmd_object(cmdname)
 
-    doc = getdoc(cmdclass)
+    doc = cmd_object.help()
     if doc == None:
         raise NotImplementedError("sorry, no detailed help yet for %r" % cmdname)
 
-    outfile.write('usage: ' + command_usage(topic, cmdclass) + '\n')
-
-    if cmdclass.aliases:
-        outfile.write('aliases: ' + ', '.join(cmdclass.aliases) + '\n')
-    
+    print >>outfile, 'usage:', command_usage(cmd_object) 
+
+    if cmd_object.aliases:
+        print >>outfile, 'aliases:',
+        print >>outfile, ', '.join(cmd_object.aliases)
+
+    print >>outfile
+
     outfile.write(doc)
     if doc[-1] != '\n':
         outfile.write('\n')
     
-    help_on_option(cmdclass.takes_options, outfile = None)
-
-
-def help_on_option(options, outfile = None):
-    import commands
+    help_on_options(cmd_object.takes_options, outfile=None)
+
+
+def help_on_options(options, outfile=None):
+    from bzrlib.commands import SHORT_OPTIONS
     
     if not options:
         return
@@ -130,32 +128,35 @@
     outfile.write('\noptions:\n')
     for on in options:
         l = '    --' + on
-        for shortname, longname in commands.SHORT_OPTIONS.items():
+        for shortname, longname in SHORT_OPTIONS.items():
             if longname == on:
                 l += ', -' + shortname
                 break
         outfile.write(l + '\n')
 
 
-def help_commands(outfile = None):
+def help_commands(outfile=None):
     """List all commands"""
-    import inspect
-    import commands
+    from bzrlib.commands import (builtin_command_names,
+                                 plugin_command_names,
+                                 get_cmd_object)
 
     if outfile == None:
         outfile = sys.stdout
-    
-    accu = []
-    for cmdname, cmdclass in commands.get_all_cmds():
-        accu.append((cmdname, cmdclass))
-    accu.sort()
-    for cmdname, cmdclass in accu:
-        if cmdclass.hidden:
+
+    names = set()                       # to eliminate duplicates
+    names.update(builtin_command_names())
+    names.update(plugin_command_names())
+    names = list(names)
+    names.sort()
+
+    for cmd_name in names:
+        cmd_object = get_cmd_object(cmd_name)
+        if cmd_object.hidden:
             continue
-        outfile.write(command_usage(cmdname, cmdclass) + '\n')
-        help = inspect.getdoc(cmdclass)
-        if help:
-            outfile.write("    " + help.split('\n', 1)[0] + '\n')
-
-            
-
+        print >>outfile, command_usage(cmd_object)
+        cmd_help = cmd_object.help()
+        if cmd_help:
+            firstline = cmd_help.split('\n', 1)[0]
+            print >>outfile, '    ' + firstline
+        

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2005-07-22 19:32:46 +0000
+++ bzrlib/inventory.py	2005-09-06 07:26:13 +0000
@@ -27,6 +27,8 @@
 
 from bzrlib.osutils import uuid, quotefn, splitpath, joinpath, appendpath
 from bzrlib.trace import mutter
+from bzrlib.errors import NotVersionedError
+        
 
 class InventoryEntry(object):
     """Description of a versioned file.
@@ -53,7 +55,9 @@
     >>> i.path2id('')
     'TREE_ROOT'
     >>> i.add(InventoryEntry('123', 'src', 'directory', ROOT_ID))
+    InventoryEntry('123', 'src', kind='directory', parent_id='TREE_ROOT')
     >>> i.add(InventoryEntry('2323', 'hello.c', 'file', parent_id='123'))
+    InventoryEntry('2323', 'hello.c', kind='file', parent_id='123')
     >>> for j in i.iter_entries():
     ...   print j
     ... 
@@ -64,12 +68,15 @@
     ...
     BzrError: inventory already contains entry with id {2323}
     >>> i.add(InventoryEntry('2324', 'bye.c', 'file', '123'))
+    InventoryEntry('2324', 'bye.c', kind='file', parent_id='123')
     >>> i.add(InventoryEntry('2325', 'wibble', 'directory', '123'))
+    InventoryEntry('2325', 'wibble', kind='directory', parent_id='123')
     >>> i.path2id('src/wibble')
     '2325'
     >>> '2325' in i
     True
     >>> i.add(InventoryEntry('2326', 'wibble.c', 'file', '2325'))
+    InventoryEntry('2326', 'wibble.c', kind='file', parent_id='2325')
     >>> i['2326']
     InventoryEntry('2326', 'wibble.c', kind='file', parent_id='2325')
     >>> for j in i.iter_entries():
@@ -93,7 +100,9 @@
     # directories, etc etc.
 
     __slots__ = ['text_sha1', 'text_size', 'file_id', 'name', 'kind',
-                 'text_id', 'parent_id', 'children', ]
+                 'text_id', 'parent_id', 'children',
+                 'text_version', 'entry_version', ]
+
 
     def __init__(self, file_id, name, kind, parent_id, text_id=None):
         """Create an InventoryEntry
@@ -110,12 +119,14 @@
         Traceback (most recent call last):
         BzrCheckError: InventoryEntry name 'src/hello.c' is invalid
         """
+        assert isinstance(name, basestring), name
         if '/' in name or '\\' in name:
             raise BzrCheckError('InventoryEntry name %r is invalid' % name)
         
+        self.text_version = None
+        self.entry_version = None
         self.text_sha1 = None
         self.text_size = None
-    
         self.file_id = file_id
         self.name = name
         self.kind = kind
@@ -155,60 +166,6 @@
                    self.parent_id))
 
     
-    def to_element(self):
-        """Convert to XML element"""
-        from bzrlib.xml import Element
-        
-        e = Element('entry')
-
-        e.set('name', self.name)
-        e.set('file_id', self.file_id)
-        e.set('kind', self.kind)
-
-        if self.text_size != None:
-            e.set('text_size', '%d' % self.text_size)
-            
-        for f in ['text_id', 'text_sha1']:
-            v = getattr(self, f)
-            if v != None:
-                e.set(f, v)
-
-        # to be conservative, we don't externalize the root pointers
-        # for now, leaving them as null in the xml form.  in a future
-        # version it will be implied by nested elements.
-        if self.parent_id != ROOT_ID:
-            assert isinstance(self.parent_id, basestring)
-            e.set('parent_id', self.parent_id)
-
-        e.tail = '\n'
-            
-        return e
-
-
-    def from_element(cls, elt):
-        assert elt.tag == 'entry'
-
-        ## original format inventories don't have a parent_id for
-        ## nodes in the root directory, but it's cleaner to use one
-        ## internally.
-        parent_id = elt.get('parent_id')
-        if parent_id == None:
-            parent_id = ROOT_ID
-
-        self = cls(elt.get('file_id'), elt.get('name'), elt.get('kind'), parent_id)
-        self.text_id = elt.get('text_id')
-        self.text_sha1 = elt.get('text_sha1')
-        
-        ## mutter("read inventoryentry: %r" % (elt.attrib))
-
-        v = elt.get('text_size')
-        self.text_size = v and int(v)
-
-        return self
-            
-
-    from_element = classmethod(from_element)
-
     def __eq__(self, other):
         if not isinstance(other, InventoryEntry):
             return NotImplemented
@@ -219,7 +176,9 @@
                and (self.text_size == other.text_size) \
                and (self.text_id == other.text_id) \
                and (self.parent_id == other.parent_id) \
-               and (self.kind == other.kind)
+               and (self.kind == other.kind) \
+               and (self.text_version == other.text_version) \
+               and (self.entry_version == other.entry_version)
 
 
     def __ne__(self, other):
@@ -266,6 +225,7 @@
 
     >>> inv = Inventory()
     >>> inv.add(InventoryEntry('123-123', 'hello.c', 'file', ROOT_ID))
+    InventoryEntry('123-123', 'hello.c', kind='file', parent_id='TREE_ROOT')
     >>> inv['123-123'].name
     'hello.c'
 
@@ -282,6 +242,7 @@
     ['hello.c']
     >>> inv = Inventory('TREE_ROOT-12345678-12345678')
     >>> inv.add(InventoryEntry('123-123', 'hello.c', 'file', ROOT_ID))
+    InventoryEntry('123-123', 'hello.c', kind='file', parent_id='TREE_ROOT-12345678-12345678')
     """
     def __init__(self, root_id=ROOT_ID):
         """Create or read an inventory.
@@ -369,6 +330,7 @@
 
         >>> inv = Inventory()
         >>> inv.add(InventoryEntry('123', 'foo.c', 'file', ROOT_ID))
+        InventoryEntry('123', 'foo.c', kind='file', parent_id='TREE_ROOT')
         >>> '123' in inv
         True
         >>> '456' in inv
@@ -382,6 +344,7 @@
 
         >>> inv = Inventory()
         >>> inv.add(InventoryEntry('123123', 'hello.c', 'file', ROOT_ID))
+        InventoryEntry('123123', 'hello.c', kind='file', parent_id='TREE_ROOT')
         >>> inv['123123'].name
         'hello.c'
         """
@@ -405,7 +368,10 @@
         """Add entry to inventory.
 
         To add  a file to a branch ready to be committed, use Branch.add,
-        which calls this."""
+        which calls this.
+
+        Returns the new entry object.
+        """
         if entry.file_id in self._byid:
             raise BzrError("inventory already contains entry with id {%s}" % entry.file_id)
 
@@ -423,20 +389,22 @@
 
         self._byid[entry.file_id] = entry
         parent.children[entry.name] = entry
+        return entry
 
 
     def add_path(self, relpath, kind, file_id=None):
         """Add entry from a path.
 
-        The immediate parent must already be versioned"""
-        from bzrlib.errors import NotVersionedError
+        The immediate parent must already be versioned.
+
+        Returns the new entry object."""
+        from bzrlib.branch import gen_file_id
         
         parts = bzrlib.osutils.splitpath(relpath)
         if len(parts) == 0:
             raise BzrError("cannot re-add root of inventory")
 
         if file_id == None:
-            from bzrlib.branch import gen_file_id
             file_id = gen_file_id(relpath)
 
         parent_path = parts[:-1]
@@ -454,6 +422,7 @@
 
         >>> inv = Inventory()
         >>> inv.add(InventoryEntry('123', 'foo.c', 'file', ROOT_ID))
+        InventoryEntry('123', 'foo.c', kind='file', parent_id='TREE_ROOT')
         >>> '123' in inv
         True
         >>> del inv['123']
@@ -475,43 +444,6 @@
         del self[ie.parent_id].children[ie.name]
 
 
-    def to_element(self):
-        """Convert to XML Element"""
-        from bzrlib.xml import Element
-        
-        e = Element('inventory')
-        e.text = '\n'
-        if self.root.file_id not in (None, ROOT_ID):
-            e.set('file_id', self.root.file_id)
-        for path, ie in self.iter_entries():
-            e.append(ie.to_element())
-        return e
-    
-
-    def from_element(cls, elt):
-        """Construct from XML Element
-        
-        >>> inv = Inventory()
-        >>> inv.add(InventoryEntry('foo.c-123981239', 'foo.c', 'file', ROOT_ID))
-        >>> elt = inv.to_element()
-        >>> inv2 = Inventory.from_element(elt)
-        >>> inv2 == inv
-        True
-        """
-        # XXXX: doctest doesn't run this properly under python2.3
-        assert elt.tag == 'inventory'
-        root_id = elt.get('file_id') or ROOT_ID
-        o = cls(root_id)
-        for e in elt:
-            ie = InventoryEntry.from_element(e)
-            if ie.parent_id == ROOT_ID:
-                ie.parent_id = root_id
-            o.add(ie)
-        return o
-        
-    from_element = classmethod(from_element)
-
-
     def __eq__(self, other):
         """Compare two sets by comparing their contents.
 
@@ -520,9 +452,11 @@
         >>> i1 == i2
         True
         >>> i1.add(InventoryEntry('123', 'foo', 'file', ROOT_ID))
+        InventoryEntry('123', 'foo', kind='file', parent_id='TREE_ROOT')
         >>> i1 == i2
         False
         >>> i2.add(InventoryEntry('123', 'foo', 'file', ROOT_ID))
+        InventoryEntry('123', 'foo', kind='file', parent_id='TREE_ROOT')
         >>> i1 == i2
         True
         """
@@ -544,7 +478,6 @@
         raise ValueError('not hashable')
 
 
-
     def get_idpath(self, file_id):
         """Return a list of file_ids for the path to an entry.
 
@@ -643,7 +576,11 @@
 
 
 
-_NAME_RE = re.compile(r'^[^/\\]+$')
+_NAME_RE = None
 
 def is_valid_name(name):
+    global _NAME_RE
+    if _NAME_RE == None:
+        _NAME_RE = re.compile(r'^[^/\\]+$')
+        
     return bool(_NAME_RE.match(name))

=== modified file 'bzrlib/log.py'
--- bzrlib/log.py	2005-07-16 00:07:40 +0000
+++ bzrlib/log.py	2005-09-05 05:35:25 +0000
@@ -28,15 +28,32 @@
 
 * with file-ids and revision-ids shown
 
-* from last to first or (not anymore) from first to last;
-  the default is "reversed" because it shows the likely most
-  relevant and interesting information first
-
-* (not yet) in XML format
+Logs are actually written out through an abstract LogFormatter
+interface, which allows for different preferred formats.  Plugins can
+register formats too.
+
+Logs can be produced in either forward (oldest->newest) or reverse
+(newest->oldest) order.
+
+Logs can be filtered to show only revisions matching a particular
+search string, or within a particular range of revisions.  The range
+can be given as date/times, which are reduced to revisions before
+calling in here.
+
+In verbose mode we show a summary of what changed in each particular
+revision.  Note that this is the delta for changes in that revision
+relative to its mainline parent, not the delta relative to the last
+logged revision.  So for example if you ask for a verbose log of
+changes touching hello.c you will get a list of those revisions also
+listing other things that were changed in the same revision, but not
+all the changes since the previous revision that touched hello.c.
 """
 
 
-from trace import mutter
+from bzrlib.tree import EmptyTree
+from bzrlib.delta import compare_trees
+from bzrlib.trace import mutter
+
 
 def find_touching_revisions(branch, file_id):
     """Yield a description of revisions which affect the file_id.
@@ -83,6 +100,15 @@
 
 
 
+def _enumerate_history(branch):
+    rh = []
+    revno = 1
+    for rev_id in branch.revision_history():
+        rh.append((revno, rev_id))
+        revno += 1
+    return rh
+
+
 def show_log(branch,
              lf,
              specific_fileid=None,
@@ -131,20 +157,32 @@
     else:
         searchRE = None
 
-    which_revs = branch.enum_history(direction)
-    which_revs = [x for x in which_revs if (
-            (start_revision is None or x[0] >= start_revision)
-            and (end_revision is None or x[0] <= end_revision))]
-
-    if not (verbose or specific_fileid):
-        # no need to know what changed between revisions
-        with_deltas = deltas_for_log_dummy(branch, which_revs)
-    elif direction == 'reverse':
-        with_deltas = deltas_for_log_reverse(branch, which_revs)
-    else:        
-        with_deltas = deltas_for_log_forward(branch, which_revs)
-
-    for revno, rev, delta in with_deltas:
+    which_revs = _enumerate_history(branch)
+    
+    if start_revision is None:
+        start_revision = 1
+    else:
+        branch.check_real_revno(start_revision)
+    
+    if end_revision is None:
+        end_revision = len(which_revs)
+    else:
+        branch.check_real_revno(end_revision)
+
+    # list indexes are 0-based; revisions are 1-based
+    cut_revs = which_revs[(start_revision-1):(end_revision)]
+
+    if direction == 'reverse':
+        cut_revs.reverse()
+    elif direction == 'forward':
+        pass
+    else:
+        raise ValueError('invalid direction %r' % direction)
+
+    for revno, rev_id in cut_revs:
+        if verbose or specific_fileid:
+            delta = branch.get_revision_delta(revno)
+            
         if specific_fileid:
             if not delta.touches_file_id(specific_fileid):
                 continue
@@ -153,29 +191,42 @@
             # although we calculated it, throw it away without display
             delta = None
 
-        if searchRE is None or searchRE.search(rev.message):
-            lf.show(revno, rev, delta)
+        rev = branch.get_revision(rev_id)
+
+        if searchRE:
+            if not searchRE.search(rev.message):
+                continue
+
+        lf.show(revno, rev, delta)
 
 
 
 def deltas_for_log_dummy(branch, which_revs):
+    """Return all the revisions without intermediate deltas.
+
+    Useful for log commands that won't need the delta information.
+    """
+    
     for revno, revision_id in which_revs:
         yield revno, branch.get_revision(revision_id), None
 
 
 def deltas_for_log_reverse(branch, which_revs):
-    """Compute deltas for display in reverse log.
-
-    Given a sequence of (revno, revision_id) pairs, return
-    (revno, rev, delta).
+    """Compute deltas for display in latest-to-earliest order.
+
+    branch
+        Branch to traverse
+
+    which_revs
+        Sequence of (revno, revision_id) for the subset of history to examine
+
+    returns 
+        Sequence of (revno, rev, delta)
 
     The delta is from the given revision to the next one in the
     sequence, which makes sense if the log is being displayed from
     newest to oldest.
     """
-    from tree import EmptyTree
-    from diff import compare_trees
-    
     last_revno = last_revision_id = last_tree = None
     for revno, revision_id in which_revs:
         this_tree = branch.revision_tree(revision_id)
@@ -210,9 +261,6 @@
     sequence, which makes sense if the log is being displayed from
     newest to oldest.
     """
-    from tree import EmptyTree
-    from diff import compare_trees
-
     last_revno = last_revision_id = last_tree = None
     prev_tree = EmptyTree(branch.get_root_id())
 
@@ -237,10 +285,14 @@
 
 class LogFormatter(object):
     """Abstract class to display log messages."""
-    def __init__(self, to_file, show_ids=False, show_timezone=False):
+    def __init__(self, to_file, show_ids=False, show_timezone='original'):
         self.to_file = to_file
         self.show_ids = show_ids
         self.show_timezone = show_timezone
+
+
+    def show(self, revno, rev, delta):
+        raise NotImplementedError('not implemented in abstract base')
         
 
 
@@ -257,9 +309,16 @@
         print >>to_file,  'revno:', revno
         if self.show_ids:
             print >>to_file,  'revision-id:', rev.revision_id
+
+            for parent in rev.parents:
+                print >>to_file, 'parent:', parent.revision_id
+            
         print >>to_file,  'committer:', rev.committer
-        print >>to_file,  'timestamp: %s' % (format_date(rev.timestamp, rev.timezone or 0,
-                                             self.show_timezone))
+
+        date_str = format_date(rev.timestamp,
+                               rev.timezone or 0,
+                               self.show_timezone)
+        print >>to_file,  'timestamp: %s' % date_str
 
         print >>to_file,  'message:'
         if not rev.message:
@@ -290,6 +349,8 @@
             for l in rev.message.split('\n'):
                 print >>to_file,  '      ' + l
 
+        # TODO: Why not show the modified files in a shorter form as
+        # well? rewrap them single lines of appropriate length
         if delta != None:
             delta.show(to_file, self.show_ids)
         print
@@ -308,3 +369,8 @@
         return FORMATTERS[name](*args, **kwargs)
     except IndexError:
         raise BzrCommandError("unknown log formatter: %r" % name)
+
+def show_one_log(revno, rev, delta, verbose, to_file, show_timezone):
+    # deprecated; for compatability
+    lf = LongLogFormatter(to_file=to_file, show_timezone=show_timezone)
+    lf.show(revno, rev, delta)

=== modified file 'bzrlib/mdiff.py'
--- bzrlib/mdiff.py	2005-04-09 06:21:44 +0000
+++ bzrlib/mdiff.py	2005-08-25 05:58:05 +0000
@@ -25,12 +25,56 @@
 
 # TODO: maybe work on files not strings?
 
+# FIXME: doesn't work properly on files without trailing newlines
 
+import unittest
 import difflib, sys, struct
 from cStringIO import StringIO
 
+def linesplit(a):
+    """Split into two lists: content and line positions.
+
+    This returns (al, ap).
+
+    al[i] is the string content of line i of the file, including its
+    newline (if any).
+
+    ap[i] is the byte position in the file where that line starts.
+
+    ap[-1] is the byte position of the end of the file (i.e. the
+    length of the file.)
+
+    This transformation allows us to do a line-based diff and then map
+    back to byte positions.
+    """
+
+    al, ap = [], []
+    last = 0
+
+    n = a.find("\n") + 1
+    while n > 0:
+        ap.append(last)
+        al.append(a[last:n])
+        last = n
+        n = a.find("\n", n) + 1
+
+    if last < len(a):
+        al.append(a[last:])
+        ap.append(last)
+
+    # position at the end
+    ap.append(len(a))
+
+    return (al, ap)
+
+
 def diff(a, b):
-    d = difflib.SequenceMatcher(None, a, b)
+    # TODO: Use different splits, perhaps rsync-like, for binary files?
+    
+    (al, ap) = linesplit(a)
+    (bl, bp) = linesplit(b)
+
+    d = difflib.SequenceMatcher(None, al, bl)
     
     ## sys.stderr.write('  ~ real_quick_ratio: %.4f\n' % d.real_quick_ratio())
     
@@ -38,9 +82,9 @@
         if o == 'equal': continue
         # a[m:n] should be replaced by b[s:t]
         if s == t:
-            yield m, n, ''
+            yield ap[m], ap[n], ''
         else:
-            yield m, n, b[s:t]
+            yield ap[m], ap[n], ''.join(bl[s:t])
 
 
 def tobinary(ops):
@@ -94,3 +138,26 @@
 
 
 
+class TestDiffPatch(unittest.TestCase):
+    def doDiffPatch(self, old, new):
+        diff = bdiff(old, new)
+        result = bpatch(old, diff)
+        self.assertEquals(new, result)
+
+
+    def testSimpleDiff(self):
+        """Simply add a line at the end"""
+        self.doDiffPatch('a\nb\n', 'a\nb\nc\n')
+        
+
+        
+    def testTrailingLine(self):
+        """Test diff that adds an unterminated line.
+
+        (Old versions didn't do this properly.)"""
+        self.doDiffPatch('a\nb\nc\n',
+                         'a\nb\nc\nd')
+
+
+if __name__ == '__main__':
+    unittest.main()

=== modified file 'bzrlib/merge.py'
--- bzrlib/merge.py	2005-07-29 17:19:16 +0000
+++ bzrlib/merge.py	2005-09-07 01:34:07 +0000
@@ -1,25 +1,51 @@
-from merge_core import merge_flex, ApplyMerge3, BackupBeforeChange
-from changeset import generate_changeset, ExceptionConflictHandler
-from changeset import Inventory, Diff3Merge
-from bzrlib import find_branch
-import bzrlib.osutils
-from bzrlib.errors import BzrCommandError
-from bzrlib.diff import compare_trees
-from trace import mutter, warning
-import os.path
+# Copyright (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+import os
 import tempfile
 import shutil
 import errno
-
-class UnrelatedBranches(BzrCommandError):
-    def __init__(self):
-        msg = "Branches have no common ancestor, and no base revision"\
-            " specified."
-        BzrCommandError.__init__(self, msg)
-
+from fetch import greedy_fetch
+
+import bzrlib.osutils
+import bzrlib.revision
+from bzrlib.merge_core import merge_flex, ApplyMerge3, BackupBeforeChange
+from bzrlib.changeset import generate_changeset, ExceptionConflictHandler
+from bzrlib.changeset import Inventory, Diff3Merge
+from bzrlib.branch import find_branch
+from bzrlib.errors import BzrCommandError, UnrelatedBranches
+from bzrlib.delta import compare_trees
+from bzrlib.trace import mutter, warning
+from bzrlib.fetch import greedy_fetch
+from bzrlib.revision import is_ancestor
+
+# comments from abentley on irc: merge happens in two stages, each
+# of which generates a changeset object
+
+# stage 1: generate OLD->OTHER,
+# stage 2: use MINE and OLD->OTHER to generate MINE -> RESULT
 
 class MergeConflictHandler(ExceptionConflictHandler):
-    """Handle conflicts encountered while merging"""
+    """Handle conflicts encountered while merging.
+
+    This subclasses ExceptionConflictHandler, so that any types of
+    conflict that are not explicitly handled cause an exception and
+    terminate the merge.
+    """
     def __init__(self, dir, ignore_zero=False):
         ExceptionConflictHandler.__init__(self, dir)
         self.conflicts = 0
@@ -36,6 +62,15 @@
             d_file.write(line)
         os.chmod(dest, 0777 & os.stat(source).st_mode)
 
+    def dump(self, lines, dest):
+        """Copy the text and mode of a file
+        :param source: The path of the file to copy
+        :param dest: The distination file to create
+        """
+        d_file = file(dest, "wb")
+        for line in lines:
+            d_file.write(line)
+
     def add_suffix(self, name, suffix, last_new_name=None):
         """Rename a file to append a suffix.  If the new name exists, the
         suffix is added repeatedly until a non-existant name is found
@@ -60,7 +95,7 @@
         self.conflicts += 1
         
 
-    def merge_conflict(self, new_file, this_path, base_path, other_path):
+    def merge_conflict(self, new_file, this_path, base_lines, other_lines):
         """
         Handle diff3 conflicts by producing a .THIS, .BASE and .OTHER.  The
         main file will be a version with diff3 conflicts.
@@ -70,11 +105,17 @@
         :param other_path: Path to the file text for the OTHER tree
         """
         self.add_suffix(this_path, ".THIS")
-        self.copy(base_path, this_path+".BASE")
-        self.copy(other_path, this_path+".OTHER")
+        self.dump(base_lines, this_path+".BASE")
+        self.dump(other_lines, this_path+".OTHER")
         os.rename(new_file, this_path)
         self.conflict("Diff3 conflict encountered in %s" % this_path)
 
+    def new_contents_conflict(self, filename, other_contents):
+        """Conflicting contents for newly added file."""
+        self.copy(other_contents, filename + ".OTHER")
+        self.conflict("Conflict in newly added file %s" % filename)
+    
+
     def target_exists(self, entry, target, old_path):
         """Handle the case when the target file or dir exists"""
         moved_path = self.add_suffix(target, ".moved")
@@ -90,47 +131,35 @@
         if not self.ignore_zero:
             print "%d conflicts encountered.\n" % self.conflicts
             
-class SourceFile(object):
-    def __init__(self, path, id, present=None, isdir=None):
-        self.path = path
-        self.id = id
-        self.present = present
-        self.isdir = isdir
-        self.interesting = True
-
-    def __repr__(self):
-        return "SourceFile(%s, %s)" % (self.path, self.id)
-
-def get_tree(treespec, temp_root, label):
+def get_tree(treespec, temp_root, label, local_branch=None):
     location, revno = treespec
     branch = find_branch(location)
     if revno is None:
+        revision = None
+    elif revno == -1:
+        revision = branch.last_patch()
+    else:
+        revision = branch.lookup_revision(revno)
+    return branch, get_revid_tree(branch, revision, temp_root, label,
+                                  local_branch)
+
+def get_revid_tree(branch, revision, temp_root, label, local_branch):
+    if revision is None:
         base_tree = branch.working_tree()
-    elif revno == -1:
-        base_tree = branch.basis_tree()
     else:
-        base_tree = branch.revision_tree(branch.lookup_revision(revno))
+        if local_branch is not None:
+            greedy_fetch(local_branch, branch, revision)
+            base_tree = local_branch.revision_tree(revision)
+        else:
+            base_tree = branch.revision_tree(revision)
     temp_path = os.path.join(temp_root, label)
     os.mkdir(temp_path)
-    return branch, MergeTree(base_tree, temp_path)
-
-
-def abspath(tree, file_id):
-    path = tree.inventory.id2path(file_id)
-    if path == "":
-        return "./."
-    return "./" + path
+    return MergeTree(base_tree, temp_path)
+
 
 def file_exists(tree, file_id):
     return tree.has_filename(tree.id2path(file_id))
     
-def inventory_map(tree):
-    inventory = {}
-    for file_id in tree.inventory:
-        path = abspath(tree, file_id)
-        inventory[path] = SourceFile(path, file_id)
-    return inventory
-
 
 class MergeTree(object):
     def __init__(self, tree, tempdir):
@@ -139,12 +168,39 @@
             self.root = tree.basedir
         else:
             self.root = None
-        self.inventory = inventory_map(tree)
         self.tree = tree
         self.tempdir = tempdir
         os.mkdir(os.path.join(self.tempdir, "texts"))
         self.cached = {}
 
+    def __iter__(self):
+        return self.tree.__iter__()
+
+    def __contains__(self, file_id):
+        return file_id in self.tree
+
+    def get_file(self, file_id):
+        return self.tree.get_file(file_id)
+
+    def get_file_sha1(self, id):
+        return self.tree.get_file_sha1(id)
+
+    def id2path(self, file_id):
+        return self.tree.id2path(file_id)
+
+    def has_id(self, file_id):
+        return self.tree.has_id(file_id)
+
+    def has_or_had_id(self, file_id):
+        if file_id == self.tree.inventory.root.file_id:
+            return True
+        return self.tree.inventory.has_id(file_id)
+
+    def has_or_had_id(self, file_id):
+        if file_id == self.tree.inventory.root.file_id:
+            return True
+        return self.tree.inventory.has_id(file_id)
+
     def readonly_path(self, id):
         if id not in self.tree:
             return None
@@ -165,77 +221,120 @@
 
 def merge(other_revision, base_revision,
           check_clean=True, ignore_zero=False,
-          this_dir=None, backup_files=False, merge_type=ApplyMerge3):
+          this_dir=None, backup_files=False, merge_type=ApplyMerge3,
+          file_list=None):
     """Merge changes into a tree.
 
     base_revision
-        Base for three-way merge.
+        tuple(path, revision) Base for three-way merge.
     other_revision
-        Other revision for three-way merge.
+        tuple(path, revision) Other revision for three-way merge.
     this_dir
         Directory to merge changes into; '.' by default.
     check_clean
         If true, this_dir must have no uncommitted changes before the
         merge begins.
+    all available ancestors of other_revision and base_revision are
+    automatically pulled into the branch.
     """
+    from bzrlib.revision import common_ancestor, MultipleRevisionSources
+    from bzrlib.errors import NoSuchRevision
     tempdir = tempfile.mkdtemp(prefix="bzr-")
     try:
         if this_dir is None:
             this_dir = '.'
         this_branch = find_branch(this_dir)
+        this_rev_id = this_branch.last_patch()
+        if this_rev_id is None:
+            raise BzrCommandError("This branch has no commits")
         if check_clean:
             changes = compare_trees(this_branch.working_tree(), 
                                     this_branch.basis_tree(), False)
             if changes.has_changed():
                 raise BzrCommandError("Working tree has uncommitted changes.")
-        other_branch, other_tree = get_tree(other_revision, tempdir, "other")
+        other_branch, other_tree = get_tree(other_revision, tempdir, "other",
+                                            this_branch)
+        if other_revision[1] == -1:
+            other_rev_id = other_branch.last_patch()
+            other_basis = other_rev_id
+        elif other_revision[1] is not None:
+            other_rev_id = other_branch.lookup_revision(other_revision[1])
+            other_basis = other_rev_id
+        else:
+            other_rev_id = None
+            other_basis = other_branch.last_patch()
         if base_revision == [None, None]:
-            if other_revision[1] == -1:
-                o_revno = None
-            else:
-                o_revno = other_revision[1]
-            base_revno = this_branch.common_ancestor(other_branch, 
-                                                     other_revno=o_revno)[0]
-            if base_revno is None:
+            base_rev_id = common_ancestor(this_rev_id, other_basis, 
+                                          this_branch)
+            if base_rev_id is None:
                 raise UnrelatedBranches()
-            base_revision = ['.', base_revno]
-        base_branch, base_tree = get_tree(base_revision, tempdir, "base")
+            base_tree = get_revid_tree(this_branch, base_rev_id, tempdir, 
+                                       "base", None)
+            base_is_ancestor = True
+        else:
+            base_branch, base_tree = get_tree(base_revision, tempdir, "base")
+            if base_revision[1] == -1:
+                base_rev_id = base_branch.last_patch()
+            elif base_revision[1] is None:
+                base_rev_id = None
+            else:
+                base_rev_id = base_branch.lookup_revision(base_revision[1])
+            if base_rev_id is not None:
+                base_is_ancestor = is_ancestor(this_rev_id, base_rev_id, 
+                                               MultipleRevisionSources(this_branch, 
+                                                                       base_branch))
+            else:
+                base_is_ancestor = False
+        if file_list is None:
+            interesting_ids = None
+        else:
+            interesting_ids = set()
+            this_tree = this_branch.working_tree()
+            for fname in file_list:
+                path = this_branch.relpath(fname)
+                found_id = False
+                for tree in (this_tree, base_tree.tree, other_tree.tree):
+                    file_id = tree.inventory.path2id(path)
+                    if file_id is not None:
+                        interesting_ids.add(file_id)
+                        found_id = True
+                if not found_id:
+                    raise BzrCommandError("%s is not a source file in any"
+                                          " tree." % fname)
         merge_inner(this_branch, other_tree, base_tree, tempdir, 
                     ignore_zero=ignore_zero, backup_files=backup_files, 
-                    merge_type=merge_type)
+                    merge_type=merge_type, interesting_ids=interesting_ids)
+        if base_is_ancestor and other_rev_id is not None\
+            and other_rev_id not in this_branch.revision_history():
+            this_branch.add_pending_merge(other_rev_id)
     finally:
         shutil.rmtree(tempdir)
 
 
-def generate_cset_optimized(tree_a, tree_b, inventory_a, inventory_b):
-    """Generate a changeset, using the text_id to mark really-changed files.
-    This permits blazing comparisons when text_ids are present.  It also
-    disables metadata comparison for files with identical texts.
+def set_interesting(inventory_a, inventory_b, interesting_ids):
+    """Mark files whose ids are in interesting_ids as interesting
+    """
+    for inventory in (inventory_a, inventory_b):
+        for path, source_file in inventory.iteritems():
+             source_file.interesting = source_file.id in interesting_ids
+
+
+def generate_cset_optimized(tree_a, tree_b, interesting_ids=None):
+    """Generate a changeset.  If interesting_ids is supplied, only changes
+    to those files will be shown.  Metadata changes are stripped.
     """ 
-    for file_id in tree_a.tree.inventory:
-        if file_id not in tree_b.tree.inventory:
-            continue
-        entry_a = tree_a.tree.inventory[file_id]
-        entry_b = tree_b.tree.inventory[file_id]
-        if (entry_a.kind, entry_b.kind) != ("file", "file"):
-            continue
-        if None in (entry_a.text_id, entry_b.text_id):
-            continue
-        if entry_a.text_id != entry_b.text_id:
-            continue
-        inventory_a[abspath(tree_a.tree, file_id)].interesting = False
-        inventory_b[abspath(tree_b.tree, file_id)].interesting = False
-    cset =  generate_changeset(tree_a, tree_b, inventory_a, inventory_b)
+    cset =  generate_changeset(tree_a, tree_b, interesting_ids)
     for entry in cset.entries.itervalues():
         entry.metadata_change = None
     return cset
 
 
 def merge_inner(this_branch, other_tree, base_tree, tempdir, 
-                ignore_zero=False, merge_type=ApplyMerge3, backup_files=False):
+                ignore_zero=False, merge_type=ApplyMerge3, backup_files=False,
+                interesting_ids=None):
 
-    def merge_factory(base_file, other_file):
-        contents_change = merge_type(base_file, other_file)
+    def merge_factory(file_id, base, other):
+        contents_change = merge_type(file_id, base, other)
         if backup_files:
             contents_change = BackupBeforeChange(contents_change)
         return contents_change
@@ -243,13 +342,14 @@
     this_tree = get_tree((this_branch.base, None), tempdir, "this")[1]
 
     def get_inventory(tree):
-        return tree.inventory
+        return tree.tree.inventory
 
     inv_changes = merge_flex(this_tree, base_tree, other_tree,
                              generate_cset_optimized, get_inventory,
                              MergeConflictHandler(base_tree.root,
                                                   ignore_zero=ignore_zero),
-                             merge_factory=merge_factory)
+                             merge_factory=merge_factory, 
+                             interesting_ids=interesting_ids)
 
     adjust_ids = []
     for id, path in inv_changes.iteritems():
@@ -257,19 +357,36 @@
             if path == '.':
                 path = ''
             else:
-                assert path.startswith('./')
+                assert path.startswith('.' + os.sep), "path is %s" % path
             path = path[2:]
         adjust_ids.append((path, id))
-    this_branch.set_inventory(regen_inventory(this_branch, this_tree.root, adjust_ids))
+    if len(adjust_ids) > 0:
+        this_branch.set_inventory(regen_inventory(this_branch, this_tree.root,
+                                                  adjust_ids))
 
 
 def regen_inventory(this_branch, root, new_entries):
     old_entries = this_branch.read_working_inventory()
     new_inventory = {}
     by_path = {}
+    new_entries_map = {} 
+    for path, file_id in new_entries:
+        if path is None:
+            continue
+        new_entries_map[file_id] = path
+
+    def id2path(file_id):
+        path = new_entries_map.get(file_id)
+        if path is not None:
+            return path
+        entry = old_entries[file_id]
+        if entry.parent_id is None:
+            return entry.name
+        return os.path.join(id2path(entry.parent_id), entry.name)
+        
     for file_id in old_entries:
         entry = old_entries[file_id]
-        path = old_entries.id2path(file_id)
+        path = id2path(file_id)
         new_inventory[file_id] = (path, file_id, entry.parent_id, entry.kind)
         by_path[path] = file_id
     

=== modified file 'bzrlib/merge3.py'
--- bzrlib/merge3.py	2005-07-06 04:09:02 +0000
+++ bzrlib/merge3.py	2005-09-04 02:59:56 +0000
@@ -76,9 +76,9 @@
     def merge_lines(self,
                     name_a=None,
                     name_b=None,
-                    start_marker='<<<<<<<<',
-                    mid_marker='========',
-                    end_marker='>>>>>>>>',
+                    start_marker='<<<<<<<',
+                    mid_marker='=======',
+                    end_marker='>>>>>>>',
                     show_base=False):
         """Return merge in cvs-like form.
         """

=== modified file 'bzrlib/merge_core.py'
--- bzrlib/merge_core.py	2005-07-28 18:46:35 +0000
+++ bzrlib/merge_core.py	2005-08-27 04:42:41 +0000
@@ -6,15 +6,16 @@
 
 class ApplyMerge3:
     """Contents-change wrapper around merge3.Merge3"""
-    def __init__(self, base_file, other_file):
-        self.base_file = base_file
-        self.other_file = other_file
+    def __init__(self, file_id, base, other):
+        self.file_id = file_id
+        self.base = base
+        self.other = other
  
     def __eq__(self, other):
         if not isinstance(other, ApplyMerge3):
             return False
-        return (self.base_file == other.base_file and 
-                self.other_file == other.other_file)
+        return (self.base == other.base and 
+                self.other == other.other and self.file_id == other.file_id)
 
     def __ne__(self, other):
         return not (self == other)
@@ -23,14 +24,19 @@
     def apply(self, filename, conflict_handler, reverse=False):
         new_file = filename+".new" 
         if not reverse:
-            base = self.base_file
-            other = self.other_file
+            base = self.base
+            other = self.other
         else:
-            base = self.other_file
-            other = self.base_file
-        m3 = Merge3(file(base, "rb").readlines(), 
-                    file(filename, "rb").readlines(), 
-                    file(other, "rb").readlines())
+            base = self.other
+            other = self.base
+        def get_lines(tree):
+            if self.file_id not in tree:
+                raise Exception("%s not in tree" % self.file_id)
+                return ()
+            return tree.get_file(self.file_id).readlines()
+        base_lines = get_lines(base)
+        other_lines = get_lines(other)
+        m3 = Merge3(base_lines, file(filename, "rb").readlines(), other_lines)
 
         new_conflicts = False
         output_file = file(new_file, "wb")
@@ -39,7 +45,7 @@
                        start_marker=start_marker):
             if line.startswith(start_marker):
                 new_conflicts = True
-                output_file.write(line.replace(start_marker, '<<<<<<<<'))
+                output_file.write(line.replace(start_marker, '<' * 7))
             else:
                 output_file.write(line)
         output_file.close()
@@ -48,7 +54,8 @@
             os.rename(new_file, filename)
             return
         else:
-            conflict_handler.merge_conflict(new_file, filename, base, other)
+            conflict_handler.merge_conflict(new_file, filename, base_lines,
+                                            other_lines)
 
 
 class BackupBeforeChange:
@@ -69,44 +76,35 @@
         self.contents_change.apply(filename, conflict_handler, reverse)
 
 
-class ThreewayInventory(object):
-    def __init__(self, this_inventory, base_inventory, other_inventory):
-        self.this = this_inventory
-        self.base = base_inventory
-        self.other = other_inventory
 def invert_invent(inventory):
     invert_invent = {}
-    for key, value in inventory.iteritems():
-        invert_invent[value.id] = key
+    for file_id in inventory:
+        path = inventory.id2path(file_id)
+        if path == '':
+            path = './.'
+        else:
+            path = './' + path
+        invert_invent[file_id] = path
     return invert_invent
 
-def make_inv(inventory):
-    return Inventory(invert_invent(inventory))
-        
 
 def merge_flex(this, base, other, changeset_function, inventory_function,
-               conflict_handler, merge_factory):
-    this_inventory = inventory_function(this)
-    base_inventory = inventory_function(base)
-    other_inventory = inventory_function(other)
-    inventory = ThreewayInventory(make_inv(this_inventory),
-                                  make_inv(base_inventory), 
-                                  make_inv(other_inventory))
-    cset = changeset_function(base, other, base_inventory, other_inventory)
-    new_cset = make_merge_changeset(cset, inventory, this, base, other, 
+               conflict_handler, merge_factory, interesting_ids):
+    cset = changeset_function(base, other, interesting_ids)
+    new_cset = make_merge_changeset(cset, this, base, other, 
                                     conflict_handler, merge_factory)
-    result = apply_changeset(new_cset, invert_invent(this_inventory),
+    result = apply_changeset(new_cset, invert_invent(this.tree.inventory),
                              this.root, conflict_handler, False)
     conflict_handler.finalize()
     return result
 
     
 
-def make_merge_changeset(cset, inventory, this, base, other, 
+def make_merge_changeset(cset, this, base, other, 
                          conflict_handler, merge_factory):
     new_cset = changeset.Changeset()
     def get_this_contents(id):
-        path = os.path.join(this.root, inventory.this.get_path(id))
+        path = this.readonly_path(id)
         if os.path.isdir(path):
             return changeset.dir_create
         else:
@@ -116,9 +114,10 @@
         if entry.is_boring():
             new_cset.add_entry(entry)
         else:
-            new_entry = make_merged_entry(entry, inventory, conflict_handler)
+            new_entry = make_merged_entry(entry, this, base, other, 
+                                          conflict_handler)
             new_contents = make_merged_contents(entry, this, base, other, 
-                                                inventory, conflict_handler, 
+                                                conflict_handler,
                                                 merge_factory)
             new_entry.contents_change = new_contents
             new_entry.metadata_change = make_merged_metadata(entry, base, other)
@@ -126,65 +125,90 @@
 
     return new_cset
 
-def make_merged_entry(entry, inventory, conflict_handler):
+class ThreeWayConflict(Exception):
+    def __init__(self, this, base, other):
+        self.this = this
+        self.base = base
+        self.other = other
+        msg = "Conflict merging %s %s and %s" % (this, base, other)
+        Exception.__init__(self, msg)
+
+def threeway_select(this, base, other):
+    """Returns a value selected by the three-way algorithm.
+    Raises ThreewayConflict if the algorithm yields a conflict"""
+    if base == other:
+        return this
+    elif base == this:
+        return other
+    elif other == this:
+        return this
+    else:
+        raise ThreeWayConflict(this, base, other)
+
+
+def make_merged_entry(entry, this, base, other, conflict_handler):
     from bzrlib.trace import mutter
-    this_name = inventory.this.get_name(entry.id)
-    this_parent = inventory.this.get_parent(entry.id)
-    this_dir = inventory.this.get_dir(entry.id)
-    if this_dir is None:
-        this_dir = ""
-
-    base_name = inventory.base.get_name(entry.id)
-    base_parent = inventory.base.get_parent(entry.id)
-    base_dir = inventory.base.get_dir(entry.id)
-    if base_dir is None:
-        base_dir = ""
-    other_name = inventory.other.get_name(entry.id)
-    other_parent = inventory.other.get_parent(entry.id)
-    other_dir = inventory.base.get_dir(entry.id)
-    if other_dir is None:
-        other_dir = ""
+    def entry_data(file_id, tree):
+        assert hasattr(tree, "__contains__"), "%s" % tree
+        if not tree.has_or_had_id(file_id):
+            return (None, None, "")
+        entry = tree.tree.inventory[file_id]
+        my_dir = tree.id2path(entry.parent_id)
+        if my_dir is None:
+            my_dir = ""
+        return entry.name, entry.parent_id, my_dir 
+    this_name, this_parent, this_dir = entry_data(entry.id, this)
+    base_name, base_parent, base_dir = entry_data(entry.id, base)
+    other_name, other_parent, other_dir = entry_data(entry.id, other)
     mutter("Dirs: this, base, other %r %r %r" % (this_dir, base_dir, other_dir))
     mutter("Names: this, base, other %r %r %r" % (this_name, base_name, other_name))
-    if base_name == other_name:
-        old_name = this_name
-        new_name = this_name
-    else:
-        if this_name != base_name and this_name != other_name:
-            conflict_handler.rename_conflict(entry.id, this_name, base_name,
-                                             other_name)
-        else:
-            old_name = this_name
-            new_name = other_name
-
-    if base_parent == other_parent:
-        old_parent = this_parent
-        new_parent = this_parent
-        old_dir = this_dir
-        new_dir = this_dir
-    else:
-        if this_parent != base_parent and this_parent != other_parent:
-            conflict_handler.move_conflict(entry.id, inventory)
-        else:
-            old_parent = this_parent
-            old_dir = this_dir
-            new_parent = other_parent
-            new_dir = other_dir
-    if old_name is not None and old_parent is not None:
-        old_path = os.path.join(old_dir, old_name)
-    else:
-        old_path = None
+    old_name = this_name
+    try:
+        new_name = threeway_select(this_name, base_name, other_name)
+    except ThreeWayConflict:
+        new_name = conflict_handler.rename_conflict(entry.id, this_name, 
+                                                    base_name, other_name)
+
+    old_parent = this_parent
+    try:
+        new_parent = threeway_select(this_parent, base_parent, other_parent)
+    except ThreeWayConflict:
+        new_parent = conflict_handler.move_conflict(entry.id, this_dir,
+                                                    base_dir, other_dir)
+    def get_path(name, parent):
+        if name is not None:
+            if name == "":
+                assert parent is None
+                return './.'
+            parent_dir = {this_parent: this_dir, other_parent: other_dir, 
+                          base_parent: base_dir}
+            directory = parent_dir[parent]
+            return os.path.join(directory, name)
+        else:
+            assert parent is None
+            return None
+
+    old_path = get_path(old_name, old_parent)
+        
     new_entry = changeset.ChangesetEntry(entry.id, old_parent, old_path)
-    if new_name is not None and new_parent is not None:
-        new_entry.new_path = os.path.join(new_dir, new_name)
-    else:
-        new_entry.new_path = None
+    new_entry.new_path = get_path(new_name, new_parent)
     new_entry.new_parent = new_parent
     mutter(repr(new_entry))
     return new_entry
 
 
-def make_merged_contents(entry, this, base, other, inventory, conflict_handler,
+def get_contents(entry, tree):
+    """Get a contents change element suitable for use with ReplaceContents
+    """
+    tree_entry = tree.tree.inventory[entry.id]
+    if tree_entry.kind == "file":
+        return changeset.FileCreate(tree.get_file(entry.id).read())
+    else:
+        assert tree_entry.kind in ("root_directory", "directory")
+        return changeset.dir_create
+
+
+def make_merged_contents(entry, this, base, other, conflict_handler,
                          merge_factory):
     contents = entry.contents_change
     if contents is None:
@@ -192,13 +216,10 @@
     this_path = this.readonly_path(entry.id)
     def make_merge():
         if this_path is None:
-            return conflict_handler.missing_for_merge(entry.id, inventory)
-        base_path = base.readonly_path(entry.id)
-        other_path = other.readonly_path(entry.id)    
-        return merge_factory(base_path, other_path)
+            return conflict_handler.missing_for_merge(entry.id, 
+                                                      other.id2path(entry.id))
+        return merge_factory(entry.id, base, other)
 
-    if isinstance(contents, changeset.PatchApply):
-        return make_merge()
     if isinstance(contents, changeset.ReplaceContents):
         if contents.old_contents is None and contents.new_contents is None:
             return None
@@ -211,7 +232,7 @@
             if this_path is None or not os.path.exists(this_path):
                 return contents
             else:
-                this_contents = file(this_path, "rb").read()
+                this_contents = get_contents(entry, this)
                 if this_contents == contents.new_contents:
                     return None
                 else:
@@ -254,420 +275,3 @@
             os.chmod(filename, other_stat)
         else:
             conflict_handler.permission_conflict(filename, base, other)
-
-
-import unittest
-import tempfile
-import shutil
-class MergeTree(object):
-    def __init__(self, dir):
-        self.dir = dir;
-        os.mkdir(dir)
-        self.inventory = {'0': ""}
-    
-    def child_path(self, parent, name):
-        return os.path.join(self.inventory[parent], name)
-
-    def add_file(self, id, parent, name, contents, mode):
-        path = self.child_path(parent, name)
-        full_path = self.abs_path(path)
-        assert not os.path.exists(full_path)
-        file(full_path, "wb").write(contents)
-        os.chmod(self.abs_path(path), mode)
-        self.inventory[id] = path
-
-    def remove_file(self, id):
-        os.unlink(self.full_path(id))
-        del self.inventory[id]
-
-    def add_dir(self, id, parent, name, mode):
-        path = self.child_path(parent, name)
-        full_path = self.abs_path(path)
-        assert not os.path.exists(full_path)
-        os.mkdir(self.abs_path(path))
-        os.chmod(self.abs_path(path), mode)
-        self.inventory[id] = path
-
-    def abs_path(self, path):
-        return os.path.join(self.dir, path)
-
-    def full_path(self, id):
-        try:
-            tree_path = self.inventory[id]
-        except KeyError:
-            return None
-        return self.abs_path(tree_path)
-
-    def readonly_path(self, id):
-        return self.full_path(id)
-
-    def change_path(self, id, path):
-        new = os.path.join(self.dir, self.inventory[id])
-        os.rename(self.abs_path(self.inventory[id]), self.abs_path(path))
-        self.inventory[id] = path
-
-class MergeBuilder(object):
-    def __init__(self):
-        self.dir = tempfile.mkdtemp(prefix="BaZing")
-        self.base = MergeTree(os.path.join(self.dir, "base"))
-        self.this = MergeTree(os.path.join(self.dir, "this"))
-        self.other = MergeTree(os.path.join(self.dir, "other"))
-        
-        self.cset = changeset.Changeset()
-        self.cset.add_entry(changeset.ChangesetEntry("0", 
-                                                     changeset.NULL_ID, "./."))
-    def get_cset_path(self, parent, name):
-        if name is None:
-            assert (parent is None)
-            return None
-        return os.path.join(self.cset.entries[parent].path, name)
-
-    def add_file(self, id, parent, name, contents, mode):
-        self.base.add_file(id, parent, name, contents, mode)
-        self.this.add_file(id, parent, name, contents, mode)
-        self.other.add_file(id, parent, name, contents, mode)
-        path = self.get_cset_path(parent, name)
-        self.cset.add_entry(changeset.ChangesetEntry(id, parent, path))
-
-    def remove_file(self, id, base=False, this=False, other=False):
-        for option, tree in ((base, self.base), (this, self.this), 
-                             (other, self.other)):
-            if option:
-                tree.remove_file(id)
-            if other or base:
-                change = self.cset.entries[id].contents_change
-                assert isinstance(change, changeset.ReplaceContents)
-                if other:
-                    change.new_contents=None
-                if base:
-                    change.old_contents=None
-                if change.old_contents is None and change.new_contents is None:
-                    change = None
-
-
-    def add_dir(self, id, parent, name, mode):
-        path = self.get_cset_path(parent, name)
-        self.base.add_dir(id, parent, name, mode)
-        self.cset.add_entry(changeset.ChangesetEntry(id, parent, path))
-        self.this.add_dir(id, parent, name, mode)
-        self.other.add_dir(id, parent, name, mode)
-
-
-    def change_name(self, id, base=None, this=None, other=None):
-        if base is not None:
-            self.change_name_tree(id, self.base, base)
-            self.cset.entries[id].name = base
-
-        if this is not None:
-            self.change_name_tree(id, self.this, this)
-
-        if other is not None:
-            self.change_name_tree(id, self.other, other)
-            self.cset.entries[id].new_name = other
-
-    def change_parent(self, id, base=None, this=None, other=None):
-        if base is not None:
-            self.change_parent_tree(id, self.base, base)
-            self.cset.entries[id].parent = base
-            self.cset.entries[id].dir = self.cset.entries[base].path
-
-        if this is not None:
-            self.change_parent_tree(id, self.this, this)
-
-        if other is not None:
-            self.change_parent_tree(id, self.other, other)
-            self.cset.entries[id].new_parent = other
-            self.cset.entries[id].new_dir = \
-                self.cset.entries[other].new_path
-
-    def change_contents(self, id, base=None, this=None, other=None):
-        if base is not None:
-            self.change_contents_tree(id, self.base, base)
-
-        if this is not None:
-            self.change_contents_tree(id, self.this, this)
-
-        if other is not None:
-            self.change_contents_tree(id, self.other, other)
-
-        if base is not None or other is not None:
-            old_contents = file(self.base.full_path(id)).read()
-            new_contents = file(self.other.full_path(id)).read()
-            contents = changeset.ReplaceFileContents(old_contents, 
-                                                     new_contents)
-            self.cset.entries[id].contents_change = contents
-
-    def change_perms(self, id, base=None, this=None, other=None):
-        if base is not None:
-            self.change_perms_tree(id, self.base, base)
-
-        if this is not None:
-            self.change_perms_tree(id, self.this, this)
-
-        if other is not None:
-            self.change_perms_tree(id, self.other, other)
-
-        if base is not None or other is not None:
-            old_perms = os.stat(self.base.full_path(id)).st_mode &077
-            new_perms = os.stat(self.other.full_path(id)).st_mode &077
-            contents = changeset.ChangeUnixPermissions(old_perms, 
-                                                       new_perms)
-            self.cset.entries[id].metadata_change = contents
-
-    def change_name_tree(self, id, tree, name):
-        new_path = tree.child_path(self.cset.entries[id].parent, name)
-        tree.change_path(id, new_path)
-
-    def change_parent_tree(self, id, tree, parent):
-        new_path = tree.child_path(parent, self.cset.entries[id].name)
-        tree.change_path(id, new_path)
-
-    def change_contents_tree(self, id, tree, contents):
-        path = tree.full_path(id)
-        mode = os.stat(path).st_mode
-        file(path, "w").write(contents)
-        os.chmod(path, mode)
-
-    def change_perms_tree(self, id, tree, mode):
-        os.chmod(tree.full_path(id), mode)
-
-    def merge_changeset(self, merge_factory):
-        all_inventory = ThreewayInventory(Inventory(self.this.inventory),
-                                          Inventory(self.base.inventory), 
-                                          Inventory(self.other.inventory))
-        conflict_handler = changeset.ExceptionConflictHandler(self.this.dir)
-        return make_merge_changeset(self.cset, all_inventory, self.this,
-                                    self.base, self.other, conflict_handler,
-                                    merge_factory)
-
-    def apply_inv_change(self, inventory_change, orig_inventory):
-        orig_inventory_by_path = {}
-        for file_id, path in orig_inventory.iteritems():
-            orig_inventory_by_path[path] = file_id
-
-        def parent_id(file_id):
-            try:
-                parent_dir = os.path.dirname(orig_inventory[file_id])
-            except:
-                print file_id
-                raise
-            if parent_dir == "":
-                return None
-            return orig_inventory_by_path[parent_dir]
-        
-        def new_path(file_id):
-            if inventory_change.has_key(file_id):
-                return inventory_change[file_id]
-            else:
-                parent = parent_id(file_id)
-                if parent is None:
-                    return orig_inventory[file_id]
-                dirname = new_path(parent)
-                return os.path.join(dirname, orig_inventory[file_id])
-
-        new_inventory = {}
-        for file_id in orig_inventory.iterkeys():
-            path = new_path(file_id)
-            if path is None:
-                continue
-            new_inventory[file_id] = path
-
-        for file_id, path in inventory_change.iteritems():
-            if orig_inventory.has_key(file_id):
-                continue
-            new_inventory[file_id] = path
-        return new_inventory
-
-        
-
-    def apply_changeset(self, cset, conflict_handler=None, reverse=False):
-        inventory_change = changeset.apply_changeset(cset,
-                                                     self.this.inventory,
-                                                     self.this.dir,
-                                                     conflict_handler, reverse)
-        self.this.inventory =  self.apply_inv_change(inventory_change, 
-                                                     self.this.inventory)
-
-                    
-        
-
-        
-    def cleanup(self):
-        shutil.rmtree(self.dir)
-
-
-class MergeTest(unittest.TestCase):
-    def test_change_name(self):
-        """Test renames"""
-        builder = MergeBuilder()
-        builder.add_file("1", "0", "name1", "hello1", 0755)
-        builder.change_name("1", other="name2")
-        builder.add_file("2", "0", "name3", "hello2", 0755)
-        builder.change_name("2", base="name4")
-        builder.add_file("3", "0", "name5", "hello3", 0755)
-        builder.change_name("3", this="name6")
-        cset = builder.merge_changeset(ApplyMerge3)
-        assert(cset.entries["2"].is_boring())
-        assert(cset.entries["1"].name == "name1")
-        assert(cset.entries["1"].new_name == "name2")
-        assert(cset.entries["3"].is_boring())
-        for tree in (builder.this, builder.other, builder.base):
-            assert(tree.dir != builder.dir and 
-                   tree.dir.startswith(builder.dir))
-            for path in tree.inventory.itervalues():
-                fullpath = tree.abs_path(path)
-                assert(fullpath.startswith(tree.dir))
-                assert(not path.startswith(tree.dir))
-                assert os.path.exists(fullpath)
-        builder.apply_changeset(cset)
-        builder.cleanup()
-        builder = MergeBuilder()
-        builder.add_file("1", "0", "name1", "hello1", 0644)
-        builder.change_name("1", other="name2", this="name3")
-        self.assertRaises(changeset.RenameConflict, 
-                          builder.merge_changeset, ApplyMerge3)
-        builder.cleanup()
-        
-    def test_file_moves(self):
-        """Test moves"""
-        builder = MergeBuilder()
-        builder.add_dir("1", "0", "dir1", 0755)
-        builder.add_dir("2", "0", "dir2", 0755)
-        builder.add_file("3", "1", "file1", "hello1", 0644)
-        builder.add_file("4", "1", "file2", "hello2", 0644)
-        builder.add_file("5", "1", "file3", "hello3", 0644)
-        builder.change_parent("3", other="2")
-        assert(Inventory(builder.other.inventory).get_parent("3") == "2")
-        builder.change_parent("4", this="2")
-        assert(Inventory(builder.this.inventory).get_parent("4") == "2")
-        builder.change_parent("5", base="2")
-        assert(Inventory(builder.base.inventory).get_parent("5") == "2")
-        cset = builder.merge_changeset(ApplyMerge3)
-        for id in ("1", "2", "4", "5"):
-            assert(cset.entries[id].is_boring())
-        assert(cset.entries["3"].parent == "1")
-        assert(cset.entries["3"].new_parent == "2")
-        builder.apply_changeset(cset)
-        builder.cleanup()
-
-        builder = MergeBuilder()
-        builder.add_dir("1", "0", "dir1", 0755)
-        builder.add_dir("2", "0", "dir2", 0755)
-        builder.add_dir("3", "0", "dir3", 0755)
-        builder.add_file("4", "1", "file1", "hello1", 0644)
-        builder.change_parent("4", other="2", this="3")
-        self.assertRaises(changeset.MoveConflict, 
-                          builder.merge_changeset, ApplyMerge3)
-        builder.cleanup()
-
-    def test_contents_merge(self):
-        """Test merge3 merging"""
-        self.do_contents_test(ApplyMerge3)
-
-    def test_contents_merge2(self):
-        """Test diff3 merging"""
-        self.do_contents_test(changeset.Diff3Merge)
-
-    def test_contents_merge3(self):
-        """Test diff3 merging"""
-        def backup_merge(base_file, other_file):
-            return BackupBeforeChange(ApplyMerge3(base_file, other_file))
-        builder = self.contents_test_success(backup_merge)
-        def backup_exists(file_id):
-            return os.path.exists(builder.this.full_path(file_id)+"~")
-        assert backup_exists("1")
-        assert backup_exists("2")
-        assert not backup_exists("3")
-        builder.cleanup()
-
-    def do_contents_test(self, merge_factory):
-        """Test merging with specified ContentsChange factory"""
-        builder = self.contents_test_success(merge_factory)
-        builder.cleanup()
-        self.contents_test_conflicts(merge_factory)
-
-    def contents_test_success(self, merge_factory):
-        from inspect import isclass
-        builder = MergeBuilder()
-        builder.add_file("1", "0", "name1", "text1", 0755)
-        builder.change_contents("1", other="text4")
-        builder.add_file("2", "0", "name3", "text2", 0655)
-        builder.change_contents("2", base="text5")
-        builder.add_file("3", "0", "name5", "text3", 0744)
-        builder.change_contents("3", this="text6")
-        cset = builder.merge_changeset(merge_factory)
-        assert(cset.entries["1"].contents_change is not None)
-        if isclass(merge_factory):
-            assert(isinstance(cset.entries["1"].contents_change,
-                          merge_factory))
-            assert(isinstance(cset.entries["2"].contents_change,
-                          merge_factory))
-        assert(cset.entries["3"].is_boring())
-        builder.apply_changeset(cset)
-        assert(file(builder.this.full_path("1"), "rb").read() == "text4" )
-        assert(file(builder.this.full_path("2"), "rb").read() == "text2" )
-        assert(os.stat(builder.this.full_path("1")).st_mode &0777 == 0755)
-        assert(os.stat(builder.this.full_path("2")).st_mode &0777 == 0655)
-        assert(os.stat(builder.this.full_path("3")).st_mode &0777 == 0744)
-        return builder
-
-    def contents_test_conflicts(self, merge_factory):
-        builder = MergeBuilder()
-        builder.add_file("1", "0", "name1", "text1", 0755)
-        builder.change_contents("1", other="text4", this="text3")
-        cset = builder.merge_changeset(merge_factory)
-        self.assertRaises(changeset.MergeConflict, builder.apply_changeset,
-                          cset)
-        builder.cleanup()
-
-        builder = MergeBuilder()
-        builder.add_file("1", "0", "name1", "text1", 0755)
-        builder.change_contents("1", other="text4", base="text3")
-        builder.remove_file("1", base=True)
-        self.assertRaises(changeset.NewContentsConflict,
-                          builder.merge_changeset, merge_factory)
-        builder.cleanup()
-
-        builder = MergeBuilder()
-        builder.add_file("1", "0", "name1", "text1", 0755)
-        builder.change_contents("1", other="text4", base="text3")
-        builder.remove_file("1", this=True)
-        self.assertRaises(changeset.MissingForMerge, builder.merge_changeset, 
-                          merge_factory)
-        builder.cleanup()
-
-    def test_perms_merge(self):
-        builder = MergeBuilder()
-        builder.add_file("1", "0", "name1", "text1", 0755)
-        builder.change_perms("1", other=0655)
-        builder.add_file("2", "0", "name2", "text2", 0755)
-        builder.change_perms("2", base=0655)
-        builder.add_file("3", "0", "name3", "text3", 0755)
-        builder.change_perms("3", this=0655)
-        cset = builder.merge_changeset(ApplyMerge3)
-        assert(cset.entries["1"].metadata_change is not None)
-        assert(isinstance(cset.entries["1"].metadata_change,
-                          PermissionsMerge))
-        assert(isinstance(cset.entries["2"].metadata_change,
-                          PermissionsMerge))
-        assert(cset.entries["3"].is_boring())
-        builder.apply_changeset(cset)
-        assert(os.stat(builder.this.full_path("1")).st_mode &0777 == 0655)
-        assert(os.stat(builder.this.full_path("2")).st_mode &0777 == 0755)
-        assert(os.stat(builder.this.full_path("3")).st_mode &0777 == 0655)
-        builder.cleanup();
-        builder = MergeBuilder()
-        builder.add_file("1", "0", "name1", "text1", 0755)
-        builder.change_perms("1", other=0655, base=0555)
-        cset = builder.merge_changeset(ApplyMerge3)
-        self.assertRaises(changeset.MergePermissionConflict, 
-                     builder.apply_changeset, cset)
-        builder.cleanup()
-
-def test():        
-    changeset_suite = unittest.makeSuite(MergeTest, 'test_')
-    runner = unittest.TextTestRunner()
-    runner.run(changeset_suite)
-        
-if __name__ == "__main__":
-    test()

=== added file 'bzrlib/missing.py'
--- bzrlib/missing.py	1970-01-01 00:00:00 +0000
+++ bzrlib/missing.py	2005-08-30 05:30:43 +0000
@@ -0,0 +1,86 @@
+#!/usr/bin/env python
+"""\
+A plugin for displaying what revisions are in 'other' but not in local.
+"""
+
+import bzrlib
+
+try:
+    set
+except NameError:
+    from sets import Set as set
+
+
+def show_missing(br_local, br_remote, verbose=False, quiet=False):
+    """Show the revisions which exist in br_remote, that 
+    do not exist in br_local.
+    """
+    from bzrlib.log import show_one_log
+    import sys
+    local_history = br_local.revision_history()
+    remote_history = br_remote.revision_history()
+    if local_history == remote_history:
+        if not quiet:
+            print 'Trees are identical.'
+        return 0
+    if local_history[:len(remote_history)] == remote_history:
+        # Local is not missing anything from remote, so consider it
+        # up-to-date
+        if not quiet:
+            print 'Local tree has all of remote revisions (remote is missing local)'
+        return 0
+    if quiet:
+        return 1
+
+    # Check for divergence
+    common_idx = min(len(local_history), len(remote_history)) - 1
+    if common_idx >= 0 and local_history[common_idx] != remote_history[common_idx]:
+        print 'Trees have diverged'
+
+    local_rev_set = set(local_history)
+
+    # Find the last common revision between the two trees
+    revno = 0
+    for revno, (local_rev, remote_rev) in enumerate(zip(local_history, remote_history)):
+        if local_rev != remote_rev:
+            break
+
+    missing_remote = []
+    for rno, rev_id in enumerate(remote_history[revno:]):
+        # This assumes that you can have a revision in the
+        # local history, which does not have the same ancestry
+        # as the remote ancestry.
+        # This may or may not be possible.
+        # In the future this should also checked for merged revisions.
+        if rev_id not in local_rev_set:
+            missing_remote.append((rno+revno+1, rev_id))
+
+    print 'Missing %d revisions' %  len(missing_remote)
+    print
+
+    if verbose:
+        from bzrlib.diff import compare_trees
+        from bzrlib.tree import EmptyTree
+        show_ids = True
+        last_tree = EmptyTree
+        last_rev_id = None
+    else:
+        show_ids = False
+    for revno, rev_id in missing_remote:
+        rev = br_remote.get_revision(rev_id)
+        if verbose:
+            parent_rev_id = rev.parents[0].revision_id
+            if last_rev_id == parent_rev_id:
+                parent_tree = last_tree
+            else:
+                parent_tree = br_remote.revision_tree(parent_rev_id)
+            revision_tree = br_remote.revision_tree(rev_id)
+            last_rev_id = rev_id
+            last_tree = revision_tree
+            delta = compare_trees(revision_tree, parent_tree)
+        else:
+            delta = None
+
+        show_one_log(revno, rev, delta, verbose, sys.stdout, 'original')
+    return 1
+

=== added file 'bzrlib/msgeditor.py'
--- bzrlib/msgeditor.py	1970-01-01 00:00:00 +0000
+++ bzrlib/msgeditor.py	2005-09-07 01:34:07 +0000
@@ -0,0 +1,122 @@
+# Bazaar-NG -- distributed version control
+
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Commit message editor support."""
+
+import os
+from subprocess import call
+from bzrlib.errors import BzrError
+
+def _get_editor():
+    """Return a sequence of possible editor binaries for the current platform"""
+    from bzrlib.osutils import _read_config_value
+    
+    e = _read_config_value("editor")
+    if e is not None:
+        yield e
+        
+    try:
+        yield os.environ["EDITOR"]
+    except KeyError:
+        if os.name == "nt":
+            yield "notepad.exe"
+        elif os.name == "posix":
+            yield "/usr/bin/vi"
+
+
+def _run_editor(filename):
+    """Try to execute an editor to edit the commit message."""
+    for e in _get_editor():
+        edargs = e.split(' ')
+        x = call(edargs + [filename])
+        if x == 0:
+            return True
+        elif x == 127:
+            continue
+        else:
+            break
+    raise BzrError("Could not start any editor. "
+                   "Please specify $EDITOR or use ~/.bzr.conf/editor")
+                          
+
+def edit_commit_message(infotext, ignoreline=None):
+    """Let the user edit a commit message in a temp file.
+
+    This is run if they don't give a message or
+    message-containing file on the command line.
+
+    infotext:
+        Text to be displayed at bottom of message for
+        the user's reference; currently similar to
+        'bzr status'.
+    """
+    import tempfile
+    
+    if ignoreline is None:
+        ignoreline = "-- This line and the following will be ignored --"
+        
+    try:
+        tmp_fileno, msgfilename = tempfile.mkstemp()
+        msgfile = os.close(tmp_fileno)
+        if infotext is not None and infotext != "":
+            hasinfo = True
+            msgfile = file(msgfilename, "w")
+            msgfile.write("\n\n%s\n\n%s" % (ignoreline, infotext))
+            msgfile.close()
+        else:
+            hasinfo = False
+
+        if not _run_editor(msgfilename):
+            return None
+        
+        started = False
+        msg = []
+        lastline, nlines = 0, 0
+        for line in file(msgfilename, "r"):
+            stripped_line = line.strip()
+            # strip empty line before the log message starts
+            if not started:
+                if stripped_line != "":
+                    started = True
+                else:
+                    continue
+            # check for the ignore line only if there
+            # is additional information at the end
+            if hasinfo and stripped_line == ignoreline:
+                break
+            nlines += 1
+            # keep track of the last line that had some content
+            if stripped_line != "":
+                lastline = nlines
+            msg.append(line)
+            
+        if len(msg) == 0:
+            return None
+        # delete empty lines at the end
+        del msg[lastline:]
+        # add a newline at the end, if needed
+        if not msg[-1].endswith("\n"):
+            return "%s%s" % ("".join(msg), "\n")
+        else:
+            return "".join(msg)
+    finally:
+        # delete the msg file in any case
+        try: os.unlink(msgfilename)
+        except IOError: pass
+

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2005-07-23 13:59:30 +0000
+++ bzrlib/osutils.py	2005-09-01 11:19:08 +0000
@@ -37,9 +37,8 @@
     os.chmod(filename, mod)
 
 
-_QUOTE_RE = re.compile(r'([^a-zA-Z0-9.,:/_~-])')
+_QUOTE_RE = None
 
-_SLASH_RE = re.compile(r'[\\/]+')
 
 def quotefn(f):
     """Return a quoted filename filename
@@ -47,6 +46,10 @@
     This previously used backslash quoting, but that works poorly on
     Windows."""
     # TODO: I'm not really sure this is the best format either.x
+    global _QUOTE_RE
+    if _QUOTE_RE == None:
+        _QUOTE_RE = re.compile(r'([^a-zA-Z0-9.,:/_~-])')
+        
     if _QUOTE_RE.search(f):
         return '"' + f + '"'
     else:
@@ -139,6 +142,9 @@
     that . and .. and repeated slashes are eliminated, and the separators
     are canonical for the platform.
     
+    The empty string as a dir name is taken as top-of-tree and matches 
+    everything.
+    
     >>> is_inside('src', 'src/foo.c')
     True
     >>> is_inside('src', 'srccontrol')
@@ -147,13 +153,19 @@
     True
     >>> is_inside('foo.c', 'foo.c')
     True
+    >>> is_inside('foo.c', '')
+    False
+    >>> is_inside('', 'foo.c')
+    True
     """
     # XXX: Most callers of this can actually do something smarter by 
     # looking at the inventory
-
     if dir == fname:
         return True
     
+    if dir == '':
+        return True
+    
     if dir[-1] != os.sep:
         dir += os.sep
     
@@ -260,14 +272,36 @@
     return realname, (username + '@' + socket.gethostname())
 
 
-def _get_user_id():
+def _get_user_id(branch):
     """Return the full user id from a file or environment variable.
 
-    TODO: Allow taking this from a file in the branch directory too
-    for per-branch ids."""
+    e.g. "John Hacker <jhacker@foo.org>"
+
+    branch
+        A branch to use for a per-branch configuration, or None.
+
+    The following are searched in order:
+
+    1. $BZREMAIL
+    2. .bzr/email for this branch.
+    3. ~/.bzr.conf/email
+    4. $EMAIL
+    """
     v = os.environ.get('BZREMAIL')
     if v:
         return v.decode(bzrlib.user_encoding)
+
+    if branch:
+        try:
+            return (branch.controlfile("email", "r") 
+                    .read()
+                    .decode(bzrlib.user_encoding)
+                    .rstrip("\r\n"))
+        except IOError, e:
+            if e.errno != errno.ENOENT:
+                raise
+        except BzrError, e:
+            pass
     
     try:
         return (open(os.path.join(config_dir(), "email"))
@@ -285,14 +319,14 @@
         return None
 
 
-def username():
+def username(branch):
     """Return email-style username.
 
     Something similar to 'Martin Pool <mbp@sourcefrog.net>'
 
     TODO: Check it's reasonably well-formed.
     """
-    v = _get_user_id()
+    v = _get_user_id(branch)
     if v:
         return v
     
@@ -303,12 +337,11 @@
         return email
 
 
-_EMAIL_RE = re.compile(r'[\w+.-]+@[\w+.-]+')
-def user_email():
+def user_email(branch):
     """Return just the email component of a username."""
-    e = _get_user_id()
+    e = _get_user_id(branch)
     if e:
-        m = _EMAIL_RE.search(e)
+        m = re.search(r'[\w+.-]+@[\w+.-]+', e)
         if not m:
             raise BzrError("%r doesn't seem to contain a reasonable email address" % e)
         return m.group(0)
@@ -358,8 +391,8 @@
         tt = time.localtime(t)
         offset = local_time_offset(t)
     else:
-        raise BzrError("unsupported timezone format %r",
-                ['options are "utc", "original", "local"'])
+        raise BzrError("unsupported timezone format %r" % timezone,
+                       ['options are "utc", "original", "local"'])
 
     return (time.strftime("%a %Y-%m-%d %H:%M:%S", tt)
             + ' %+03d%02d' % (offset / 3600, (offset / 60) % 60))
@@ -459,87 +492,3 @@
         raise
 
 
-def _get_editor():
-    """Return a sequence of possible editor binaries for the current platform"""
-    e = _read_config_value("editor")
-    if e is not None:
-        yield e
-        
-    if os.name == "windows":
-        yield "notepad.exe"
-    elif os.name == "posix":
-        try:
-            yield os.environ["EDITOR"]
-        except KeyError:
-            yield "/usr/bin/vi"
-
-
-def _run_editor(filename):
-    """Try to execute an editor to edit the commit message. Returns True on success,
-    False on failure"""
-    for e in _get_editor():
-        x = os.spawnvp(os.P_WAIT, e, (e, filename))
-        if x == 0:
-            return True
-        elif x == 127:
-            continue
-        else:
-            break
-    raise BzrError("Could not start any editor. Please specify $EDITOR or use ~/.bzr.conf/editor")
-    return False
-                          
-
-def get_text_message(infotext, ignoreline = "default"):
-    import tempfile
-    
-    if ignoreline == "default":
-        ignoreline = "-- This line and the following will be ignored --"
-        
-    try:
-        tmp_fileno, msgfilename = tempfile.mkstemp()
-        msgfile = os.close(tmp_fileno)
-        if infotext is not None and infotext != "":
-            hasinfo = True
-            msgfile = file(msgfilename, "w")
-            msgfile.write("\n\n%s\n\n%s" % (ignoreline, infotext))
-            msgfile.close()
-        else:
-            hasinfo = False
-
-        if not _run_editor(msgfilename):
-            return None
-        
-        started = False
-        msg = []
-        lastline, nlines = 0, 0
-        for line in file(msgfilename, "r"):
-            stripped_line = line.strip()
-            # strip empty line before the log message starts
-            if not started:
-                if stripped_line != "":
-                    started = True
-                else:
-                    continue
-            # check for the ignore line only if there
-            # is additional information at the end
-            if hasinfo and stripped_line == ignoreline:
-                break
-            nlines += 1
-            # keep track of the last line that had some content
-            if stripped_line != "":
-                lastline = nlines
-            msg.append(line)
-            
-        if len(msg) == 0:
-            return None
-        # delete empty lines at the end
-        del msg[lastline:]
-        # add a newline at the end, if needed
-        if not msg[-1].endswith("\n"):
-            return "%s%s" % ("".join(msg), "\n")
-        else:
-            return "".join(msg)
-    finally:
-        # delete the msg file in any case
-        try: os.unlink(msgfilename)
-        except IOError: pass

=== modified file 'bzrlib/patch.py'
--- bzrlib/patch.py	2005-07-26 15:25:58 +0000
+++ bzrlib/patch.py	2005-08-25 05:58:05 +0000
@@ -40,28 +40,6 @@
     return status 
 
 
-def diff(orig_file, mod_str, orig_label=None, mod_label=None):
-    """Compare two files, and produce a patch.
-
-    :param orig_file: path to the old file
-    :type orig_file: str
-    :param mod_str: Contents of the new file
-    :type mod_str: str
-    :param orig_label: The label to use for the old file
-    :type orig_label: str
-    :param mod_label: The label to use for the new file
-    :type mod_label: str
-    """
-    args = ["diff", "-u" ]
-    if orig_label is not None and mod_label is not None:
-        args.extend(("-L", orig_label, "-L", mod_label))
-    args.extend(("--", orig_file, "-"))
-    patch, stderr, status = write_to_cmd(args, mod_str)
-    if status == 0:
-        return None
-    else:
-        return patch
-
 def diff3(out_file, mine_path, older_path, yours_path):
     def add_label(args, label):
         args.extend(("-L", label))

=== modified file 'bzrlib/plugin.py'
--- bzrlib/plugin.py	2005-07-22 22:31:40 +0000
+++ bzrlib/plugin.py	2005-08-29 05:13:54 +0000
@@ -59,8 +59,10 @@
     
     from bzrlib.trace import log_error, mutter, log_exception
     from bzrlib.errors import BzrError
+    from bzrlib import plugins
 
-    bzrpath = os.environ.get('BZR_PLUGIN_PATH', DEFAULT_PLUGIN_PATH)
+    dirs = os.environ.get('BZR_PLUGIN_PATH', DEFAULT_PLUGIN_PATH).split(":")
+    dirs.insert(0, os.path.dirname(plugins.__file__))
 
     # The problem with imp.get_suffixes() is that it doesn't include
     # .pyo which is technically valid
@@ -70,7 +72,7 @@
     suffixes = imp.get_suffixes()
     suffixes.append(('.pyo', 'rb', imp.PY_COMPILED))
     package_entries = ['__init__.py', '__init__.pyc', '__init__.pyo']
-    for d in bzrpath.split(os.pathsep):
+    for d in dirs:
         # going through them one by one allows different plugins with the same
         # filename in different directories in the path
         mutter('looking for plugins in %s' % d)
@@ -114,8 +116,9 @@
                 finally:
                     if plugin_info[0] is not None:
                         plugin_info[0].close()
-            except Exception, e:
+
+                mutter('loaded succesfully')
+            except:
                 log_error('Unable to load plugin %r from %r' % (name, d))
-                log_error(str(e))
                 log_exception()
 

=== renamed directory 'plugins' => 'bzrlib/plugins'
=== added file 'bzrlib/plugins/__init__.py'
--- bzrlib/plugins/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/plugins/__init__.py	2005-08-30 03:29:32 +0000
@@ -0,0 +1,1 @@
+"""Null placeholder plugin"""

=== modified file 'bzrlib/progress.py'
--- bzrlib/progress.py	2005-07-22 23:06:25 +0000
+++ bzrlib/progress.py	2005-08-25 12:46:42 +0000
@@ -94,7 +94,22 @@
         self.show_eta = show_eta
         self.show_bar = show_bar
         self.show_count = show_count
-        
+
+
+
+class DummyProgress(_BaseProgressBar):
+    """Progress-bar standin that does nothing.
+
+    This can be used as the default argument for methods that
+    take an optional progress indicator."""
+    def tick(self):
+        pass
+
+    def update(self, msg=None, current=None, total=None):
+        pass
+
+    def clear(self):
+        pass
         
     
 class DotsProgressBar(_BaseProgressBar):

=== modified file 'bzrlib/remotebranch.py'
--- bzrlib/remotebranch.py	2005-06-28 11:14:41 +0000
+++ bzrlib/remotebranch.py	2005-09-07 08:06:16 +0000
@@ -26,30 +26,39 @@
 
 import gzip
 from cStringIO import StringIO
+import os
 import urllib2
-
-from errors import BzrError, BzrCheckError
-from branch import Branch, BZR_BRANCH_FORMAT
-from trace import mutter
-
-# velocitynet.com.au transparently proxies connections and thereby
-# breaks keep-alive -- sucks!
-
-
-ENABLE_URLGRABBER = True
-
+import urlparse
+
+from bzrlib.errors import BzrError, BzrCheckError
+from bzrlib.branch import Branch, BZR_BRANCH_FORMAT
+from bzrlib.trace import mutter
+from bzrlib.xml import serializer_v4
+
+
+ENABLE_URLGRABBER = False
+
+from bzrlib.errors import BzrError, NoSuchRevision
+
+class GetFailed(BzrError):
+    def __init__(self, url, status):
+        BzrError.__init__(self, "Get %s failed with status %s" % (url, status))
+        self.url = url
+        self.status = status
 
 if ENABLE_URLGRABBER:
-    import urlgrabber
-    import urlgrabber.keepalive
-    urlgrabber.keepalive.DEBUG = 0
+    import util.urlgrabber
+    import util.urlgrabber.keepalive
+    util.urlgrabber.keepalive.DEBUG = 0
     def get_url(path, compressed=False):
         try:
             url = path
             if compressed:
                 url += '.gz'
             mutter("grab url %s" % url)
-            url_f = urlgrabber.urlopen(url, keepalive=1, close_connection=0)
+            url_f = util.urlgrabber.urlopen(url, keepalive=1, close_connection=0)
+            if url_f.status != 200:
+                raise GetFailed(url, url_f.status)
             if not compressed:
                 return url_f
             else:
@@ -75,8 +84,8 @@
     orig_url = url
     while True:
         try:
-            ff = get_url(url + '/.bzr/branch-format')
-
+            fmt_url = url + '/.bzr/branch-format'
+            ff = get_url(fmt_url)
             fmt = ff.read()
             ff.close()
 
@@ -89,12 +98,17 @@
         except urllib2.URLError:
             pass
 
-        try:
-            idx = url.rindex('/')
-        except ValueError:
-            raise BzrError('no branch root found for URL %s' % orig_url)
-
-        url = url[:idx]        
+        scheme, host, path = list(urlparse.urlparse(url))[:3]
+        # discard params, query, fragment
+        
+        # strip off one component of the path component
+        idx = path.rfind('/')
+        if idx == -1 or path == '/':
+            raise BzrError('no branch root found for URL %s'
+                           ' or enclosing directories'
+                           % orig_url)
+        path = path[:idx]
+        url = urlparse.urlunparse((scheme, host, path, '', '', ''))
         
 
 
@@ -145,10 +159,11 @@
 
 
     def get_revision(self, revision_id):
-        from bzrlib.revision import Revision
-        from bzrlib.xml import unpack_xml
-        revf = self.revision_store[revision_id]
-        r = unpack_xml(Revision, revf)
+        try:
+            revf = self.revision_store[revision_id]
+        except KeyError:
+            raise NoSuchRevision(self, revision_id)
+        r = serializer_v4.read_revision(revf)
         if r.revision_id != revision_id:
             raise BzrCheckError('revision stored as {%s} actually contains {%s}'
                                 % (revision_id, r.revision_id))
@@ -167,7 +182,10 @@
         
     def __getitem__(self, fileid):
         p = self._path(fileid)
-        return get_url(p, compressed=True)
+        try:
+            return get_url(p, compressed=True)
+        except:
+            raise KeyError(fileid)
     
 
     

=== modified file 'bzrlib/revfile.py'
--- bzrlib/revfile.py	2005-05-26 02:13:57 +0000
+++ bzrlib/revfile.py	2005-08-25 05:58:05 +0000
@@ -52,6 +52,22 @@
 is that sequence numbers are stable references.  But not every
 repository in the world will assign the same sequence numbers,
 therefore the SHA-1 is the only universally unique reference.
+
+This is meant to scale to hold 100,000 revisions of a single file, by
+which time the index file will be ~4.8MB and a bit big to read
+sequentially.
+
+Some of the reserved fields could be used to implement a (semi?)
+balanced tree indexed by SHA1 so we can much more efficiently find the
+index associated with a particular hash.  For 100,000 revs we would be
+able to find it in about 17 random reads, which is not too bad.
+
+This performs pretty well except when trying to calculate deltas of
+really large files.  For that the main thing would be to plug in
+something faster than difflib, which is after all pure Python.
+Another approach is to just store the gzipped full text of big files,
+though perhaps that's too perverse?
+
 The iter method here will generally read through the whole index file
 in one go.  With readahead in the kernel and python/libc (typically
 128kB) this means that there should be no seeks and often only one
@@ -73,13 +89,13 @@
 # if there are gaps and that can happen if we're interrupted while
 # writing to the datafile.  Overlapping would be very bad though.
 
-
+# TODO: Shouldn't need to lock if we always write in append mode and
+# then ftell after writing to see where it went.  In any case we
+# assume the whole branch is protected by a lock.
 
 import sys, zlib, struct, mdiff, stat, os, sha
 from binascii import hexlify, unhexlify
 
-factor = 10
-
 _RECORDSIZE = 48
 
 _HEADER = "bzr revfile v1\n"
@@ -96,7 +112,7 @@
 FL_GZIP = 1
 
 # maximum number of patches in a row before recording a whole text.
-CHAIN_LIMIT = 50
+CHAIN_LIMIT = 10
 
 
 class RevfileError(Exception):
@@ -133,7 +149,6 @@
             self.idxfile = open(idxname, 'w+b')
             self.datafile = open(dataname, 'w+b')
             
-            print 'init empty file'
             self.idxfile.write(_HEADER)
             self.idxfile.flush()
         else:
@@ -227,16 +242,38 @@
         return self._add_compressed(text_sha, text, _NO_RECORD, compress)
 
 
+    # NOT USED
+    def _choose_base(self, seed, base):
+        while seed & 3 == 3:
+            if base == _NO_RECORD:
+                return _NO_RECORD
+            idxrec = self[base]
+            if idxrec[I_BASE] == _NO_RECORD:
+                return base
+
+            base = idxrec[I_BASE]
+            seed >>= 2
+                
+        return base        # relative to this full text
+        
+
+
     def _add_delta(self, text, text_sha, base, compress):
         """Add a text stored relative to a previous text."""
         self._check_index(base)
-        
+
         try:
-            base_text = self.get(base, recursion_limit=CHAIN_LIMIT)
+            base_text = self.get(base, CHAIN_LIMIT)
         except LimitHitException:
             return self._add_full_text(text, text_sha, compress)
         
         data = mdiff.bdiff(base_text, text)
+
+
+        if True: # paranoid early check for bad diff
+            result = mdiff.bpatch(base_text, data)
+            assert result == text
+            
         
         # If the delta is larger than the text, we might as well just
         # store the text.  (OK, the delta might be more compressible,
@@ -248,7 +285,7 @@
             return self._add_compressed(text_sha, data, base, compress)
 
 
-    def add(self, text, base=_NO_RECORD, compress=True):
+    def add(self, text, base=None, compress=True):
         """Add a new text to the revfile.
 
         If the text is already present them its existing id is
@@ -262,6 +299,9 @@
         only be used if it would be a size win and if the existing
         base is not at too long of a delta chain already.
         """
+        if base == None:
+            base = _NO_RECORD
+        
         self._check_write()
         
         text_sha = sha.new(text).digest()
@@ -272,6 +312,8 @@
             # it's the same, in case someone ever breaks SHA-1.
             return idx                  # already present
         
+        # base = self._choose_base(ord(text_sha[0]), base)
+
         if base == _NO_RECORD:
             return self._add_full_text(text, text_sha, compress)
         else:
@@ -296,8 +338,8 @@
             text = self._get_patched(idx, idxrec, recursion_limit)
 
         if sha.new(text).digest() != idxrec[I_SHA]:
-            raise RevfileError("corrupt SHA-1 digest on record %d"
-                               % idx)
+            raise RevfileError("corrupt SHA-1 digest on record %d in %s"
+                               % (idx, self.basename))
 
         return text
 
@@ -372,7 +414,7 @@
         self._seek_index(idx)
         idxrec = self._read_next_index()
         if idxrec == None:
-            raise IndexError()
+            raise IndexError("no index %d" % idx)
         else:
             return idxrec
 
@@ -388,7 +430,7 @@
         """Read back all index records.
 
         Do not seek the index file while this is underway!"""
-        sys.stderr.write(" ** iter called ** \n")
+        ## sys.stderr.write(" ** iter called ** \n")
         self._seek_index(0)
         while True:
             idxrec = self._read_next_index()
@@ -436,51 +478,76 @@
         for idx in range(len(self)):
             t += len(self.get(idx))
         return t
+
+
+    def check(self, pb=None):
+        """Extract every version and check its hash."""
+        total = len(self)
+        for i in range(total):
+            if pb:
+                pb.update("check revision", i, total)
+            # the get method implicitly checks the SHA-1
+            self.get(i)
+        if pb:
+            pb.clear()
         
 
 
 def main(argv):
     try:
         cmd = argv[1]
+        filename = argv[2]
     except IndexError:
-        sys.stderr.write("usage: revfile dump\n"
-                         "       revfile add\n"
-                         "       revfile add-delta BASE\n"
-                         "       revfile get IDX\n"
-                         "       revfile find-sha HEX\n"
-                         "       revfile total-text-size\n"
-                         "       revfile last\n")
+        sys.stderr.write("usage: revfile dump REVFILE\n"
+                         "       revfile add REVFILE < INPUT\n"
+                         "       revfile add-delta REVFILE BASE < INPUT\n"
+                         "       revfile add-series REVFILE BASE FILE...\n"
+                         "       revfile get REVFILE IDX\n"
+                         "       revfile find-sha REVFILE HEX\n"
+                         "       revfile total-text-size REVFILE\n"
+                         "       revfile last REVFILE\n")
         return 1
 
+    if filename.endswith('.drev') or filename.endswith('.irev'):
+        filename = filename[:-5]
+
     def rw():
-        return Revfile('testrev', 'w')
+        return Revfile(filename, 'w')
 
     def ro():
-        return Revfile('testrev', 'r')
+        return Revfile(filename, 'r')
 
     if cmd == 'add':
         print rw().add(sys.stdin.read())
     elif cmd == 'add-delta':
-        print rw().add(sys.stdin.read(), int(argv[2]))
+        print rw().add(sys.stdin.read(), int(argv[3]))
+    elif cmd == 'add-series':
+        r = rw()
+        rev = int(argv[3])
+        for fn in argv[4:]:
+            print rev
+            rev = r.add(file(fn).read(), rev)
     elif cmd == 'dump':
         ro().dump()
     elif cmd == 'get':
         try:
-            idx = int(argv[2])
+            idx = int(argv[3])
         except IndexError:
-            sys.stderr.write("usage: revfile get IDX\n")
+            sys.stderr.write("usage: revfile get FILE IDX\n")
             return 1
 
+        r = ro()
+
         if idx < 0 or idx >= len(r):
             sys.stderr.write("invalid index %r\n" % idx)
             return 1
 
-        sys.stdout.write(ro().get(idx))
+        sys.stdout.write(r.get(idx))
     elif cmd == 'find-sha':
         try:
-            s = unhexlify(argv[2])
+            s = unhexlify(argv[3])
         except IndexError:
-            sys.stderr.write("usage: revfile find-sha HEX\n")
+            sys.stderr.write("usage: revfile find-sha FILE HEX\n")
             return 1
 
         idx = ro().find_sha(s)
@@ -493,6 +560,10 @@
         print ro().total_text_size()
     elif cmd == 'last':
         print len(ro())-1
+    elif cmd == 'check':
+        import bzrlib.progress
+        pb = bzrlib.progress.ProgressBar()
+        ro().check(pb)
     else:
         sys.stderr.write("unknown command %r\n" % cmd)
         return 1

=== modified file 'bzrlib/revision.py'
--- bzrlib/revision.py	2005-07-04 11:57:18 +0000
+++ bzrlib/revision.py	2005-09-12 08:21:24 +0000
@@ -15,7 +15,8 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-
+import bzrlib.errors
+from bzrlib.graph import farthest_nodes, node_distances, all_descendants
 
 class RevisionReference(object):
     """
@@ -69,92 +70,21 @@
     def __repr__(self):
         return "<Revision id %s>" % self.revision_id
 
-        
-    def to_element(self):
-        from bzrlib.xml import Element, SubElement
-        
-        root = Element('revision',
-                       committer = self.committer,
-                       timestamp = '%.9f' % self.timestamp,
-                       revision_id = self.revision_id,
-                       inventory_id = self.inventory_id,
-                       inventory_sha1 = self.inventory_sha1,
-                       )
-        if self.timezone:
-            root.set('timezone', str(self.timezone))
-        root.text = '\n'
-        
-        msg = SubElement(root, 'message')
-        msg.text = self.message
-        msg.tail = '\n'
-
-        if self.parents:
-            pelts = SubElement(root, 'parents')
-            pelts.tail = pelts.text = '\n'
-            for rr in self.parents:
-                assert isinstance(rr, RevisionReference)
-                p = SubElement(pelts, 'revision_ref')
-                p.tail = '\n'
-                assert rr.revision_id
-                p.set('revision_id', rr.revision_id)
-                if rr.revision_sha1:
-                    p.set('revision_sha1', rr.revision_sha1)
-
-        return root
-
-
-    def from_element(cls, elt):
-        return unpack_revision(elt)
-
-    from_element = classmethod(from_element)
-
-
-
-def unpack_revision(elt):
-    """Convert XML element into Revision object."""
-    # <changeset> is deprecated...
-    from bzrlib.errors import BzrError
-    
-    if elt.tag not in ('revision', 'changeset'):
-        raise BzrError("unexpected tag in revision file: %r" % elt)
-
-    rev = Revision(committer = elt.get('committer'),
-                   timestamp = float(elt.get('timestamp')),
-                   revision_id = elt.get('revision_id'),
-                   inventory_id = elt.get('inventory_id'),
-                   inventory_sha1 = elt.get('inventory_sha1')
-                   )
-
-    precursor = elt.get('precursor')
-    precursor_sha1 = elt.get('precursor_sha1')
-
-    pelts = elt.find('parents')
-
-    if pelts:
-        for p in pelts:
-            assert p.tag == 'revision_ref', \
-                   "bad parent node tag %r" % p.tag
-            rev_ref = RevisionReference(p.get('revision_id'),
-                                        p.get('revision_sha1'))
-            rev.parents.append(rev_ref)
-
-        if precursor:
-            # must be consistent
-            prec_parent = rev.parents[0].revision_id
-            assert prec_parent == precursor
-    elif precursor:
-        # revisions written prior to 0.0.5 have a single precursor
-        # give as an attribute
-        rev_ref = RevisionReference(precursor, precursor_sha1)
-        rev.parents.append(rev_ref)
-
-    v = elt.get('timezone')
-    rev.timezone = v and int(v)
-
-    rev.message = elt.findtext('message') # text of <message>
-    return rev
-
-
+    def __eq__(self, other):
+        if not isinstance(other, Revision):
+            return False
+        return (self.inventory_id == other.inventory_id
+                and self.inventory_sha1 == other.inventory_sha1
+                and self.revision_id == other.revision_id
+                and self.timestamp == other.timestamp
+                and self.message == other.message
+                and self.timezone == other.timezone
+                and self.committer == other.committer)
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
+        
 
 REVISION_ID_RE = None
 
@@ -167,4 +97,220 @@
 
     if not REVISION_ID_RE.match(rid):
         raise ValueError("malformed revision-id %r" % rid)
-    
+
+def is_ancestor(revision_id, candidate_id, revision_source):
+    """Return true if candidate_id is an ancestor of revision_id.
+    A false negative will be returned if any intermediate descendent of
+    candidate_id is not present in any of the revision_sources.
+    
+    revisions_source is an object supporting a get_revision operation that
+    behaves like Branch's.
+    """
+
+    for ancestor_id, distance in iter_ancestors(revision_id, revision_source):
+        if ancestor_id == candidate_id:
+            return True
+    return False
+
+def iter_ancestors(revision_id, revision_source, only_present=False):
+    ancestors = (revision_id,)
+    distance = 0
+    while len(ancestors) > 0:
+        new_ancestors = []
+        for ancestor in ancestors:
+            if not only_present:
+                yield ancestor, distance
+            try:
+                revision = revision_source.get_revision(ancestor)
+            except bzrlib.errors.NoSuchRevision, e:
+                if e.revision == revision_id:
+                    raise 
+                else:
+                    continue
+            if only_present:
+                yield ancestor, distance
+            new_ancestors.extend([p.revision_id for p in revision.parents])
+        ancestors = new_ancestors
+        distance += 1
+
+
+def find_present_ancestors(revision_id, revision_source):
+    """Return the ancestors of a revision present in a branch.
+
+    It's possible that a branch won't have the complete ancestry of
+    one of its revisions.  
+
+    """
+    found_ancestors = {}
+    anc_iter = enumerate(iter_ancestors(revision_id, revision_source,
+                         only_present=True))
+    for anc_order, (anc_id, anc_distance) in anc_iter:
+        if not found_ancestors.has_key(anc_id):
+            found_ancestors[anc_id] = (anc_order, anc_distance)
+    return found_ancestors
+    
+
+def __get_closest(intersection):
+    intersection.sort()
+    matches = [] 
+    for entry in intersection:
+        if entry[0] == intersection[0][0]:
+            matches.append(entry[2])
+    return matches
+
+
+def old_common_ancestor(revision_a, revision_b, revision_source):
+    """Find the ancestor common to both revisions that is closest to both.
+    """
+    from bzrlib.trace import mutter
+    a_ancestors = find_present_ancestors(revision_a, revision_source)
+    b_ancestors = find_present_ancestors(revision_b, revision_source)
+    a_intersection = []
+    b_intersection = []
+    # a_order is used as a tie-breaker when two equally-good bases are found
+    for revision, (a_order, a_distance) in a_ancestors.iteritems():
+        if b_ancestors.has_key(revision):
+            a_intersection.append((a_distance, a_order, revision))
+            b_intersection.append((b_ancestors[revision][1], a_order, revision))
+    mutter("a intersection: %r" % a_intersection)
+    mutter("b intersection: %r" % b_intersection)
+
+    a_closest = __get_closest(a_intersection)
+    if len(a_closest) == 0:
+        return None
+    b_closest = __get_closest(b_intersection)
+    assert len(b_closest) != 0
+    mutter ("a_closest %r" % a_closest)
+    mutter ("b_closest %r" % b_closest)
+    if a_closest[0] in b_closest:
+        return a_closest[0]
+    elif b_closest[0] in a_closest:
+        return b_closest[0]
+    else:
+        raise bzrlib.errors.AmbiguousBase((a_closest[0], b_closest[0]))
+    return a_closest[0]
+
+def revision_graph(revision, revision_source):
+    """Produce a graph of the ancestry of the specified revision.
+    Return root, ancestors map, descendants map
+
+    TODO: Produce graphs with the NULL revision as root, so that we can find
+    a common even when trees are not branches don't represent a single line
+    of descent.
+    """
+    ancestors = {}
+    descendants = {}
+    lines = [revision]
+    root = None
+    descendants[revision] = {}
+    while len(lines) > 0:
+        new_lines = set()
+        for line in lines:
+            try:
+                rev = revision_source.get_revision(line)
+                parents = [p.revision_id for p in rev.parents]
+                if len(parents) == 0:
+                    root = line
+            except bzrlib.errors.NoSuchRevision:
+                if line == revision:
+                    raise
+                parents = None
+            if parents is not None:
+                for parent in parents:
+                    if parent not in ancestors:
+                        new_lines.add(parent)
+                    if parent not in descendants:
+                        descendants[parent] = {}
+                    descendants[parent][line] = 1
+            if parents is not None:
+                ancestors[line] = set(parents)
+        lines = new_lines
+    assert root not in descendants[root]
+    assert root not in ancestors[root]
+    return root, ancestors, descendants
+
+def combined_graph(revision_a, revision_b, revision_source):
+    """Produce a combined ancestry graph.
+    Return graph root, ancestors map, descendants map, set of common nodes"""
+    root, ancestors, descendants = revision_graph(revision_a, revision_source)
+    root_b, ancestors_b, descendants_b = revision_graph(revision_b, 
+                                                        revision_source)
+    assert root == root_b
+    common = set()
+    for node, node_anc in ancestors_b.iteritems():
+        if node in ancestors:
+            common.add(node)
+        else:
+            ancestors[node] = set()
+        ancestors[node].update(node_anc)
+    for node, node_dec in descendants_b.iteritems():
+        if node not in descendants:
+            descendants[node] = set()
+        descendants[node].update(node_dec)
+    return root, ancestors, descendants, common
+
+def common_ancestor(revision_a, revision_b, revision_source):
+    root, ancestors, descendants, common = \
+        combined_graph(revision_a, revision_b, revision_source)
+    nodes = farthest_nodes(descendants, ancestors, root)
+    for node in nodes:
+        if node in common:
+            return node
+
+class MultipleRevisionSources(object):
+    """Proxy that looks in multiple branches for revisions."""
+    def __init__(self, *args):
+        object.__init__(self)
+        assert len(args) != 0
+        self._revision_sources = args
+
+    def get_revision(self, revision_id):
+        for source in self._revision_sources:
+            try:
+                return source.get_revision(revision_id)
+            except bzrlib.errors.NoSuchRevision, e:
+                pass
+        raise e
+
+def get_intervening_revisions(ancestor_id, rev_id, rev_source, 
+                              revision_history=None):
+    """Find the longest line of descent from maybe_ancestor to revision.
+    Revision history is followed where possible.
+
+    If ancestor_id == rev_id, list will be empty.
+    Otherwise, rev_id will be the last entry.  ancestor_id will never appear.
+    If ancestor_id is not an ancestor, NotAncestor will be thrown
+    """
+    root, ancestors, descendants = revision_graph(rev_id, rev_source)
+    if len(descendants) == 0:
+        raise NoSuchRevision(rev_source, rev_id)
+    if ancestor_id not in descendants:
+        rev_source.get_revision(ancestor_id)
+        raise bzrlib.errors.NotAncestor(rev_id, ancestor_id)
+    root_descendants = all_descendants(descendants, ancestor_id)
+    root_descendants.add(ancestor_id)
+    if rev_id not in root_descendants:
+        raise bzrlib.errors.NotAncestor(rev_id, ancestor_id)
+    distances = node_distances(descendants, ancestors, ancestor_id,
+                               root_descendants=root_descendants)
+
+    def best_ancestor(rev_id):
+        best = None
+        for anc_id in ancestors[rev_id]:
+            try:
+                distance = distances[anc_id]
+            except KeyError:
+                continue
+            if revision_history is not None and anc_id in revision_history:
+                return anc_id
+            elif best is None or distance > best[1]:
+                best = (anc_id, distance)
+        return best[0]
+
+    next = rev_id
+    path = []
+    while next != ancestor_id:
+        path.append(next)
+        next = best_ancestor(next)
+    path.reverse()
+    return path

=== added file 'bzrlib/selftest/TestUtil.py'
--- bzrlib/selftest/TestUtil.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/TestUtil.py	2005-08-24 08:34:10 +0000
@@ -0,0 +1,80 @@
+# Copyright (c) 2004 Canonical Limited
+#       Author: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#
+
+import sys
+import logging
+import unittest
+
+
+class LogCollector(logging.Handler):
+    def __init__(self):
+        logging.Handler.__init__(self)
+        self.records=[]
+    def emit(self, record):
+        self.records.append(record.getMessage())
+
+
+def makeCollectingLogger():
+    """I make a logger instance that collects its logs for programmatic analysis
+    -> (logger, collector)"""
+    logger=logging.Logger("collector")
+    handler=LogCollector()
+    handler.setFormatter(logging.Formatter("%(levelname)s: %(message)s"))
+    logger.addHandler(handler)
+    return logger, handler
+
+
+def visitTests(suite, visitor):
+    """A foreign method for visiting the tests in a test suite."""
+    for test in suite._tests:
+        #Abusing types to avoid monkey patching unittest.TestCase. 
+        # Maybe that would be better?
+        try:
+            test.visit(visitor)
+        except AttributeError:
+            if isinstance(test, unittest.TestCase):
+                visitor.visitCase(test)
+            elif isinstance(test, unittest.TestSuite):
+                visitor.visitSuite(test)
+                visitTests(test, visitor)
+            else:
+                print "unvisitable non-unittest.TestCase element %r (%r)" % (test, test.__class__)
+    
+
+class TestSuite(unittest.TestSuite):
+    """I am an extended TestSuite with a visitor interface.
+    This is primarily to allow filtering of tests - and suites or
+    more in the future. An iterator of just tests wouldn't scale..."""
+
+    def visit(self, visitor):
+        """visit the composite. Visiting is depth-first.
+        current callbacks are visitSuite and visitCase."""
+        visitor.visitSuite(self)
+        visitTests(self, visitor)
+
+
+class TestLoader(unittest.TestLoader):
+    """Custome TestLoader to set the right TestSuite class."""
+    suiteClass = TestSuite
+
+class TestVisitor(object):
+    """A visitor for Tests"""
+    def visitSuite(self, aTestSuite):
+        pass
+    def visitCase(self, aTestCase):
+        pass

=== modified file 'bzrlib/selftest/__init__.py'
--- bzrlib/selftest/__init__.py	2005-07-27 16:42:41 +0000
+++ bzrlib/selftest/__init__.py	2005-09-12 08:21:24 +0000
@@ -15,77 +15,343 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-from testsweet import TestBase, run_suite, InTempDir
+import logging
+import unittest
+import tempfile
+import os
+import sys
+import subprocess
+
+from testsweet import run_suite
+import bzrlib.commands
+
+import bzrlib.trace
+import bzrlib.fetch
+
 
 MODULES_TO_TEST = []
 MODULES_TO_DOCTEST = []
 
-def selftest(verbose=False):
-    from unittest import TestLoader, TestSuite
+from logging import debug, warning, error
+
+class CommandFailed(Exception):
+    pass
+
+class TestCase(unittest.TestCase):
+    """Base class for bzr unit tests.
+    
+    Tests that need access to disk resources should subclass 
+    TestCaseInTempDir not TestCase.
+
+    Error and debug log messages are redirected from their usual
+    location into a temporary file, the contents of which can be
+    retrieved by _get_log().
+       
+    There are also convenience functions to invoke bzr's command-line
+    routine, and to build and check bzr trees."""
+
+    BZRPATH = 'bzr'
+
+    def setUp(self):
+        # this replaces the default testsweet.TestCase; we don't want logging changed
+        unittest.TestCase.setUp(self)
+        bzrlib.trace.disable_default_logging()
+        self._enable_file_logging()
+
+
+    def _enable_file_logging(self):
+        fileno, name = tempfile.mkstemp(suffix='.log', prefix='testbzr')
+
+        self._log_file = os.fdopen(fileno, 'w+')
+
+        hdlr = logging.StreamHandler(self._log_file)
+        hdlr.setLevel(logging.DEBUG)
+        hdlr.setFormatter(logging.Formatter('%(levelname)4.4s  %(message)s'))
+        logging.getLogger('').addHandler(hdlr)
+        logging.getLogger('').setLevel(logging.DEBUG)
+        self._log_hdlr = hdlr
+        debug('opened log file %s', name)
+        
+        self._log_file_name = name
+
+    def run(self, result):
+        self.apply_redirected(None, None, None,
+                              unittest.TestCase.run, self, result)
+        
+    def tearDown(self):
+        logging.getLogger('').removeHandler(self._log_hdlr)
+        bzrlib.trace.enable_default_logging()
+        logging.debug('%s teardown', self.id())
+        self._log_file.close()
+        unittest.TestCase.tearDown(self)
+
+
+    def log(self, *args):
+        logging.debug(*args)
+
+    def _get_log(self):
+        """Return as a string the log for this test"""
+        return open(self._log_file_name).read()
+
+    def run_bzr(self, *args, **kwargs):
+        """Invoke bzr, as if it were run from the command line.
+
+        This should be the main method for tests that want to exercise the
+        overall behavior of the bzr application (rather than a unit test
+        or a functional test of the library.)
+
+        Much of the old code runs bzr by forking a new copy of Python, but
+        that is slower, harder to debug, and generally not necessary.
+        """
+        retcode = kwargs.get('retcode', 0)
+        result = self.apply_redirected(None, None, None,
+                                       bzrlib.commands.run_bzr, args)
+        self.assertEquals(result, retcode)
+        
+        
+    def check_inventory_shape(self, inv, shape):
+        """
+        Compare an inventory to a list of expected names.
+
+        Fail if they are not precisely equal.
+        """
+        extras = []
+        shape = list(shape)             # copy
+        for path, ie in inv.entries():
+            name = path.replace('\\', '/')
+            if ie.kind == 'dir':
+                name = name + '/'
+            if name in shape:
+                shape.remove(name)
+            else:
+                extras.append(name)
+        if shape:
+            self.fail("expected paths not found in inventory: %r" % shape)
+        if extras:
+            self.fail("unexpected paths found in inventory: %r" % extras)
+
+    def apply_redirected(self, stdin=None, stdout=None, stderr=None,
+                         a_callable=None, *args, **kwargs):
+        """Call callable with redirected std io pipes.
+
+        Returns the return code."""
+        from StringIO import StringIO
+        if not callable(a_callable):
+            raise ValueError("a_callable must be callable.")
+        if stdin is None:
+            stdin = StringIO("")
+        if stdout is None:
+            if hasattr(self, "_log_file"):
+                stdout = self._log_file
+            else:
+                stdout = StringIO()
+        if stderr is None:
+            if hasattr(self, "_log_file"):
+                stderr = self._log_file
+            else:
+                stderr = StringIO()
+        real_stdin = sys.stdin
+        real_stdout = sys.stdout
+        real_stderr = sys.stderr
+        try:
+            sys.stdout = stdout
+            sys.stderr = stderr
+            sys.stdin = stdin
+            return a_callable(*args, **kwargs)
+        finally:
+            sys.stdout = real_stdout
+            sys.stderr = real_stderr
+            sys.stdin = real_stdin
+
+
+BzrTestBase = TestCase
+
+     
+class TestCaseInTempDir(TestCase):
+    """Derived class that runs a test within a temporary directory.
+
+    This is useful for tests that need to create a branch, etc.
+
+    The directory is created in a slightly complex way: for each
+    Python invocation, a new temporary top-level directory is created.
+    All test cases create their own directory within that.  If the
+    tests complete successfully, the directory is removed.
+
+    InTempDir is an old alias for FunctionalTestCase.
+    """
+
+    TEST_ROOT = None
+    _TEST_NAME = 'test'
+    OVERRIDE_PYTHON = 'python'
+
+    def check_file_contents(self, filename, expect):
+        self.log("check contents of file %s" % filename)
+        contents = file(filename, 'r').read()
+        if contents != expect:
+            self.log("expected: %r" % expect)
+            self.log("actually: %r" % contents)
+            self.fail("contents of %s not as expected")
+
+    def _make_test_root(self):
+        import os
+        import shutil
+        import tempfile
+        
+        if TestCaseInTempDir.TEST_ROOT is not None:
+            return
+        TestCaseInTempDir.TEST_ROOT = os.path.abspath(
+                                 tempfile.mkdtemp(suffix='.tmp',
+                                                  prefix=self._TEST_NAME + '-',
+                                                  dir=os.curdir))
+    
+        # make a fake bzr directory there to prevent any tests propagating
+        # up onto the source directory's real branch
+        os.mkdir(os.path.join(TestCaseInTempDir.TEST_ROOT, '.bzr'))
+
+    def setUp(self):
+        super(TestCaseInTempDir, self).setUp()
+        import os
+        self._make_test_root()
+        self._currentdir = os.getcwdu()
+        self.test_dir = os.path.join(self.TEST_ROOT, self.id())
+        os.mkdir(self.test_dir)
+        os.chdir(self.test_dir)
+        
+    def tearDown(self):
+        import os
+        os.chdir(self._currentdir)
+        super(TestCaseInTempDir, self).tearDown()
+
+    def _formcmd(self, cmd):
+        if isinstance(cmd, basestring):
+            cmd = cmd.split()
+        if cmd[0] == 'bzr':
+            cmd[0] = self.BZRPATH
+            if self.OVERRIDE_PYTHON:
+                cmd.insert(0, self.OVERRIDE_PYTHON)
+        self.log('$ %r' % cmd)
+        return cmd
+
+    def runcmd(self, cmd, retcode=0):
+        """Run one command and check the return code.
+
+        Returns a tuple of (stdout,stderr) strings.
+
+        If a single string is based, it is split into words.
+        For commands that are not simple space-separated words, please
+        pass a list instead."""
+        cmd = self._formcmd(cmd)
+        self.log('$ ' + ' '.join(cmd))
+        actual_retcode = subprocess.call(cmd, stdout=self._log_file,
+                                         stderr=self._log_file)
+        if retcode != actual_retcode:
+            raise CommandFailed("test failed: %r returned %d, expected %d"
+                                % (cmd, actual_retcode, retcode))
+
+    def backtick(self, cmd, retcode=0):
+        """Run a command and return its output"""
+        cmd = self._formcmd(cmd)
+        child = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=self._log_file)
+        outd, errd = child.communicate()
+        self.log(outd)
+        actual_retcode = child.wait()
+
+        outd = outd.replace('\r', '')
+
+        if retcode != actual_retcode:
+            raise CommandFailed("test failed: %r returned %d, expected %d"
+                                % (cmd, actual_retcode, retcode))
+
+        return outd
+
+
+
+    def build_tree(self, shape):
+        """Build a test tree according to a pattern.
+
+        shape is a sequence of file specifications.  If the final
+        character is '/', a directory is created.
+
+        This doesn't add anything to a branch.
+        """
+        # XXX: It's OK to just create them using forward slashes on windows?
+        import os
+        for name in shape:
+            assert isinstance(name, basestring)
+            if name[-1] == '/':
+                os.mkdir(name[:-1])
+            else:
+                f = file(name, 'wt')
+                print >>f, "contents of", name
+                f.close()
+                
+
+
+class MetaTestLog(TestCase):
+    def test_logging(self):
+        """Test logs are captured when a test fails."""
+        logging.info('an info message')
+        warning('something looks dodgy...')
+        logging.debug('hello, test is running')
+        ##assert 0
+
+
+def selftest(verbose=False, pattern=".*"):
+    return run_suite(test_suite(), 'testbzr', verbose=verbose, pattern=pattern)
+
+
+def test_suite():
+    from bzrlib.selftest.TestUtil import TestLoader, TestSuite
     import bzrlib, bzrlib.store, bzrlib.inventory, bzrlib.branch
     import bzrlib.osutils, bzrlib.commands, bzrlib.merge3, bzrlib.plugin
-    global MODULES_TO_TEST, MODULES_TO_DOCTEST
-
-    import bzrlib.selftest.whitebox
-    import bzrlib.selftest.blackbox
-    import bzrlib.selftest.versioning
-    import bzrlib.selftest.testmerge3
-    import bzrlib.selftest.testhashcache
-    import bzrlib.selftest.testrevisionnamespaces
-    import bzrlib.selftest.testbranch
-    import bzrlib.selftest.teststatus
-    import bzrlib.selftest.testinv
-    import bzrlib.selftest.testdiff
-    import bzrlib.merge_core
     from doctest import DocTestSuite
     import os
     import shutil
     import time
     import sys
-    import unittest
+
+    global MODULES_TO_TEST, MODULES_TO_DOCTEST
+
+    testmod_names = \
+                  ['bzrlib.selftest.MetaTestLog',
+                   'bzrlib.selftest.test_parent',
+                   'bzrlib.selftest.testinv',
+                   'bzrlib.selftest.testfetch',
+                   'bzrlib.selftest.versioning',
+                   'bzrlib.selftest.whitebox',
+                   'bzrlib.selftest.testmerge3',
+                   'bzrlib.selftest.testhashcache',
+                   'bzrlib.selftest.teststatus',
+                   'bzrlib.selftest.testlog',
+                   'bzrlib.selftest.blackbox',
+                   'bzrlib.selftest.testrevisionnamespaces',
+                   'bzrlib.selftest.testbranch',
+                   'bzrlib.selftest.testrevision',
+                   'bzrlib.selftest.test_merge_core',
+                   'bzrlib.selftest.test_smart_add',
+                   'bzrlib.selftest.testdiff',
+                   'bzrlib.selftest.test_xml',
+                   'bzrlib.fetch',
+                   'bzrlib.selftest.teststore',
+                   'bzrlib.selftest.testgraph',
+                   ]
 
     for m in (bzrlib.store, bzrlib.inventory, bzrlib.branch,
               bzrlib.osutils, bzrlib.commands, bzrlib.merge3):
         if m not in MODULES_TO_DOCTEST:
             MODULES_TO_DOCTEST.append(m)
-    
-    # ugly ugly!
-    for m in (bzrlib.selftest.whitebox,
-              bzrlib.selftest.versioning,
-              bzrlib.selftest.testinv,
-              bzrlib.selftest.testmerge3,
-              bzrlib.selftest.testhashcache,
-              bzrlib.selftest.teststatus,
-              bzrlib.selftest.blackbox,
-              bzrlib.selftest.testhashcache,
-              bzrlib.selftest.testrevisionnamespaces,
-              bzrlib.selftest.testbranch,
-              bzrlib.selftest.testdiff,
-              ):
-        if m not in MODULES_TO_TEST:
-            MODULES_TO_TEST.append(m)
-
-
-    TestBase.BZRPATH = os.path.join(os.path.realpath(os.path.dirname(bzrlib.__path__[0])), 'bzr')
-    print '%-30s %s' % ('bzr binary', TestBase.BZRPATH)
-
+
+    TestCase.BZRPATH = os.path.join(os.path.realpath(os.path.dirname(bzrlib.__path__[0])), 'bzr')
+    print '%-30s %s' % ('bzr binary', TestCase.BZRPATH)
     print
-
     suite = TestSuite()
-
+    suite.addTest(TestLoader().loadTestsFromNames(testmod_names))
     for m in MODULES_TO_TEST:
          suite.addTest(TestLoader().loadTestsFromModule(m))
-
     for m in (MODULES_TO_DOCTEST):
         suite.addTest(DocTestSuite(m))
-
     for p in bzrlib.plugin.all_plugins:
         if hasattr(p, 'test_suite'):
             suite.addTest(p.test_suite())
-
-    suite.addTest(unittest.makeSuite(bzrlib.merge_core.MergeTest, 'test_'))
-
-    return run_suite(suite, 'testbzr', verbose=verbose)
-
-
+    return suite
 

=== modified file 'bzrlib/selftest/blackbox.py'
--- bzrlib/selftest/blackbox.py	2005-07-15 13:43:54 +0000
+++ bzrlib/selftest/blackbox.py	2005-09-12 02:55:34 +0000
@@ -26,83 +26,71 @@
 it's normally invoked.
 """
 
-# this code was previously in testbzr
-
-from unittest import TestCase
-from bzrlib.selftest import TestBase, InTempDir
-
-
-
-class ExternalBase(InTempDir):
-    def runbzr(self, args, retcode=0):
-        try:
-            import shutil
-            from subprocess import call
-        except ImportError, e:
-            _need_subprocess()
-            raise
-
+import sys
+import os
+
+from bzrlib.selftest import TestCaseInTempDir, BzrTestBase
+from bzrlib.branch import Branch
+from bzrlib.commands import run_bzr
+
+
+class ExternalBase(TestCaseInTempDir):
+    def runbzr(self, args, retcode=0,backtick=False):
         if isinstance(args, basestring):
             args = args.split()
-            
-        return self.runcmd(['python', self.BZRPATH,] + args,
-                           retcode=retcode)
-
-
-
-class TestVersion(ExternalBase):
-    def runTest(self):
-        # output is intentionally passed through to stdout so that we
-        # can see the version being tested
-        self.runbzr(['version'])
-
-
-
-class HelpCommands(ExternalBase):
-    def runTest(self):
+
+        if backtick:
+            return self.backtick(['python', self.BZRPATH,] + args,
+                           retcode=retcode)
+        else:
+            return self.runcmd(['python', self.BZRPATH,] + args,
+                           retcode=retcode)
+
+
+class TestCommands(ExternalBase):
+
+    def test_help_commands(self):
         self.runbzr('--help')
         self.runbzr('help')
         self.runbzr('help commands')
         self.runbzr('help help')
         self.runbzr('commit -h')
 
-
-class InitBranch(ExternalBase):
-    def runTest(self):
-        import os
+    def test_init_branch(self):
         self.runbzr(['init'])
 
-
-
-class UserIdentity(ExternalBase):
-    def runTest(self):
+    def test_whoami(self):
         # this should always identify something, if only "john@localhost"
         self.runbzr("whoami")
         self.runbzr("whoami --email")
-        self.assertEquals(self.backtick("bzr whoami --email").count('@'),
-                          1)
-
-
-class InvalidCommands(ExternalBase):
-    def runTest(self):
+
+        self.assertEquals(self.runbzr("whoami --email",
+                                      backtick=True).count('@'), 1)
+        
+    def test_whoami_branch(self):
+        """branch specific user identity works."""
+        self.runbzr('init')
+        f = file('.bzr/email', 'wt')
+        f.write('Branch Identity <branch@identi.ty>')
+        f.close()
+        whoami = self.runbzr("whoami",backtick=True)
+        whoami_email = self.runbzr("whoami --email",backtick=True)
+        self.assertTrue(whoami.startswith('Branch Identity <branch@identi.ty>'))
+        self.assertTrue(whoami_email.startswith('branch@identi.ty'))
+
+    def test_invalid_commands(self):
         self.runbzr("pants", retcode=1)
         self.runbzr("--pants off", retcode=1)
         self.runbzr("diff --message foo", retcode=1)
 
-
-
-class EmptyCommit(ExternalBase):
-    def runTest(self):
+    def test_empty_commit(self):
         self.runbzr("init")
         self.build_tree(['hello.txt'])
         self.runbzr("commit -m empty", retcode=1)
         self.runbzr("add hello.txt")
         self.runbzr("commit -m added")
 
-
-
-class IgnorePatterns(ExternalBase):
-    def runTest(self):
+    def test_ignore_patterns(self):
         from bzrlib.branch import Branch
         
         b = Branch('.', init=True)
@@ -133,16 +121,161 @@
         self.runbzr('ignore garh')
         self.assertEquals(list(b.unknowns()), [])
         assert file('.bzrignore', 'rb').read() == '*.blah\ngarh\n'
-        
-
+
+    def test_revert(self):
+        self.runbzr('init')
+
+        file('hello', 'wt').write('foo')
+        self.runbzr('add hello')
+        self.runbzr('commit -m setup hello')
+
+        file('goodbye', 'wt').write('baz')
+        self.runbzr('add goodbye')
+        self.runbzr('commit -m setup goodbye')
+        
+        file('hello', 'wt').write('bar')
+        file('goodbye', 'wt').write('qux')
+        self.runbzr('revert hello')
+        self.check_file_contents('hello', 'foo')
+        self.check_file_contents('goodbye', 'qux')
+        self.runbzr('revert')
+        self.check_file_contents('goodbye', 'baz')
+
+        os.mkdir('revertdir')
+        self.runbzr('add revertdir')
+        self.runbzr('commit -m f')
+        os.rmdir('revertdir')
+        self.runbzr('revert')
+
+    def skipped_test_mv_modes(self):
+        """Test two modes of operation for mv"""
+        from bzrlib.branch import Branch
+        b = Branch('.', init=True)
+        self.build_tree(['a', 'c', 'subdir/'])
+        self.run_bzr('mv', 'a', 'b')
+        self.run_bzr('mv', 'b', 'subdir')
+        self.run_bzr('mv', 'subdir/b', 'a')
+        self.run_bzr('mv', 'a', 'b', 'subdir')
+        self.run_bzr('mv', 'subdir/a', 'subdir/newa')
+
+    def test_main_version(self):
+        """Check output from version command and master option is reasonable"""
+        # output is intentionally passed through to stdout so that we
+        # can see the version being tested
+        output = self.runbzr('version', backtick=1)
+        self.log('bzr version output:')
+        self.log(output)
+        self.assert_(output.startswith('bzr (bazaar-ng) '))
+        self.assertNotEqual(output.index('Canonical'), -1)
+        # make sure --version is consistent
+        tmp_output = self.runbzr('--version', backtick=1)
+        self.log('bzr --version output:')
+        self.log(tmp_output)
+        self.assertEquals(output, tmp_output)
+
+    def example_branch(test):
+        test.runbzr('init')
+        file('hello', 'wt').write('foo')
+        test.runbzr('add hello')
+        test.runbzr('commit -m setup hello')
+        file('goodbye', 'wt').write('baz')
+        test.runbzr('add goodbye')
+        test.runbzr('commit -m setup goodbye')
+
+    def test_revert(self):
+        self.example_branch()
+        file('hello', 'wt').write('bar')
+        file('goodbye', 'wt').write('qux')
+        self.runbzr('revert hello')
+        self.check_file_contents('hello', 'foo')
+        self.check_file_contents('goodbye', 'qux')
+        self.runbzr('revert')
+        self.check_file_contents('goodbye', 'baz')
+
+    def test_merge(self):
+        from bzrlib.branch import Branch
+        
+        os.mkdir('a')
+        os.chdir('a')
+
+        self.example_branch()
+        os.chdir('..')
+        self.runbzr('branch a b')
+        os.chdir('b')
+        file('goodbye', 'wt').write('quux')
+        self.runbzr(['commit',  '-m',  "more u's are always good"])
+
+        os.chdir('../a')
+        file('hello', 'wt').write('quuux')
+        # We can't merge when there are in-tree changes
+        self.runbzr('merge ../b', retcode=1)
+        self.runbzr(['commit', '-m', "Like an epidemic of u's"])
+        self.runbzr('merge ../b')
+        self.check_file_contents('goodbye', 'quux')
+        # Merging a branch pulls its revision into the tree
+        a = Branch('.')
+        b = Branch('../b')
+        a.get_revision_xml(b.last_patch())
+
+        self.log('pending merges: %s', a.pending_merges())
+        #        assert a.pending_merges() == [b.last_patch()], "Assertion %s %s" \
+        #        % (a.pending_merges(), b.last_patch())
+
+    def test_pull(self):
+        """Pull changes from one branch to another."""
+        os.mkdir('a')
+        os.chdir('a')
+
+        self.example_branch()
+        os.chdir('..')
+        self.runbzr('branch a b')
+        os.chdir('b')
+        self.runbzr('commit -m blah --unchanged')
+        os.chdir('../a')
+        a = Branch('.')
+        b = Branch('../b')
+        assert a.revision_history() == b.revision_history()[:-1]
+        self.runbzr('pull ../b')
+        assert a.revision_history() == b.revision_history()
+        self.runbzr('commit -m blah2 --unchanged')
+        os.chdir('../b')
+        self.runbzr('commit -m blah3 --unchanged')
+        self.runbzr('pull ../a', retcode=1)
+        os.chdir('../a')
+        self.runbzr('merge ../b')
+        self.runbzr('commit -m blah4 --unchanged')
+        os.chdir('../b')
+        self.runbzr('pull ../a')
+        assert a.revision_history()[-1] == b.revision_history()[-1]
+        
+
+    def test_add_reports(self):
+        """add command prints the names of added files."""
+        b = Branch('.', init=True)
+        self.build_tree(['top.txt', 'dir/', 'dir/sub.txt'])
+
+        from cStringIO import StringIO
+        out = StringIO()
+
+        ret = self.apply_redirected(None, out, None,
+                                    run_bzr,
+                                    ['add'])
+        self.assertEquals(ret, 0)
+
+        # the ordering is not defined at the moment
+        results = sorted(out.getvalue().rstrip('\n').split('\n'))
+        self.assertEquals(['added dir',
+                           'added dir/sub.txt',
+                           'added top.txt',],
+                          results)
 
 
 class OldTests(ExternalBase):
-    # old tests moved from ./testbzr
-    def runTest(self):
+    """old tests moved from ./testbzr."""
+
+    def test_bzr(self):
         from os import chdir, mkdir
         from os.path import exists
-        import os
 
         runbzr = self.runbzr
         backtick = self.backtick
@@ -307,102 +440,3 @@
 
         runbzr('info')
 
-
-
-
-
-
-        chdir('..')
-        chdir('..')
-        progress('branch')
-        assert os.path.exists('branch1')
-        assert not os.path.exists('branch2')
-        # Can't create a branch if it already exists
-        runbzr('branch branch1', retcode=1)
-        # Can't create a branch if its parent doesn't exist
-        runbzr('branch /unlikely/to/exist', retcode=1)
-        runbzr('branch branch1 branch2')
-        assert exists('branch2')
-        assert exists('branch2/sub1')
-        assert exists('branch2/sub1/hello.txt')
-        
-        runbzr('branch --revision 0 branch1 branch3')
-        assert not exists('branch3/sub1/hello.txt')
-        runbzr('branch --revision 0..3 branch1 branch4', retcode=1)
-
-        progress("pull")
-        chdir('branch1')
-        runbzr('pull', retcode=1)
-        runbzr('pull ../branch2')
-        chdir('.bzr')
-        runbzr('pull')
-        runbzr('commit --unchanged -m empty')
-        runbzr('pull')
-        chdir('../../branch2')
-        runbzr('pull')
-        runbzr('commit --unchanged -m empty')
-        chdir('../branch1')
-        runbzr('commit --unchanged -m empty')
-        runbzr('pull', retcode=1)
-        chdir ('..')
-
-        progress('status after remove')
-        mkdir('status-after-remove')
-        # see mail from William Dodé, 2005-05-25
-        # $ bzr init; touch a; bzr add a; bzr commit -m "add a"
-        #     * looking for changes...
-        #     added a
-        #     * commited r1
-        #     $ bzr remove a
-        #     $ bzr status
-        #     bzr: local variable 'kind' referenced before assignment
-        #     at /vrac/python/bazaar-ng/bzrlib/diff.py:286 in compare_trees()
-        #     see ~/.bzr.log for debug information
-        chdir('status-after-remove')
-        runbzr('init')
-        file('a', 'w').write('foo')
-        runbzr('add a')
-        runbzr(['commit', '-m', 'add a'])
-        runbzr('remove a')
-        runbzr('status')
-
-        chdir('..')
-
-
-
-        progress("recursive and non-recursive add")
-        mkdir('no-recurse')
-        chdir('no-recurse')
-        runbzr('init')
-        mkdir('foo')
-        fp = os.path.join('foo', 'test.txt')
-        f = file(fp, 'w')
-        f.write('hello!\n')
-        f.close()
-        runbzr('add --no-recurse foo')
-        runbzr('file-id foo')
-        runbzr('file-id ' + fp, 1)      # not versioned yet
-        runbzr('commit -m add-dir-only')
-
-        self.runbzr('file-id ' + fp, 1)      # still not versioned 
-
-        self.runbzr('add foo')
-        self.runbzr('file-id ' + fp)
-        self.runbzr('commit -m add-sub-file')
-
-        chdir('..')
-
-
-
-class RevertCommand(ExternalBase):
-    def runTest(self):
-        self.runbzr('init')
-
-        file('hello', 'wt').write('foo')
-        self.runbzr('add hello')
-        self.runbzr('commit -m setup hello')
-        
-        file('hello', 'wt').write('bar')
-        self.runbzr('revert hello')
-        self.check_file_contents('hello', 'foo')
-

=== modified file 'bzrlib/selftest/plugins.py'
--- bzrlib/selftest/plugins.py	2005-06-22 07:57:56 +0000
+++ bzrlib/selftest/plugins.py	2005-08-30 03:10:32 +0000
@@ -17,20 +17,25 @@
 
 """Tests for plugins"""
 
+
+
+# **************************************************
 # NOT RUN YET
-
-
-
-
-
-
-
-from bzrlib.selftest import InTempDir
-
-
-def PluginTest(InTempDir):
+# **************************************************
+
+
+
+
+
+
+
+
+from bzrlib.selftest import TestCaseInTempDir
+
+
+class PluginTest(TestCaseInTempDir):
     """Create an external plugin and test loading."""
-    def runTest(self):
+    def test_plugin_loading(self):
         import os
         
         orig_help = self.backtick('bzr help commands') # No plugins yet
@@ -51,36 +56,36 @@
 
 
 
-PLUGIN_TEXT = \
-"""import bzrlib, bzrlib.commands
-class cmd_myplug(bzrlib.commands.Command):
-    '''Just a simple test plugin.'''
-    aliases = ['mplg']
-    def run(self):
-        print 'Hello from my plugin'
-""")
-    f.close()
-
-    os.environ['BZRPLUGINPATH'] = os.path.abspath('plugin_test')
-    help = backtick('bzr help commands')
-    assert help.find('myplug') != -1
-    assert help.find('Just a simple test plugin.') != -1
-
-
-    assert backtick('bzr myplug') == 'Hello from my plugin\n'
-    assert backtick('bzr mplg') == 'Hello from my plugin\n'
-
-    f = open(os.path.join('plugin_test', 'override.py'), 'wb')
-    f.write("""import bzrlib, bzrlib.commands
-class cmd_commit(bzrlib.commands.cmd_commit):
-    '''Commit changes into a new revision.'''
-    def run(self, *args, **kwargs):
-        print "I'm sorry dave, you can't do that"
-
-class cmd_help(bzrlib.commands.cmd_help):
-    '''Show help on a command or other topic.'''
-    def run(self, *args, **kwargs):
-        print "You have been overridden"
-        bzrlib.commands.cmd_help.run(self, *args, **kwargs)
-
-    """
+#         PLUGIN_TEXT = \
+#         """import bzrlib, bzrlib.commands
+#         class cmd_myplug(bzrlib.commands.Command):
+#             '''Just a simple test plugin.'''
+#             aliases = ['mplg']
+#             def run(self):
+#                 print 'Hello from my plugin'
+#         """
+#         f.close()
+
+#         os.environ['BZRPLUGINPATH'] = os.path.abspath('plugin_test')
+#         help = backtick('bzr help commands')
+#         assert help.find('myplug') != -1
+#         assert help.find('Just a simple test plugin.') != -1
+
+
+#         assert backtick('bzr myplug') == 'Hello from my plugin\n'
+#         assert backtick('bzr mplg') == 'Hello from my plugin\n'
+
+#         f = open(os.path.join('plugin_test', 'override.py'), 'wb')
+#         f.write("""import bzrlib, bzrlib.commands
+#     class cmd_commit(bzrlib.commands.cmd_commit):
+#         '''Commit changes into a new revision.'''
+#         def run(self, *args, **kwargs):
+#             print "I'm sorry dave, you can't do that"
+
+#     class cmd_help(bzrlib.commands.cmd_help):
+#         '''Show help on a command or other topic.'''
+#         def run(self, *args, **kwargs):
+#             print "You have been overridden"
+#             bzrlib.commands.cmd_help.run(self, *args, **kwargs)
+
+#         """

=== added file 'bzrlib/selftest/test_merge_core.py'
--- bzrlib/selftest/test_merge_core.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/test_merge_core.py	2005-09-01 06:33:48 +0000
@@ -0,0 +1,501 @@
+import os
+import unittest
+
+from bzrlib.selftest import TestCaseInTempDir, TestCase
+from bzrlib.branch import ScratchBranch, Branch
+from bzrlib.errors import NotBranchError, NotVersionedError
+
+
+import tempfile
+import shutil
+from bzrlib.inventory import InventoryEntry, RootEntry
+from bzrlib.osutils import file_kind
+from bzrlib import changeset
+from bzrlib.merge_core import (ApplyMerge3, make_merge_changeset,
+                                BackupBeforeChange, PermissionsMerge)
+from bzrlib.changeset import Inventory, apply_changeset, invert_dict
+
+class FalseTree(object):
+    def __init__(self, realtree):
+        self._realtree = realtree
+        self.inventory = self
+
+    def __getitem__(self, file_id):
+        entry = self.make_inventory_entry(file_id)
+        if entry is None:
+            raise KeyError(file_id)
+        return entry
+        
+    def make_inventory_entry(self, file_id):
+        path = self._realtree.inventory.get(file_id)
+        if path is None:
+            return None
+        if path == "":
+            return RootEntry(file_id)
+        dir, name = os.path.split(path)
+        kind = file_kind(self._realtree.abs_path(path))
+        for parent_id, path in self._realtree.inventory.iteritems():
+            if path == dir:
+                break
+        if path != dir:
+            raise Exception("Can't find parent for %s" % name)
+        return InventoryEntry(file_id, name, kind, parent_id)
+
+
+class MergeTree(object):
+    def __init__(self, dir):
+        self.dir = dir;
+        os.mkdir(dir)
+        self.inventory = {'0': ""}
+        self.tree = FalseTree(self)
+    
+    def child_path(self, parent, name):
+        return os.path.join(self.inventory[parent], name)
+
+    def add_file(self, id, parent, name, contents, mode):
+        path = self.child_path(parent, name)
+        full_path = self.abs_path(path)
+        assert not os.path.exists(full_path)
+        file(full_path, "wb").write(contents)
+        os.chmod(self.abs_path(path), mode)
+        self.inventory[id] = path
+
+    def remove_file(self, id):
+        os.unlink(self.full_path(id))
+        del self.inventory[id]
+
+    def add_dir(self, id, parent, name, mode):
+        path = self.child_path(parent, name)
+        full_path = self.abs_path(path)
+        assert not os.path.exists(full_path)
+        os.mkdir(self.abs_path(path))
+        os.chmod(self.abs_path(path), mode)
+        self.inventory[id] = path
+
+    def abs_path(self, path):
+        return os.path.join(self.dir, path)
+
+    def full_path(self, id):
+        try:
+            tree_path = self.inventory[id]
+        except KeyError:
+            return None
+        return self.abs_path(tree_path)
+
+    def readonly_path(self, id):
+        return self.full_path(id)
+
+    def __contains__(self, file_id):
+        return file_id in self.inventory
+
+    def has_or_had_id(self, file_id):
+        return file_id in self
+
+    def get_file(self, file_id):
+        path = self.readonly_path(file_id)
+        return file(path, "rb")
+
+    def id2path(self, file_id):
+        return self.inventory[file_id]
+
+    def change_path(self, id, path):
+        old_path = os.path.join(self.dir, self.inventory[id])
+        os.rename(old_path, self.abs_path(path))
+        self.inventory[id] = path
+
+
+class MergeBuilder(object):
+    def __init__(self):
+        self.dir = tempfile.mkdtemp(prefix="BaZing")
+        self.base = MergeTree(os.path.join(self.dir, "base"))
+        self.this = MergeTree(os.path.join(self.dir, "this"))
+        self.other = MergeTree(os.path.join(self.dir, "other"))
+        
+        self.cset = changeset.Changeset()
+        self.cset.add_entry(changeset.ChangesetEntry("0", 
+                                                     changeset.NULL_ID, "./."))
+    def get_cset_path(self, parent, name):
+        if name is None:
+            assert (parent is None)
+            return None
+        return os.path.join(self.cset.entries[parent].path, name)
+
+    def add_file(self, id, parent, name, contents, mode):
+        self.base.add_file(id, parent, name, contents, mode)
+        self.this.add_file(id, parent, name, contents, mode)
+        self.other.add_file(id, parent, name, contents, mode)
+        path = self.get_cset_path(parent, name)
+        self.cset.add_entry(changeset.ChangesetEntry(id, parent, path))
+
+    def remove_file(self, id, base=False, this=False, other=False):
+        for option, tree in ((base, self.base), (this, self.this), 
+                             (other, self.other)):
+            if option:
+                tree.remove_file(id)
+            if other or base:
+                change = self.cset.entries[id].contents_change
+                if change is None:
+                    change = changeset.ReplaceContents(None, None)
+                    self.cset.entries[id].contents_change = change
+                    def create_file(tree):
+                        return changeset.FileCreate(tree.get_file(id).read())
+                    if not other:
+                        change.new_contents = create_file(self.other)
+                    if not base:
+                        change.old_contents = create_file(self.base)
+                else:
+                    assert isinstance(change, changeset.ReplaceContents)
+                if other:
+                    change.new_contents=None
+                if base:
+                    change.old_contents=None
+                if change.old_contents is None and change.new_contents is None:
+                    change = None
+
+
+    def add_dir(self, id, parent, name, mode):
+        path = self.get_cset_path(parent, name)
+        self.base.add_dir(id, parent, name, mode)
+        self.cset.add_entry(changeset.ChangesetEntry(id, parent, path))
+        self.this.add_dir(id, parent, name, mode)
+        self.other.add_dir(id, parent, name, mode)
+
+
+    def change_name(self, id, base=None, this=None, other=None):
+        if base is not None:
+            self.change_name_tree(id, self.base, base)
+            self.cset.entries[id].name = base
+
+        if this is not None:
+            self.change_name_tree(id, self.this, this)
+
+        if other is not None:
+            self.change_name_tree(id, self.other, other)
+            self.cset.entries[id].new_name = other
+
+    def change_parent(self, id, base=None, this=None, other=None):
+        if base is not None:
+            self.change_parent_tree(id, self.base, base)
+            self.cset.entries[id].parent = base
+            self.cset.entries[id].dir = self.cset.entries[base].path
+
+        if this is not None:
+            self.change_parent_tree(id, self.this, this)
+
+        if other is not None:
+            self.change_parent_tree(id, self.other, other)
+            self.cset.entries[id].new_parent = other
+            self.cset.entries[id].new_dir = \
+                self.cset.entries[other].new_path
+
+    def change_contents(self, id, base=None, this=None, other=None):
+        if base is not None:
+            self.change_contents_tree(id, self.base, base)
+
+        if this is not None:
+            self.change_contents_tree(id, self.this, this)
+
+        if other is not None:
+            self.change_contents_tree(id, self.other, other)
+
+        if base is not None or other is not None:
+            old_contents = file(self.base.full_path(id)).read()
+            new_contents = file(self.other.full_path(id)).read()
+            contents = changeset.ReplaceFileContents(old_contents, 
+                                                     new_contents)
+            self.cset.entries[id].contents_change = contents
+
+    def change_perms(self, id, base=None, this=None, other=None):
+        if base is not None:
+            self.change_perms_tree(id, self.base, base)
+
+        if this is not None:
+            self.change_perms_tree(id, self.this, this)
+
+        if other is not None:
+            self.change_perms_tree(id, self.other, other)
+
+        if base is not None or other is not None:
+            old_perms = os.stat(self.base.full_path(id)).st_mode &077
+            new_perms = os.stat(self.other.full_path(id)).st_mode &077
+            contents = changeset.ChangeUnixPermissions(old_perms, 
+                                                       new_perms)
+            self.cset.entries[id].metadata_change = contents
+
+    def change_name_tree(self, id, tree, name):
+        new_path = tree.child_path(self.cset.entries[id].parent, name)
+        tree.change_path(id, new_path)
+
+    def change_parent_tree(self, id, tree, parent):
+        new_path = tree.child_path(parent, self.cset.entries[id].name)
+        tree.change_path(id, new_path)
+
+    def change_contents_tree(self, id, tree, contents):
+        path = tree.full_path(id)
+        mode = os.stat(path).st_mode
+        file(path, "w").write(contents)
+        os.chmod(path, mode)
+
+    def change_perms_tree(self, id, tree, mode):
+        os.chmod(tree.full_path(id), mode)
+
+    def merge_changeset(self, merge_factory):
+        conflict_handler = changeset.ExceptionConflictHandler(self.this.dir)
+        return make_merge_changeset(self.cset, self.this, self.base,
+                                    self.other, conflict_handler,
+                                    merge_factory)
+
+    def apply_inv_change(self, inventory_change, orig_inventory):
+        orig_inventory_by_path = {}
+        for file_id, path in orig_inventory.iteritems():
+            orig_inventory_by_path[path] = file_id
+
+        def parent_id(file_id):
+            try:
+                parent_dir = os.path.dirname(orig_inventory[file_id])
+            except:
+                print file_id
+                raise
+            if parent_dir == "":
+                return None
+            return orig_inventory_by_path[parent_dir]
+        
+        def new_path(file_id):
+            if inventory_change.has_key(file_id):
+                return inventory_change[file_id]
+            else:
+                parent = parent_id(file_id)
+                if parent is None:
+                    return orig_inventory[file_id]
+                dirname = new_path(parent)
+                return os.path.join(dirname, orig_inventory[file_id])
+
+        new_inventory = {}
+        for file_id in orig_inventory.iterkeys():
+            path = new_path(file_id)
+            if path is None:
+                continue
+            new_inventory[file_id] = path
+
+        for file_id, path in inventory_change.iteritems():
+            if orig_inventory.has_key(file_id):
+                continue
+            new_inventory[file_id] = path
+        return new_inventory
+
+    def apply_changeset(self, cset, conflict_handler=None, reverse=False):
+        inventory_change = changeset.apply_changeset(cset,
+                                                     self.this.inventory,
+                                                     self.this.dir,
+                                                     conflict_handler, reverse)
+        self.this.inventory =  self.apply_inv_change(inventory_change, 
+                                                     self.this.inventory)
+
+    def cleanup(self):
+        shutil.rmtree(self.dir)
+
+class MergeTest(unittest.TestCase):
+    def test_change_name(self):
+        """Test renames"""
+        builder = MergeBuilder()
+        builder.add_file("1", "0", "name1", "hello1", 0755)
+        builder.change_name("1", other="name2")
+        builder.add_file("2", "0", "name3", "hello2", 0755)
+        builder.change_name("2", base="name4")
+        builder.add_file("3", "0", "name5", "hello3", 0755)
+        builder.change_name("3", this="name6")
+        cset = builder.merge_changeset(ApplyMerge3)
+        assert(cset.entries["2"].is_boring())
+        assert(cset.entries["1"].name == "name1")
+        assert(cset.entries["1"].new_name == "name2")
+        assert(cset.entries["3"].is_boring())
+        for tree in (builder.this, builder.other, builder.base):
+            assert(tree.dir != builder.dir and 
+                   tree.dir.startswith(builder.dir))
+            for path in tree.inventory.itervalues():
+                fullpath = tree.abs_path(path)
+                assert(fullpath.startswith(tree.dir))
+                assert(not path.startswith(tree.dir))
+                assert os.path.exists(fullpath)
+        builder.apply_changeset(cset)
+        builder.cleanup()
+        builder = MergeBuilder()
+        builder.add_file("1", "0", "name1", "hello1", 0644)
+        builder.change_name("1", other="name2", this="name3")
+        self.assertRaises(changeset.RenameConflict, 
+                          builder.merge_changeset, ApplyMerge3)
+        builder.cleanup()
+        
+    def test_file_moves(self):
+        """Test moves"""
+        builder = MergeBuilder()
+        builder.add_dir("1", "0", "dir1", 0755)
+        builder.add_dir("2", "0", "dir2", 0755)
+        builder.add_file("3", "1", "file1", "hello1", 0644)
+        builder.add_file("4", "1", "file2", "hello2", 0644)
+        builder.add_file("5", "1", "file3", "hello3", 0644)
+        builder.change_parent("3", other="2")
+        assert(Inventory(builder.other.inventory).get_parent("3") == "2")
+        builder.change_parent("4", this="2")
+        assert(Inventory(builder.this.inventory).get_parent("4") == "2")
+        builder.change_parent("5", base="2")
+        assert(Inventory(builder.base.inventory).get_parent("5") == "2")
+        cset = builder.merge_changeset(ApplyMerge3)
+        for id in ("1", "2", "4", "5"):
+            assert(cset.entries[id].is_boring())
+        assert(cset.entries["3"].parent == "1")
+        assert(cset.entries["3"].new_parent == "2")
+        builder.apply_changeset(cset)
+        builder.cleanup()
+
+        builder = MergeBuilder()
+        builder.add_dir("1", "0", "dir1", 0755)
+        builder.add_dir("2", "0", "dir2", 0755)
+        builder.add_dir("3", "0", "dir3", 0755)
+        builder.add_file("4", "1", "file1", "hello1", 0644)
+        builder.change_parent("4", other="2", this="3")
+        self.assertRaises(changeset.MoveConflict, 
+                          builder.merge_changeset, ApplyMerge3)
+        builder.cleanup()
+
+    def test_contents_merge(self):
+        """Test merge3 merging"""
+        self.do_contents_test(ApplyMerge3)
+
+    def test_contents_merge2(self):
+        """Test diff3 merging"""
+        self.do_contents_test(changeset.Diff3Merge)
+
+    def test_contents_merge3(self):
+        """Test diff3 merging"""
+        def backup_merge(file_id, base, other):
+            return BackupBeforeChange(ApplyMerge3(file_id, base, other))
+        builder = self.contents_test_success(backup_merge)
+        def backup_exists(file_id):
+            return os.path.exists(builder.this.full_path(file_id)+"~")
+        assert backup_exists("1")
+        assert backup_exists("2")
+        assert not backup_exists("3")
+        builder.cleanup()
+
+    def do_contents_test(self, merge_factory):
+        """Test merging with specified ContentsChange factory"""
+        builder = self.contents_test_success(merge_factory)
+        builder.cleanup()
+        self.contents_test_conflicts(merge_factory)
+
+    def contents_test_success(self, merge_factory):
+        from inspect import isclass
+        builder = MergeBuilder()
+        builder.add_file("1", "0", "name1", "text1", 0755)
+        builder.change_contents("1", other="text4")
+        builder.add_file("2", "0", "name3", "text2", 0655)
+        builder.change_contents("2", base="text5")
+        builder.add_file("3", "0", "name5", "text3", 0744)
+        builder.add_file("4", "0", "name6", "text4", 0744)
+        builder.remove_file("4", base=True)
+        assert not builder.cset.entries["4"].is_boring()
+        builder.change_contents("3", this="text6")
+        cset = builder.merge_changeset(merge_factory)
+        assert(cset.entries["1"].contents_change is not None)
+        if isclass(merge_factory):
+            assert(isinstance(cset.entries["1"].contents_change,
+                          merge_factory))
+            assert(isinstance(cset.entries["2"].contents_change,
+                          merge_factory))
+        assert(cset.entries["3"].is_boring())
+        assert(cset.entries["4"].is_boring())
+        builder.apply_changeset(cset)
+        assert(file(builder.this.full_path("1"), "rb").read() == "text4" )
+        assert(file(builder.this.full_path("2"), "rb").read() == "text2" )
+        assert(os.stat(builder.this.full_path("1")).st_mode &0777 == 0755)
+        assert(os.stat(builder.this.full_path("2")).st_mode &0777 == 0655)
+        assert(os.stat(builder.this.full_path("3")).st_mode &0777 == 0744)
+        return builder
+
+    def contents_test_conflicts(self, merge_factory):
+        builder = MergeBuilder()
+        builder.add_file("1", "0", "name1", "text1", 0755)
+        builder.change_contents("1", other="text4", this="text3")
+        cset = builder.merge_changeset(merge_factory)
+        self.assertRaises(changeset.MergeConflict, builder.apply_changeset,
+                          cset)
+        builder.cleanup()
+
+        builder = MergeBuilder()
+        builder.add_file("1", "0", "name1", "text1", 0755)
+        builder.change_contents("1", other="text4", base="text3")
+        builder.remove_file("1", base=True)
+        self.assertRaises(changeset.NewContentsConflict,
+                          builder.merge_changeset, merge_factory)
+        builder.cleanup()
+
+        builder = MergeBuilder()
+        builder.add_file("1", "0", "name1", "text1", 0755)
+        builder.change_contents("1", other="text4", base="text3")
+        builder.remove_file("1", this=True)
+        self.assertRaises(changeset.MissingForMerge, builder.merge_changeset, 
+                          merge_factory)
+        builder.cleanup()
+
+    def test_perms_merge(self):
+        builder = MergeBuilder()
+        builder.add_file("1", "0", "name1", "text1", 0755)
+        builder.change_perms("1", other=0655)
+        builder.add_file("2", "0", "name2", "text2", 0755)
+        builder.change_perms("2", base=0655)
+        builder.add_file("3", "0", "name3", "text3", 0755)
+        builder.change_perms("3", this=0655)
+        cset = builder.merge_changeset(ApplyMerge3)
+        assert(cset.entries["1"].metadata_change is not None)
+        assert(isinstance(cset.entries["1"].metadata_change,
+                          PermissionsMerge))
+        assert(isinstance(cset.entries["2"].metadata_change,
+                          PermissionsMerge))
+        assert(cset.entries["3"].is_boring())
+        builder.apply_changeset(cset)
+        assert(os.stat(builder.this.full_path("1")).st_mode &0777 == 0655)
+        assert(os.stat(builder.this.full_path("2")).st_mode &0777 == 0755)
+        assert(os.stat(builder.this.full_path("3")).st_mode &0777 == 0655)
+        builder.cleanup();
+        builder = MergeBuilder()
+        builder.add_file("1", "0", "name1", "text1", 0755)
+        builder.change_perms("1", other=0655, base=0555)
+        cset = builder.merge_changeset(ApplyMerge3)
+        self.assertRaises(changeset.MergePermissionConflict, 
+                     builder.apply_changeset, cset)
+        builder.cleanup()
+
+class FunctionalMergeTest(TestCaseInTempDir):
+
+    def test_trivial_star_merge(self):
+        """Test that merges in a star shape Just Work.""" 
+        from bzrlib.add import smart_add_branch, add_reporter_null
+        from bzrlib.branch import copy_branch
+        from bzrlib.merge import merge
+        # John starts a branch
+        self.build_tree(("original/", "original/file1", "original/file2"))
+        branch = Branch("original", init=True)
+        smart_add_branch(branch, ["original"], True, add_reporter_null)
+        branch.commit("start branch.", verbose=False)
+        # Mary branches it.
+        self.build_tree(("mary/",))
+        copy_branch(branch, "mary")
+        # Now John commits a change
+        file = open("original/file1", "wt")
+        file.write("John\n")
+        file.close()
+        branch.commit("change file1")
+        # Mary does too
+        mary_branch = Branch("mary")
+        file = open("mary/file2", "wt")
+        file.write("Mary\n")
+        file.close()
+        mary_branch.commit("change file2")
+        # john should be able to merge with no conflicts.
+        merge_type = ApplyMerge3
+        base = [None, None]
+        other = ("mary", -1)
+        merge(other, base, check_clean=True, merge_type=merge_type, this_dir="original")
+        self.assertEqual("John\n", open("original/file1", "rt").read())
+        self.assertEqual("Mary\n", open("original/file2", "rt").read())

=== added file 'bzrlib/selftest/test_parent.py'
--- bzrlib/selftest/test_parent.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/test_parent.py	2005-08-30 05:55:13 +0000
@@ -0,0 +1,55 @@
+# Copyright (C) 2004, 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+import os
+from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.branch import Branch, copy_branch
+
+
+
+class TestParent(TestCaseInTempDir):
+    def test_no_default_parent(self):
+        """Branches should have no parent by default"""
+        b = Branch('.', init=True)
+        self.assertEquals(b.get_parent(), None)
+        
+    
+    def test_set_get_parent(self):
+        """Set and then re-get the parent"""
+        b = Branch('.', init=True)
+        url = 'http://bazaar-ng.org/bzr/bzr.dev'
+        b.set_parent(url)
+        self.assertEquals(b.get_parent(), url)
+
+    def test_branch_sets_parent(self):
+        """The branch command should set the new branch's parent"""
+        from bzrlib.commands import run_bzr
+
+        os.mkdir('from')
+        branch_from = Branch('from', init=True)
+        file('from/foo', 'wt').write('contents of foo')
+        branch_from.add('foo')
+        branch_from.commit('initial commit')
+        
+        os.mkdir('to')
+        copy_branch(branch_from, 'to', None)
+
+        branch_to = Branch('to')
+        abspath = os.path.abspath('from')
+        self.assertEquals(branch_to.get_parent(), abspath)
+        
+

=== added file 'bzrlib/selftest/test_smart_add.py'
--- bzrlib/selftest/test_smart_add.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/test_smart_add.py	2005-09-01 06:33:48 +0000
@@ -0,0 +1,160 @@
+import os
+import unittest
+
+from bzrlib.selftest import TestCaseInTempDir, TestCase
+from bzrlib.branch import Branch
+from bzrlib.errors import NotBranchError, NotVersionedError
+
+class TestSmartAdd(TestCaseInTempDir):
+
+    def test_add_dot_from_root(self):
+        """Test adding . from the root of the tree.""" 
+        from bzrlib.add import smart_add
+        paths = ("original/", "original/file1", "original/file2")
+        self.build_tree(paths)
+        branch = Branch(".", init=True)
+        smart_add((".",), recurse=True)
+        for path in paths:
+            self.assertNotEqual(branch.inventory.path2id(path), None)
+
+    def test_add_dot_from_subdir(self):
+        """Test adding . from a subdir of the tree.""" 
+        from bzrlib.add import smart_add
+        paths = ("original/", "original/file1", "original/file2")
+        self.build_tree(paths)
+        branch = Branch(".", init=True)
+        os.chdir("original")
+        smart_add((".",), recurse=True)
+        for path in paths:
+            self.assertNotEqual(branch.inventory.path2id(path), None)
+
+    def test_add_tree_from_above_tree(self):
+        """Test adding a tree from above the tree.""" 
+        from bzrlib.add import smart_add
+        paths = ("original/", "original/file1", "original/file2")
+        branch_paths = ("branch/", "branch/original/", "branch/original/file1",
+                        "branch/original/file2")
+        self.build_tree(branch_paths)
+        branch = Branch("branch", init=True)
+        smart_add(("branch",))
+        for path in paths:
+            self.assertNotEqual(branch.inventory.path2id(path), None)
+
+    def test_add_above_tree_preserves_tree(self):
+        """Test nested trees are not affect by an add above them."""
+        from bzrlib.add import smart_add, add_reporter_null
+        
+        paths = ("original/", "original/file1", "original/file2")
+        child_paths = ("path",)
+        full_child_paths = ("original/child", "original/child/path")
+        build_paths = ("original/", "original/file1", "original/file2", 
+                       "original/child/", "original/child/path")
+        
+        self.build_tree(build_paths)
+        branch = Branch(".", init=True)
+        child_branch = Branch("original/child", init=True)
+        smart_add((".",), True, add_reporter_null)
+        for path in paths:
+            self.assertNotEqual((path, branch.inventory.path2id(path)),
+                                (path, None))
+        for path in full_child_paths:
+            self.assertEqual((path, branch.inventory.path2id(path)),
+                             (path, None))
+        for path in child_paths:
+            self.assertEqual(child_branch.inventory.path2id(path), None)
+
+    def test_add_paths(self):
+        """Test smart-adding a list of paths."""
+        from bzrlib.add import smart_add
+        paths = ("file1", "file2")
+        self.build_tree(paths)
+        branch = Branch(".", init=True)
+        smart_add(paths)
+        for path in paths:
+            self.assertNotEqual(branch.inventory.path2id(path), None)
+            
+class TestSmartAddBranch(TestCaseInTempDir):
+    """Test smart adds with a specified branch."""
+
+    def test_add_dot_from_root(self):
+        """Test adding . from the root of the tree.""" 
+        from bzrlib.add import smart_add_branch
+        paths = ("original/", "original/file1", "original/file2")
+        self.build_tree(paths)
+        branch = Branch(".", init=True)
+        smart_add_branch(branch, (".",))
+        for path in paths:
+            self.assertNotEqual(branch.inventory.path2id(path), None)
+
+    def test_add_dot_from_subdir(self):
+        """Test adding . from a subdir of the tree.""" 
+        from bzrlib.add import smart_add_branch
+        paths = ("original/", "original/file1", "original/file2")
+        self.build_tree(paths)
+        branch = Branch(".", init=True)
+        os.chdir("original")
+        smart_add_branch(branch, (".",))
+        for path in paths:
+            self.assertNotEqual(branch.inventory.path2id(path), None)
+
+    def test_add_tree_from_above_tree(self):
+        """Test adding a tree from above the tree.""" 
+        from bzrlib.add import smart_add_branch
+        paths = ("original/", "original/file1", "original/file2")
+        branch_paths = ("branch/", "branch/original/", "branch/original/file1",
+                        "branch/original/file2")
+        self.build_tree(branch_paths)
+        branch = Branch("branch", init=True)
+        smart_add_branch(branch, ("branch",))
+        for path in paths:
+            self.assertNotEqual(branch.inventory.path2id(path), None)
+
+    def test_add_above_tree_preserves_tree(self):
+        """Test nested trees are not affect by an add above them."""
+        from bzrlib.add import smart_add_branch
+        paths = ("original/", "original/file1", "original/file2")
+        child_paths = ("path")
+        full_child_paths = ("original/child", "original/child/path")
+        build_paths = ("original/", "original/file1", "original/file2", 
+                       "original/child/", "original/child/path")
+        self.build_tree(build_paths)
+        branch = Branch(".", init=True)
+        child_branch = Branch("original/child", init=True)
+        smart_add_branch(branch, (".",))
+        for path in paths:
+            self.assertNotEqual((path, branch.inventory.path2id(path)),
+                                (path, None))
+        for path in full_child_paths:
+            self.assertEqual((path, branch.inventory.path2id(path)),
+                             (path, None))
+        for path in child_paths:
+            self.assertEqual(child_branch.inventory.path2id(path), None)
+
+    def test_add_paths(self):
+        """Test smart-adding a list of paths."""
+        from bzrlib.add import smart_add_branch
+        paths = ("file1", "file2")
+        self.build_tree(paths)
+        branch = Branch(".", init=True)
+        smart_add_branch(branch, paths)
+        for path in paths:
+            self.assertNotEqual(branch.inventory.path2id(path), None)
+
+class TestAddCallbacks(TestCaseInTempDir):
+
+    def setUp(self):
+        from bzrlib.inventory import InventoryEntry
+        super(TestAddCallbacks, self).setUp()
+        self.entry = InventoryEntry("id", "name", "file", None)
+
+    def test_null_callback(self):
+        from bzrlib.add import add_reporter_null
+        add_reporter_null('path', 'file', self.entry)
+
+    def test_print_callback(self):
+        from bzrlib.add import add_reporter_print
+        from StringIO import StringIO
+        stdout = StringIO()
+        self.apply_redirected(None, stdout, None, add_reporter_print,
+                              'path', 'file', self.entry)
+        self.assertEqual(stdout.getvalue(), "added path\n")

=== added file 'bzrlib/selftest/test_xml.py'
--- bzrlib/selftest/test_xml.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/test_xml.py	2005-09-06 07:26:13 +0000
@@ -0,0 +1,140 @@
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+from cStringIO import StringIO
+
+from bzrlib.selftest import TestCase
+from bzrlib.inventory import Inventory, InventoryEntry
+from bzrlib.xml import serializer_v4, serializer_v5
+
+_working_inventory_v4 = """<inventory file_id="TREE_ROOT">
+<entry file_id="bar-20050901064931-73b4b1138abc9cd2" kind="file" name="bar" parent_id="TREE_ROOT" />
+<entry file_id="foo-20050801201819-4139aa4a272f4250" kind="directory" name="foo" parent_id="TREE_ROOT" />
+<entry file_id="bar-20050824000535-6bc48cfad47ed134" kind="file" name="bar" parent_id="foo-20050801201819-4139aa4a272f4250" />
+</inventory>"""
+
+
+_revision_v4 = """<revision committer="Martin Pool &lt;mbp@sourcefrog.net&gt;"
+    inventory_id="mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9"
+    inventory_sha1="e79c31c1deb64c163cf660fdedd476dd579ffd41"
+    revision_id="mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9"
+    timestamp="1125907235.211783886"
+    timezone="36000">
+<message>- start splitting code for xml (de)serialization away from objects
+  preparatory to supporting multiple formats by a single library
+</message>
+<parents>
+<revision_ref revision_id="mbp@sourcefrog.net-20050905063503-43948f59fa127d92" revision_sha1="7bdf4cc8c5bdac739f8cf9b10b78cf4b68f915ff" />
+</parents>
+</revision>
+"""
+
+_revision_v5 = """<revision committer="Martin Pool &lt;mbp@sourcefrog.net&gt;"
+    inventory_id="mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9"
+    inventory_sha1="e79c31c1deb64c163cf660fdedd476dd579ffd41"
+    revision_id="mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9"
+    timestamp="1125907235.211783886"
+    timezone="36000">
+<message>- start splitting code for xml (de)serialization away from objects
+  preparatory to supporting multiple formats by a single library
+</message>
+<parents>
+<revision_ref revision_id="mbp@sourcefrog.net-20050905063503-43948f59fa127d92"/>
+</parents>
+</revision>
+"""
+
+_committed_inv_v5 = """<inventory>
+<file file_id="bar-20050901064931-73b4b1138abc9cd2" 
+      name="bar" parent_id="TREE_ROOT" 
+      text_version="mbp@foo-123123" entry_version="mbp@foo-123123"
+      />
+<directory name="subdir"
+           file_id="foo-20050801201819-4139aa4a272f4250"
+           parent_id="TREE_ROOT" 
+           entry_version="mbp@foo-00"/>
+<file file_id="bar-20050824000535-6bc48cfad47ed134" 
+      name="bar" parent_id="foo-20050801201819-4139aa4a272f4250" 
+      entry_version="mbp@foo-00"
+      text_version="mbp@foo-123123"/>
+</inventory>
+"""
+
+class TestSerializer(TestCase):
+    """Test XML serialization"""
+    def test_canned_inventory(self):
+        """Test unpacked a canned inventory v4 file."""
+        inp = StringIO(_working_inventory_v4)
+        inv = serializer_v4.read_inventory(inp)
+        self.assertEqual(len(inv), 4)
+        self.assert_('bar-20050901064931-73b4b1138abc9cd2' in inv)
+
+    def test_unpack_revision(self):
+        """Test unpacking a canned revision v4"""
+        inp = StringIO(_revision_v4)
+        rev = serializer_v4.read_revision(inp)
+        eq = self.assertEqual
+        eq(rev.committer,
+           "Martin Pool <mbp@sourcefrog.net>")
+        eq(rev.inventory_id,
+           "mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9")
+        eq(len(rev.parents), 1)
+        eq(rev.parents[0].revision_id,
+           "mbp@sourcefrog.net-20050905063503-43948f59fa127d92")
+
+    def test_unpack_revision_5(self):
+        """Test unpacking a canned revision v5"""
+        inp = StringIO(_revision_v5)
+        rev = serializer_v5.read_revision(inp)
+        eq = self.assertEqual
+        eq(rev.committer,
+           "Martin Pool <mbp@sourcefrog.net>")
+        eq(rev.inventory_id,
+           "mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9")
+        eq(len(rev.parents), 1)
+        eq(rev.timezone, 36000)
+        eq(rev.parents[0].revision_id,
+           "mbp@sourcefrog.net-20050905063503-43948f59fa127d92")
+
+    def test_unpack_inventory_5(self):
+        """Unpack canned new-style inventory"""
+        inp = StringIO(_committed_inv_v5)
+        inv = serializer_v5.read_inventory(inp)
+        eq = self.assertEqual
+        eq(len(inv), 4)
+        ie = inv['bar-20050824000535-6bc48cfad47ed134']
+        eq(ie.kind, 'file')
+        eq(ie.text_version, 'mbp@foo-123123')
+        eq(ie.entry_version, 'mbp@foo-00')
+        eq(ie.name, 'bar')
+        eq(inv[ie.parent_id].kind, 'directory')
+
+    def test_repack_inventory_5(self):
+        inp = StringIO(_committed_inv_v5)
+        inv = serializer_v5.read_inventory(inp)
+        outp = StringIO()
+        serializer_v5.write_inventory(inv, outp)
+        inv2 = serializer_v5.read_inventory(StringIO(outp.getvalue()))
+        self.assertEqual(inv, inv2)
+
+    def test_repack_revision_5(self):
+        inp = StringIO(_revision_v5)
+        rev = serializer_v5.read_revision(inp)
+        outp = StringIO()
+        serializer_v5.write_revision(rev, outp)
+        rev2 = serializer_v5.read_revision(StringIO(outp.getvalue()))
+        self.assertEqual(rev, rev2)
+

=== modified file 'bzrlib/selftest/testbranch.py'
--- bzrlib/selftest/testbranch.py	2005-07-11 07:05:34 +0000
+++ bzrlib/selftest/testbranch.py	2005-08-30 01:35:40 +0000
@@ -14,22 +14,35 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-from bzrlib.selftest import InTempDir
-
-
-
-class TestAppendRevisions(InTempDir):
+from bzrlib.selftest import TestCaseInTempDir
+
+
+class TestAppendRevisions(TestCaseInTempDir):
     """Test appending more than one revision"""
-    def runTest(self):
+    def test_append_revisions(self):
         from bzrlib.branch import Branch
         br = Branch(".", init=True)
         br.append_revision("rev1")
         self.assertEquals(br.revision_history(), ["rev1",])
         br.append_revision("rev2", "rev3")
         self.assertEquals(br.revision_history(), ["rev1", "rev2", "rev3"])
-        
-
-
-TEST_CLASSES = [
-    TestAppendRevisions,
-    ]
+
+
+# TODO: rewrite this as a regular unittest, without relying on the displayed output        
+#         >>> from bzrlib.commit import commit
+#         >>> bzrlib.trace.silent = True
+#         >>> br1 = ScratchBranch(files=['foo', 'bar'])
+#         >>> br1.add('foo')
+#         >>> br1.add('bar')
+#         >>> commit(br1, "lala!", rev_id="REVISION-ID-1", verbose=False)
+#         >>> br2 = ScratchBranch()
+#         >>> br2.update_revisions(br1)
+#         Added 2 texts.
+#         Added 1 inventories.
+#         Added 1 revisions.
+#         >>> br2.revision_history()
+#         [u'REVISION-ID-1']
+#         >>> br2.update_revisions(br1)
+#         Added 0 revisions.
+#         >>> br1.text_store.total_size() == br2.text_store.total_size()
+#         True

=== modified file 'bzrlib/selftest/testdiff.py'
--- bzrlib/selftest/testdiff.py	2005-07-27 16:44:44 +0000
+++ bzrlib/selftest/testdiff.py	2005-08-25 12:46:42 +0000
@@ -1,4 +1,4 @@
-from bzrlib.selftest import TestBase
+from bzrlib.selftest import TestCase
 from bzrlib.diff import internal_diff
 from cStringIO import StringIO
 def udiff_lines(old, new):
@@ -21,41 +21,28 @@
     assert '@@' in lines[2][2:], \
         "Unterminated hunk header for patch:\n%s" % "".join(lines)
 
-class AddNL(TestBase):
-    """
-    diff generates a valid diff for patches that add a newline
-    """
-    def runTest(self):
+class TestDiff(TestCase):
+    def test_add_nl(self):
+        """diff generates a valid diff for patches that add a newline"""
         lines = udiff_lines(['boo'], ['boo\n'])
         check_patch(lines)
         assert lines[4] == '\\ No newline at end of file\n', \
             "expected no-nl, got %r" % lines[4]
 
-
-class AddNL2(TestBase):
-    """
-    diff generates a valid diff for patches that change last line and add a
-    newline
-    """
-    def runTest(self):
+    def test_add_nl_2(self):
+        """diff generates a valid diff for patches that change last line and
+        add a newline.
+        """
         lines = udiff_lines(['boo'], ['goo\n'])
         check_patch(lines)
         assert lines[4] == '\\ No newline at end of file\n', \
             "expected no-nl, got %r" % lines[4]
 
-class RemoveNL(TestBase):
-    """
-    diff generates a valid diff for patches that change last line and add a
-    newline
-    """
-    def runTest(self):
+    def test_remove_nl(self):
+        """diff generates a valid diff for patches that change last line and
+        add a newline.
+        """
         lines = udiff_lines(['boo\n'], ['boo'])
         check_patch(lines)
         assert lines[5] == '\\ No newline at end of file\n', \
             "expected no-nl, got %r" % lines[5]
-
-TEST_CLASSES = [
-    AddNL, 
-    AddNL2, 
-    RemoveNL,
-]

=== added file 'bzrlib/selftest/testfetch.py'
--- bzrlib/selftest/testfetch.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testfetch.py	2005-08-30 03:10:32 +0000
@@ -0,0 +1,95 @@
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+import bzrlib.errors
+from bzrlib.selftest.testrevision import make_branches
+from bzrlib.trace import mutter
+from bzrlib.branch import Branch
+import sys
+import os
+
+from bzrlib.selftest import TestCaseInTempDir
+        
+
+class TestFetch(TestCaseInTempDir):
+    def runTest(self):
+        from bzrlib.fetch import greedy_fetch, has_revision
+
+        def new_branch(name):
+            os.mkdir(name)
+            return Branch(name, init=True)
+            
+        #highest indices a: 5, b: 7
+        br_a, br_b = make_branches()
+        assert not has_revision(br_b, br_a.revision_history()[3])
+        assert has_revision(br_b, br_a.revision_history()[2])
+        assert len(br_b.revision_history()) == 7
+        assert greedy_fetch(br_b, br_a, br_a.revision_history()[2])[0] == 0
+
+        # greedy_fetch is not supposed to alter the revision history
+        assert len(br_b.revision_history()) == 7
+        assert not has_revision(br_b, br_a.revision_history()[3])
+
+        assert len(br_b.revision_history()) == 7
+        assert greedy_fetch(br_b, br_a, br_a.revision_history()[3])[0] == 1
+        assert has_revision(br_b, br_a.revision_history()[3])
+        assert not has_revision(br_a, br_b.revision_history()[3])
+        assert not has_revision(br_a, br_b.revision_history()[4])
+
+        # When a non-branch ancestor is missing, it should be a failure, not
+        # exception
+        br_a4 = new_branch('br_a4')
+        count, failures = greedy_fetch(br_a4, br_a)
+        assert count == 6
+        assert failures == set((br_b.revision_history()[4],
+                                br_b.revision_history()[5])) 
+
+        assert greedy_fetch(br_a, br_b)[0] == 4
+        assert has_revision(br_a, br_b.revision_history()[3])
+        assert has_revision(br_a, br_b.revision_history()[4])
+
+        br_b2 = new_branch('br_b2')
+        assert greedy_fetch(br_b2, br_b)[0] == 7
+        assert has_revision(br_b2, br_b.revision_history()[4])
+        assert has_revision(br_b2, br_a.revision_history()[2])
+        assert not has_revision(br_b2, br_a.revision_history()[3])
+
+        br_a2 = new_branch('br_a2')
+        assert greedy_fetch(br_a2, br_a)[0] == 9
+        assert has_revision(br_a2, br_b.revision_history()[4])
+        assert has_revision(br_a2, br_a.revision_history()[3])
+
+        br_a3 = new_branch('br_a3')
+        assert greedy_fetch(br_a3, br_a2)[0] == 0
+        for revno in range(4):
+            assert not has_revision(br_a3, br_a.revision_history()[revno])
+        assert greedy_fetch(br_a3, br_a2, br_a.revision_history()[2])[0] == 3
+        fetched = greedy_fetch(br_a3, br_a2, br_a.revision_history()[3])[0]
+        assert fetched == 3, "fetched %d instead of 3" % fetched
+        # InstallFailed should be raised if the branch is missing the revision
+        # that was requested.
+        self.assertRaises(bzrlib.errors.InstallFailed, greedy_fetch, br_a3,
+                          br_a2, 'pizza')
+        # InstallFailed should be raised if the branch is missing a revision
+        # from its own revision history
+        br_a2.append_revision('a-b-c')
+        self.assertRaises(bzrlib.errors.InstallFailed, greedy_fetch, br_a3,
+                          br_a2)
+
+
+
+if __name__ == '__main__':
+    import unittest
+    sys.exit(unittest.run_suite(unittest.makeSuite()))

=== added file 'bzrlib/selftest/testgraph.py'
--- bzrlib/selftest/testgraph.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testgraph.py	2005-09-10 23:46:35 +0000
@@ -0,0 +1,36 @@
+from bzrlib.selftest import TestCase
+from bzrlib.graph import farthest_nodes
+
+class TestBase(TestCase):
+    def edge_add(self, *args):
+        for start, end in zip(args[:-1], args[1:]):
+            if start not in self.graph:
+                self.graph[start] = {}
+            if end not in self.graph:
+                self.graph[end] = {}
+            self.graph[start][end] = 1
+
+    def setUp(self):
+        TestCase.setUp(self)
+        self.graph = {}
+        self.edge_add('A', 'B', 'C', 'D')
+        self.edge_add('A', 'E', 'F', 'C')
+        self.edge_add('A', 'G', 'H', 'I', 'B')
+        self.edge_add('A', 'J', 'K', 'L', 'M', 'N')
+        self.edge_add('O', 'N')
+    
+    def test_farthest(self):
+        descendants = {'A':set()}
+        for node in self.graph:
+            for ancestor in self.graph[node]:
+                if ancestor not in descendants:
+                    descendants[ancestor] = set()
+                descendants[ancestor].add(node)
+        nodes = farthest_nodes(self.graph, descendants, 'A')
+        self.assertEqual(nodes[0], 'D')
+        assert nodes[1] in ('N', 'C')
+        assert nodes[2] in ('N', 'C')
+        assert nodes[3] in ('B', 'M')
+        assert nodes[4] in ('B', 'M')
+
+

=== modified file 'bzrlib/selftest/testhashcache.py'
--- bzrlib/selftest/testhashcache.py	2005-07-08 06:54:58 +0000
+++ bzrlib/selftest/testhashcache.py	2005-08-30 03:10:32 +0000
@@ -14,7 +14,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-from bzrlib.selftest import InTempDir
+from bzrlib.selftest import TestCaseInTempDir
 
 
 
@@ -31,13 +31,12 @@
         time.sleep(0.2)
     
 
+class TestHashCache(TestCaseInTempDir):
 
-class TestHashCache(InTempDir):
-    """Functional tests for hashcache"""
-    def runTest(self):
+    def test_hashcache(self):
+        """Functional tests for hashcache"""
         from bzrlib.hashcache import HashCache
         import os
-        import time
 
         # make a dummy bzr directory just to hold the cache
         os.mkdir('.bzr')
@@ -112,8 +111,3 @@
         self.assertEquals(hc.hit_count, 1)
         self.assertEquals(hc.miss_count, 0)
         self.assertEquals(hc.get_sha1('foo2'), sha1('new content'))
-
-        
-
-        
-

=== modified file 'bzrlib/selftest/testinv.py'
--- bzrlib/selftest/testinv.py	2005-07-23 00:18:14 +0000
+++ bzrlib/selftest/testinv.py	2005-09-05 08:00:35 +0000
@@ -14,13 +14,14 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-from bzrlib.selftest import TestBase
+from bzrlib.selftest import TestCase
 
 from bzrlib.inventory import Inventory, InventoryEntry
 
 
-class TestIsWithin(TestBase):
-    def runTest(self):
+class TestInventory(TestCase):
+
+    def test_is_within(self):
         from bzrlib.osutils import is_inside_any
         
         for dirs, fn in [(['src', 'doc'], 'src/foo.c'),
@@ -33,10 +34,7 @@
                          (['src'], 'srccontrol/foo')]:
             self.assertFalse(is_inside_any(dirs, fn))
             
-            
-            
-class TestInventoryIds(TestBase):
-    def runTest(self):
+    def test_ids(self):
         """Test detection of files within selected directories."""
         inv = Inventory()
         
@@ -51,3 +49,11 @@
         self.assertEqual(inv.path2id('src/bye.c'), 'bye-id')
         
         self.assert_('src-id' in inv)
+
+
+    def test_version(self):
+        """Inventory remembers the text's version."""
+        inv = Inventory()
+        ie = inv.add_path('foo.txt', 'file')
+        ## XXX
+

=== added file 'bzrlib/selftest/testlog.py'
--- bzrlib/selftest/testlog.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testlog.py	2005-09-05 01:31:37 +0000
@@ -0,0 +1,134 @@
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import os
+from cStringIO import StringIO
+
+from bzrlib.selftest import BzrTestBase, TestCaseInTempDir
+from bzrlib.log import LogFormatter, show_log, LongLogFormatter
+from bzrlib.branch import Branch
+from bzrlib.errors import InvalidRevisionNumber
+
+class _LogEntry(object):
+    # should probably move into bzrlib.log?
+    pass
+
+
+class LogCatcher(LogFormatter):
+    """Pull log messages into list rather than displaying them.
+
+    For ease of testing we save log messages here rather than actually
+    formatting them, so that we can precisely check the result without
+    being too dependent on the exact formatting.
+
+    We should also test the LogFormatter.
+    """
+    def __init__(self):
+        super(LogCatcher, self).__init__(to_file=None)
+        self.logs = []
+        
+        
+    def show(self, revno, rev, delta):
+        le = _LogEntry()
+        le.revno = revno
+        le.rev = rev
+        le.delta = delta
+        self.logs.append(le)
+
+
+class SimpleLogTest(TestCaseInTempDir):
+
+    def checkDelta(self, delta, **kw):
+        """Check the filenames touched by a delta are as expected."""
+        for n in 'added', 'removed', 'renamed', 'modified', 'unchanged':
+            expected = kw.get(n, [])
+
+            # tests are written with unix paths; fix them up for windows
+            if os.sep != '/':
+                expected = [x.replace('/', os.sep) for x in expected]
+
+            # strip out only the path components
+            got = [x[0] for x in getattr(delta, n)]
+            self.assertEquals(expected, got)
+
+    def test_cur_revno(self):
+        b = Branch('.', init=True)
+
+        lf = LogCatcher()
+        b.commit('empty commit')
+        show_log(b, lf, verbose=True, start_revision=1, end_revision=1)
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=2, end_revision=1) 
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=1, end_revision=2) 
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=0, end_revision=2) 
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=1, end_revision=0) 
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=-1, end_revision=1) 
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=1, end_revision=-1) 
+
+    def test_simple_log(self):
+        eq = self.assertEquals
+        
+        b = Branch('.', init=True)
+
+        lf = LogCatcher()
+        show_log(b, lf)
+        # no entries yet
+        eq(lf.logs, [])
+
+
+        b.commit('empty commit')
+        lf = LogCatcher()
+        show_log(b, lf, verbose=True)
+        eq(len(lf.logs), 1)
+        eq(lf.logs[0].revno, 1)
+        eq(lf.logs[0].rev.message, 'empty commit')
+        d = lf.logs[0].delta
+        self.log('log delta: %r' % d)
+        self.checkDelta(d)
+
+
+        self.build_tree(['hello'])
+        b.add('hello')
+        b.commit('add one file')
+
+        lf = StringIO()
+        # log using regular thing
+        show_log(b, LongLogFormatter(lf))
+        lf.seek(0)
+        for l in lf.readlines():
+            self.log(l)
+
+        # get log as data structure
+        lf = LogCatcher()
+        show_log(b, lf, verbose=True)
+        eq(len(lf.logs), 2)
+        self.log('log entries:')
+        for logentry in lf.logs:
+            self.log('%4d %s' % (logentry.revno, logentry.rev.message))
+        
+        # first one is most recent
+        logentry = lf.logs[0]
+        eq(logentry.revno, 2)
+        eq(logentry.rev.message, 'add one file')
+        d = logentry.delta
+        self.log('log 2 delta: %r' % d)
+        # self.checkDelta(d, added=['hello'])
+        

=== added file 'bzrlib/selftest/testmerge.py'
--- bzrlib/selftest/testmerge.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testmerge.py	2005-09-05 07:10:59 +0000
@@ -0,0 +1,12 @@
+from bzrlib.branch import Branch
+from bzrlib.commit import commit
+from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.merge import merge
+class TestMerge(TestCaseInTempDir):
+    """Test appending more than one revision"""
+    def test_pending(self):
+        br = Branch(".", init=True)
+        commit(br, "lala!")
+        self.assertEquals(len(br.pending_merges()), 0)
+        merge(['.', -1], [None, None])
+        self.assertEquals(len(br.pending_merges()), 0)

=== modified file 'bzrlib/selftest/testmerge3.py'
--- bzrlib/selftest/testmerge3.py	2005-07-06 05:24:29 +0000
+++ bzrlib/selftest/testmerge3.py	2005-09-04 02:59:56 +0000
@@ -15,16 +15,85 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-from bzrlib.selftest import InTempDir, TestBase
+from bzrlib.selftest import TestCaseInTempDir, TestCase
 from bzrlib.merge3 import Merge3
 
-
-
-
-
-class NoChanges(TestBase):
-    """No conflicts because nothing changed"""
-    def runTest(self):
+def split_lines(t):
+    from cStringIO import StringIO
+    return StringIO(t).readlines()
+
+############################################################
+# test case data from the gnu diffutils manual
+# common base
+TZU = split_lines("""     The Nameless is the origin of Heaven and Earth;
+     The named is the mother of all things.
+     
+     Therefore let there always be non-being,
+       so we may see their subtlety,
+     And let there always be being,
+       so we may see their outcome.
+     The two are the same,
+     But after they are produced,
+       they have different names.
+     They both may be called deep and profound.
+     Deeper and more profound,
+     The door of all subtleties!
+""")
+
+LAO = split_lines("""     The Way that can be told of is not the eternal Way;
+     The name that can be named is not the eternal name.
+     The Nameless is the origin of Heaven and Earth;
+     The Named is the mother of all things.
+     Therefore let there always be non-being,
+       so we may see their subtlety,
+     And let there always be being,
+       so we may see their outcome.
+     The two are the same,
+     But after they are produced,
+       they have different names.
+""")
+
+
+TAO = split_lines("""     The Way that can be told of is not the eternal Way;
+     The name that can be named is not the eternal name.
+     The Nameless is the origin of Heaven and Earth;
+     The named is the mother of all things.
+     
+     Therefore let there always be non-being,
+       so we may see their subtlety,
+     And let there always be being,
+       so we may see their result.
+     The two are the same,
+     But after they are produced,
+       they have different names.
+     
+       -- The Way of Lao-Tzu, tr. Wing-tsit Chan
+
+""")
+
+MERGED_RESULT = split_lines("""     The Way that can be told of is not the eternal Way;
+     The name that can be named is not the eternal name.
+     The Nameless is the origin of Heaven and Earth;
+     The Named is the mother of all things.
+     Therefore let there always be non-being,
+       so we may see their subtlety,
+     And let there always be being,
+       so we may see their result.
+     The two are the same,
+     But after they are produced,
+       they have different names.
+<<<<<<< LAO
+=======
+     
+       -- The Way of Lao-Tzu, tr. Wing-tsit Chan
+
+>>>>>>> TAO
+""")
+
+class TestMerge3(TestCase):
+
+    def test_no_changes(self):
+        """No conflicts because nothing changed"""
         m3 = Merge3(['aaa', 'bbb'],
                     ['aaa', 'bbb'],
                     ['aaa', 'bbb'])
@@ -44,9 +113,7 @@
         self.assertEquals(list(m3.merge_groups()),
                           [('unchanged', ['aaa', 'bbb'])])
 
-
-class FrontInsert(TestBase):
-    def runTest(self):
+    def test_front_insert(self):
         m3 = Merge3(['zz'],
                     ['aaa', 'bbb', 'zz'],
                     ['zz'])
@@ -65,14 +132,10 @@
                           [('a', ['aaa', 'bbb']),
                            ('unchanged', ['zz'])])
         
-    
-
-class NullInsert(TestBase):
-    def runTest(self):
+    def test_null_insert(self):
         m3 = Merge3([],
                     ['aaa', 'bbb'],
                     [])
-
         # todo: should use a sentinal at end as from get_matching_blocks
         # to match without zz
         self.assertEquals(list(m3.find_sync_regions()),
@@ -83,12 +146,9 @@
 
         self.assertEquals(list(m3.merge_lines()),
                           ['aaa', 'bbb'])
-        
-    
 
-class NoConflicts(TestBase):
-    """No conflicts because only one side changed"""
-    def runTest(self):
+    def test_no_conflicts(self):
+        """No conflicts because only one side changed"""
         m3 = Merge3(['aaa', 'bbb'],
                     ['aaa', '111', 'bbb'],
                     ['aaa', 'bbb'])
@@ -106,10 +166,7 @@
                            ('a', 1, 2),
                            ('unchanged', 1, 2),])
 
-
-
-class AppendA(TestBase):
-    def runTest(self):
+    def test_append_a(self):
         m3 = Merge3(['aaa\n', 'bbb\n'],
                     ['aaa\n', 'bbb\n', '222\n'],
                     ['aaa\n', 'bbb\n'])
@@ -117,8 +174,7 @@
         self.assertEquals(''.join(m3.merge_lines()),
                           'aaa\nbbb\n222\n')
 
-class AppendB(TestBase):
-    def runTest(self):
+    def test_append_b(self):
         m3 = Merge3(['aaa\n', 'bbb\n'],
                     ['aaa\n', 'bbb\n'],
                     ['aaa\n', 'bbb\n', '222\n'])
@@ -126,8 +182,7 @@
         self.assertEquals(''.join(m3.merge_lines()),
                           'aaa\nbbb\n222\n')
 
-class AppendAgreement(TestBase):
-    def runTest(self):
+    def test_append_agreement(self):
         m3 = Merge3(['aaa\n', 'bbb\n'],
                     ['aaa\n', 'bbb\n', '222\n'],
                     ['aaa\n', 'bbb\n', '222\n'])
@@ -135,8 +190,7 @@
         self.assertEquals(''.join(m3.merge_lines()),
                           'aaa\nbbb\n222\n')
 
-class AppendClash(TestBase):
-    def runTest(self):
+    def test_append_clash(self):
         m3 = Merge3(['aaa\n', 'bbb\n'],
                     ['aaa\n', 'bbb\n', '222\n'],
                     ['aaa\n', 'bbb\n', '333\n'])
@@ -157,9 +211,7 @@
 >> b
 ''')
 
-
-class InsertAgreement(TestBase):
-    def runTest(self):
+    def test_insert_agreement(self):
         m3 = Merge3(['aaa\n', 'bbb\n'],
                     ['aaa\n', '222\n', 'bbb\n'],
                     ['aaa\n', '222\n', 'bbb\n'])
@@ -169,15 +221,11 @@
                             start_marker='<<',
                             mid_marker='--',
                             end_marker='>>')
-        self.assertEquals(''.join(m3.merge_lines()),
-                          'aaa\n222\nbbb\n')
-
-
-
-
-class InsertClash(TestBase):
-    """Both try to insert lines in the same place."""
-    def runTest(self):
+        self.assertEquals(''.join(ml), 'aaa\n222\nbbb\n')
+        
+
+    def test_insert_clash(self):
+        """Both try to insert lines in the same place."""
         m3 = Merge3(['aaa\n', 'bbb\n'],
                     ['aaa\n', '111\n', 'bbb\n'],
                     ['aaa\n', '222\n', 'bbb\n'])
@@ -216,11 +264,8 @@
 bbb
 ''')
 
-
-
-class ReplaceClash(TestBase):
-    """Both try to insert lines in the same place."""
-    def runTest(self):
+    def test_replace_clash(self):
+        """Both try to insert lines in the same place."""
         m3 = Merge3(['aaa', '000', 'bbb'],
                     ['aaa', '111', 'bbb'],
                     ['aaa', '222', 'bbb'])
@@ -233,11 +278,8 @@
                            (2,3, 2,3, 2,3),
                            (3,3, 3,3, 3,3),])
 
-
-
-class ReplaceMulti(TestBase):
-    """Replacement with regions of different size."""
-    def runTest(self):
+    def test_replace_multi(self):
+        """Replacement with regions of different size."""
         m3 = Merge3(['aaa', '000', '000', 'bbb'],
                     ['aaa', '111', '111', '111', 'bbb'],
                     ['aaa', '222', '222', '222', '222', 'bbb'])
@@ -251,112 +293,10 @@
                            (3,4, 4,5, 5,6),
                            (4,4, 5,5, 6,6),])
 
-        
-        
-
-
-
-
-def split_lines(t):
-    from cStringIO import StringIO
-    return StringIO(t).readlines()
-
-
-
-############################################################
-# test case from the gnu diffutils manual
-
-# common base
-TZU = split_lines("""     The Nameless is the origin of Heaven and Earth;
-     The named is the mother of all things.
-     
-     Therefore let there always be non-being,
-       so we may see their subtlety,
-     And let there always be being,
-       so we may see their outcome.
-     The two are the same,
-     But after they are produced,
-       they have different names.
-     They both may be called deep and profound.
-     Deeper and more profound,
-     The door of all subtleties!
-""")
-
-LAO = split_lines("""     The Way that can be told of is not the eternal Way;
-     The name that can be named is not the eternal name.
-     The Nameless is the origin of Heaven and Earth;
-     The Named is the mother of all things.
-     Therefore let there always be non-being,
-       so we may see their subtlety,
-     And let there always be being,
-       so we may see their outcome.
-     The two are the same,
-     But after they are produced,
-       they have different names.
-""")
-
-
-TAO = split_lines("""     The Way that can be told of is not the eternal Way;
-     The name that can be named is not the eternal name.
-     The Nameless is the origin of Heaven and Earth;
-     The named is the mother of all things.
-     
-     Therefore let there always be non-being,
-       so we may see their subtlety,
-     And let there always be being,
-       so we may see their result.
-     The two are the same,
-     But after they are produced,
-       they have different names.
-     
-       -- The Way of Lao-Tzu, tr. Wing-tsit Chan
-
-""")
-
-MERGED_RESULT = split_lines("""     The Way that can be told of is not the eternal Way;
-     The name that can be named is not the eternal name.
-     The Nameless is the origin of Heaven and Earth;
-     The Named is the mother of all things.
-     Therefore let there always be non-being,
-       so we may see their subtlety,
-     And let there always be being,
-       so we may see their result.
-     The two are the same,
-     But after they are produced,
-       they have different names.
-<<<<<<<< LAO
-========
-     
-       -- The Way of Lao-Tzu, tr. Wing-tsit Chan
-
->>>>>>>> TAO
-""")
-
-
-
-class MergePoem(TestBase):
-    """Test case from diff3 manual"""
-    def runTest(self):
+    def test_merge_poem(self):
+        """Test case from diff3 manual"""
         m3 = Merge3(TZU, LAO, TAO)
         ml = list(m3.merge_lines('LAO', 'TAO'))
         self.log('merge result:')
         self.log(''.join(ml))
         self.assertEquals(ml, MERGED_RESULT)
-
-
-
-TEST_CLASSES = [
-    NoChanges,
-    FrontInsert,
-    NullInsert,
-    NoConflicts,
-    AppendA,
-    AppendB,
-    AppendAgreement,
-    AppendClash,
-    InsertAgreement,
-    InsertClash,
-    ReplaceClash,
-    ReplaceMulti,
-    MergePoem,
-    ]

=== added file 'bzrlib/selftest/testrevision.py'
--- bzrlib/selftest/testrevision.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testrevision.py	2005-09-10 23:37:36 +0000
@@ -0,0 +1,214 @@
+# (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+from bzrlib.selftest import TestCaseInTempDir
+
+
+def make_branches():
+    from bzrlib.branch import Branch
+    from bzrlib.commit import commit
+    import os
+    os.mkdir("branch1")
+    br1 = Branch("branch1", init=True)
+    
+    commit(br1, "Commit one", rev_id="a@u-0-0")
+    commit(br1, "Commit two", rev_id="a@u-0-1")
+    commit(br1, "Commit three", rev_id="a@u-0-2")
+
+    os.mkdir("branch2")
+    br2 = Branch("branch2", init=True)
+    br2.update_revisions(br1)
+    commit(br2, "Commit four", rev_id="b@u-0-3")
+    commit(br2, "Commit five", rev_id="b@u-0-4")
+    revisions_2 = br2.revision_history()
+    br1.add_pending_merge(revisions_2[4])
+    commit(br1, "Commit six", rev_id="a@u-0-3")
+    commit(br1, "Commit seven", rev_id="a@u-0-4")
+    commit(br2, "Commit eight", rev_id="b@u-0-5")
+    br1.add_pending_merge(br2.revision_history()[5])
+    commit(br1, "Commit nine", rev_id="a@u-0-5")
+    br2.add_pending_merge(br1.revision_history()[4])
+    commit(br2, "Commit ten", rev_id="b@u-0-6")
+    return br1, br2
+
+
+class TestIsAncestor(TestCaseInTempDir):
+    def test_is_ancestor(self):
+        """Test checking whether a revision is an ancestor of another revision"""
+        from bzrlib.revision import is_ancestor, MultipleRevisionSources
+        from bzrlib.errors import NoSuchRevision
+        br1, br2 = make_branches()
+        revisions = br1.revision_history()
+        revisions_2 = br2.revision_history()
+        sources = MultipleRevisionSources(br1, br2)
+
+        assert is_ancestor(revisions[0], revisions[0], sources)
+        assert is_ancestor(revisions[1], revisions[0], sources)
+        assert not is_ancestor(revisions[0], revisions[1], sources)
+        assert is_ancestor(revisions_2[3], revisions[0], sources)
+        self.assertRaises(NoSuchRevision, is_ancestor, revisions_2[3],
+                          revisions[0], br1)        
+        assert is_ancestor(revisions[3], revisions_2[4], sources)
+        assert is_ancestor(revisions[3], revisions_2[4], br1)
+        assert is_ancestor(revisions[3], revisions_2[3], sources)
+        assert not is_ancestor(revisions[3], revisions_2[3], br1)
+
+class TestIntermediateRevisions(TestCaseInTempDir):
+
+    def setUp(self):
+        from bzrlib.commit import commit
+        TestCaseInTempDir.setUp(self)
+        self.br1, self.br2 = make_branches()
+        commit(self.br2, "Commit eleven", rev_id="b@u-0-7")
+        commit(self.br2, "Commit twelve", rev_id="b@u-0-8")
+        commit(self.br2, "Commit thirtteen", rev_id="b@u-0-9")
+        self.br1.add_pending_merge(self.br2.revision_history()[6])
+        commit(self.br1, "Commit fourtten", rev_id="a@u-0-6")
+        self.br2.add_pending_merge(self.br1.revision_history()[6])
+        commit(self.br2, "Commit fifteen", rev_id="b@u-0-10")
+
+        from bzrlib.revision import MultipleRevisionSources
+        self.sources = MultipleRevisionSources(self.br1, self.br2)
+
+    def intervene(self, ancestor, revision, revision_history=None):
+        from bzrlib.revision import get_intervening_revisions
+        return get_intervening_revisions(ancestor,revision, self.sources, 
+                                         revision_history)
+
+    def test_intervene(self):
+        """Find intermediate revisions, without requiring history"""
+        from bzrlib.errors import NotAncestor, NoSuchRevision
+        assert len(self.intervene('a@u-0-0', 'a@u-0-0')) == 0
+        self.assertEqual(self.intervene('a@u-0-0', 'a@u-0-1'), ['a@u-0-1'])
+        self.assertEqual(self.intervene('a@u-0-0', 'a@u-0-2'), 
+                         ['a@u-0-1', 'a@u-0-2'])
+        self.assertEqual(self.intervene('a@u-0-0', 'b@u-0-3'), 
+                         ['a@u-0-1', 'a@u-0-2', 'b@u-0-3'])
+        self.assertEqual(self.intervene('b@u-0-3', 'a@u-0-3'), 
+                         ['b@u-0-4', 'a@u-0-3'])
+        self.assertEqual(self.intervene('a@u-0-2', 'a@u-0-3', 
+                                        self.br1.revision_history()), 
+                         ['a@u-0-3'])
+        self.assertEqual(self.intervene('a@u-0-0', 'a@u-0-5', 
+                                        self.br1.revision_history()), 
+                         ['a@u-0-1', 'a@u-0-2', 'a@u-0-3', 'a@u-0-4', 
+                          'a@u-0-5'])
+        self.assertEqual(self.intervene('a@u-0-0', 'b@u-0-6', 
+                         self.br1.revision_history()), 
+                         ['a@u-0-1', 'a@u-0-2', 'a@u-0-3', 'a@u-0-4', 
+                          'b@u-0-6'])
+        self.assertEqual(self.intervene('a@u-0-0', 'b@u-0-5'), 
+                         ['a@u-0-1', 'a@u-0-2', 'b@u-0-3', 'b@u-0-4', 
+                          'b@u-0-5'])
+        self.assertEqual(self.intervene('b@u-0-3', 'b@u-0-6', 
+                         self.br2.revision_history()), 
+                         ['b@u-0-4', 'b@u-0-5', 'b@u-0-6'])
+        self.assertEqual(self.intervene('b@u-0-6', 'b@u-0-10'), 
+                         ['b@u-0-7', 'b@u-0-8', 'b@u-0-9', 'b@u-0-10'])
+        self.assertEqual(self.intervene('b@u-0-6', 'b@u-0-10', 
+                                        self.br2.revision_history()), 
+                         ['b@u-0-7', 'b@u-0-8', 'b@u-0-9', 'b@u-0-10'])
+        self.assertRaises(NotAncestor, self.intervene, 'b@u-0-10', 'b@u-0-6', 
+                          self.br2.revision_history())
+        self.assertRaises(NoSuchRevision, self.intervene, 'c@u-0-10', 
+                          'b@u-0-6', self.br2.revision_history())
+        self.assertRaises(NoSuchRevision, self.intervene, 'b@u-0-10', 
+                          'c@u-0-6', self.br2.revision_history())
+
+
+class TestCommonAncestor(TestCaseInTempDir):
+    """Test checking whether a revision is an ancestor of another revision"""
+
+    def test_old_common_ancestor(self):
+        """Pick a resonable merge base using the old functionality"""
+        from bzrlib.revision import find_present_ancestors
+        from bzrlib.revision import old_common_ancestor as common_ancestor
+        from bzrlib.revision import MultipleRevisionSources
+        br1, br2 = make_branches()
+        revisions = br1.revision_history()
+        revisions_2 = br2.revision_history()
+        sources = MultipleRevisionSources(br1, br2)
+
+        expected_ancestors_list = {revisions[3]:(0, 0), 
+                                   revisions[2]:(1, 1),
+                                   revisions_2[4]:(2, 1), 
+                                   revisions[1]:(3, 2),
+                                   revisions_2[3]:(4, 2),
+                                   revisions[0]:(5, 3) }
+        ancestors_list = find_present_ancestors(revisions[3], sources)
+        assert len(expected_ancestors_list) == len(ancestors_list)
+        for key, value in expected_ancestors_list.iteritems():
+            self.assertEqual(ancestors_list[key], value, 
+                              "key %r, %r != %r" % (key, ancestors_list[key],
+                                                    value))
+
+        self.assertEqual(common_ancestor(revisions[0], revisions[0], sources),
+                          revisions[0])
+        self.assertEqual(common_ancestor(revisions[1], revisions[2], sources),
+                          revisions[1])
+        self.assertEqual(common_ancestor(revisions[1], revisions[1], sources),
+                          revisions[1])
+        self.assertEqual(common_ancestor(revisions[2], revisions_2[4], sources),
+                          revisions[2])
+        self.assertEqual(common_ancestor(revisions[3], revisions_2[4], sources),
+                          revisions_2[4])
+        self.assertEqual(common_ancestor(revisions[4], revisions_2[5], sources),
+                          revisions_2[4])
+        self.assertEqual(common_ancestor(revisions[5], revisions_2[6], sources),
+                          revisions[4])
+        self.assertEqual(common_ancestor(revisions_2[6], revisions[5], sources),
+                          revisions_2[5])
+
+    def test_common_ancestor(self):
+        """Pick a reasonable merge base"""
+        from bzrlib.revision import find_present_ancestors
+        from bzrlib.revision import common_ancestor
+        from bzrlib.revision import MultipleRevisionSources
+        br1, br2 = make_branches()
+        revisions = br1.revision_history()
+        revisions_2 = br2.revision_history()
+        sources = MultipleRevisionSources(br1, br2)
+
+        expected_ancestors_list = {revisions[3]:(0, 0), 
+                                   revisions[2]:(1, 1),
+                                   revisions_2[4]:(2, 1), 
+                                   revisions[1]:(3, 2),
+                                   revisions_2[3]:(4, 2),
+                                   revisions[0]:(5, 3) }
+        ancestors_list = find_present_ancestors(revisions[3], sources)
+        assert len(expected_ancestors_list) == len(ancestors_list)
+        for key, value in expected_ancestors_list.iteritems():
+            self.assertEqual(ancestors_list[key], value, 
+                              "key %r, %r != %r" % (key, ancestors_list[key],
+                                                    value))
+
+        self.assertEqual(common_ancestor(revisions[0], revisions[0], sources),
+                          revisions[0])
+        self.assertEqual(common_ancestor(revisions[1], revisions[2], sources),
+                          revisions[1])
+        self.assertEqual(common_ancestor(revisions[1], revisions[1], sources),
+                          revisions[1])
+        self.assertEqual(common_ancestor(revisions[2], revisions_2[4], sources),
+                          revisions[2])
+        self.assertEqual(common_ancestor(revisions[3], revisions_2[4], sources),
+                          revisions_2[4])
+        self.assertEqual(common_ancestor(revisions[4], revisions_2[5], sources),
+                          revisions_2[4])
+        self.assertEqual(common_ancestor(revisions[5], revisions_2[6], sources),
+                          revisions[4])
+        self.assertEqual(common_ancestor(revisions_2[6], revisions[5], sources),
+                          revisions[4])
+

=== modified file 'bzrlib/selftest/testrevisionnamespaces.py'
--- bzrlib/selftest/testrevisionnamespaces.py	2005-07-11 06:13:18 +0000
+++ bzrlib/selftest/testrevisionnamespaces.py	2005-09-04 02:59:56 +0000
@@ -14,16 +14,14 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-
-from bzrlib.selftest import InTempDir, TestBase
-
-class TestRevisionNamespaces(InTempDir):
-    """Functional tests for hashcache"""
-    def runTest(self):
-        from bzrlib.errors import BzrError
+import os
+from bzrlib.selftest import TestCaseInTempDir
+
+class TestRevisionNamespaces(TestCaseInTempDir):
+    def test_revision_namespaces(self):
+        """Functional tests for hashcache"""
+        from bzrlib.errors import NoSuchRevision
         from bzrlib.branch import Branch
-        import os
-        import time
 
         b = Branch('.', init=True)
 
@@ -31,16 +29,19 @@
         b.commit('Commit two', rev_id='a@r-0-2')
         b.commit('Commit three', rev_id='a@r-0-3')
 
+        self.assertEquals(b.get_revision_info(None), (0, None))
         self.assertEquals(b.get_revision_info(1), (1, 'a@r-0-1'))
         self.assertEquals(b.get_revision_info('revno:1'), (1, 'a@r-0-1'))
         self.assertEquals(b.get_revision_info('revid:a@r-0-1'), (1, 'a@r-0-1'))
-        self.assertRaises(BzrError, b.get_revision_info, 'revid:a@r-0-0')
+        self.assertRaises(NoSuchRevision, b.get_revision_info, 'revid:a@r-0-0')
+        self.assertRaises(TypeError, b.get_revision_info, object)
 
         self.assertEquals(b.get_revision_info('date:-tomorrow'), (3, 'a@r-0-3'))
         self.assertEquals(b.get_revision_info('date:+today'), (1, 'a@r-0-1'))
 
         self.assertEquals(b.get_revision_info('last:1'), (3, 'a@r-0-3'))
+        self.assertEquals(b.get_revision_info('-1'), (3, 'a@r-0-3'))
 
-TEST_CLASSES = [
-    TestRevisionNamespaces
-    ]
+        os.mkdir('newbranch')
+        b2 = Branch('newbranch', init=True)
+        self.assertEquals(b2.lookup_revision('revid:a@r-0-1'), 'a@r-0-1')

=== modified file 'bzrlib/selftest/teststatus.py'
--- bzrlib/selftest/teststatus.py	2005-07-12 01:44:23 +0000
+++ bzrlib/selftest/teststatus.py	2005-09-04 02:59:56 +0000
@@ -21,10 +21,11 @@
 """
 
 
-from bzrlib.selftest import InTempDir
+from bzrlib.selftest import TestCaseInTempDir
 
-class BranchStatus(InTempDir):
-    def runTest(self): 
+class BranchStatus(TestCaseInTempDir):
+    
+    def test_branch_status(self): 
         """Basic 'bzr mkdir' operation"""
         from cStringIO import StringIO
         from bzrlib.status import show_status
@@ -39,11 +40,14 @@
 
         tof = StringIO()
         self.build_tree(['hello.c', 'bye.c'])
+        b.add_pending_merge('pending@pending-0-0')
         show_status(b, to_file=tof)
         tof.seek(0)
         self.assertEquals(tof.readlines(),
                           ['unknown:\n',
                            '  bye.c\n',
                            '  hello.c\n',
+                           'pending merges:\n',
+                           '  pending@pending-0-0\n'
                            ])
 

=== added file 'bzrlib/selftest/teststore.py'
--- bzrlib/selftest/teststore.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/teststore.py	2005-09-04 02:59:56 +0000
@@ -0,0 +1,33 @@
+# Copyright (C) 2005 by Canonical Development Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Test Store implementation
+"""
+from bzrlib.store import ImmutableStore
+from bzrlib.selftest import TestCaseInTempDir
+from StringIO import StringIO
+from bzrlib.errors import BzrError
+
+class TestStore(TestCaseInTempDir):
+    def test_multiple_add(self):
+        """Multiple add with same ID should raise a BzrError"""
+        store = ImmutableStore('.')
+        store.add(StringIO('goodbye'), '123123')
+        self.assertRaises(BzrError, store.add, StringIO('goodbye'), '123123')
+
+TEST_CLASSES = [
+    TestStore,
+    ]

=== modified file 'bzrlib/selftest/versioning.py'
--- bzrlib/selftest/versioning.py	2005-07-23 14:06:37 +0000
+++ bzrlib/selftest/versioning.py	2005-09-12 01:29:49 +0000
@@ -17,14 +17,24 @@
 
 """Tests of simple versioning operations"""
 
-
-from bzrlib.selftest import InTempDir
-
-class Mkdir(InTempDir):
-    def runTest(self): 
+# TODO: test trying to commit within a directory that is not yet added
+
+
+import os
+
+from bzrlib.selftest import BzrTestBase, TestCaseInTempDir
+from bzrlib.branch import Branch
+
+import logging
+logger = logging.getLogger('bzr.test.versioning')
+debug = logger.debug
+
+
+class TestVersioning(TestCaseInTempDir):
+    
+    def test_mkdir(self): 
         """Basic 'bzr mkdir' operation"""
         from bzrlib.commands import run_bzr
-        import os
 
         run_bzr(['init'])
         run_bzr(['mkdir', 'foo'])
@@ -32,27 +42,25 @@
 
         self.assertRaises(OSError, run_bzr, ['mkdir', 'foo'])
 
-        from bzrlib.diff import compare_trees, TreeDelta
+        from bzrlib.diff import compare_trees
         from bzrlib.branch import Branch
         b = Branch('.')
         
         delta = compare_trees(b.basis_tree(), b.working_tree())
 
+        self.log('delta.added = %r' % delta.added)
+
         self.assertEquals(len(delta.added), 1)
         self.assertEquals(delta.added[0][0], 'foo')
         self.failIf(delta.modified)
 
-
-
-class AddInUnversioned(InTempDir):
-    def runTest(self):
+    def test_branch_add_in_unversioned(self):
         """Try to add a file in an unversioned directory.
 
-        smart_add may eventually add the parent as necessary, but simple
-        branch add doesn't do that.
+        "bzr add" adds the parent as necessary, but simple branch add
+        doesn't do that.
         """
         from bzrlib.branch import Branch
-        import os
         from bzrlib.errors import NotVersionedError
 
         b = Branch('.', init=True)
@@ -64,29 +72,153 @@
                           b.add,
                           'foo/hello')
         
-        
-class SubdirCommit(InTempDir):
-    def runTest(self):
-        """Various commits from a subdirectory"""
-        
-        from bzrlib.branch import Branch
-        from bzrlib.commands import run_bzr
-        import os
-        
-        b = Branch('.', init=True)
-        self.build_tree(['src/', 'src/foo.c', 'README'])
-        
-        self.assertEqual(sorted(b.unknowns()),
-                         ['README', 'src'])
-        
-        self.assertEqual(run_bzr(['version']), 0)
-        
-        self.assertEqual(run_bzr(['add']), 0)
-        
-        
-
-
-TEST_CLASSES = [
-    Mkdir,
-    AddInUnversioned,
-    ]
+        self.check_and_upgrade()
+
+
+    def test_add_in_unversioned(self):
+        """Try to add a file in an unversioned directory.
+
+        "bzr add" should add the parent(s) as necessary.
+        """
+        from bzrlib.branch import Branch
+        from bzrlib.commands import run_bzr
+        eq = self.assertEqual
+
+        b = Branch('.', init=True)
+
+        self.build_tree(['inertiatic/', 'inertiatic/esp'])
+        eq(list(b.unknowns()), ['inertiatic'])
+        run_bzr(['add', 'inertiatic/esp'])
+        eq(list(b.unknowns()), [])
+
+        # Multiple unversioned parents
+        self.build_tree(['veil/', 'veil/cerpin/', 'veil/cerpin/taxt'])
+        eq(list(b.unknowns()), ['veil'])
+        run_bzr(['add', 'veil/cerpin/taxt'])
+        eq(list(b.unknowns()), [])
+
+        # Check whacky paths work
+        self.build_tree(['cicatriz/', 'cicatriz/esp'])
+        eq(list(b.unknowns()), ['cicatriz'])
+        run_bzr(['add', 'inertiatic/../cicatriz/esp'])
+        eq(list(b.unknowns()), [])
+
+    def test_add_in_versioned(self):
+        """Try to add a file in a versioned directory.
+
+        "bzr add" should do this happily.
+        """
+        from bzrlib.branch import Branch
+        from bzrlib.commands import run_bzr
+        eq = self.assertEqual
+
+        b = Branch('.', init=True)
+
+        self.build_tree(['inertiatic/', 'inertiatic/esp'])
+        eq(list(b.unknowns()), ['inertiatic'])
+        run_bzr(['add', '--no-recurse', 'inertiatic'])
+        eq(list(b.unknowns()), ['inertiatic/esp'])
+        run_bzr(['add', 'inertiatic/esp'])
+        eq(list(b.unknowns()), [])
+
+    def test_subdir_add(self):
+        """Add in subdirectory should add only things from there down"""
+        
+        from bzrlib.branch import Branch
+        from bzrlib.commands import run_bzr
+        
+        eq = self.assertEqual
+        ass = self.assert_
+        chdir = os.chdir
+        
+        b = Branch('.', init=True)
+        self.build_tree(['src/', 'README'])
+        
+        eq(sorted(b.unknowns()),
+           ['README', 'src'])
+        
+        eq(run_bzr(['add', 'src']), 0)
+        
+        self.build_tree(['src/foo.c'])
+        
+        chdir('src')
+        eq(run_bzr(['add']), 0)
+        
+        eq(sorted(b.unknowns()), 
+           ['README'])
+        eq(len(b.inventory), 3)
+                
+        chdir('..')
+        eq(run_bzr(['add']), 0)
+        eq(list(b.unknowns()), [])
+
+        self.check_and_upgrade()
+
+
+    def check_and_upgrade(self):
+        """After all the above changes, run the check and upgrade commands.
+
+        The upgrade should be a no-op."""
+        b = Branch('.')
+        debug('branch has %d revisions', b.revno())
+        
+        debug('check branch...')
+        from bzrlib.check import check
+        check(b)
+        
+        debug('upgrade branch...')
+        from bzrlib.upgrade import upgrade
+        upgrade(b)
+        
+        debug('check branch...')
+        from bzrlib.check import check
+        check(b)
+        
+
+        
+class SubdirCommit(TestCaseInTempDir):
+
+    def test_subdir_commit(self):
+        """Test committing a subdirectory, and committing within a directory."""
+        run_bzr = self.run_bzr
+        eq = self.assertEqual
+
+        self.build_tree(['a/', 'b/'])
+        
+        run_bzr('init')
+        b = Branch('.')
+        
+        for fn in ('a/one', 'b/two', 'top'):
+            file(fn, 'w').write('old contents')
+            
+        run_bzr('add')
+        run_bzr('commit', '-m', 'first revision')
+        
+        for fn in ('a/one', 'b/two', 'top'):
+            file(fn, 'w').write('new contents')
+            
+        run_bzr('commit', 'a', '-m', 'commit a only')
+        
+        old = b.revision_tree(b.lookup_revision(1))
+        new = b.revision_tree(b.lookup_revision(2))
+        
+        eq(new.get_file_by_path('b/two').read(), 'old contents')
+        eq(new.get_file_by_path('top').read(), 'old contents')
+        eq(new.get_file_by_path('a/one').read(), 'new contents')
+        
+        os.chdir('a')
+        # commit from here should do nothing
+        run_bzr('commit', '.', '-m', 'commit subdir only', '--unchanged')
+        v3 = b.revision_tree(b.lookup_revision(3))
+        eq(v3.get_file_by_path('b/two').read(), 'old contents')
+        eq(v3.get_file_by_path('top').read(), 'old contents')
+        eq(v3.get_file_by_path('a/one').read(), 'new contents')
+                
+        # commit in subdirectory commits whole tree
+        run_bzr('commit', '-m', 'commit whole tree from subdir')
+        v4 = b.revision_tree(b.lookup_revision(4))
+        eq(v4.get_file_by_path('b/two').read(), 'new contents')        
+        eq(v4.get_file_by_path('top').read(), 'new contents')
+        
+        # TODO: factor out some kind of assert_tree_state() method
+        

=== modified file 'bzrlib/selftest/whitebox.py'
--- bzrlib/selftest/whitebox.py	2005-07-11 22:20:04 +0000
+++ bzrlib/selftest/whitebox.py	2005-08-30 01:35:40 +0000
@@ -3,13 +3,14 @@
 import os
 import unittest
 
-from bzrlib.selftest import InTempDir, TestBase
+from bzrlib.selftest import TestCaseInTempDir, TestCase
 from bzrlib.branch import ScratchBranch, Branch
 from bzrlib.errors import NotBranchError, NotVersionedError
 
 
-class Unknowns(InTempDir):
-    def runTest(self):
+class TestBranch(TestCaseInTempDir):
+
+    def test_unknowns(self):
         b = Branch('.', init=True)
 
         self.build_tree(['hello.txt',
@@ -18,10 +19,7 @@
         self.assertEquals(list(b.unknowns()),
                           ['hello.txt'])
 
-
-
-class NoChanges(InTempDir):
-    def runTest(self):
+    def test_no_changes(self):
         from bzrlib.errors import PointlessCommit
         
         b = Branch('.', init=True)
@@ -55,69 +53,52 @@
         
 
 
-class ValidateRevisionId(TestBase):
-    def runTest(self):
+class TestRevisionId(TestCase):
+    
+    def test_validate_revision_id(self):
         from bzrlib.revision import validate_revision_id
         validate_revision_id('mbp@sourcefrog.net-20050311061123-96a255005c7c9dbe')
-        
         self.assertRaises(ValueError,
                           validate_revision_id,
                           ' asdkjas')
-
-
         self.assertRaises(ValueError,
                           validate_revision_id,
                           'mbp@sourcefrog.net-20050311061123-96a255005c7c9dbe\n')
-
-
         self.assertRaises(ValueError,
                           validate_revision_id,
                           ' mbp@sourcefrog.net-20050311061123-96a255005c7c9dbe')
-
         self.assertRaises(ValueError,
                           validate_revision_id,
                           'Martin Pool <mbp@sourcefrog.net>-20050311061123-96a255005c7c9dbe')
 
 
+class PendingMerges(TestCaseInTempDir):
 
-class PendingMerges(InTempDir):
-    """Tracking pending-merged revisions."""
-    def runTest(self):
+    def test_pending_merges(self):
+        """Tracking pending-merged revisions."""
         b = Branch('.', init=True)
 
         self.assertEquals(b.pending_merges(), [])
-        
-        b.add_pending_merge('foo@azkhazan-123123-abcabc')
-        
-        self.assertEquals(b.pending_merges(), ['foo@azkhazan-123123-abcabc'])
-    
-        b.add_pending_merge('foo@azkhazan-123123-abcabc')
-        
-        self.assertEquals(b.pending_merges(), ['foo@azkhazan-123123-abcabc'])
-
+        b.add_pending_merge('foo@azkhazan-123123-abcabc')
+        self.assertEquals(b.pending_merges(), ['foo@azkhazan-123123-abcabc'])
+        b.add_pending_merge('foo@azkhazan-123123-abcabc')
+        self.assertEquals(b.pending_merges(), ['foo@azkhazan-123123-abcabc'])
         b.add_pending_merge('wibble@fofof--20050401--1928390812')
         self.assertEquals(b.pending_merges(),
                           ['foo@azkhazan-123123-abcabc',
                            'wibble@fofof--20050401--1928390812'])
-
         b.commit("commit from base with two merges")
-
         rev = b.get_revision(b.revision_history()[0])
         self.assertEquals(len(rev.parents), 2)
         self.assertEquals(rev.parents[0].revision_id,
                           'foo@azkhazan-123123-abcabc')
         self.assertEquals(rev.parents[1].revision_id,
                            'wibble@fofof--20050401--1928390812')
-
         # list should be cleared when we do a commit
         self.assertEquals(b.pending_merges(), [])
         
-        
-        
-
-class Revert(InTempDir):
-    """Test selected-file revert"""
-    def runTest(self):
+    def test_revert(self):
+        """Test selected-file revert"""
         b = Branch('.', init=True)
 
         self.build_tree(['hello.txt'])
@@ -143,11 +124,8 @@
         self.check_file_contents('hello.txt', 'initial hello')
         self.check_file_contents('hello.txt~', 'initial hello')
 
-
-
-class RenameDirs(InTempDir):
-    """Test renaming directories and the files within them."""
-    def runTest(self):
+    def test_rename_dirs(self):
+        """Test renaming directories and the files within them."""
         b = Branch('.', init=True)
         self.build_tree(['dir/', 'dir/sub/', 'dir/sub/file'])
         b.add(['dir', 'dir/sub', 'dir/sub/file'])
@@ -176,18 +154,13 @@
                                    ['newdir', 'newdir/newsub',
                                     'newdir/newsub/file'])
 
-        
-
-
-class BranchPathTestCase(TestBase):
-    """test for branch path lookups
-
-    Branch.relpath and bzrlib.branch._relpath do a simple but subtle
-    job: given a path (either relative to cwd or absolute), work out
-    if it is inside a branch and return the path relative to the base.
-    """
-
-    def runTest(self):
+    def test_relpath(self):
+        """test for branch path lookups
+    
+        Branch.relpath and bzrlib.branch._relpath do a simple but subtle
+        job: given a path (either relative to cwd or absolute), work out
+        if it is inside a branch and return the path relative to the base.
+        """
         from bzrlib.branch import _relpath
         import tempfile, shutil
         
@@ -238,14 +211,3 @@
         finally:
             os.chdir(savedir)
             shutil.rmtree(dtmp)
-
-
-
-
-TEST_CLASSES = [Unknowns,
-                ValidateRevisionId,
-                PendingMerges,
-                Revert,
-                RenameDirs,
-                BranchPathTestCase,
-                ]

=== added file 'bzrlib/shellcomplete.py'
--- bzrlib/shellcomplete.py	1970-01-01 00:00:00 +0000
+++ bzrlib/shellcomplete.py	2005-08-22 16:31:16 +0000
@@ -0,0 +1,74 @@
+import sys
+
+
+def shellcomplete(context=None, outfile = None):
+    if outfile == None:
+        outfile = sys.stdout
+    if context == None:
+        shellcomplete_commands(outfile = outfile)
+    else:
+        shellcomplete_on_command(context, outfile = outfile)
+
+def shellcomplete_on_command(cmdname, outfile = None):
+    cmdname = str(cmdname)
+
+    if outfile == None:
+        outfile = sys.stdout
+
+    from inspect import getdoc
+    import commands
+    context, cmdclass = commands.get_cmd_class(cmdname)
+
+    doc = getdoc(cmdclass)
+    if doc == None:
+        raise NotImplementedError("sorry, no detailed shellcomplete yet for %r" % cmdname)
+
+    shellcomplete_on_option(cmdclass.takes_options, outfile = None)
+    for aname in cmdclass.takes_args:
+	    outfile.write(aname + '\n')
+
+
+def shellcomplete_on_option(options, outfile = None):
+    import commands
+    
+    if not options:
+        return
+    
+    if outfile == None:
+        outfile = sys.stdout
+
+    for on in options:
+        for shortname, longname in commands.SHORT_OPTIONS.items():
+            if longname == on:
+                l = '"(--' + on + ' -' + shortname + ')"{--' + on + ',-' + shortname + '}'
+                break
+	    else:
+		l = '--' + on
+        outfile.write(l + '\n')
+
+
+def shellcomplete_commands(outfile = None):
+    """List all commands"""
+    import inspect
+    import commands
+    from inspect import getdoc
+    
+    if outfile == None:
+        outfile = sys.stdout
+    
+    cmds = []
+    for cmdname, cmdclass in commands.get_all_cmds():
+        cmds.append((cmdname, cmdclass))
+	for alias in cmdclass.aliases:
+	    cmds.append((alias, cmdclass))
+    cmds.sort()
+    for cmdname, cmdclass in cmds:
+        if cmdclass.hidden:
+            continue
+        doc = getdoc(cmdclass)
+        if doc == None:
+	    outfile.write(cmdname + '\n')
+        else:
+	    doclines = doc.splitlines()
+	    firstline = doclines[0].lower()
+	    outfile.write(cmdname + ':' + firstline[0:-1] + '\n')

=== modified file 'bzrlib/status.py'
--- bzrlib/status.py	2005-07-12 01:44:23 +0000
+++ bzrlib/status.py	2005-09-07 23:14:30 +0000
@@ -19,20 +19,27 @@
 def show_status(branch, show_unchanged=False,
                 specific_files=None,
                 show_ids=False,
-                to_file=None):
-    """Display single-line status for non-ignored working files.
+                to_file=None,
+                show_pending=True):
+    """Display status for non-ignored working files.
 
-    show_all
-        If true, show unmodified files too.
+    show_unchanged
+        If set, includes unchanged files.
 
     specific_files
         If set, only show the status of files in this list.
 
+    show_ids
+        If set, includes each file's id.
+
     to_file
         If set, write to this file (default stdout.)
+
+    show_pending
+        If set, write pending merges.
     """
     import sys
-    from bzrlib.diff import compare_trees
+    from bzrlib.delta import compare_trees
 
     if to_file == None:
         to_file = sys.stdout
@@ -62,6 +69,10 @@
                 print >>to_file, 'unknown:'
                 done_header = True
             print >>to_file, ' ', path
+        if show_pending and len(branch.pending_merges()) > 0:
+            print >>to_file, 'pending merges:'
+            for merge in branch.pending_merges():
+                print >> to_file, ' ', merge
     finally:
         branch.unlock()
         

=== modified file 'bzrlib/store.py'
--- bzrlib/store.py	2005-07-11 07:33:45 +0000
+++ bzrlib/store.py	2005-09-06 23:19:08 +0000
@@ -24,7 +24,9 @@
 import os, tempfile, types, osutils, gzip, errno
 from stat import ST_SIZE
 from StringIO import StringIO
-from trace import mutter
+from bzrlib.errors import BzrError
+from bzrlib.trace import mutter
+import bzrlib.ui
 
 ######################################################################
 # stores
@@ -66,10 +68,12 @@
     def __init__(self, basedir):
         self._basedir = basedir
 
-    def _path(self, id):
-        if '\\' in id or '/' in id:
-            raise ValueError("invalid store id %r" % id)
-        return os.path.join(self._basedir, id)
+    def _path(self, entry_id):
+        if not isinstance(entry_id, basestring):
+            raise TypeError(type(entry_id))
+        if '\\' in entry_id or '/' in entry_id:
+            raise ValueError("invalid store id %r" % entry_id)
+        return os.path.join(self._basedir, entry_id)
 
     def __repr__(self):
         return "%s(%r)" % (self.__class__.__name__, self._basedir)
@@ -90,7 +94,6 @@
             
         p = self._path(fileid)
         if os.access(p, os.F_OK) or os.access(p + '.gz', os.F_OK):
-            from bzrlib.errors import bailout
             raise BzrError("store %r already contains id %r" % (self._basedir, fileid))
 
         fn = p
@@ -110,31 +113,44 @@
             af.close()
 
 
-    def copy_multi(self, other, ids):
+    def copy_multi(self, other, ids, permit_failure=False):
         """Copy texts for ids from other into self.
 
-        If an id is present in self, it is skipped.  A count of copied
-        ids is returned, which may be less than len(ids).
+        If an id is present in self, it is skipped.
+
+        Returns (count_copied, failed), where failed is a collection of ids
+        that could not be copied.
         """
-        from bzrlib.progress import ProgressBar
-        pb = ProgressBar()
+        pb = bzrlib.ui.ui_factory.progress_bar()
+        
         pb.update('preparing to copy')
         to_copy = [id for id in ids if id not in self]
         if isinstance(other, ImmutableStore):
             return self.copy_multi_immutable(other, to_copy, pb)
         count = 0
+        failed = set()
         for id in to_copy:
             count += 1
             pb.update('copy', count, len(to_copy))
-            self.add(other[id], id)
-        assert count == len(to_copy)
+            if not permit_failure:
+                self.add(other[id], id)
+            else:
+                try:
+                    entry = other[id]
+                except IndexError:
+                    failed.add(id)
+                    continue
+                self.add(entry, id)
+                
+        if not permit_failure:
+            assert count == len(to_copy)
         pb.clear()
-        return count
-
-
-    def copy_multi_immutable(self, other, to_copy, pb):
+        return count, failed
+
+    def copy_multi_immutable(self, other, to_copy, pb, permit_failure=False):
         from shutil import copyfile
         count = 0
+        failed = set()
         for id in to_copy:
             p = self._path(id)
             other_p = other._path(id)
@@ -142,7 +158,16 @@
                 copyfile(other_p, p)
             except IOError, e:
                 if e.errno == errno.ENOENT:
-                    copyfile(other_p+".gz", p+".gz")
+                    if not permit_failure:
+                        copyfile(other_p+".gz", p+".gz")
+                    else:
+                        try:
+                            copyfile(other_p+".gz", p+".gz")
+                        except IOError, e:
+                            if e.errno == errno.ENOENT:
+                                failed.add(id)
+                            else:
+                                raise
                 else:
                     raise
             
@@ -150,7 +175,7 @@
             pb.update('copy', count, len(to_copy))
         assert count == len(to_copy)
         pb.clear()
-        return count
+        return count, failed
     
 
     def __contains__(self, fileid):
@@ -172,16 +197,24 @@
     def __len__(self):
         return len(os.listdir(self._basedir))
 
+
     def __getitem__(self, fileid):
         """Returns a file reading from a particular entry."""
         p = self._path(fileid)
         try:
             return gzip.GzipFile(p + '.gz', 'rb')
         except IOError, e:
-            if e.errno == errno.ENOENT:
-                return file(p, 'rb')
-            else:
-                raise e
+            if e.errno != errno.ENOENT:
+                raise
+
+        try:
+            return file(p, 'rb')
+        except IOError, e:
+            if e.errno != errno.ENOENT:
+                raise
+
+        raise IndexError(fileid)
+
 
     def total_size(self):
         """Return (count, bytes)

=== modified file 'bzrlib/trace.py'
--- bzrlib/trace.py	2005-07-20 17:31:25 +0000
+++ bzrlib/trace.py	2005-08-29 10:57:01 +0000
@@ -13,69 +13,85 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-
-# TODO: Could probably replace this with something based on Python logging
-# module.
-
-
-
-
-__copyright__ = "Copyright (C) 2005 Canonical Ltd."
-__author__ = "Martin Pool <mbp@canonical.com>"
-
-
 """Messages and logging for bazaar-ng
 
-Nothing is actually logged unless you call bzrlib.open_tracefile().
+Messages are sent out through the Python logging library.
+
+They can be sent to two places: to stderr, and to ~/.bzr.log.
+
+~/.bzr.log gets all messages, and tracebacks of all uncaught
+exceptions.
+
+Normally stderr only gets messages of level INFO and higher, and gets
+only a summary of exceptions, not the traceback.
 """
 
 
-import sys, os
-
-######################################################################
-# messages and logging
-
-global _tracefile, _starttime
-_tracefile = None
-
-# used to have % (os.environ['USER'], time.time(), os.getpid()), 'w')
-_starttime = None
-
-# If false, notes also go to stdout; should replace this with --silent
-# at some point.
-silent = False
-
-
-# fix this if we ever fork within python
-_mypid = os.getpid()
-_logprefix = '[%d] ' % _mypid
-
-
-def _write_trace(msg):
-    if 0:
-        if _tracefile:
-            _tracefile.write(_logprefix + msg + '\n')
-
-
-def warning(msg):
-    sys.stderr.write('bzr: warning: ' + msg + '\n')
-    _write_trace('warning: ' + msg)
-
-
-mutter = _write_trace
-
-
-def note(msg):
-    b = '* ' + str(msg) + '\n'
-    if not silent:
-        sys.stderr.write(b)
-    _write_trace('note: ' + msg)
-
-
-def log_error(msg):
-    sys.stderr.write(msg + '\n')
-    _write_trace(msg)
-
+# TODO: in debug mode, stderr should get full tracebacks and also
+# debug messages.  (Is this really needed?)
+
+# TODO: When running the test suites, we should add an additional
+# logger that sends messages into the test log file.
+
+
+import sys
+import os
+import logging
+import traceback
+
+
+_file_handler = None
+_stderr_handler = None
+
+
+class QuietFormatter(logging.Formatter):
+    """Formatter that supresses the details of errors.
+
+    This is used by default on stderr so as not to scare the user.
+    """
+    # At first I tried overriding formatException to suppress the
+    # exception details, but that has global effect: no loggers
+    # can get the exception details is we suppress them here.
+
+    def format(self, record):
+        s = 'bzr: '
+        if record.levelno >= logging.WARNING:
+            s += record.levelname + ': '
+            
+        s += record.getMessage() 
+            
+        if record.exc_info:
+            # give just a summary of the exception, not the whole thing
+            exinfo = traceback.extract_tb(record.exc_info[2])
+            # the format of this string matches one of the REs
+            s += '\n'
+            s += ('  at %s line %d, in %s()\n' % exinfo[-1][:3])
+            s += '  see ~/.bzr.log for debug information'
+
+        return s
+        
+
+
+
+################
+# configure convenient aliases for output routines
+
+_bzr_logger = logging.getLogger('bzr')
+_bzr_logger.setLevel(logging.DEBUG) 
+
+info = note = _bzr_logger.info
+warning =   _bzr_logger.warning
+log_error = _bzr_logger.error
+error =     _bzr_logger.error
+mutter =    _bzr_logger.debug
+debug =     _bzr_logger.debug
+
+
+
+
+# we do the rollover using this code, rather than the default from python
+# logging, because we only want to rollover at program startup, not on each
+# message.  maybe that's not a good enough reason.
 
 def _rollover_trace_maybe(trace_fname):
     import stat
@@ -101,51 +117,95 @@
 
 
 
-def open_tracefile(argv=[], tracefilename='~/.bzr.log'):
+def open_tracefile(tracefilename='~/.bzr.log'):
     # Messages are always written to here, so that we have some
     # information if something goes wrong.  In a future version this
     # file will be removed on successful completion.
-    global _starttime, _tracefile
+    global _file_handler
     import stat, codecs
 
-    _starttime = os.times()[4]
-
     trace_fname = os.path.join(os.path.expanduser(tracefilename))
     _rollover_trace_maybe(trace_fname)
 
     # buffering=1 means line buffered
     try:
-        _tracefile = codecs.open(trace_fname, 'at', 'utf8', buffering=1)
-        t = _tracefile
-
-        if os.fstat(t.fileno())[stat.ST_SIZE] == 0:
-            t.write("\nthis is a debug log for diagnosing/reporting problems in bzr\n")
-            t.write("you can delete or truncate this file, or include sections in\n")
-            t.write("bug reports to bazaar-ng@lists.canonical.com\n\n")
-
-        import bzrlib
-        _write_trace('bzr %s invoked on python %s (%s)'
-                     % (bzrlib.__version__,
-                        '.'.join(map(str, sys.version_info)),
-                        sys.platform))
-
-        _write_trace('  arguments: %r' % argv)
-        _write_trace('  working dir: ' + os.getcwdu())
+        tf = codecs.open(trace_fname, 'at', 'utf8', buffering=1)
+
+        if os.fstat(tf.fileno())[stat.ST_SIZE] == 0:
+            tf.write("\nthis is a debug log for diagnosing/reporting problems in bzr\n")
+            tf.write("you can delete or truncate this file, or include sections in\n")
+            tf.write("bug reports to bazaar-ng@lists.canonical.com\n\n")
+        
+        _file_handler = logging.StreamHandler(tf)
+        fmt = r'[%(process)5d] %(asctime)s.%(msecs)03d %(levelname)s: %(message)s'
+        datefmt = r'%a %H:%M:%S'
+        _file_handler.setFormatter(logging.Formatter(fmt, datefmt))
+        _file_handler.setLevel(logging.DEBUG)
+        logging.getLogger('').addHandler(_file_handler)
+
     except IOError, e:
         warning("failed to open trace file: %s" % (e))
 
-def close_trace():
-    times = os.times()
-    mutter("finished, %.3fu/%.3fs cpu, %.3fu/%.3fs cum, %.3f elapsed"
-           % (times[:4] + ((times[4] - _starttime),)))
-
-
-
-def log_exception():
-    """Log the last exception into the trace file."""
-    import cgitb
-    s = cgitb.text(sys.exc_info())
-    for l in s.split('\n'):
-        _write_trace(l)
+
+def log_startup(argv):
+    import bzrlib
+
+    debug('bzr %s invoked on python %s (%s)',
+          bzrlib.__version__,
+          '.'.join(map(str, sys.version_info)),
+          sys.platform)
+
+    debug('  arguments: %r', argv)
+    debug('  working dir: %s', os.getcwdu())
+
+
+def log_exception(msg=None):
+    """Log the last exception into the trace file.
+
+    The exception string representation is used as the error
+    summary, unless msg is given.
+    """
+    if msg == None:
+        ei = sys.exc_info()
+        msg = str(ei[1])
+
+    if msg and (msg[-1] == '\n'):
+        msg = msg[:-1]
         
-    
+    _bzr_logger.exception(msg)
+
+
+
+def enable_default_logging():
+    """Configure default logging to stderr and .bzr.log"""
+    global _stderr_handler, _file_handler
+
+    _stderr_handler = logging.StreamHandler()
+    _stderr_handler.setFormatter(QuietFormatter())
+
+    if not _file_handler:
+        open_tracefile()
+
+    if os.environ.get('BZR_DEBUG'):
+        level = logging.DEBUG
+    else:
+        level = logging.INFO
+
+    _stderr_handler.setLevel(logging.INFO)
+    _file_handler.setLevel(level)
+
+    logging.getLogger('').addHandler(_stderr_handler)
+
+
+
+def disable_default_logging():
+    """Turn off default log handlers.
+
+    This is intended to be used by the test framework, which doesn't
+    want leakage from the code-under-test into the main logs.
+    """
+
+    l = logging.getLogger('')
+    l.removeHandler(_stderr_handler)
+    if _file_handler:
+        l.removeHandler(_file_handler)

=== modified file 'bzrlib/tree.py'
--- bzrlib/tree.py	2005-07-11 07:33:45 +0000
+++ bzrlib/tree.py	2005-08-25 05:58:05 +0000
@@ -17,13 +17,14 @@
 """Tree classes, representing directory at point in time.
 """
 
-from osutils import pumpfile, appendpath, fingerprint_file
 import os
 
+import bzrlib
 from bzrlib.trace import mutter, note
 from bzrlib.errors import BzrError
+from bzrlib.inventory import Inventory
+from bzrlib.osutils import pumpfile, appendpath, fingerprint_file
 
-import bzrlib
 
 exporters = {}
 
@@ -66,6 +67,9 @@
 
     def _get_inventory(self):
         return self._inventory
+    
+    def get_file_by_path(self, path):
+        return self.get_file(self._inventory.path2id(path))
 
     inventory = property(_get_inventory,
                          doc="Inventory of this Tree")
@@ -131,7 +135,8 @@
 
     def get_file_sha1(self, file_id):
         ie = self._inventory[file_id]
-        return ie.text_sha1
+        if ie.kind == "file":
+            return ie.text_sha1
 
     def has_filename(self, filename):
         return bool(self.inventory.path2id(filename))
@@ -143,9 +148,8 @@
 
 
 class EmptyTree(Tree):
-    def __init__(self, root_id):
-        from bzrlib.inventory import Inventory
-        self._inventory = Inventory(root_id)
+    def __init__(self):
+        self._inventory = Inventory()
 
     def has_filename(self, filename):
         return False
@@ -154,6 +158,14 @@
         if False:  # just to make it a generator
             yield None
     
+    def __contains__(self, file_id):
+        return file_id in self._inventory
+
+    def get_file_sha1(self, file_id):
+        assert self._inventory[file_id].kind == "root_directory"
+        return None
+
+
 
 
 ######################################################################

=== added file 'bzrlib/ui.py'
--- bzrlib/ui.py	1970-01-01 00:00:00 +0000
+++ bzrlib/ui.py	2005-08-25 12:46:42 +0000
@@ -0,0 +1,51 @@
+# Copyright (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+
+"""UI abstraction.
+
+This tells the library how to display things to the user.  Through this
+layer different applications can choose the style of UI.
+
+At the moment this layer is almost trivial: the application can just
+choose the style of progress bar.
+
+Set the ui_factory member to define the behaviour.  The default
+displays no output.
+"""
+
+
+
+
+
+import bzrlib.progress
+
+
+class TextUIFactory(object):
+    def progress_bar(self):
+
+        # this in turn is abstract, and creates either a tty or dots
+        # bar depending on what we think of the terminal
+        return bzrlib.progress.ProgressBar()
+
+
+class SilentUIFactory(object):
+    def progress_bar(self):
+        return bzrlib.progress.DummyProgress()
+
+
+ui_factory = SilentUIFactory()

=== modified file 'bzrlib/upgrade.py'
--- bzrlib/upgrade.py	2005-06-28 05:33:40 +0000
+++ bzrlib/upgrade.py	2005-09-05 09:27:11 +0000
@@ -15,7 +15,11 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-
+import tempfile, os, errno
+                
+import bzrlib.errors
+import bzrlib.progress
+from bzrlib.xml import serializer_v4
 
 
 def upgrade(branch):
@@ -34,12 +38,13 @@
 
     from bzrlib.trace import mutter
     from bzrlib.errors import BzrCheckError
-    from bzrlib.progress import ProgressBar
+    import bzrlib.ui
 
     branch.lock_write()
 
+    pb = bzrlib.ui.ui_factory.progress_bar()
+
     try:
-        pb = ProgressBar(show_spinner=True)
         last_rev_id = None
 
         history = branch.revision_history()
@@ -71,17 +76,20 @@
                 mutter("  set inventory_sha1 on {%s}" % rev_id)
 
             for prr in rev.parents:
-                actual_sha1 = branch.get_revision_sha1(prr.revision_id)
-                if (updated_previous_revision
-                    or prr.revision_sha1 is None):
-                    if prr.revision_sha1 != actual_sha1:
-                        prr.revision_sha1 = actual_sha1
-                        updated = True
-                elif actual_sha1 != prr.revision_sha1:
-                    raise BzrCheckError("parent {%s} of {%s} sha1 mismatch: "
-                                        "%s vs %s"
-                                        % (prr.revision_id, rev_id,
-                                           actual_sha1, prr.revision_sha1))
+                try:
+                    actual_sha1 = branch.get_revision_sha1(prr.revision_id)
+                except bzrlib.errors.NoSuchRevision:
+                    mutter("parent {%s} of {%s} not present in branch; skipped"
+                           % (prr.revision_id, rev_id))
+                    continue
+                    
+                if actual_sha1 != prr.revision_sha1:
+                    mutter("parent {%s} of {%s} sha1 mismatch: "
+                           "%s vs %s; fixed"
+                           % (prr.revision_id, rev_id,
+                              actual_sha1, prr.revision_sha1))
+                    prr.revision_sha1 = actual_sha1
+                    updated = True
 
             if updated:
                 updated_previous_revision = True
@@ -91,11 +99,8 @@
                 # revision entry. I'm not supremely happy about it, but
                 # there should be *some* way of making old entries have
                 # the full meta information.
-                import tempfile, os, errno
-                from bzrlib.xml import pack_xml
-                
                 rev_tmp = tempfile.TemporaryFile()
-                pack_xml(rev, rev_tmp)
+                serializer_v4.write_revision(rev, rev_tmp)
                 rev_tmp.seek(0)
 
                 tmpfd, tmp_path = tempfile.mkstemp(prefix=rev_id, suffix='.gz',

=== added directory 'bzrlib/util'
=== added file 'bzrlib/util/__init__.py'
=== renamed directory 'effbot' => 'bzrlib/util/effbot'
=== renamed directory 'elementtree' => 'bzrlib/util/elementtree'
=== renamed directory 'urlgrabber' => 'bzrlib/util/urlgrabber'
=== modified file 'bzrlib/weave.py'
--- bzrlib/weave.py	2005-07-18 18:53:46 +0000
+++ bzrlib/weave.py	2005-08-25 05:58:05 +0000
@@ -45,7 +45,10 @@
 
 # with delta folded in and mutation of the list, 36.13s
 
-# with all this and simplification of add code, 33s 
+# with all this and simplification of add code, 33s
+
+
+
 
 
 # TODO: Perhaps have copy method for Weave instances?
@@ -59,8 +62,6 @@
 # binaries, perhaps by naively splitting on \n or perhaps using
 # something like a rolling checksum.
 
-# TODO: Track version names as well as indexes. 
-
 # TODO: End marker for each version so we can stop reading?
 
 # TODO: Check that no insertion occurs inside a deletion that was
@@ -75,6 +76,17 @@
 # description of which revisions include it.  Nice for checking all
 # shas in parallel.
 
+# TODO: Using a single _extract routine and then processing the output
+# is probably inefficient.  It's simple enough that we can afford to
+# have slight specializations for different ways its used: annotate,
+# basis for add, get, etc.
+
+# TODO: Perhaps the API should work only in names to hide the integer
+# indexes from the user?
+
+
+
+import sha
 
 
 
@@ -101,7 +113,7 @@
 
     * a nonnegative index number.
 
-    * a version-id string.
+    * a version-id string. (not implemented yet)
 
     Typically the index number will be valid only inside this weave and
     the version-id is used to reference it in the larger world.
@@ -118,7 +130,9 @@
     The instruction can be '{' or '}' for an insertion block, and '['
     and ']' for a deletion block respectively.  The version is the
     integer version index.  There is no replace operator, only deletes
-    and inserts.
+    and inserts.  For '}', the end of an insertion, there is no
+    version parameter because it always closes the most recently
+    opened insertion.
 
     Constraints/notes:
 
@@ -160,52 +174,78 @@
         each version; the parent's parents are implied.
 
     _sha1s
-        List of hex SHA-1 of each version, or None if not recorded.
+        List of hex SHA-1 of each version.
+
+    _names
+        List of symbolic names for each version.  Each should be unique.
+
+    _name_map
+        For each name, the version number.
     """
 
-    __slots__ = ['_weave', '_parents', '_sha1s']
+    __slots__ = ['_weave', '_parents', '_sha1s', '_names', '_name_map']
     
     def __init__(self):
         self._weave = []
         self._parents = []
         self._sha1s = []
+        self._names = []
+        self._name_map = {}
 
 
     def __eq__(self, other):
         if not isinstance(other, Weave):
             return False
         return self._parents == other._parents \
-               and self._weave == other._weave
+               and self._weave == other._weave \
+               and self._sha1s == other._sha1s 
+
     
-
     def __ne__(self, other):
         return not self.__eq__(other)
 
+
+    def lookup(self, name):
+        try:
+            return self._name_map[name]
+        except KeyError:
+            raise WeaveError("name %s not present in weave" % name)
+
         
-    def add(self, parents, text):
+    def add(self, name, parents, text):
         """Add a single text on top of the weave.
   
         Returns the index number of the newly added version.
 
+        name
+            Symbolic name for this version.
+            (Typically the revision-id of the revision that added it.)
+
         parents
             List or set of direct parent version numbers.
             
         text
             Sequence of lines to be added in the new version."""
 
+        assert isinstance(name, basestring)
+        if name in self._name_map:
+            raise WeaveError("name %r already present in weave" % name)
+        
         self._check_versions(parents)
         ## self._check_lines(text)
         new_version = len(self._parents)
 
-        import sha
         s = sha.new()
         map(s.update, text)
         sha1 = s.hexdigest()
         del s
 
-        # if we abort after here the weave will be corrupt
-        self._parents.append(frozenset(parents))
+        # if we abort after here the (in-memory) weave will be corrupt because only
+        # some fields are updated
+        self._parents.append(parents[:])
         self._sha1s.append(sha1)
+        self._names.append(name)
+        self._name_map[name] = new_version
 
             
         if not parents:
@@ -216,7 +256,7 @@
             if text:
                 self._weave.append(('{', new_version))
                 self._weave.extend(text)
-                self._weave.append(('}', new_version))
+                self._weave.append(('}', None))
         
             return new_version
 
@@ -238,7 +278,8 @@
             basis_lineno.append(lineno)
             basis_lines.append(line)
 
-        # another small special case: a merge, producing the same text as auto-merge
+        # another small special case: a merge, producing the same text
+        # as auto-merge
         if text == basis_lines:
             return new_version            
 
@@ -287,8 +328,8 @@
                 # we don't destroy ourselves
                 i = i2 + offset
                 self._weave[i:i] = ([('{', new_version)] 
-                                + text[j1:j2] 
-                                + [('}', new_version)])
+                                    + text[j1:j2] 
+                                    + [('}', None)])
                 offset += 2 + (j2 - j1)
 
         return new_version
@@ -384,7 +425,7 @@
                 if c == '{':
                     istack.append(v)
                 elif c == '}':
-                    oldv = istack.pop()
+                    istack.pop()
                 elif c == '[':
                     assert v not in dset
                     dset.add(v)
@@ -431,8 +472,7 @@
                     assert v not in istack
                     istack.append(v)
                 elif c == '}':
-                    oldv = istack.pop()
-                    assert oldv == v
+                    istack.pop()
                 elif c == '[':
                     if v in included:
                         assert v not in dset
@@ -508,7 +548,6 @@
 
         # try extracting all versions; this is a bit slow and parallel
         # extraction could be used
-        import sha
         nv = self.numversions()
         for version in range(nv):
             if progress_bar:
@@ -670,15 +709,17 @@
 
 
 
-def weave_info(w):
-    """Show some text information about the weave."""
-    print '%6s %40s %20s' % ('ver', 'sha1', 'parents')
-    for i in (6, 40, 20):
+def weave_toc(w):
+    """Show the weave's table-of-contents"""
+    print '%6s %50s %10s %10s' % ('ver', 'name', 'sha1', 'parents')
+    for i in (6, 50, 10, 10):
         print '-' * i,
     print
     for i in range(w.numversions()):
         sha1 = w._sha1s[i]
-        print '%6d %40s %s' % (i, sha1, ' '.join(map(str, w._parents[i])))
+        name = w._names[i]
+        parent_str = ' '.join(map(str, w._parents[i]))
+        print '%6d %-50.50s %10.10s %s' % (i, name, sha1, parent_str)
 
 
 
@@ -706,7 +747,10 @@
     print 'weave file        %9d bytes' % weave_size
     print 'total contents    %9d bytes' % total
     print 'compression ratio %9.2fx' % (float(total) / float(weave_size))
-
+    if vers:
+        avg = total/vers
+        print 'average size      %9d bytes' % avg
+        print 'relative size     %9.2fx' % (float(weave_size) / float(avg))
 
 
 def usage():
@@ -721,9 +765,9 @@
         Write out specified version.
     weave check WEAVEFILE
         Check consistency of all versions.
-    weave info WEAVEFILE
+    weave toc WEAVEFILE
         Display table of contents.
-    weave add WEAVEFILE [BASE...] < NEWTEXT
+    weave add WEAVEFILE NAME [BASE...] < NEWTEXT
         Add NEWTEXT, with specified parent versions.
     weave annotate WEAVEFILE VERSION
         Display origin of each line.
@@ -736,23 +780,23 @@
 
     % weave init foo.weave
     % vi foo.txt
-    % weave add foo.weave < foo.txt
+    % weave add foo.weave ver0 < foo.txt
     added version 0
 
     (create updated version)
     % vi foo.txt
     % weave get foo.weave 0 | diff -u - foo.txt
-    % weave add foo.weave 0 < foo.txt
+    % weave add foo.weave ver1 0 < foo.txt
     added version 1
 
     % weave get foo.weave 0 > foo.txt       (create forked version)
     % vi foo.txt
-    % weave add foo.weave 0 < foo.txt
+    % weave add foo.weave ver2 0 < foo.txt
     added version 2
 
     % weave merge foo.weave 1 2 > foo.txt   (merge them)
     % vi foo.txt                            (resolve conflicts)
-    % weave add foo.weave 1 2 < foo.txt     (commit merged version)     
+    % weave add foo.weave merged 1 2 < foo.txt     (commit merged version)     
     
 """
     
@@ -764,8 +808,15 @@
     from weavefile import write_weave, read_weave
     from bzrlib.progress import ProgressBar
 
-    #import psyco
-    #psyco.full()
+    try:
+        import psyco
+        psyco.full()
+    except ImportError:
+        pass
+
+    if len(argv) < 2:
+        usage()
+        return 0
 
     cmd = argv[1]
 
@@ -777,11 +828,12 @@
     elif cmd == 'add':
         w = readit()
         # at the moment, based on everything in the file
-        parents = map(int, argv[3:])
+        name = argv[3]
+        parents = map(int, argv[4:])
         lines = sys.stdin.readlines()
-        ver = w.add(parents, lines)
+        ver = w.add(name, parents, lines)
         write_weave(w, file(argv[2], 'wb'))
-        print 'added version %d' % ver
+        print 'added version %r %d' % (name, ver)
     elif cmd == 'init':
         fn = argv[2]
         if os.path.exists(fn):
@@ -809,8 +861,8 @@
                 print '%5d | %s' % (origin, text)
                 lasto = origin
                 
-    elif cmd == 'info':
-        weave_info(readit())
+    elif cmd == 'toc':
+        weave_toc(readit())
 
     elif cmd == 'stats':
         weave_stats(argv[2])
@@ -865,6 +917,33 @@
         raise ValueError('unknown command %r' % cmd)
     
 
+
+def profile_main(argv): 
+    import tempfile, hotshot, hotshot.stats
+
+    prof_f = tempfile.NamedTemporaryFile()
+
+    prof = hotshot.Profile(prof_f.name)
+
+    ret = prof.runcall(main, argv)
+    prof.close()
+
+    stats = hotshot.stats.load(prof_f.name)
+    #stats.strip_dirs()
+    stats.sort_stats('cumulative')
+    ## XXX: Might like to write to stderr or the trace file instead but
+    ## print_stats seems hardcoded to stdout
+    stats.print_stats(20)
+            
+    return ret
+
+
 if __name__ == '__main__':
     import sys
-    sys.exit(main(sys.argv))
+    if '--profile' in sys.argv:
+        args = sys.argv[:]
+        args.remove('--profile')
+        sys.exit(profile_main(args))
+    else:
+        sys.exit(main(sys.argv))
+

=== modified file 'bzrlib/weavefile.py'
--- bzrlib/weavefile.py	2005-07-18 13:38:13 +0000
+++ bzrlib/weavefile.py	2005-08-25 12:46:42 +0000
@@ -26,9 +26,13 @@
 There is one format marker followed by a blank line, followed by a
 series of version headers, followed by the weave itself.
 
-Each version marker has 'i' and the included previous versions, then
-'1' and the SHA-1 of the text, if known.  The inclusions do not need
-to list versions included by a parent.
+Each version marker has
+
+ 'i'   parent version indexes
+ '1'   SHA-1 of text
+ 'n'   name
+
+The inclusions do not need to list versions included by a parent.
 
 The weave is bracketed by 'w' and 'W' lines, and includes the '{}[]'
 processing instructions.  Lines of text are prefixed by '.' if the
@@ -36,19 +40,20 @@
 """
 
 # TODO: When extracting a single version it'd be enough to just pass
-# an iterator returning the weave lines...
+# an iterator returning the weave lines...  We don't really need to
+# deserialize it into memory.
 
-FORMAT_1 = '# bzr weave file v3\n'
+FORMAT_1 = '# bzr weave file v5\n'
 
 
 def write_weave(weave, f, format=None):
     if format == None or format == 1:
-        return write_weave_v1(weave, f)
+        return write_weave_v5(weave, f)
     else:
         raise ValueError("unknown weave format %r" % format)
 
 
-def write_weave_v1(weave, f):
+def write_weave_v5(weave, f):
     """Write weave to file f."""
     print >>f, FORMAT_1,
 
@@ -63,6 +68,7 @@
         else:
             print >>f, 'i'
         print >>f, '1', weave._sha1s[version]
+        print >>f, 'n', weave._names[version]
         print >>f
 
     print >>f, 'w'
@@ -70,7 +76,10 @@
     for l in weave._weave:
         if isinstance(l, tuple):
             assert l[0] in '{}[]'
-            print >>f, '%s %d' % l
+            if l[0] == '}':
+                print >>f, '}'
+            else:
+                print >>f, '%s %d' % l
         else: # text line
             if not l:
                 print >>f, ', '
@@ -86,14 +95,13 @@
 
 
 def read_weave(f):
-    return read_weave_v1(f)
-
-
-def read_weave_v1(f):
+    return read_weave_v5(f)
+
+
+def read_weave_v5(f):
     from weave import Weave, WeaveFormatError
     w = Weave()
 
-    wfe = WeaveFormatError
     l = f.readline()
     if l != FORMAT_1:
         raise WeaveFormatError('invalid weave file header: %r' % l)
@@ -102,19 +110,26 @@
     while True:
         l = f.readline()
         if l[0] == 'i':
-            ver += 1
-
             if len(l) > 2:
-                w._parents.append(frozenset(map(int, l[2:].split(' '))))
+                w._parents.append(map(int, l[2:].split(' ')))
             else:
-                w._parents.append(frozenset())
+                w._parents.append([])
 
             l = f.readline()[:-1]
             assert l.startswith('1 ')
             w._sha1s.append(l[2:])
                 
             l = f.readline()
+            assert l.startswith('n ')
+            name = l[2:-1]
+            assert name not in w._name_map
+            w._names.append(name)
+            w._name_map[name] = ver
+                
+            l = f.readline()
             assert l == '\n'
+
+            ver += 1
         elif l == 'w\n':
             break
         else:
@@ -128,8 +143,10 @@
             w._weave.append(l[2:])  # include newline
         elif l.startswith(', '):
             w._weave.append(l[2:-1])        # exclude newline
+        elif l == '}\n':
+            w._weave.append(('}', None))
         else:
-            assert l[0] in '{}[]', l
+            assert l[0] in '{[]', l
             assert l[1] == ' ', l
             w._weave.append((intern(l[0]), int(l[2:])))
 

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2005-07-22 19:32:52 +0000
+++ bzrlib/workingtree.py	2005-08-29 13:51:52 +0000
@@ -21,10 +21,12 @@
 # it's not predictable when it will be written out.
 
 import os
-    
+import fnmatch
+        
 import bzrlib.tree
-from errors import BzrCheckError
-from trace import mutter
+from bzrlib.osutils import appendpath, file_kind, isdir, splitpath
+from bzrlib.errors import BzrCheckError
+from bzrlib.trace import mutter
 
 class WorkingTree(bzrlib.tree.Tree):
     """Working copy tree.
@@ -135,9 +137,6 @@
 
         Skips the control directory.
         """
-        from osutils import appendpath, file_kind
-        import os
-
         inv = self._inventory
 
         def descend(from_dir_relpath, from_dir_id, dp):
@@ -205,8 +204,6 @@
         Currently returned depth-first, sorted by name within directories.
         """
         ## TODO: Work from given directory downwards
-        from osutils import isdir, appendpath
-        
         for path, dir_entry in self.inventory.directories():
             mutter("search for unknowns in %r" % path)
             dirabs = self.abspath(path)
@@ -269,9 +266,6 @@
         # Eventually it should be replaced with something more
         # accurate.
         
-        import fnmatch
-        from osutils import splitpath
-        
         for pat in self.get_ignore_list():
             if '/' in pat or '\\' in pat:
                 
@@ -290,4 +284,4 @@
                     return pat
         else:
             return None
-        
\ No newline at end of file
+        

=== modified file 'bzrlib/xml.py'
--- bzrlib/xml.py	2005-06-28 05:33:40 +0000
+++ bzrlib/xml.py	2005-09-06 06:44:11 +0000
@@ -20,21 +20,344 @@
 # "XML is like violence: if it doesn't solve your problem, you aren't
 # using enough of it." -- various
 
-# importing this module is fairly slow because it has to load several ElementTree bits
+# importing this module is fairly slow because it has to load several
+# ElementTree bits
+
 try:
-    from cElementTree import ElementTree, SubElement, Element
+    from util.cElementTree import ElementTree, SubElement, Element
 except ImportError:
-    from elementtree.ElementTree import ElementTree, SubElement, Element
-
-
-def pack_xml(o, f):
-    """Write object o to file f as XML.
-
-    o must provide a to_element method.
-    """
-    ElementTree(o.to_element()).write(f, 'utf-8')
-    f.write('\n')
-
-
-def unpack_xml(cls, f):
-    return cls.from_element(ElementTree().parse(f))
+    from util.elementtree.ElementTree import ElementTree, SubElement, Element
+
+from bzrlib.inventory import ROOT_ID, Inventory, InventoryEntry
+from bzrlib.revision import Revision, RevisionReference        
+from bzrlib.errors import BzrError
+
+
+class Serializer(object):
+    """Abstract object serialize/deserialize"""
+    def write_inventory(self, inv, f):
+        """Write inventory to a file"""
+        elt = self._pack_inventory(inv)
+        self._write_element(elt, f)
+
+    def read_inventory(self, f):
+        return self._unpack_inventory(self._read_element(f))
+
+    def write_revision(self, rev, f):
+        self._write_element(self._pack_revision(rev), f)
+
+    def read_revision(self, f):
+        return self._unpack_revision(self._read_element(f))
+
+    def _write_element(self, elt, f):
+        ElementTree(elt).write(f, 'utf-8')
+        f.write('\n')
+
+    def _read_element(self, f):
+        return ElementTree().parse(f)
+
+
+
+class _Serializer_v4(Serializer):
+    """Version 0.0.4 serializer
+
+    You should use the serialzer_v4 singleton."""
+    
+    __slots__ = []
+    
+    def _pack_inventory(self, inv):
+        """Convert to XML Element"""
+        e = Element('inventory')
+        e.text = '\n'
+        if inv.root.file_id not in (None, ROOT_ID):
+            e.set('file_id', inv.root.file_id)
+        for path, ie in inv.iter_entries():
+            e.append(self._pack_entry(ie))
+        return e
+
+
+    def _pack_entry(self, ie):
+        """Convert InventoryEntry to XML element"""
+        e = Element('entry')
+        e.set('name', ie.name)
+        e.set('file_id', ie.file_id)
+        e.set('kind', ie.kind)
+
+        if ie.text_size != None:
+            e.set('text_size', '%d' % ie.text_size)
+
+        for f in ['text_id', 'text_sha1']:
+            v = getattr(ie, f)
+            if v != None:
+                e.set(f, v)
+
+        # to be conservative, we don't externalize the root pointers
+        # for now, leaving them as null in the xml form.  in a future
+        # version it will be implied by nested elements.
+        if ie.parent_id != ROOT_ID:
+            assert isinstance(ie.parent_id, basestring)
+            e.set('parent_id', ie.parent_id)
+
+        e.tail = '\n'
+
+        return e
+
+
+    def _unpack_inventory(self, elt):
+        """Construct from XML Element
+        """
+        assert elt.tag == 'inventory'
+        root_id = elt.get('file_id') or ROOT_ID
+        inv = Inventory(root_id)
+        for e in elt:
+            ie = self._unpack_entry(e)
+            if ie.parent_id == ROOT_ID:
+                ie.parent_id = root_id
+            inv.add(ie)
+        return inv
+
+
+    def _unpack_entry(self, elt):
+        assert elt.tag == 'entry'
+
+        ## original format inventories don't have a parent_id for
+        ## nodes in the root directory, but it's cleaner to use one
+        ## internally.
+        parent_id = elt.get('parent_id')
+        if parent_id == None:
+            parent_id = ROOT_ID
+
+        ie = InventoryEntry(elt.get('file_id'),
+                              elt.get('name'),
+                              elt.get('kind'),
+                              parent_id)
+        ie.text_id = elt.get('text_id')
+        ie.text_sha1 = elt.get('text_sha1')
+
+        ## mutter("read inventoryentry: %r" % (elt.attrib))
+
+        v = elt.get('text_size')
+        ie.text_size = v and int(v)
+
+        return ie
+
+
+    def _pack_revision(self, rev):
+        """Revision object -> xml tree"""
+        root = Element('revision',
+                       committer = rev.committer,
+                       timestamp = '%.9f' % rev.timestamp,
+                       revision_id = rev.revision_id,
+                       inventory_id = rev.inventory_id,
+                       inventory_sha1 = rev.inventory_sha1,
+                       )
+        if rev.timezone:
+            root.set('timezone', str(rev.timezone))
+        root.text = '\n'
+
+        msg = SubElement(root, 'message')
+        msg.text = rev.message
+        msg.tail = '\n'
+
+        if rev.parents:
+            pelts = SubElement(root, 'parents')
+            pelts.tail = pelts.text = '\n'
+            for rr in rev.parents:
+                assert isinstance(rr, RevisionReference)
+                p = SubElement(pelts, 'revision_ref')
+                p.tail = '\n'
+                assert rr.revision_id
+                p.set('revision_id', rr.revision_id)
+                if rr.revision_sha1:
+                    p.set('revision_sha1', rr.revision_sha1)
+
+        return root
+
+    
+    def _unpack_revision(self, elt):
+        """XML Element -> Revision object"""
+        
+        # <changeset> is deprecated...
+        if elt.tag not in ('revision', 'changeset'):
+            raise BzrError("unexpected tag in revision file: %r" % elt)
+
+        rev = Revision(committer = elt.get('committer'),
+                       timestamp = float(elt.get('timestamp')),
+                       revision_id = elt.get('revision_id'),
+                       inventory_id = elt.get('inventory_id'),
+                       inventory_sha1 = elt.get('inventory_sha1')
+                       )
+
+        precursor = elt.get('precursor')
+        precursor_sha1 = elt.get('precursor_sha1')
+
+        pelts = elt.find('parents')
+
+        if pelts:
+            for p in pelts:
+                assert p.tag == 'revision_ref', \
+                       "bad parent node tag %r" % p.tag
+                rev_ref = RevisionReference(p.get('revision_id'),
+                                            p.get('revision_sha1'))
+                rev.parents.append(rev_ref)
+
+            if precursor:
+                # must be consistent
+                prec_parent = rev.parents[0].revision_id
+                assert prec_parent == precursor
+        elif precursor:
+            # revisions written prior to 0.0.5 have a single precursor
+            # give as an attribute
+            rev_ref = RevisionReference(precursor, precursor_sha1)
+            rev.parents.append(rev_ref)
+
+        v = elt.get('timezone')
+        rev.timezone = v and int(v)
+
+        rev.message = elt.findtext('message') # text of <message>
+        return rev
+
+
+
+class _Serializer_v5(Serializer):
+    """Version 5 serializer
+
+    Packs objects into XML and vice versa.
+
+    You should use the serialzer_v5 singleton."""
+    
+    __slots__ = []
+    
+    def _pack_inventory(self, inv):
+        """Convert to XML Element"""
+        e = Element('inventory')
+        e.text = '\n'
+        if inv.root.file_id not in (None, ROOT_ID):
+            e.set('file_id', inv.root.file_id)
+        for path, ie in inv.iter_entries():
+            e.append(self._pack_entry(ie))
+        return e
+
+
+    def _pack_entry(self, ie):
+        """Convert InventoryEntry to XML element"""
+        assert ie.kind == 'directory' or ie.kind == 'file'
+        e = Element(ie.kind)
+        e.set('name', ie.name)
+        e.set('file_id', ie.file_id)
+
+        if ie.text_size != None:
+            e.set('text_size', '%d' % ie.text_size)
+
+        for f in ['text_version', 'text_sha1', 'entry_version']:
+            v = getattr(ie, f)
+            if v != None:
+                e.set(f, v)
+
+        # to be conservative, we don't externalize the root pointers
+        # for now, leaving them as null in the xml form.  in a future
+        # version it will be implied by nested elements.
+        if ie.parent_id != ROOT_ID:
+            assert isinstance(ie.parent_id, basestring)
+            e.set('parent_id', ie.parent_id)
+
+        e.tail = '\n'
+
+        return e
+
+
+    def _pack_revision(self, rev):
+        """Revision object -> xml tree"""
+        root = Element('revision',
+                       committer = rev.committer,
+                       timestamp = '%.9f' % rev.timestamp,
+                       revision_id = rev.revision_id,
+                       inventory_id = rev.inventory_id,
+                       inventory_sha1 = rev.inventory_sha1,
+                       )
+        if rev.timezone:
+            root.set('timezone', str(rev.timezone))
+        root.text = '\n'
+
+        msg = SubElement(root, 'message')
+        msg.text = rev.message
+        msg.tail = '\n'
+
+        if rev.parents:
+            pelts = SubElement(root, 'parents')
+            pelts.tail = pelts.text = '\n'
+            for rr in rev.parents:
+                assert isinstance(rr, RevisionReference)
+                p = SubElement(pelts, 'revision_ref')
+                p.tail = '\n'
+                assert rr.revision_id
+                p.set('revision_id', rr.revision_id)
+
+        return root
+
+    
+
+    def _unpack_inventory(self, elt):
+        """Construct from XML Element
+        """
+        assert elt.tag == 'inventory'
+        root_id = elt.get('file_id') or ROOT_ID
+        inv = Inventory(root_id)
+        for e in elt:
+            ie = self._unpack_entry(e)
+            if ie.parent_id == ROOT_ID:
+                ie.parent_id = root_id
+            inv.add(ie)
+        return inv
+
+
+    def _unpack_entry(self, elt):
+        kind = elt.tag
+        assert kind == 'directory' or kind == 'file'
+
+        parent_id = elt.get('parent_id')
+        if parent_id == None:
+            parent_id = ROOT_ID
+
+        ie = InventoryEntry(elt.get('file_id'),
+                            elt.get('name'),
+                            kind,
+                            parent_id)
+        ie.text_version = elt.get('text_version')
+        ie.entry_version = elt.get('entry_version')
+        ie.text_sha1 = elt.get('text_sha1')
+        v = elt.get('text_size')
+        ie.text_size = v and int(v)
+
+        return ie
+
+
+    def _unpack_revision(self, elt):
+        """XML Element -> Revision object"""
+        assert elt.tag == 'revision'
+        
+        rev = Revision(committer = elt.get('committer'),
+                       timestamp = float(elt.get('timestamp')),
+                       revision_id = elt.get('revision_id'),
+                       inventory_id = elt.get('inventory_id'),
+                       inventory_sha1 = elt.get('inventory_sha1')
+                       )
+
+        for p in elt.find('parents'):
+            assert p.tag == 'revision_ref', \
+                   "bad parent node tag %r" % p.tag
+            rev_ref = RevisionReference(p.get('revision_id'))
+            rev.parents.append(rev_ref)
+
+        v = elt.get('timezone')
+        rev.timezone = v and int(v)
+
+        rev.message = elt.findtext('message') # text of <message>
+        return rev
+
+
+
+"""singleton instance"""
+serializer_v4 = _Serializer_v4()
+
+serializer_v5 = _Serializer_v5()

=== added directory 'contrib/emacs'
=== added file 'contrib/emacs/bzr-mode.el'
--- contrib/emacs/bzr-mode.el	1970-01-01 00:00:00 +0000
+++ contrib/emacs/bzr-mode.el	2005-08-25 05:58:05 +0000
@@ -0,0 +1,163 @@
+;;; bzr.el -- version control commands for Bazaar-NG.
+;;; Copyright 2005  Luke Gorrie <luke@member.fsf.org>
+;;;
+;;; bzr.el is free software distributed under the terms of the GNU
+;;; General Public Licence, version 2. For details see the file
+;;; COPYING in the GNU Emacs distribution.
+;;;
+;;; This is MAJOR copy & paste job from darcs.el
+
+(eval-when-compile
+  (unless (fboundp 'define-minor-mode)
+    (require 'easy-mmode)
+    (defalias 'define-minor-mode 'easy-mmode-define-minor-mode))
+  (when (featurep 'xemacs)
+    (require 'cl)))
+
+;;;; Configurables
+
+(defvar bzr-command-prefix "\C-cb"
+  ;; This default value breaks the Emacs rules and uses a sequence
+  ;; reserved for the user's own custom bindings. That's not good but
+  ;; I can't think of a decent standard one. -luke (14/Mar/2005)
+  "Prefix sequence for bzr-mode commands.")
+
+(defvar bzr-command "bzr"
+  "*Shell command to execute bzr.")
+
+(defvar bzr-buffer "*bzr-command*"
+  "Buffer for user-visible bzr command output.")
+
+;;;; Minor-mode
+
+(define-minor-mode bzr-mode
+  "\\{bzr-mode-map}"
+  nil
+  " bzr"
+  ;; Coax define-minor-mode into creating a keymap.
+  ;; We'll fill it in manually though because define-minor-mode seems
+  ;; hopeless for changing bindings without restarting Emacs.
+  `((,bzr-command-prefix . fake)))
+
+(defvar bzr-mode-commands-map nil
+  "Keymap for bzr-mode commands.
+This map is bound to a prefix sequence in `bzr-mode-map'.")
+
+(defconst bzr-command-keys '(("l" bzr-log)
+                             ("d" bzr-diff)
+                             ("s" bzr-status)
+                             ("c" bzr-commit))
+  "Keys to bind in `bzr-mode-commands-map'.")
+
+(defun bzr-init-command-keymap ()
+  "Bind the bzr-mode keys.
+This command can be called interactively to redefine the keys from
+`bzr-commands-keys'."
+  (interactive)
+  (setq bzr-mode-commands-map (make-sparse-keymap))
+  (dolist (spec bzr-command-keys)
+    (define-key bzr-mode-commands-map (car spec) (cadr spec)))
+  (define-key bzr-mode-map bzr-command-prefix bzr-mode-commands-map))
+
+(bzr-init-command-keymap)
+
+
+;;;; Commands
+
+(defun bzr-log ()
+  "Run \"bzr log\" in the repository top-level."
+  (interactive)
+  (bzr "log"))
+
+(defun bzr-diff ()
+  "Run \"bzr diff\" in the repository top-level."
+  (interactive)
+  (save-some-buffers)
+  (bzr-run-command (bzr-command "diff") 'diff-mode))
+
+(defun bzr-status ()
+  "Run \"bzr diff\" in the repository top-level."
+  (interactive)
+  (bzr "status"))
+
+(defun bzr-commit (message)
+  "Run \"bzr diff\" in the repository top-level."
+  (interactive "sCommit message: ")
+  (save-some-buffers)
+  (bzr "commit -m %s" (shell-quote-argument message)))
+
+;;;; Utilities
+
+(defun bzr (format &rest args)
+  (bzr-run-command (apply #'bzr-command format args)))
+
+(defun bzr-command (format &rest args)
+  (concat bzr-command " " (apply #'format format args)))
+
+(defun bzr-run-command (command &optional pre-view-hook)
+  "Run COMMAND at the top-level and view the result in another window.
+PRE-VIEW-HOOK is an optional function to call before entering
+view-mode. This is useful to set the major-mode of the result buffer,
+because if you did it afterwards then it would zap view-mode."
+  (bzr-cleanup)
+  (let ((toplevel (bzr-toplevel)))
+    (with-current-buffer (get-buffer-create bzr-buffer)
+      ;; prevent `shell-command' from printing output in a message
+      (let ((max-mini-window-height 0))
+        (let ((default-directory toplevel))
+          (shell-command command t)))
+      (goto-char (point-min))
+      (when pre-view-hook
+        (funcall pre-view-hook))))
+  (if (zerop (buffer-size (get-buffer bzr-buffer)))
+      (message "(bzr command finished with no output.)")
+    (view-buffer-other-window bzr-buffer)
+    ;; Bury the buffer when dismissed.
+    (with-current-buffer (get-buffer bzr-buffer)
+      (setq view-exit-action #'bury-buffer))))
+
+(defun bzr-current-file ()
+  (or (buffer-file-name)
+      (error "Don't know what file to use!")))
+
+(defun bzr-cleanup (&optional buffer-name)
+  "Cleanup before executing a command.
+BUFFER-NAME is the command's output buffer."
+  (let ((name (or buffer-name bzr-buffer)))
+    (when (get-buffer bzr-buffer)
+      (kill-buffer bzr-buffer))))
+
+(defun bzr-toplevel ()
+  "Return the top-level directory of the repository."
+  (let ((dir (bzr-find-repository)))
+    (if dir
+        (file-name-directory dir)
+      (error "Can't find bzr repository top-level."))))
+  
+(defun bzr-find-repository (&optional start-directory)
+  "Return the enclosing \".bzr\" directory, or nil if there isn't one."
+  (when (and (buffer-file-name)
+             (file-directory-p (file-name-directory (buffer-file-name))))
+    (let ((dir (or start-directory
+                   default-directory
+                   (error "No start directory given."))))
+      (or (car (directory-files dir t "^\\.bzr$"))
+          (let ((next-dir (file-name-directory (directory-file-name dir))))
+            (unless (equal dir next-dir)
+              (bzr-find-repository next-dir)))))))
+
+;;;; Hook setup
+;;;
+;;; Automaticaly enter bzr-mode when we open a file that's under bzr
+;;; control, i.e. if the .bzr directory can be found.
+
+(defun bzr-find-file-hook ()
+  "Enable bzr-mode if the file is inside a bzr repository."
+  ;; Note: This function is called for every file that Emacs opens so
+  ;; it mustn't make any mistakes.
+  (when (bzr-find-repository) (bzr-mode 1)))
+
+(add-hook 'find-file-hooks 'bzr-find-file-hook)
+
+(provide 'bzr)
+

=== modified file 'doc/index.txt'
--- doc/index.txt	2005-05-17 00:33:59 +0000
+++ doc/index.txt	2005-09-01 02:39:02 +0000
@@ -14,10 +14,9 @@
 __ http://canonical.com/
 
 
-**Note:** These documents are in a very preliminary state, and so may
-be internally or externally inconsistent or redundant.  Comments are
-still very welcome.  Please send them to <mbp@sourcefrog.net>.
-
+**Note: These documents describe the initial design and are out of
+date in parts.**  For more up-to-date information see the wiki at
+http://bazaar.canonical.com/
 
 For more information, see the homepage at http://bazaar-ng.org/
 

=== added file 'doc/split-join-files.txt'
--- doc/split-join-files.txt	1970-01-01 00:00:00 +0000
+++ doc/split-join-files.txt	2005-08-25 05:58:05 +0000
@@ -0,0 +1,59 @@
+Split and join files
+====================
+
+Version control systems tend to have trouble when files are split or
+joined.  Here is one possibility for handling that.
+
+
+Splitting files
+---------------
+
+This can work like 'svn cp': copy with history::
+
+  % bzr cp a b
+
+This creates a new file b, whose file-history is the same as a, but
+with a different file-id, and present at a different name.
+
+This creates the slightly strange situation that b claims to have text
+versions for revisions where b was not actually present.
+
+
+Joining files
+-------------
+
+We might want to also join files in various cases:
+
+ * Multiple imports of the same file
+
+ * Two source files are being refactored into a single one
+
+ * Two people apply an external patch that creates the same file
+
+I'm not aware of any other system that handles this.
+
+  % bzr join a b
+
+a and b must be versioned files.  b is removed from the working tree
+and inventory.  b's file id is added as a secondary id as a, so that
+later merges between them will detect them as being the same.  
+
+b's weave is (???) incorporated into a.  
+
+This means that we have a file which has multiple history paths.  For
+example, asking to compare that file to its state in a previous
+version has to arbitrarily pick one to use.  That's not necessarily
+fatal if we assign one as primary, but it might be a bit confusing.
+Trying to merge from another tree might can't really choose a good
+basis.
+
+
+
+File aliases
+------------
+
+Rather than trying to join files we might just add notes that files
+are also known by a different file-id in other trees.
+
+
+

=== modified file 'doc/todo-from-arch.txt'
--- doc/todo-from-arch.txt	2005-04-15 01:31:21 +0000
+++ doc/todo-from-arch.txt	2005-09-01 02:34:38 +0000
@@ -2,10 +2,11 @@
 Opportunities for improvement on GNU Arch
 ***************************************** 
 
+[note that this document is rather out of date in 2005-08]
 
-Bazaar-NG is based on the GNU Arch system, and inherits a lot of its
-design from Arch.  However, there are several things we will change in
-Baz to (we hope) improve the user experience.
+GNU Arch is one influence on bazaar-ng.  There are several things we
+would change from Arch in Bazaar to (we hope) improve the user
+experience.
 
 The core design of Arch is good, brilliant even.  It can scale from
 small projects too large ones, and is a good foundation for building

=== added file 'notes/inventory-v2-sample.xml'
--- notes/inventory-v2-sample.xml	1970-01-01 00:00:00 +0000
+++ notes/inventory-v2-sample.xml	2005-08-19 21:34:32 +0000
@@ -0,0 +1,10 @@
+<inventory>
+  <root>
+    <file name="readme.txt" id="readme-123" size="400" sha1="abcdef" 
+          revision="2050808"/>
+    <directory name="src" id="src-123"> 
+      <directory name="libwibble" id="libwibble-123">
+      </directory>
+    </directory>
+  </root>
+</inventory>
\ No newline at end of file

=== added file 'notes/inventory-v2.rnc'
--- notes/inventory-v2.rnc	1970-01-01 00:00:00 +0000
+++ notes/inventory-v2.rnc	2005-08-19 21:34:32 +0000
@@ -0,0 +1,30 @@
+grammar {
+  file = element file {
+    attribute id { text },
+    attribute name { text }, 
+
+    # the following attributes are absent on the working-inventory
+    attribute sha1 { text }?,
+    attribute size { text }?,
+
+    attribute revision { text }?,
+
+    empty
+  }
+
+  directory = element directory {
+    attribute id { text },
+    attribute name { text },
+    
+    entry*
+  }
+
+  entry = 
+    (file | directory )
+
+  start = element inventory {
+    element root {
+      entry*
+    }
+  }
+}

=== added file 'notes/revfile.txt'
--- notes/revfile.txt	1970-01-01 00:00:00 +0000
+++ notes/revfile.txt	2005-08-25 05:58:05 +0000
@@ -0,0 +1,45 @@
+importing all inventories:
+
+with 50% chance of choosing earlier parent:
+
+-rw-r--r--  1 mbp mbp 296961 2005-04-09 22:37 inventory.drev
+-rw-r--r--  1 mbp mbp  11184 2005-04-09 22:37 inventory.irev
+
+imported in 2.490s
+
+
+with no chance:
+
+clearly slows down, 11.163s to import
+
+-rw-r--r--  1 mbp mbp 63590 2005-04-09 22:39 inventory.drev
+-rw-r--r--  1 mbp mbp 11184 2005-04-09 22:39 inventory.irev
+
+
+
+with 25% chance:
+
+revfile.py add-series inventory 0 mbp*  7.86s user 0.12s system 96% cpu 8.247 total
+-rw-r--r--  1 mbp mbp 79535 2005-04-09 22:42 inventory.drev
+-rw-r--r--  1 mbp mbp 11184 2005-04-09 22:42 inventory.irev
+
+
+
+with full text inserted every 20 revisions:
+
+revfile.py add-series inventory 0 mbp*  1.98s user 0.06s system 95% cpu 2.139 total
+
+-rw-r--r--  1 mbp mbp 133719 2005-04-09 22:44 inventory.drev
+-rw-r--r--  1 mbp mbp  11184 2005-04-09 22:44 inventory.irev
+
+So pretty quick, simple, and somewhat easily justified.
+
+
+
+every 50 revisions:
+
+revfile.py add-series inventory 0 mbp*  3.35s user 0.08s system 93% cpu 3.667 total
+
+-rw-r--r--  1 mbp mbp 91895 2005-04-09 22:46 inventory.drev
+-rw-r--r--  1 mbp mbp 11184 2005-04-09 22:46 inventory.irev
+

=== added file 'notes/schemas.xml'
--- notes/schemas.xml	1970-01-01 00:00:00 +0000
+++ notes/schemas.xml	2005-08-19 21:34:32 +0000
@@ -0,0 +1,4 @@
+<?xml version="1.0"?>
+<locatingRules xmlns="http://thaiopensource.com/ns/locating-rules/1.0">
+  <uri resource="inventory-v2-sample.xml" uri="inventory-v2.rnc"/>
+</locatingRules>

=== added file 'patches/pending-merge.patch'
--- patches/pending-merge.patch	1970-01-01 00:00:00 +0000
+++ patches/pending-merge.patch	2005-08-25 05:58:05 +0000
@@ -0,0 +1,104 @@
+*** modified file 'bzrlib/merge.py'
+--- bzrlib/merge.py 
++++ bzrlib/merge.py 
+@@ -105,14 +105,21 @@
+     location, revno = treespec
+     branch = find_branch(location)
+     if revno is None:
++        rev_id = None
++    elif revno == -1:
++        rev_id = branch.last_patch()
++    else:
++        rev_id = branch.lookup_revision(revno)
++    return branch, get_revid_tree(branch, rev_id, temp_root, label)
++
++def get_revid_tree(branch, rev_id, temp_root, label):
++    if rev_id is None:
+         base_tree = branch.working_tree()
+-    elif revno == -1:
+-        base_tree = branch.basis_tree()
+     else:
+-        base_tree = branch.revision_tree(branch.lookup_revision(revno))
++        base_tree = branch.revision_tree(rev_id)
+     temp_path = os.path.join(temp_root, label)
+     os.mkdir(temp_path)
+-    return branch, MergeTree(base_tree, temp_path)
++    return MergeTree(base_tree, temp_path)
+ 
+ 
+ def abspath(tree, file_id):
+@@ -178,30 +185,68 @@
+         If true, this_dir must have no uncommitted changes before the
+         merge begins.
+     """
++    from bzrlib.revision import common_ancestor, MultipleRevisionSources
++    from bzrlib.branch import NoSuchRevision
+     tempdir = tempfile.mkdtemp(prefix="bzr-")
+     try:
+         if this_dir is None:
+             this_dir = '.'
+         this_branch = find_branch(this_dir)
++        this_rev_id = this_branch.last_patch()
++        if this_rev_id is None:
++            raise BzrCommandError("This branch has no commits")
+         if check_clean:
+             changes = compare_trees(this_branch.working_tree(), 
+                                     this_branch.basis_tree(), False)
+             if changes.has_changed():
+                 raise BzrCommandError("Working tree has uncommitted changes.")
+         other_branch, other_tree = get_tree(other_revision, tempdir, "other")
++        if other_revision[1] == -1:
++            other_rev_id = other_branch.last_patch()
++            other_basis = other_rev_id
++        elif other_revision[1] is not None:
++            other_rev_id = other_branch.lookup_revision(other_revision[1])
++            other_basis = other_rev_id
++        else:
++            other_rev_id = None
++            other_basis = other_branch.last_patch()
+         if base_revision == [None, None]:
+             if other_revision[1] == -1:
+                 o_revno = None
+             else:
+                 o_revno = other_revision[1]
+-            base_revno = this_branch.common_ancestor(other_branch, 
+-                                                     other_revno=o_revno)[0]
+-            if base_revno is None:
++            base_rev_id = common_ancestor(this_rev_id, other_basis,
++                                          MultipleRevisionSources(this_branch, 
++                                          other_branch))
++            if base_rev_id is None:
+                 raise UnrelatedBranches()
+-            base_revision = ['.', base_revno]
+-        base_branch, base_tree = get_tree(base_revision, tempdir, "base")
++            try:
++                base_revision = this_branch.get_revision(base_rev_id)
++                base_branch = this_branch
++            except NoSuchRevision:
++                base_branch = other_branch
++            base_tree = get_revid_tree(base_branch, base_rev_id, tempdir, 
++                                       "base")
++            base_is_ancestor = True
++        else:
++            base_branch, base_tree = get_tree(base_revision, tempdir, "base")
++            if base_revision[1] == -1:
++                base_rev_id = base_branch.last_patch()
++            elif base_revision[1] is None:
++                base_rev_id = None
++            else:
++                base_rev_id = base_branch.lookup_revision(base_revision[1])
++            if base_rev_id is not None:
++                base_is_ancestor = is_ancestor(this_rev_id, base_rev_id, 
++                                               MultipleRevisionSources(
++                                               this_branch, 
++                                               base_branch))
++            else:
++                base_is_ancestor = False
+         merge_inner(this_branch, other_tree, base_tree, tempdir, 
+                     ignore_zero=ignore_zero)
++        if base_is_ancestor and other_rev_id is not None:
++            this_branch.add_pending_merge(other_rev_id)
+     finally:
+         shutil.rmtree(tempdir)
+ 
+

=== removed directory 'plugins/changeset'
=== removed file 'plugins/changeset/__init__.py'
--- plugins/changeset/__init__.py	2005-06-22 09:34:59 +0000
+++ plugins/changeset/__init__.py	1970-01-01 00:00:00 +0000
@@ -1,105 +0,0 @@
-#!/usr/bin/env python
-"""\
-This is an attempt to take the internal delta object, and represent
-it as a single-file text-only changeset.
-This should have commands for both generating a changeset,
-and for applying a changeset.
-"""
-
-import bzrlib, bzrlib.commands
-
-class cmd_changeset(bzrlib.commands.Command):
-    """Generate a bundled up changeset.
-
-    This changeset contains all of the meta-information of a
-    diff, rather than just containing the patch information.
-
-    If a file is specified, only changes affecting that file are
-    specified; this gives a changeset that is probably not useful.
-
-    Right now, rollup changesets, or working tree changesets are
-    not supported. This will only generate a changeset that has been
-    committed. You can use "--revision" to specify a certain change
-    to display.
-    """
-    takes_options = ['revision', 'diff-options']
-    takes_args = ['file*']
-    aliases = ['cset']
-
-    def run(self, revision=None, file_list=None, diff_options=None):
-        from bzrlib import find_branch
-        import gen_changeset
-        import sys
-
-        if isinstance(revision, (list, tuple)):
-            if len(revision) > 1:
-                raise BzrCommandError('We do not support rollup-changesets yet.')
-            revision = revision[0]
-        if file_list:
-            b = find_branch(file_list[0])
-            file_list = [b.relpath(f) for f in file_list]
-            if file_list == ['']:
-                # just pointing to top-of-tree
-                file_list = None
-        else:
-            b = find_branch('.')
-
-        gen_changeset.show_changeset(b, revision,
-                specific_files=file_list,
-                external_diff_options=diff_options,
-                to_file=sys.stdout)
-
-class cmd_verify_changeset(bzrlib.commands.Command):
-    """Read a written changeset, and make sure it is valid.
-
-    """
-    takes_args = ['filename?']
-
-    def run(self, filename=None):
-        import sys, read_changeset
-        if filename is None or filename == '-':
-            f = sys.stdin
-        else:
-            f = open(filename, 'rb')
-
-        cset_info = read_changeset.read_changeset(f)
-        print cset_info
-        cset = cset_info.get_changeset()
-        print cset.entries
-
-class cmd_apply_changeset(bzrlib.commands.Command):
-    """Read in the given changeset, and apply it to the
-    current tree.
-
-    """
-    takes_args = ['filename?']
-    takes_options = []
-
-    def run(self, filename=None, reverse=False, auto_commit=False):
-        from bzrlib import find_branch
-        import sys
-        import apply_changeset
-
-        b = find_branch('.') # Make sure we are in a branch
-        if filename is None or filename == '-':
-            f = sys.stdin
-        else:
-            f = open(filename, 'rb')
-
-        apply_changeset.apply_changeset(b, f, reverse=reverse,
-                auto_commit=auto_commit)
-
-
-##if hasattr(bzrlib.commands, 'register_plugin_cmd'):
-
-## print dir(bzrlib.commands)
-
-bzrlib.commands.register_command(cmd_changeset)
-bzrlib.commands.register_command(cmd_verify_changeset)
-bzrlib.commands.register_command(cmd_apply_changeset)
-
-bzrlib.commands.OPTIONS['reverse'] = None
-bzrlib.commands.OPTIONS['auto-commit'] = None
-cmd_apply_changeset.takes_options.append('reverse')
-cmd_apply_changeset.takes_options.append('auto-commit')
-

=== removed file 'plugins/changeset/apply_changeset.py'
--- plugins/changeset/apply_changeset.py	2005-06-22 09:08:43 +0000
+++ plugins/changeset/apply_changeset.py	1970-01-01 00:00:00 +0000
@@ -1,41 +0,0 @@
-#!/usr/bin/env python
-"""\
-This contains the apply changset function for bzr
-"""
-
-import bzrlib
-
-def apply_changeset(branch, from_file, reverse=False, auto_commit=False):
-    from bzrlib.changeset import apply_changeset as _apply_changeset
-    from bzrlib.merge import regen_inventory
-    import sys, read_changeset
-
-
-    cset_info = read_changeset.read_changeset(from_file)
-    cset = cset_info.get_changeset()
-    inv = {}
-    for file_id in branch.inventory:
-        inv[file_id] = branch.inventory.id2path(file_id)
-    changes = _apply_changeset(cset, inv, branch.base,
-            reverse=reverse)
-
-    adjust_ids = []
-    for id, path in changes.iteritems():
-        if path is not None:
-            if path == '.':
-                path = ''
-        adjust_ids.append((path, id))
-
-    branch.set_inventory(regen_inventory(branch, branch.base, adjust_ids))
-
-    if auto_commit:
-        from bzrlib.commit import commit
-        if branch.last_patch() == cset_info.precursor:
-            # This patch can be applied directly
-            commit(branch, message = cset_info.message,
-                    timestamp=float(cset_info.timestamp),
-                    timezone=float(cset_info.timezone),
-                    committer=cset_info.committer,
-                    rev_id=cset_info.revision)
-
-

=== removed file 'plugins/changeset/common.py'
--- plugins/changeset/common.py	2005-06-22 09:08:43 +0000
+++ plugins/changeset/common.py	1970-01-01 00:00:00 +0000
@@ -1,15 +0,0 @@
-#!/usr/bin/env python
-"""\
-Common entries, like strings, etc, for the changeset reading + writing code.
-"""
-
-header_str = 'Bazaar-NG (bzr) changeset v'
-version = (0, 0, 5)
-
-def get_header():
-    return [
-        header_str + '.'.join([str(v) for v in version]),
-        'This changeset can be applied with bzr apply-changeset',
-        ''
-    ]
-

=== removed file 'plugins/changeset/gen_changeset.py'
--- plugins/changeset/gen_changeset.py	2005-06-22 09:08:43 +0000
+++ plugins/changeset/gen_changeset.py	1970-01-01 00:00:00 +0000
@@ -1,345 +0,0 @@
-#!/usr/bin/env python
-"""\
-Just some work for generating a changeset.
-"""
-
-import bzrlib, bzrlib.errors
-
-import common
-
-from bzrlib.inventory import ROOT_ID
-
-try:
-    set
-except NameError:
-    from sets import Set as set
-
-def _canonicalize_revision(branch, revno):
-    """Turn some sort of revision information into a single
-    set of from-to revision ids.
-
-    A revision id can be None if there is no associated revison.
-
-    :return: (old, new)
-    """
-    # This is a little clumsy because revision parsing may return
-    # a single entry, or a list
-    if revno is None:
-        new = branch.last_patch()
-    else:
-        new = branch.lookup_revision(revno)
-
-    if new is None:
-        raise BzrCommandError('Cannot generate a changset with no commits in tree.')
-
-    old = branch.get_revision(new).precursor
-
-    return old, new
-
-def _get_trees(branch, revisions):
-    """Get the old and new trees based on revision.
-    """
-    from bzrlib.tree import EmptyTree
-    if revisions[0] is None:
-        if hasattr(branch, 'get_root_id'): # Watch out for trees with labeled ROOT ids
-            old_tree = EmptyTree(branch.get_root_id) 
-        else:
-            old_tree = EmptyTree()
-    else:
-        old_tree = branch.revision_tree(revisions[0])
-
-    if revisions[1] is None:
-        # This is for the future, once we support rollup revisions
-        # Or working tree revisions
-        new_tree = branch.working_tree()
-    else:
-        new_tree = branch.revision_tree(revisions[1])
-    return old_tree, new_tree
-
-def _fake_working_revision(branch):
-    """Fake a Revision object for the working tree.
-    
-    This is for the future, to support changesets against the working tree.
-    """
-    from bzrlib.revision import Revision
-    import time
-    from bzrlib.osutils import local_time_offset, \
-            username
-
-    precursor = branch.last_patch()
-    precursor_sha1 = branch.get_revision_sha1(precursor)
-
-    return Revision(timestamp=time.time(),
-            timezone=local_time_offset(),
-            committer=username(),
-            precursor=precursor,
-            precursor_sha1=precursor_sha1)
-
-
-class MetaInfoHeader(object):
-    """Maintain all of the header information about this
-    changeset.
-    """
-
-    def __init__(self, branch, revisions, delta,
-            full_remove=True, full_rename=False,
-            external_diff_options = None,
-            new_tree=None, old_tree=None,
-            old_label = '', new_label = ''):
-        """
-        :param full_remove: Include the full-text for a delete
-        :param full_rename: Include an add+delete patch for a rename
-        """
-        self.branch = branch
-        self.delta = delta
-        self.full_remove=full_remove
-        self.full_rename=full_rename
-        self.external_diff_options = external_diff_options
-        self.old_label = old_label
-        self.new_label = new_label
-        self.old_tree = old_tree
-        self.new_tree = new_tree
-        self.to_file = None
-        self.revno = None
-        self.precursor_revno = None
-
-        self._get_revision_list(revisions)
-
-    def _get_revision_list(self, revisions):
-        """This generates the list of all revisions from->to.
-
-        This is for the future, when we support having a rollup changeset.
-        For now, the list should only be one long.
-        """
-        old_revno = None
-        new_revno = None
-        rh = self.branch.revision_history()
-        for revno, rev in enumerate(rh):
-            if rev == revisions[0]:
-                old_revno = revno
-            if rev == revisions[1]:
-                new_revno = revno
-
-        self.revision_list = []
-        if old_revno is None:
-            self.base_revision = None # Effectively the EmptyTree()
-            old_revno = -1
-        else:
-            self.base_revision = self.branch.get_revision(rh[old_revno])
-        if new_revno is None:
-            # For the future, when we support working tree changesets.
-            for rev_id in rh[old_revno+1:]:
-                self.revision_list.append(self.branch.get_revision(rev_id))
-            self.revision_list.append(_fake_working_revision(self.branch))
-        else:
-            for rev_id in rh[old_revno+1:new_revno+1]:
-                self.revision_list.append(self.branch.get_revision(rev_id))
-        self.precursor_revno = old_revno
-        self.revno = new_revno
-
-    def _write(self, txt, key=None):
-        if key:
-            self.to_file.write('# %s: %s\n' % (key, txt))
-        else:
-            self.to_file.write('# %s\n' % (txt,))
-
-    def write_meta_info(self, to_file):
-        """Write out the meta-info portion to the supplied file.
-
-        :param to_file: Write out the meta information to the supplied
-                        file
-        """
-        self.to_file = to_file
-
-        self._write_header()
-        self._write_diffs()
-        self._write_footer()
-
-    def _write_header(self):
-        """Write the stuff that comes before the patches."""
-        from bzrlib.osutils import username, format_date
-        write = self._write
-
-        for line in common.get_header():
-            write(line)
-
-        # This grabs the current username, what we really want is the
-        # username from the actual patches.
-        #write(username(), key='committer')
-        assert len(self.revision_list) == 1
-        rev = self.revision_list[0]
-        write(rev.committer, key='committer')
-        write(format_date(rev.timestamp, offset=rev.timezone), key='date')
-        write(str(self.revno), key='revno')
-        if rev.message:
-            self.to_file.write('# message:\n')
-            for line in rev.message.split('\n'):
-                self.to_file.write('#    %s\n' % line)
-        write(rev.revision_id, key='revision')
-
-        if self.base_revision:
-            write(self.base_revision.revision_id, key='precursor')
-            write(str(self.precursor_revno), key='precursor revno')
-
-
-        write('')
-        self.to_file.write('\n')
-
-    def _write_footer(self):
-        """Write the stuff that comes after the patches.
-
-        This is meant to be more meta-information, which people probably don't want
-        to read, but which is required for proper bzr operation.
-        """
-        write = self._write
-
-        write('BEGIN BZR FOOTER')
-
-        assert len(self.revision_list) == 1 # We only handle single revision entries
-        rev = self.revision_list[0]
-        write(self.branch.get_revision_sha1(rev.revision_id),
-                key='revision sha1')
-        if self.base_revision:
-            rev_id = self.base_revision.revision_id
-            write(self.branch.get_revision_sha1(rev_id),
-                    key='precursor sha1')
-
-        write('%.9f' % rev.timestamp, key='timestamp')
-        write(str(rev.timezone), key='timezone')
-
-        self._write_ids()
-
-        write('END BZR FOOTER')
-
-    def _write_revisions(self):
-        """Not used. Used for writing multiple revisions."""
-        first = True
-        for rev in self.revision_list:
-            if rev.revision_id is not None:
-                if first:
-                    self._write('revisions:')
-                    first = False
-                self._write(' '*4 + rev.revision_id + '\t' + self.branch.get_revision_sha1(rev.revision_id))
-
-
-    def _write_ids(self):
-        if hasattr(self.branch, 'get_root_id'):
-            root_id = self.branch.get_root_id()
-        else:
-            root_id = ROOT_ID
-
-        old_ids = set()
-        new_ids = set()
-
-        for path, file_id, kind in self.delta.removed:
-            old_ids.add(file_id)
-        for path, file_id, kind in self.delta.added:
-            new_ids.add(file_id)
-        for old_path, new_path, file_id, kind, text_modified in self.delta.renamed:
-            old_ids.add(file_id)
-            new_ids.add(file_id)
-        for path, file_id, kind in self.delta.modified:
-            new_ids.add(file_id)
-
-        self._write(root_id, key='tree root id')
-
-        def write_ids(tree, id_set, name):
-            if len(id_set) > 0:
-                self.to_file.write('# %s ids:\n' % name)
-            seen_ids = set([root_id])
-            while len(id_set) > 0:
-                file_id = id_set.pop()
-                if file_id in seen_ids:
-                    continue
-                seen_ids.add(file_id)
-                ie = tree.inventory[file_id]
-                if ie.parent_id not in seen_ids:
-                    id_set.add(ie.parent_id)
-                path = tree.inventory.id2path(file_id)
-                self.to_file.write('#    %s\t%s\t%s\n'
-                        % (path.encode('utf8'), file_id.encode('utf8'),
-                            ie.parent_id.encode('utf8')))
-        write_ids(self.new_tree, new_ids, 'file')
-        write_ids(self.old_tree, old_ids, 'old file')
-
-    def _write_diffs(self):
-        """Write out the specific diffs"""
-        from bzrlib.diff import internal_diff, external_diff
-        DEVNULL = '/dev/null'
-
-        if self.external_diff_options:
-            assert isinstance(self.external_diff_options, basestring)
-            opts = self.external_diff_options.split()
-            def diff_file(olab, olines, nlab, nlines, to_file):
-                external_diff(olab, olines, nlab, nlines, to_file, opts)
-        else:
-            diff_file = internal_diff
-
-        for path, file_id, kind in self.delta.removed:
-            print >>self.to_file, '*** removed %s %r' % (kind, path)
-            if kind == 'file' and self.full_remove:
-                diff_file(self.old_label + path,
-                          self.old_tree.get_file(file_id).readlines(),
-                          DEVNULL, 
-                          [],
-                          self.to_file)
-    
-        for path, file_id, kind in self.delta.added:
-            print >>self.to_file, '*** added %s %r' % (kind, path)
-            if kind == 'file':
-                diff_file(DEVNULL,
-                          [],
-                          self.new_label + path,
-                          self.new_tree.get_file(file_id).readlines(),
-                          self.to_file)
-    
-        for old_path, new_path, file_id, kind, text_modified in self.delta.renamed:
-            print >>self.to_file, '*** renamed %s %r => %r' % (kind, old_path, new_path)
-            if self.full_rename and kind == 'file':
-                diff_file(self.old_label + old_path,
-                          self.old_tree.get_file(file_id).readlines(),
-                          DEVNULL, 
-                          [],
-                          self.to_file)
-                diff_file(DEVNULL,
-                          [],
-                          self.new_label + new_path,
-                          self.new_tree.get_file(file_id).readlines(),
-                          self.to_file)
-            elif text_modified:
-                    diff_file(self.old_label + old_path,
-                              self.old_tree.get_file(file_id).readlines(),
-                              self.new_label + new_path,
-                              self.new_tree.get_file(file_id).readlines(),
-                              self.to_file)
-    
-        for path, file_id, kind in self.delta.modified:
-            print >>self.to_file, '*** modified %s %r' % (kind, path)
-            if kind == 'file':
-                diff_file(self.old_label + path,
-                          self.old_tree.get_file(file_id).readlines(),
-                          self.new_label + path,
-                          self.new_tree.get_file(file_id).readlines(),
-                          self.to_file)
-
-def show_changeset(branch, revision=None, specific_files=None,
-        external_diff_options=None, to_file=None,
-        include_full_diff=False):
-    from bzrlib.diff import compare_trees
-
-    if to_file is None:
-        import sys
-        to_file = sys.stdout
-    revisions = _canonicalize_revision(branch, revision)
-
-    old_tree, new_tree = _get_trees(branch, revisions)
-
-    delta = compare_trees(old_tree, new_tree, want_unchanged=False,
-                          specific_files=specific_files)
-
-    meta = MetaInfoHeader(branch, revisions, delta,
-            external_diff_options=external_diff_options,
-            old_tree=old_tree, new_tree=new_tree)
-    meta.write_meta_info(to_file)
-
-

=== removed file 'plugins/changeset/read_changeset.py'
--- plugins/changeset/read_changeset.py	2005-06-22 09:08:43 +0000
+++ plugins/changeset/read_changeset.py	1970-01-01 00:00:00 +0000
@@ -1,342 +0,0 @@
-#!/usr/bin/env python
-"""\
-Read in a changeset output, and process it into a Changeset object.
-"""
-
-import bzrlib, bzrlib.changeset
-import common
-
-class BadChangeset(Exception): pass
-class MalformedHeader(BadChangeset): pass
-class MalformedPatches(BadChangeset): pass
-class MalformedFooter(BadChangeset): pass
-
-def _unescape(name):
-    """Now we want to find the filename effected.
-    Unfortunately the filename is written out as
-    repr(filename), which means that it surrounds
-    the name with quotes which may be single or double
-    (single is preferred unless there is a single quote in
-    the filename). And some characters will be escaped.
-
-    TODO:   There has to be some pythonic way of undo-ing the
-            representation of a string rather than using eval.
-    """
-    delimiter = name[0]
-    if name[-1] != delimiter:
-        raise BadChangeset('Could not properly parse the'
-                ' filename: %r' % name)
-    # We need to handle escaped hexadecimals too.
-    return name[1:-1].replace('\"', '"').replace("\'", "'")
-
-class ChangesetInfo(object):
-    """This is the intermediate class that gets filled out as
-    the file is read.
-    """
-    def __init__(self):
-        self.committer = None
-        self.date = None
-        self.message = None
-        self.revno = None
-        self.revision = None
-        self.revision_sha1 = None
-        self.precursor = None
-        self.precursor_sha1 = None
-        self.precursor_revno = None
-
-        self.timestamp = None
-        self.timezone = None
-
-        self.tree_root_id = None
-        self.file_ids = None
-        self.old_file_ids = None
-
-        self.actions = [] #this is the list of things that happened
-        self.id2path = {} # A mapping from file id to path name
-        self.path2id = {} # The reverse mapping
-        self.id2parent = {} # A mapping from a given id to it's parent id
-
-        self.old_id2path = {}
-        self.old_path2id = {}
-        self.old_id2parent = {}
-
-    def __str__(self):
-        import pprint
-        return pprint.pformat(self.__dict__)
-
-    def create_maps(self):
-        """Go through the individual id sections, and generate the 
-        id2path and path2id maps.
-        """
-        # Rather than use an empty path, the changeset code seems 
-        # to like to use "./." for the tree root.
-        self.id2path[self.tree_root_id] = './.'
-        self.path2id['./.'] = self.tree_root_id
-        self.id2parent[self.tree_root_id] = bzrlib.changeset.NULL_ID
-        self.old_id2path = self.id2path.copy()
-        self.old_path2id = self.path2id.copy()
-        self.old_id2parent = self.id2parent.copy()
-
-        if self.file_ids:
-            for info in self.file_ids:
-                path, f_id, parent_id = info.split('\t')
-                self.id2path[f_id] = path
-                self.path2id[path] = f_id
-                self.id2parent[f_id] = parent_id
-        if self.old_file_ids:
-            for info in self.old_file_ids:
-                path, f_id, parent_id = info.split('\t')
-                self.old_id2path[f_id] = path
-                self.old_path2id[path] = f_id
-                self.old_id2parent[f_id] = parent_id
-
-    def get_changeset(self):
-        """Create a changeset from the data contained within."""
-        from bzrlib.changeset import Changeset, ChangesetEntry, \
-            PatchApply, ReplaceContents
-        cset = Changeset()
-        
-        entry = ChangesetEntry(self.tree_root_id, 
-                bzrlib.changeset.NULL_ID, './.')
-        cset.add_entry(entry)
-        for info, lines in self.actions:
-            parts = info.split(' ')
-            action = parts[0]
-            kind = parts[1]
-            extra = ' '.join(parts[2:])
-            if action == 'renamed':
-                old_path, new_path = extra.split(' => ')
-                old_path = _unescape(old_path)
-                new_path = _unescape(new_path)
-
-                new_id = self.path2id[new_path]
-                old_id = self.old_path2id[old_path]
-                assert old_id == new_id
-
-                new_parent = self.id2parent[new_id]
-                old_parent = self.old_id2parent[old_id]
-
-                entry = ChangesetEntry(old_id, old_parent, old_path)
-                entry.new_path = new_path
-                entry.new_parent = new_parent
-                if lines:
-                    entry.contents_change = PatchApply(''.join(lines))
-            elif action == 'removed':
-                old_path = _unescape(extra)
-                old_id = self.old_path2id[old_path]
-                old_parent = self.old_id2parent[old_id]
-                entry = ChangesetEntry(old_id, old_parent, old_path)
-                entry.new_path = None
-                entry.new_parent = None
-                if lines:
-                    # Technically a removed should be a ReplaceContents()
-                    # Where you need to have the old contents
-                    # But at most we have a remove style patch.
-                    #entry.contents_change = ReplaceContents()
-                    pass
-            elif action == 'added':
-                new_path = _unescape(extra)
-                new_id = self.path2id[new_path]
-                new_parent = self.id2parent[new_id]
-                entry = ChangesetEntry(new_id, new_parent, new_path)
-                entry.path = None
-                entry.parent = None
-                if lines:
-                    # Technically an added should be a ReplaceContents()
-                    # Where you need to have the old contents
-                    # But at most we have an add style patch.
-                    #entry.contents_change = ReplaceContents()
-                    entry.contents_change = PatchApply(''.join(lines))
-            elif action == 'modified':
-                new_path = _unescape(extra)
-                new_id = self.path2id[new_path]
-                new_parent = self.id2parent[new_id]
-                entry = ChangesetEntry(new_id, new_parent, new_path)
-                entry.path = None
-                entry.parent = None
-                if lines:
-                    # Technically an added should be a ReplaceContents()
-                    # Where you need to have the old contents
-                    # But at most we have an add style patch.
-                    #entry.contents_change = ReplaceContents()
-                    entry.contents_change = PatchApply(''.join(lines))
-            else:
-                raise BadChangeset('Unrecognized action: %r' % action)
-            cset.add_entry(entry)
-        return cset
-
-class ChangesetReader(object):
-    """This class reads in a changeset from a file, and returns
-    a Changeset object, which can then be applied against a tree.
-    """
-    def __init__(self, from_file):
-        """Read in the changeset from the file.
-
-        :param from_file: A file-like object (must have iterator support).
-        """
-        object.__init__(self)
-        self.from_file = from_file
-        
-        self.info = ChangesetInfo()
-        # We put the actual inventory ids in the footer, so that the patch
-        # is easier to read for humans.
-        # Unfortunately, that means we need to read everything before we
-        # can create a proper changeset.
-        self._read_header()
-        next_line = self._read_patches()
-        if next_line is not None:
-            self._read_footer(next_line)
-
-    def get_info(self):
-        """Create the actual changeset object.
-        """
-        self.info.create_maps()
-        return self.info
-
-    def _read_header(self):
-        """Read the bzr header"""
-        header = common.get_header()
-        for head_line, line in zip(header, self.from_file):
-            if (line[:2] != '# '
-                    or line[-1] != '\n'
-                    or line[2:-1] != head_line):
-                raise MalformedHeader('Did not read the opening'
-                    ' header information.')
-
-        for line in self.from_file:
-            if self._handle_info_line(line) is not None:
-                break
-
-    def _handle_info_line(self, line, in_footer=False):
-        """Handle reading a single line.
-
-        This may call itself, in the case that we read_multi,
-        and then had a dangling line on the end.
-        """
-        # The bzr header is terminated with a blank line
-        # which does not start with #
-        next_line = None
-        if line[:1] == '\n':
-            return 'break'
-        if line[:2] != '# ':
-            raise MalformedHeader('Opening bzr header did not start with #')
-
-        line = line[2:-1] # Remove the '# '
-        if not line:
-            return # Ignore blank lines
-
-        if in_footer and line in ('BEGIN BZR FOOTER', 'END BZR FOOTER'):
-            return
-
-        loc = line.find(': ')
-        if loc != -1:
-            key = line[:loc]
-            value = line[loc+2:]
-            if not value:
-                value, next_line = self._read_many()
-        else:
-            if line[-1:] == ':':
-                key = line[:-1]
-                value, next_line = self._read_many()
-            else:
-                raise MalformedHeader('While looking for key: value pairs,'
-                        ' did not find the colon %r' % (line))
-
-        key = key.replace(' ', '_')
-        if hasattr(self.info, key):
-            if getattr(self.info, key) is None:
-                setattr(self.info, key, value)
-            else:
-                raise MalformedHeader('Duplicated Key: %s' % key)
-        else:
-            # What do we do with a key we don't recognize
-            raise MalformedHeader('Unknown Key: %s' % key)
-        
-        if next_line:
-            self._handle_info_line(next_line, in_footer=in_footer)
-
-    def _read_many(self):
-        """If a line ends with no entry, that means that it should be
-        followed with multiple lines of values.
-
-        This detects the end of the list, because it will be a line that
-        does not start with '#    '. Because it has to read that extra
-        line, it returns the tuple: (values, next_line)
-        """
-        values = []
-        for line in self.from_file:
-            if line[:5] != '#    ':
-                return values, line
-            values.append(line[5:-1])
-        return values, None
-
-    def _read_one_patch(self, first_line=None):
-        """Read in one patch, return the complete patch, along with
-        the next line.
-
-        :return: action, lines, next_line, do_continue
-        """
-        first = True
-        action = None
-
-        def parse_firstline(line):
-            if line[:1] == '#':
-                return None
-            if line[:3] != '***':
-                raise MalformedPatches('The first line of all patches'
-                    ' should be a bzr meta line "***"')
-            return line[4:-1]
-
-        if first_line is not None:
-            action = parse_firstline(first_line)
-            first = False
-            if action is None:
-                return None, [], first_line, False
-
-        lines = []
-        for line in self.from_file:
-            if first:
-                action = parse_firstline(line)
-                first = False
-                if action is None:
-                    return None, [], line, False
-            else:
-                if line[:3] == '***':
-                    return action, lines, line, True
-                elif line[:1] == '#':
-                    return action, lines, line, False
-                lines.append(line)
-        return action, lines, None, False
-            
-    def _read_patches(self):
-        next_line = None
-        do_continue = True
-        while do_continue:
-            action, lines, next_line, do_continue = \
-                    self._read_one_patch(next_line)
-            if action is not None:
-                self.info.actions.append((action, lines))
-        return next_line
-
-    def _read_footer(self, first_line=None):
-        """Read the rest of the meta information.
-
-        :param first_line:  The previous step iterates past what it
-                            can handle. That extra line is given here.
-        """
-        if first_line is not None:
-            if self._handle_info_line(first_line, in_footer=True) is not None:
-                return
-        for line in self.from_file:
-            if self._handle_info_line(line, in_footer=True) is not None:
-                break
-
-
-def read_changeset(from_file):
-    """Read in a changeset from a filelike object (must have "readline" support), and
-    parse it into a Changeset object.
-    """
-    cr = ChangesetReader(from_file)
-    info = cr.get_info()
-    return info
-

=== removed file 'plugins/checkperms'
--- plugins/checkperms	2005-06-27 05:14:15 +0000
+++ plugins/checkperms	1970-01-01 00:00:00 +0000
@@ -1,5 +0,0 @@
-#! /bin/sh -ex
-
-root=`bzr root`
-
-find $root \! -perm -0444 
\ No newline at end of file

=== modified file 'setup.py'
--- setup.py	2005-03-14 07:07:24 +0000
+++ setup.py	2005-09-04 03:32:17 +0000
@@ -13,5 +13,11 @@
       url='http://www.bazaar-ng.org/',
       description='Friendly distributed version control system',
       license='GNU GPL v2',
-      packages=['bzrlib'],
+      packages=['bzrlib',
+                'bzrlib.plugins',
+                'bzrlib.selftest',
+                'bzrlib.util',
+                'bzrlib.util.elementtree',
+                'bzrlib.util.effbot.org',
+                ],
       scripts=['bzr'])

=== modified file 'testsweet.py'
--- testsweet.py	2005-07-22 23:31:09 +0000
+++ testsweet.py	2005-08-30 01:35:40 +0000
@@ -30,16 +30,18 @@
 * utilities to run external commands and check their return code
   and/or output
 
-Test cases should normally subclass TestBase.  The test runner should
+Test cases should normally subclass testsweet.TestCase.  The test runner should
 call runsuite().
 
 This is meant to become independent of bzr, though that's not quite
 true yet.
 """  
 
-
-from unittest import TestResult, TestCase
-
+import unittest
+import sys
+from bzrlib.selftest import TestUtil
+
+# XXX: Don't need this anymore now we depend on python2.4
 def _need_subprocess():
     sys.stderr.write("sorry, this test suite requires the subprocess module\n"
                      "this is shipped with python2.4 and available separately for 2.3\n")
@@ -49,318 +51,130 @@
     pass
 
 
-
 class TestSkipped(Exception):
     """Indicates that a test was intentionally skipped, rather than failing."""
     # XXX: Not used yet
 
 
-class TestBase(TestCase):
-    """Base class for bzr test cases.
-
-    Just defines some useful helper functions; doesn't actually test
-    anything.
-    """
-    
-    # TODO: Special methods to invoke bzr, so that we can run it
-    # through a specified Python intepreter
-
-    OVERRIDE_PYTHON = None # to run with alternative python 'python'
-    BZRPATH = 'bzr'
-
-    _log_buf = ""
-
-
-    def setUp(self):
-        super(TestBase, self).setUp()
-        self.log("%s setup" % self.id())
-
-
-    def tearDown(self):
-        super(TestBase, self).tearDown()
-        self.log("%s teardown" % self.id())
-        self.log('')
-
-
-    def formcmd(self, cmd):
-        if isinstance(cmd, basestring):
-            cmd = cmd.split()
-
-        if cmd[0] == 'bzr':
-            cmd[0] = self.BZRPATH
-            if self.OVERRIDE_PYTHON:
-                cmd.insert(0, self.OVERRIDE_PYTHON)
-
-        self.log('$ %r' % cmd)
-
-        return cmd
-
-
-    def runcmd(self, cmd, retcode=0):
-        """Run one command and check the return code.
-
-        Returns a tuple of (stdout,stderr) strings.
-
-        If a single string is based, it is split into words.
-        For commands that are not simple space-separated words, please
-        pass a list instead."""
-        try:
-            import shutil
-            from subprocess import call
-        except ImportError, e:
-            _need_subprocess()
-            raise
-
-
-        cmd = self.formcmd(cmd)
-
-        self.log('$ ' + ' '.join(cmd))
-        actual_retcode = call(cmd, stdout=self.TEST_LOG, stderr=self.TEST_LOG)
-
-        if retcode != actual_retcode:
-            raise CommandFailed("test failed: %r returned %d, expected %d"
-                                % (cmd, actual_retcode, retcode))
-
-
-    def backtick(self, cmd, retcode=0):
-        """Run a command and return its output"""
-        try:
-            import shutil
-            from subprocess import Popen, PIPE
-        except ImportError, e:
-            _need_subprocess()
-            raise
-
-        cmd = self.formcmd(cmd)
-        child = Popen(cmd, stdout=PIPE, stderr=self.TEST_LOG)
-        outd, errd = child.communicate()
-        self.log(outd)
-        actual_retcode = child.wait()
-
-        outd = outd.replace('\r', '')
-
-        if retcode != actual_retcode:
-            raise CommandFailed("test failed: %r returned %d, expected %d"
-                                % (cmd, actual_retcode, retcode))
-
-        return outd
-
-
-
-    def build_tree(self, shape):
-        """Build a test tree according to a pattern.
-
-        shape is a sequence of file specifications.  If the final
-        character is '/', a directory is created.
-
-        This doesn't add anything to a branch.
-        """
-        # XXX: It's OK to just create them using forward slashes on windows?
-        import os
-        for name in shape:
-            assert isinstance(name, basestring)
-            if name[-1] == '/':
-                os.mkdir(name[:-1])
-            else:
-                f = file(name, 'wt')
-                print >>f, "contents of", name
-                f.close()
-
-
-    def log(self, msg):
-        """Log a message to a progress file"""
-        self._log_buf = self._log_buf + str(msg) + '\n'
-        print >>self.TEST_LOG, msg
-
-
-    def check_inventory_shape(self, inv, shape):
-        """
-        Compare an inventory to a list of expected names.
-
-        Fail if they are not precisely equal.
-        """
-        extras = []
-        shape = list(shape)             # copy
-        for path, ie in inv.entries():
-            name = path.replace('\\', '/')
-            if ie.kind == 'dir':
-                name = name + '/'
-            if name in shape:
-                shape.remove(name)
-            else:
-                extras.append(name)
-        if shape:
-            self.fail("expected paths not found in inventory: %r" % shape)
-        if extras:
-            self.fail("unexpected paths found in inventory: %r" % extras)
-
-
-    def check_file_contents(self, filename, expect):
-        self.log("check contents of file %s" % filename)
-        contents = file(filename, 'r').read()
-        if contents != expect:
-            self.log("expected: %r" % expect)
-            self.log("actually: %r" % contents)
-            self.fail("contents of %s not as expected")
-            
-
-
-class InTempDir(TestBase):
-    """Base class for tests run in a temporary branch."""
-    def setUp(self):
-        import os
-        self.test_dir = os.path.join(self.TEST_ROOT, self.__class__.__name__)
-        os.mkdir(self.test_dir)
-        os.chdir(self.test_dir)
-        
-    def tearDown(self):
-        import os
-        os.chdir(self.TEST_ROOT)
-
-
-
-
-
-class _MyResult(TestResult):
+
+class EarlyStoppingTestResultAdapter(object):
+    """An adapter for TestResult to stop at the first first failure or error"""
+
+    def __init__(self, result):
+        self._result = result
+
+    def addError(self, test, err):
+        self._result.addError(test, err)
+        self._result.stop()
+
+    def addFailure(self, test, err):
+        self._result.addFailure(test, err)
+        self._result.stop()
+
+    def __getattr__(self, name):
+        return getattr(self._result, name)
+
+    def __setattr__(self, name, value):
+        if name == '_result':
+            object.__setattr__(self, name, value)
+        return setattr(self._result, name, value)
+
+
+class _MyResult(unittest._TextTestResult):
     """
     Custom TestResult.
 
     No special behaviour for now.
     """
-    def __init__(self, out, style):
-        self.out = out
-        TestResult.__init__(self)
-        assert style in ('none', 'progress', 'verbose')
-        self.style = style
-
 
     def startTest(self, test):
+        unittest.TestResult.startTest(self, test)
         # TODO: Maybe show test.shortDescription somewhere?
         what = test.id()
         # python2.3 has the bad habit of just "runit" for doctests
         if what == 'runit':
             what = test.shortDescription()
-        
-        if self.style == 'verbose':
-            print >>self.out, '%-60.60s' % what,
-            self.out.flush()
-        elif self.style == 'progress':
-            self.out.write('~')
-            self.out.flush()
-        TestResult.startTest(self, test)
-
-
-    def stopTest(self, test):
-        # print
-        TestResult.stopTest(self, test)
-
+        if self.showAll:
+            self.stream.write('%-60.60s' % what)
+        self.stream.flush()
 
     def addError(self, test, err):
-        if self.style == 'verbose':
-            print >>self.out, 'ERROR'
-        TestResult.addError(self, test, err)
-        _show_test_failure('error', test, err, self.out)
+        super(_MyResult, self).addError(test, err)
+        self.stream.flush()
 
     def addFailure(self, test, err):
-        if self.style == 'verbose':
-            print >>self.out, 'FAILURE'
-        TestResult.addFailure(self, test, err)
-        _show_test_failure('failure', test, err, self.out)
+        super(_MyResult, self).addFailure(test, err)
+        self.stream.flush()
 
     def addSuccess(self, test):
-        if self.style == 'verbose':
-            print >>self.out, 'OK'
-        TestResult.addSuccess(self, test)
-
-
-
-def run_suite(suite, name='test', verbose=False):
-    import os
+        if self.showAll:
+            self.stream.writeln('OK')
+        elif self.dots:
+            self.stream.write('~')
+        self.stream.flush()
+        unittest.TestResult.addSuccess(self, test)
+
+    def printErrorList(self, flavour, errors):
+        for test, err in errors:
+            self.stream.writeln(self.separator1)
+            self.stream.writeln("%s: %s" % (flavour,self.getDescription(test)))
+            self.stream.writeln(self.separator2)
+            self.stream.writeln("%s" % err)
+            if hasattr(test, '_get_log'):
+                self.stream.writeln()
+                self.stream.writeln('log from this test:')
+                print >>self.stream, test._get_log()
+
+
+class TextTestRunner(unittest.TextTestRunner):
+
+    def _makeResult(self):
+        result = _MyResult(self.stream, self.descriptions, self.verbosity)
+        return EarlyStoppingTestResultAdapter(result)
+
+
+class filteringVisitor(TestUtil.TestVisitor):
+    """I accruse all the testCases I visit that pass a regexp filter on id
+    into my suite
+    """
+
+    def __init__(self, filter):
+        import re
+        TestUtil.TestVisitor.__init__(self)
+        self._suite=None
+        self.filter=re.compile(filter)
+
+    def suite(self):
+        """answer the suite we are building"""
+        if self._suite is None:
+            self._suite=TestUtil.TestSuite()
+        return self._suite
+
+    def visitCase(self, aCase):
+        if self.filter.match(aCase.id()):
+            self.suite().addTest(aCase)
+
+
+def run_suite(suite, name='test', verbose=False, pattern=".*"):
     import shutil
-    import time
-    import sys
-    
-    _setup_test_log(name)
-    _setup_test_dir(name)
-    print
-
-    # save stdout & stderr so there's no leakage from code-under-test
-    real_stdout = sys.stdout
-    real_stderr = sys.stderr
-    sys.stdout = sys.stderr = TestBase.TEST_LOG
-    try:
-        if verbose:
-            style = 'verbose'
-        else:
-            style = 'progress'
-        result = _MyResult(real_stdout, style)
-        suite.run(result)
-    finally:
-        sys.stdout = real_stdout
-        sys.stderr = real_stderr
-
-    _show_results(result)
-
+    from bzrlib.selftest import TestCaseInTempDir
+    TestCaseInTempDir._TEST_NAME = name
+    if verbose:
+        verbosity = 2
+    else:
+        verbosity = 1
+    runner = TextTestRunner(stream=sys.stdout,
+                            descriptions=0,
+                            verbosity=verbosity)
+    visitor = filteringVisitor(pattern)
+    suite.visit(visitor)
+    result = runner.run(visitor.suite())
+    # This is still a little bogus, 
+    # but only a little. Folk not using our testrunner will
+    # have to delete their temp directories themselves.
+    if result.wasSuccessful():
+        if TestCaseInTempDir.TEST_ROOT:
+            shutil.rmtree(TestCaseInTempDir.TEST_ROOT) 
+    else:
+        print "Failed tests working directories are in '%s'\n" % TestCaseInTempDir.TEST_ROOT
     return result.wasSuccessful()
-
-
-
-def _setup_test_log(name):
-    import time
-    import os
-    
-    log_filename = os.path.abspath(name + '.log')
-    TestBase.TEST_LOG = open(log_filename, 'wt', buffering=1) # line buffered
-
-    print >>TestBase.TEST_LOG, "tests run at " + time.ctime()
-    print '%-30s %s' % ('test log', log_filename)
-
-
-def _setup_test_dir(name):
-    import os
-    import shutil
-    
-    TestBase.ORIG_DIR = os.getcwdu()
-    TestBase.TEST_ROOT = os.path.abspath(name + '.tmp')
-
-    print '%-30s %s' % ('running tests in', TestBase.TEST_ROOT)
-
-    if os.path.exists(TestBase.TEST_ROOT):
-        shutil.rmtree(TestBase.TEST_ROOT)
-    os.mkdir(TestBase.TEST_ROOT)
-    os.chdir(TestBase.TEST_ROOT)
-
-    # make a fake bzr directory there to prevent any tests propagating
-    # up onto the source directory's real branch
-    os.mkdir(os.path.join(TestBase.TEST_ROOT, '.bzr'))
-
-    
-
-def _show_results(result):
-     print
-     print '%4d tests run' % result.testsRun
-     print '%4d errors' % len(result.errors)
-     print '%4d failures' % len(result.failures)
-
-
-
-def _show_test_failure(kind, case, exc_info, out):
-    from traceback import print_exception
-    
-    print >>out, '-' * 60
-    print >>out, case
-    
-    desc = case.shortDescription()
-    if desc:
-        print >>out, '   (%s)' % desc
-         
-    print_exception(exc_info[0], exc_info[1], exc_info[2], None, out)
-        
-    if isinstance(case, TestBase):
-        print >>out
-        print >>out, 'log from this test:'
-        print >>out, case._log_buf
-         
-    print >>out, '-' * 60
-    
-

=== added file 'tools/history2revfiles.py'
--- tools/history2revfiles.py	1970-01-01 00:00:00 +0000
+++ tools/history2revfiles.py	2005-08-25 05:58:05 +0000
@@ -0,0 +1,99 @@
+#! /usr/bin/python
+
+# Copyright (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Experiment in converting existing bzr branches to weaves."""
+
+
+import bzrlib.branch
+from bzrlib.revfile import Revfile
+from bzrlib.progress import ProgressBar
+import tempfile
+import hotshot, hotshot.stats
+import sys
+
+def convert():
+    pb = ProgressBar()
+
+    inv_rf = Revfile('revfiles/inventory', 'w')
+    last_text_sha = {}
+    text_rfs = {}
+
+    b = bzrlib.branch.find_branch('.')
+
+    revno = 1
+    rev_history = b.revision_history()
+    last_idx = None
+    for rev_id in rev_history:
+        pb.update('converting inventory', revno, len(rev_history))
+        inv_xml = b.get_inventory_xml(rev_id).read()
+        new_idx = inv_rf.add(inv_xml, last_idx, compress=False)
+
+        tree = b.revision_tree(rev_id)
+        inv = tree.inventory
+
+        # for each file in the inventory, put it into its own revfile
+        for file_id in inv:
+            ie = inv[file_id]
+            if ie.kind != 'file':
+                continue
+            if last_text_sha.get(file_id) == ie.text_sha1:
+                # same as last time
+                continue
+            last_text_sha[file_id] = ie.text_sha1
+
+            # new text (though possibly already stored); need to store it
+            text = tree.get_file(file_id).read()
+            
+            if file_id not in text_rfs:
+                text_rfs[file_id] = Revfile('revfiles/' + file_id, 'w')
+            rf = text_rfs[file_id]
+
+            last = len(rf)
+            if last == 0:
+                last = None
+            else:
+                last -= 1
+            rf.add(text, last, compress=True)
+        
+        last_idx = new_idx
+        revno += 1
+
+    pb.clear()
+
+
+def profile_convert(): 
+    prof_f = tempfile.NamedTemporaryFile()
+
+    prof = hotshot.Profile(prof_f.name)
+
+    prof.runcall(convert) 
+    prof.close()
+
+    stats = hotshot.stats.load(prof_f.name)
+    #stats.strip_dirs()
+    stats.sort_stats('time')
+    ## XXX: Might like to write to stderr or the trace file instead but
+    ## print_stats seems hardcoded to stdout
+    stats.print_stats(20)
+            
+
+if '-p' in sys.argv[1:]:
+    profile_convert()
+else:
+    convert()
+    

=== added file 'tools/history2weaves.py'
--- tools/history2weaves.py	1970-01-01 00:00:00 +0000
+++ tools/history2weaves.py	2005-08-29 04:53:09 +0000
@@ -0,0 +1,152 @@
+#! /usr/bin/python
+
+# Copyright (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Experiment in converting existing bzr branches to weaves."""
+
+try:
+    import psyco
+    psyco.full()
+except ImportError:
+    pass
+
+
+import logging
+
+import bzrlib.branch
+from bzrlib.revfile import Revfile
+from bzrlib.weave import Weave
+from bzrlib.weavefile import read_weave, write_weave
+from bzrlib.progress import ProgressBar
+from bzrlib.atomicfile import AtomicFile
+import bzrlib.trace
+import tempfile
+import hotshot, hotshot.stats
+import sys
+
+def convert():
+    bzrlib.trace.enable_default_logging()
+
+    pb = ProgressBar()
+
+    inv_weave = Weave()
+
+    last_text_sha = {}
+
+    # holds in-memory weaves for all files
+    text_weaves = {}
+
+    b = bzrlib.branch.find_branch('.')
+
+    revno = 1
+    rev_history = b.revision_history()
+    last_idx = None
+    inv_parents = []
+    text_count = 0
+    
+    for rev_id in rev_history:
+        pb.update('converting revision', revno, len(rev_history))
+        
+        inv_xml = b.get_inventory_xml(rev_id).readlines()
+
+        new_idx = inv_weave.add(rev_id, inv_parents, inv_xml)
+        inv_parents = [new_idx]
+
+        tree = b.revision_tree(rev_id)
+        inv = tree.inventory
+
+        # for each file in the inventory, put it into its own revfile
+        for file_id in inv:
+            ie = inv[file_id]
+            if ie.kind != 'file':
+                continue
+            if last_text_sha.get(file_id) == ie.text_sha1:
+                # same as last time
+                continue
+            last_text_sha[file_id] = ie.text_sha1
+
+            # new text (though possibly already stored); need to store it
+            text_lines = tree.get_file(file_id).readlines()
+
+            # if the file's created for the first time in this
+            # revision then make a new weave; else find the old one
+            if file_id not in text_weaves:
+                text_weaves[file_id] = Weave()
+                
+            w = text_weaves[file_id]
+
+            # base the new text version off whatever was last
+            # (actually it'd be better to track this, to allow for
+            # files that are deleted and then reappear)
+            last = len(w)
+            if last == 0:
+                parents = []
+            else:
+                parents = [last-1]
+
+            w.add(rev_id, parents, text_lines)
+            text_count += 1
+
+        revno += 1
+
+    pb.clear()
+    print '%6d revisions and inventories' % revno
+    print '%6d texts' % text_count
+
+    i = 0
+    # TODO: commit them all atomically at the end, not one by one
+    write_atomic_weave(inv_weave, 'weaves/inventory.weave')
+    for file_id, file_weave in text_weaves.items():
+        pb.update('writing weave', i, len(text_weaves))
+        write_atomic_weave(file_weave, 'weaves/%s.weave' % file_id)
+        i += 1
+
+    pb.clear()
+
+
+def write_atomic_weave(weave, filename):
+    inv_wf = AtomicFile(filename)
+    try:
+        write_weave(weave, inv_wf)
+        inv_wf.commit()
+    finally:
+        inv_wf.close()
+
+    
+
+
+def profile_convert(): 
+    prof_f = tempfile.NamedTemporaryFile()
+
+    prof = hotshot.Profile(prof_f.name)
+
+    prof.runcall(convert) 
+    prof.close()
+
+    stats = hotshot.stats.load(prof_f.name)
+    #stats.strip_dirs()
+    stats.sort_stats('time')
+    ## XXX: Might like to write to stderr or the trace file instead but
+    ## print_stats seems hardcoded to stdout
+    stats.print_stats(20)
+            
+
+if '-p' in sys.argv[1:]:
+    profile_convert()
+else:
+    convert()
+    

=== added file 'tools/http_client.py'
--- tools/http_client.py	1970-01-01 00:00:00 +0000
+++ tools/http_client.py	2005-08-25 12:46:42 +0000
@@ -0,0 +1,501 @@
+#! /usr/bin/python
+
+# $Id: http_client.py 271 2004-10-09 10:50:59Z fredrik $
+# a simple asynchronous http client (based on SimpleAsyncHTTP.py from
+# "Python Standard Library" by Fredrik Lundh, O'Reilly 2001)
+#
+# HTTP/1.1 and GZIP support added in January 2003 by Fredrik Lundh.
+#
+# changes:
+# 2004-08-26 fl   unified http callback
+# 2004-10-09 fl   factored out gzip_consumer support
+# 2005-07-08 mbp  experimental support for keepalive connections
+#
+# Copyright (c) 2001-2004 by Fredrik Lundh.  All rights reserved.
+#
+
+
+
+"""async/pipelined http client
+
+Use
+===
+
+Users of this library pass in URLs they want to see, and consumer
+objects that will receive the results at some point in the future.
+Any number of requests may be queued up, and more may be added while
+the download is in progress.
+
+Requests can be both superscalar and superpipelined.  That is to say,
+for each server there can be multiple sockets open, and each socket
+may have more than one request in flight.
+
+Design
+======
+
+There is a single DownloadManager, and a connection object for each
+open socket.
+
+Request/consumer pairs are maintained in queues.  Each connection has
+a list of transmitted requests whose response has not yet been
+received.  There is also a per-server list of requests that have not
+yet been submitted.
+
+When a connection is ready to transmit a new request, it takes one
+from the unsubmitted list, sends the request, and adds the request to
+its unfulfilled list.  This should happen when the connection has
+space for more transmissions or when a new request is added by the
+user.  If the connection terminates with unfulfilled requests they are
+put back onto the unsubmitted list, to be retried elsewhere.
+
+Because responses come back precisely in order, the connection always
+knows what it should expect next: the response for the next
+unfulfilled request.
+"""
+
+# Note that (as of ubuntu python 2.4.1) every socket.connect() call
+# with a hostname does a remote DNS resolution, which is pretty sucky.
+# Shouldn't there be a cache in glibc?  We should probably cache the
+# address in, say, the DownloadManager.
+
+# TODO: A default consumer operation that writes the received data
+# into a file; by default the file is named the same as the last
+# component of the URL.
+
+# TODO: A utility function that is given a list of URLs, and downloads
+# them all parallel/pipelined.  If any fail, it raises an exception
+# (and discards the rest), or perhaps can be told to continue anyhow.
+# The content is written into temporary files.  It returns a list of
+# readable file objects.
+
+# TODO: If we try pipelined or keepalive and the connection drop out
+# then retry the request on a new connection; eventually we should perhaps
+# learn that a given host or network just won't allow keepalive.
+
+
+import asyncore
+import socket, string, time, sys
+import StringIO
+import mimetools, urlparse, urllib
+import logging
+
+logging.basicConfig(level=logging.DEBUG,
+                    format='%(asctime)s %(levelname)s %(message)s',
+                    filename='/tmp/http_client.log',
+                    filemode='w')
+
+logger = logging.getLogger('bzr.http_client')
+debug = logger.debug
+info = logger.info
+error = logger.error
+
+
+##
+# Close connection.   Request handlers can raise this exception to
+# indicate that the connection should be closed.
+
+class CloseConnection(Exception):
+    pass
+
+##
+# Redirect connection.  Request handlers can raise this exception to
+# indicate that the a new request should be issued.
+
+class Redirect(CloseConnection):
+    def __init__(self, location):
+        self.location = location
+
+
+class DownloadManager(object):
+    """Handles pipelined/overlapped downloads.
+
+    Pass in a series of URLs with handlers to receive the response.
+    This object will spread the requests over however many sockets
+    seem useful.
+
+    queued_requests
+        Requests not assigned to any channel
+
+    running_requests
+        Currently assigned to a channel
+    """
+    def __init__(self):
+        self.queued_requests = []
+        # self.channel = HttpChannel('localhost', 8000, self)
+        self.channels = []
+        self.try_pipelined = False
+        self.try_keepalive = False
+        self.max_channels = 5
+
+
+    def enqueue(self, url, consumer):
+        self.queued_requests.append((url, consumer))
+        self._wake_up_channel()
+
+
+    def _channel_closed(self, channel):
+        """Called by the channel when its socket closes.
+        """
+        self.channels.remove(channel)
+        if self.queued_requests:
+            # might recreate one
+            self._wake_up_channel()
+
+
+    def _make_channel(self):
+        # proxy2 203.17.154.69
+        # return HttpChannel('82.211.81.161', 80, self)         # bazaar-ng.org 
+        # return HttpChannel('203.17.154.69', 8080, self)
+        return HttpChannel('127.0.0.1', 8000, self)  # forwarded
+            
+
+    def _wake_up_channel(self):
+        """Try to wake up one channel to send the newly-added request.
+
+        There may be more than one request pending, and this may cause
+        more than one channel to take requests.  That's OK; some of
+        them may be frustrated.
+        """
+        from random import shuffle, choice
+        
+        # first, wake up any idle channels
+        done = False
+        for ch in self.channels:
+            if not ch.sent_requests:
+                ch.take_one()
+                done = True
+        if done:
+            debug("woke existing idle channel(s)")
+            return
+
+        if len(self.channels) < self.max_channels:
+            newch = self._make_channel()
+            self.channels.append(newch)
+            newch.take_one()
+            debug("created new channel")
+            return
+
+        if self.try_pipelined:
+            # ask existing channels to take it
+            debug("woke busy channel")
+            choice(self.channels).take_one()
+
+
+        # debug("request postponed until a channel's idle")
+        
+
+
+
+    def run(self):
+        """Run until all outstanding requests have been served."""
+        #while self.running_requests or self.queued_requests \
+        #          or not self.channel.is_idle():
+        #    asyncore.loop(count=1)
+        asyncore.loop()
+
+
+
+class Response(object):
+    """Holds in-flight response."""
+
+
+
+def _parse_response_http10(header):
+    from cStringIO import StringIO
+
+    fp = StringIO(header)
+    r = Response()
+
+    r.status = fp.readline().split(" ", 2)
+    r.headers = mimetools.Message(fp)
+
+    # we can only(?) expect to do keepalive if we got either a 
+    # content-length or chunked encoding; otherwise there's no way to know
+    # when the content ends apart from through the connection close
+    r.content_type = r.headers.get("content-type")
+    try:
+        r.content_length = int(r.headers.get("content-length"))
+    except (ValueError, TypeError):
+        r.content_length = None
+    debug("seen content length of %r" % r.content_length)
+
+    r.transfer_encoding = r.headers.get("transfer-encoding")
+    r.content_encoding = r.headers.get("content-encoding")
+    r.connection_reply = r.headers.get("connection")
+
+    # TODO: pass status code to consumer?
+
+    if r.transfer_encoding:
+        raise NotImplementedError()
+
+    if r.transfer_encoding:
+        raise NotImplementedError()
+
+    if int(r.status[1]) != 200:
+        debug("can't handle response status %r" % r.status)
+        raise NotImplementedError()
+
+    if r.content_length == None:
+        raise NotImplementedError()
+
+    if r.content_length == 0:
+        raise NotImplementedError()
+
+    r.content_remaining = r.content_length                
+
+    return r
+
+
+    
+    
+        
+
+
+class HttpChannel(asyncore.dispatcher_with_send):
+    """One http socket, pipelining if possible."""
+    # asynchronous http client
+
+    user_agent = "http_client.py 1.3ka (based on effbot)"
+
+    proxies = urllib.getproxies()
+
+    def __init__(self, ip_host, ip_port, manager):
+        asyncore.dispatcher_with_send.__init__(self)
+        self.manager = manager
+
+        # if a response header has been seen, this holds it
+        self.response = None
+        
+        self.data = ""
+
+        self.chunk_size = None
+
+        self.timestamp = time.time()
+
+        self.create_socket(socket.AF_INET, socket.SOCK_STREAM)
+        debug('connecting...')
+        self.connect((ip_host, ip_port))
+
+        # sent_requests holds (url, consumer) 
+        self.sent_requests = []
+
+        self._outbuf = ''
+
+
+    def __repr__(self):
+        return 'HttpChannel(local_port=%r)' % (self.getsockname(),)
+
+
+    def is_idle(self):
+        return (not self.sent_requests)
+
+
+    def handle_connect(self):
+        debug("connected")
+        self.take_one()
+
+
+    def take_one(self):
+        """Accept one request from the manager if possible."""
+        if self.manager.try_pipelined:
+            if len(self.sent_requests) > 4:
+                return
+        else:
+            if len(self.sent_requests) > 0:
+                return 
+        
+        try:
+            url, consumer = self.manager.queued_requests.pop(0)
+            debug('request accepted by channel')
+        except IndexError:
+            return
+        
+        # TODO: If there are too many already in flight, don't take one.
+        # TODO: If the socket's not writable (tx buffer full), don't take.
+        self._push_request_http10(url, consumer)
+
+
+
+    def _push_request_http10(self, url, consumer):
+        """Send a request, and add it to the outstanding queue."""
+        # TODO: check the url requested is appropriate for this connection
+
+        # TODO: If there are too many requests outstanding or (less likely) the 
+        # connection fails, queue it for later use.
+
+        # TODO: Keep track of requests that have been sent but not yet fulfilled,
+        # because we might need to retransmit them if the connection fails. (Or
+        # should the caller do that?)
+
+        request = self._form_request_http10(url)
+        debug('send request for %s from %r' % (url, self))
+
+        # dispatcher_with_send handles buffering the data until it can
+        # be written, and hooks handle_write.
+
+        self.send(request)
+
+        self.sent_requests.append((url, consumer))
+
+
+    def _form_request_http10(self, url):
+        # TODO: get right vhost name
+        request = [
+            "GET %s HTTP/1.0" % (url),
+            "Host: www.bazaar-ng.org",
+            ]
+
+        if self.manager.try_keepalive or self.manager.try_pipelined:
+            request.extend([
+                "Keep-Alive: 60", 
+                "Connection: keep-alive",
+                ])
+
+        # make sure to include a user agent
+        for header in request:
+            if string.lower(header).startswith("user-agent:"):
+                break
+        else:
+            request.append("User-Agent: %s" % self.user_agent)
+
+        return string.join(request, "\r\n") + "\r\n\r\n"
+
+
+    def handle_read(self):
+        # handle incoming data
+        data = self.recv(2048)
+
+        self.data = self.data + data
+
+        if len(data):
+            debug('got %d bytes from socket' % len(data))
+        else:
+            debug('server closed connection')
+
+        while self.data:
+            consumer = self.sent_requests[0][1]
+            if not self.response:
+                # do not have a full response header yet
+
+                # check if we've seen a full header
+                debug('getting header for %s' % self.sent_requests[0][0])
+
+                header = self.data.split("\r\n\r\n", 1)
+                if len(header) <= 1:
+                    return
+                header, self.data = header
+
+                self.response = _parse_response_http10(header)
+                self.content_remaining = self.response.content_length
+
+            if not self.data:
+                return
+
+            # we now know how many (more) content bytes we have, and how much
+            # is in the data buffer. there are two main possibilities:
+            # too much data, and some must be left behind containing the next
+            # response headers, or too little, or possibly just right
+
+            want = self.content_remaining
+            if want > 0:
+                got_data = self.data[:want]
+                self.data = self.data[want:]
+                
+                assert got_data
+
+                self.content_remaining -= len(got_data)
+
+                debug('pass back %d bytes of %s, %d remain'
+                      % (len(got_data),
+                         self.sent_requests[0][0],
+                         self.content_remaining))
+                consumer.feed(data)
+
+            if self.content_remaining == 0:
+                del self.sent_requests[0]
+
+                debug('content complete')
+                consumer.content_complete()
+                
+                # reset lots of things and try to get the next response header
+                if self.response.connection_reply == 'close':
+                    debug('server requested close')
+                    self.manager._channel_closed(self)
+                    self.close()
+                elif not self.manager.try_keepalive:
+                    debug('no keepalive for this socket')
+                    self.manager._channel_closed(self)
+                    self.close()
+                else:
+                    debug("ready for next header...")
+                    self.take_one()
+                self.response = None
+
+
+
+    def handle_close(self):
+        debug('async told us of close on %r' % self)
+        # if there are outstanding requests should probably reopen and 
+        # retransmit, but if we're not making any progress then give up
+        self.manager._channel_closed(self)
+        self.close()
+
+
+class DummyConsumer:
+    def __init__(self, url, pb):
+        self.url = url
+        self.outf = None
+        self._pb = pb
+
+    def feed(self, data):
+        # print "feed", repr(data)
+        # print "feed", repr(data[:20]), repr(data[-20:]), len(data)
+        if not self.outf:
+            base = self.url[self.url.rindex('/')+1:]
+            self.outf = file('/tmp/download/' + base, 'wb')
+        self.outf.write(data)
+
+    def error(self, err_info):
+        import traceback
+        error('error reported to consumer')
+        traceback.print_exception(err_info[0], err_info[1], err_info[2])
+        sys.exit(1)
+
+    def content_complete(self):
+        info('content complete from %s' % self.url)
+        self.outf.close()
+        self.outf = None
+        # using last_cnt is cheating
+        self._pb.update('downloading inventory',
+                        self._pb.last_cnt+1,
+                        self._pb.last_total)
+
+
+
+if __name__ == "__main__":
+    logging.basicConfig(level=logging.DEBUG)
+
+    mgr = DownloadManager()
+
+    from bzrlib.branch import Branch
+    from bzrlib.progress import ProgressBar
+
+    pb = ProgressBar()
+    revs = Branch('/home/mbp/work/bzr').revision_history()
+    pb.update('downloading inventories', 0, len(revs))
+
+    for rev in revs:
+        url = 'http://www.bazaar-ng.org/bzr/bzr.dev/.bzr/inventory-store/' \
+              + rev + '.gz'
+        mgr.enqueue(url, DummyConsumer(url, pb))
+
+    mgr.run()
+    
+
+
+    
+#     for url in ['http://www.bazaar-ng.org/',
+#                 'http://www.bazaar-ng.org/tutorial.html',
+#                 'http://www.bazaar-ng.org/download.html',
+#                 'http://www.bazaar-ng.org/bzr/bzr.dev/.bzr/revision-store/mbp@hope-20050415013653-3b3c9c3d33fae0a6.gz',
+#                 ]:

=== modified file 'tools/testweave.py'
--- tools/testweave.py	2005-07-18 14:28:13 +0000
+++ tools/testweave.py	2005-08-18 07:51:58 +0000
@@ -17,6 +17,8 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
+# TODO: tests regarding version names
+
 
 
 """test suite for weave algorithm"""
@@ -60,6 +62,14 @@
             tf.seek(0)
             self.log('serialized weave:')
             self.log(tf.read())
+
+            self.log('')
+            self.log('parents: %s' % (k._parents == k2._parents))
+            self.log('         %r' % k._parents)
+            self.log('         %r' % k2._parents)
+            self.log('')
+
+            
             self.fail('read/write check failed')
         
         
@@ -74,7 +84,7 @@
     """Store and retrieve a simple text."""
     def runTest(self):
         k = Weave()
-        idx = k.add([], TEXT_0)
+        idx = k.add('text0', [], TEXT_0)
         self.assertEqual(k.get(idx), TEXT_0)
         self.assertEqual(idx, 0)
 
@@ -83,7 +93,7 @@
 class AnnotateOne(TestBase):
     def runTest(self):
         k = Weave()
-        k.add([], TEXT_0)
+        k.add('text0', [], TEXT_0)
         self.assertEqual(k.annotate(0),
                          [(0, TEXT_0[0])])
 
@@ -92,10 +102,10 @@
     def runTest(self):
         k = Weave()
 
-        idx = k.add([], TEXT_0)
+        idx = k.add('text0', [], TEXT_0)
         self.assertEqual(idx, 0)
 
-        idx = k.add([], TEXT_1)
+        idx = k.add('text1', [], TEXT_1)
         self.assertEqual(idx, 1)
 
         self.assertEqual(k.get(0), TEXT_0)
@@ -112,6 +122,7 @@
 
         self.assertRaises(IndexError,
                           k.add,
+                          'text0',
                           [69],
                           ['new text!'])
 
@@ -124,8 +135,8 @@
     def runTest(self):
         k = Weave()
 
-        k.add([], ['line 1'])
-        k.add([0], ['line 1', 'line 2'])
+        k.add('text0', [], ['line 1'])
+        k.add('text1', [0], ['line 1', 'line 2'])
 
         self.assertEqual(k.annotate(0),
                          [(0, 'line 1')])
@@ -138,14 +149,15 @@
                          [(0, 'line 1'),
                           (1, 'line 2')])
 
-        k.add([0], ['line 1', 'diverged line'])
+        k.add('text2', [0], ['line 1', 'diverged line'])
 
         self.assertEqual(k.annotate(2),
                          [(0, 'line 1'),
                           (2, 'diverged line')])
 
         text3 = ['line 1', 'middle line', 'line 2']
-        k.add([0, 1],
+        k.add('text3',
+              [0, 1],
               text3)
 
         # self.log("changes to text3: " + pformat(list(k._delta(set([0, 1]), text3))))
@@ -158,7 +170,8 @@
                           (1, 'line 2')])
 
         # now multiple insertions at different places
-        k.add([0, 1, 3],
+        k.add('text4',
+              [0, 1, 3],
               ['line 1', 'aaa', 'middle line', 'bbb', 'line 2', 'ccc'])
 
         self.assertEqual(k.annotate(4), 
@@ -180,7 +193,7 @@
 
         base_text = ['one', 'two', 'three', 'four']
 
-        k.add([], base_text)
+        k.add('text0', [], base_text)
         
         texts = [['one', 'two', 'three'],
                  ['two', 'three', 'four'],
@@ -188,8 +201,11 @@
                  ['one', 'two', 'three', 'four'],
                  ]
 
+        i = 1
         for t in texts:
-            ver = k.add([0], t)
+            ver = k.add('text%d' % i,
+                        [0], t)
+            i += 1
 
         self.log('final weave:')
         self.log('k._weave=' + pformat(k._weave))
@@ -406,14 +422,14 @@
     def runTest(self):
         k = Weave()
 
-        k.add([], ["line the first",
+        k.add('text0', [], ["line the first",
                    "line 2",
                    "line 3",
                    "fine"])
 
         self.assertEqual(len(k.get(0)), 4)
 
-        k.add([0], ["line the first",
+        k.add('text1', [0], ["line the first",
                    "fine"])
 
         self.assertEqual(k.get(1),
@@ -501,8 +517,8 @@
         text0 = ['cheddar', 'stilton', 'gruyere']
         text1 = ['cheddar', 'blue vein', 'neufchatel', 'chevre']
         
-        k.add([], text0)
-        k.add([0], text1)
+        k.add('text0', [], text0)
+        k.add('text1', [0], text1)
 
         self.log('k._weave=' + pformat(k._weave))
 
@@ -522,10 +538,10 @@
                  ['header', '', 'line from 1', 'fixup line', 'line from 2'],
                  ]
 
-        k.add([], texts[0])
-        k.add([0], texts[1])
-        k.add([0], texts[2])
-        k.add([0, 1, 2], texts[3])
+        k.add('text0', [], texts[0])
+        k.add('text1', [0], texts[1])
+        k.add('text2', [0], texts[2])
+        k.add('merge', [0, 1, 2], texts[3])
 
         for i, t in enumerate(texts):
             self.assertEqual(k.get(i), t)
@@ -593,9 +609,9 @@
                  ['header', 'aaa', 'bbb', 'line from 2', 'more from 2'],
                  ]
 
-        k.add([], texts[0])
-        k.add([0], texts[1])
-        k.add([0], texts[2])
+        k.add('text0', [], texts[0])
+        k.add('text1', [0], texts[1])
+        k.add('text2', [0], texts[2])
 
         self.log('k._weave=' + pformat(k._weave))
 
@@ -641,9 +657,12 @@
 
         k = Weave()
         parents = set()
+        i = 0
         for t in texts:
-            ver = k.add(list(parents), t)
+            ver = k.add('text%d' % i,
+                        list(parents), t)
             parents.add(ver)
+            i += 1
 
         self.log("k._weave=" + pformat(k._weave))
 
@@ -663,9 +682,9 @@
             return x + '\n'
         
         w = Weave()
-        w.add([], map(addcrlf, base))
-        w.add([0], map(addcrlf, a))
-        w.add([0], map(addcrlf, b))
+        w.add('text0', [], map(addcrlf, base))
+        w.add('text1', [0], map(addcrlf, a))
+        w.add('text2', [0], map(addcrlf, b))
 
         self.log('weave is:')
         tmpf = StringIO()

=== modified file 'tools/weavebench.py'
--- tools/weavebench.py	2005-07-18 11:23:40 +0000
+++ tools/weavebench.py	2005-08-25 07:48:27 +0000
@@ -51,7 +51,7 @@
             if lines:
                 del lines[randrange(0, len(lines))]
 
-        rev_id = wf.add(parents, lines)
+        rev_id = wf.add("%s" % i, parents, lines)
         parents = [rev_id]
 
     write_weave(wf, file(WEAVE_NAME, 'wb'))

=== added file 'tutorial.txt'
--- tutorial.txt	1970-01-01 00:00:00 +0000
+++ tutorial.txt	2005-09-01 12:34:34 +0000
@@ -0,0 +1,312 @@
+==================
+Bazaar-NG Tutorial
+==================
+
+current for bzr 0.0.6pre, July 2005
+
+
+*NOTE* For a more current and user-editable version of this 
+document, see the wiki at http://bazaar.canonical.com/IntroductionToBzr
+
+
+Introduction
+============
+
+Bazaar-NG is a version control tool.  It manages trees of files and subdirectories.  In particular, it records *revisions* of trees, representing their state at a particular point in time, and information about those revisions and their relationships.  Recording and retrieving tree revisions is useful in several ways if you are writing software or documents or doing similar creative work.
+
+ * Keeping previous revisions lets you go back if you make a mistake or want to check your work.  It acts as a high-level unlimited undo.
+
+ * By recording comments on every revision, you produce an annotated history of the project, describing what, who, why, and when.
+
+ * Using a version control tool can be an aid to thinking about a project: getting to a stable state at regular intervals and then writing a description of what you did is an easy way to stay organized and on track.
+
+Bazaar-NG remembers the *ancestry* of a revision: the previous revisions that it is based upon.  A single revision may have more than one direct descendant, each with different changes, representing a divergence in the evolution of the tree.  
+By branching, Bazaar-NG allows multiple people to cooperate on the evolution of a project, without all needing to work in strict
+lock-step.  Branching can be useful even for a single developer.
+
+Bazaar-NG installs a single new command,
+*bzr*.  Everything else is a subcommand of this.  You can get
+some help with ``bzr help``.  There will be more in the future.
+
+
+
+Introducing yourself to bzr
+===========================
+
+One function of a version control system is to keep track of who
+changed what.  In a distributed system that requires an identifier for
+each author that is globally unique.  Most people already have one of
+these: an email address.
+
+[after 0.0.4] To tell bzr which email address to use, put it in the file
+``$HOME/.bzr.conf/email``, or the environment variable ``$BZREMAIL``.
+If neither of these are set, bzr will use the ``$EMAIL``
+variable, or use your username and hostname.
+
+To check this has taken effect, or if you forget your own name, use
+the ``whoami`` ("who am i?") command::
+
+  % bzr whoami
+
+Some people want to avoid sharing their email address so as not to
+get spam.  bzr will never
+disclose your email address unless you tell it to by publishing an
+archive or transmitting a changeset.  It's recommended that you do use
+a real address, so that people can contact you about your work, but
+it's not required.  You can use an address which is obfuscated, which
+bounces, or which goes through an anti-spam service such as spamgourmet.com.
+
+
+
+Creating a branch
+==================
+
+
+History is by default stored in the .bzr directory of the branch.
+There will be a facility to store it in a separate repository, which
+may be remote.  We create a new branch by running *bzr init* in
+an existing directory::
+
+    % mkdir tutorial
+    % cd tutorial
+    % ls -a
+    ls -a
+    ./  ../
+    % pwd
+    /home/mbp/work/bzr.test/tutorial
+    %
+    % bzr init
+    % ls -aF
+    ./  ../  .bzr/
+    %
+
+As for CVS, there are three classes of file: unknown, ignored, and
+versioned.  The *add* command makes a file versioned: that is,
+changes to it will be recorded by the system::
+
+    % echo 'hello world' > hello.txt
+    % bzr status
+    ?       hello.txt
+    % bzr unknowns
+    hello.txt
+    % bzr add -v hello.txt
+    A       hello.txt
+    % bzr unknowns
+
+
+If you add the wrong file, simply use ``bzr remove`` to make
+it unversioned again.  This does not delete the working copy.
+
+
+Reviewing changes
+=================
+
+Once you have completed some work, you will want to *commit*
+it to the version history.  It is good to commit fairly often:
+whenever you get a new feature working, fix a bug, or improve some
+code or documentation.  It's also a good practice to make sure that
+the code compiles and passes its test suite before committing, to make
+sure that every revision is a known-good state.  You can also review
+your changes, to make sure you're committing what you intend to, and
+as a chance to rethink your work before you permanently record it. 
+
+Two bzr commands are particularly useful here: *status* and
+*diff*.  The *status* command
+shows a listing with one line per file, indicating whether it has been
+Added, Deleted, Modified, or Renamed in the current revision.  Unknown
+files are shown as '?'.  With the ``--all`` option, the status
+command also shows unmodified versioned files as '.', and ignored
+files as 'I'::
+
+    % bzr status
+    A       hello.txt
+
+The *diff* command shows the full text of changes to all
+files as a standard unified diff.  This can be piped through many
+programs such as ``patch``, ``diffstat``,
+``filterdiff`` and ``colordiff``::
+
+    % bzr diff
+    *** added file 'hello.txt'
+    --- /dev/null 
+    +++ hello.txt 
+    @@ -1,0 +1,1 @@
+    +hello world
+
+With the ``-r`` option, the tree is compared to an earlier
+revision.
+
+[TODO: options to run external diff; to get context diff or other
+formats; to diff only selected files; to compare two historical
+revisions.]
+
+
+
+Committing changes
+==================
+
+When the working tree state is satisfactory, it can be
+*committed* to the branch, creating a new revision holding a
+snapshot of that state.  
+
+The ``commit`` command takes a message describing the changes
+in the revision.  It also records your userid, the current time and
+timezone, and the inventory and contents of the tree.  The commit
+message is specified by the ``-m`` or ``--message`` option.
+You can enter a multi-line commit message; in most shells you can
+enter this just by leaving the quotes open at the end of the line. ::
+
+    % bzr commit -m "added my first file"
+
+[TODO: commit message interactively, through an editor or from a
+file.]
+
+[TODO: commit only selected files, including renamed/added/deleted
+files.]
+
+
+
+Removing uncommitted changes
+============================
+
+If you've made some changes and don't want to keep them, use the
+``revert`` command to go back to the previous head version.  It's a
+good idea to use ``bzr diff`` first to see what will be removed. 
+By default the revert command reverts the whole tree; if file or
+directory names are given then only those ones will be affected.
+revert also clears the list of pending merges revisions.
+
+
+
+
+
+Ignoring files
+==============
+
+Many source trees contain some files that do not need to be
+versioned, such as editor backups, object or bytecode files, and built
+programs.  You can simply not add them, but then they'll always crop
+up as unknown files.  You can also tell bzr to ignore these files by
+adding them to a file called ``.bzrignore`` at the top of the
+tree.
+
+This file contains a list of file wildcards (or "globs"), one
+per line.  Typical contents are like this::
+
+    *.o
+    *~
+    *.tmp
+    *.py[co]
+
+If a glob contains a slash, it is matched against the whole path
+from the top of the tree; otherwise it is matched against only the
+filename.  So the previous example ignores ``*.o`` in all
+subdirectories, but this example ignores only config.h at the top
+level and HTML files in ``doc/``::
+
+    ./config.h
+    doc/*.html
+
+To get a list of which files are ignored and what pattern they matched, use ``bzr ignored``::
+
+    % bzr ignored
+    config.h                                           ./config.h
+    configure.in~                                      *~
+
+It is OK to have an ignore pattern match a versioned file, or to
+add an ignored file.  Ignore patterns have no effect on versioned
+files; they only determine whether unversioned files are reported as
+unknown or ignored.
+
+The ``.bzrignore`` file should normally be versioned, so that new
+copies of the branch see the same patterns::
+
+    % bzr add .bzrignore
+    % bzr commit -m "Add ignore patterns"
+
+
+Examining history
+=================
+
+bzr log
+-------
+
+The ``log`` command shows a list of previous revisions.
+
+
+Branch statistics
+=================
+
+The ``bzr info`` command shows some summary information about
+the working tree and the branch history.  
+
+
+Versioning directories
+======================
+
+bzr versions files and directories in a way that can keep track of
+renames and intelligently merge them::
+
+    % mkdir src
+    % echo 'int main() {}' > src/simple.c
+    % bzr add src
+    % bzr status
+    A       src/
+    ?       src/simple.c
+    % bzr add src/simple.c
+    % bzr status
+    A       src/
+    A       src/simple.c
+
+
+Deleting and removing files
+===========================
+
+You can delete files or directories by just deleting them from the
+working directory.  This is a bit different to CVS, which requires
+that you also do *cvs remove*.
+
+*bzr remove* makes the file un-versioned, but does not
+delete the working copy.  This is useful when you add the wrong file,
+or decide that a file should actually not be versioned. ::
+
+    % rm -r src
+    % bzr remove -v hello.txt
+    ?       hello.txt
+    % bzr status
+    ?       hello.txt
+    D       src/
+    D       src/simple.c
+
+
+Branching
+=========
+
+Often rather than starting your own project, you will want to
+submit a change to an existing project.  You can get a copy of an
+existing branch by copying its directory, expanding a tarball, or by a
+remote copy using something like rsync.  You can also use bzr to fetch
+a copy.  Because this new copy is potentially a new branch, the
+command is called *branch*::
+
+    % bzr branch http://bazaar-ng.org/bzr/bzr.dev 
+    % cd bzr.dev
+
+This copies down the complete history of this branch, so we can
+do all operations on it locally: log, annotate, making and merging
+branches.  There will be an option to get only part of the history if
+you wish.
+
+
+
+Following upstream changes
+==========================
+
+You can stay up-to-date with the parent branch by *pulling*
+in their changes::
+
+    % bzr pull
+
+This only works if your local branch includes only changes from the
+parent branch.  Otherwise, the branches are said to have *diverged*,
+and they must be merged instead.

